@@ -10,10 +10,14 @@ setAs("Matrix", "denseMatrix",  function(from) as_dense(from))
 ## Maybe TODO:
 ## setAs("Matrix", "nMatrix", function(from) ....)
 
+## Anything: we build on  as.matrix(.) :
+## ---       authors can always provide their own specific  setAs(*, "Matrix")
+setAs("ANY", "Matrix", function(from) Matrix(as.matrix(from)))
+
 ## Most of these work; this is a last resort:
-setAs(from = "Matrix", to = "matrix", # do *not* call base::as.matrix() here:
+setAs("Matrix", "matrix", # do *not* call base::as.matrix() here:
       function(from) .bail.out.2("coerce", class(from), class(to)))
-setAs(from = "matrix", to = "Matrix", function(from) Matrix(from))
+setAs("matrix", "Matrix", function(from) Matrix(from))
 
 ## ## probably not needed eventually:
 ## setAs(from = "ddenseMatrix", to = "matrix",
@@ -22,12 +26,28 @@ setAs(from = "matrix", to = "Matrix", function(from) Matrix(from))
 ## 	  array(from@x, dim = d, dimnames = dimnames(from))
 ##       })
 
-## should propagate to all subclasses:
-setMethod("as.matrix", signature(x = "Matrix"), function(x) as(x, "matrix"))
-## for 'Matrix' objects, as.array() should be equivalent:
-setMethod("as.array",  signature(x = "Matrix"), function(x) as(x, "matrix"))
+.asmatrix <- function(x) as(x, "matrix") # not better; just for those hating typing
+
 ## Such that also base functions dispatch properly on our classes:
-as.array.Matrix <- as.matrix.Matrix <- function(x, ...) as(x, "matrix")
+if(.Matrix.avoiding.as.matrix) {
+    as.matrix.Matrix <- function(x, ...) {
+        if(nonTRUEoption("Matrix.quiet.as.matrix") && nonTRUEoption("Matrix.quiet"))
+            warning("as.matrix(<Matrix>) is deprecated (to become a no-op in the future).
+Use  as(x, \"matrix\")  or .asmatrix(x) instead.")
+        as(x, "matrix")
+    }
+    as.array.Matrix <- function(x, ...) {
+        warning("as.array(<Matrix>) is deprecated. Use  as(x, \"matrix\")  or .asmatrix(x) instead.")
+        as(x, "matrix")
+    }
+} else { ## regularly -- documented since 2005 that this works
+    as.array.Matrix <- as.matrix.Matrix <- function(x, ...) as(x, "matrix")
+}
+
+## should propagate to all subclasses:
+setMethod("as.matrix", signature(x = "Matrix"), function(x) as.matrix.Matrix(x))
+## for 'Matrix' objects, as.array() should be equivalent:
+setMethod("as.array",  signature(x = "Matrix"), function(x)  as.array.Matrix(x))
 
 ## head and tail apply to all Matrix objects for which subscripting is allowed:
 setMethod("head", signature(x = "Matrix"), utils::head.matrix)
@@ -37,11 +57,21 @@ setMethod("drop", signature(x = "Matrix"),
 	  function(x) if(all(dim(x) != 1)) x else drop(as(x, "matrix")))
 
 ## slow "fall back" method {subclasses should have faster ones}:
-setMethod("as.vector", signature(x = "Matrix", mode = "missing"),
+setMethod("as.vector", "Matrix",
 	  function(x, mode) as.vector(as(x, "matrix"), mode))
 ## so base functions calling as.vector() work too:
+## S3 dispatch works for base::as.vector(), but S4 dispatch does not
 as.vector.Matrix <- function(x, mode) as.vector(as(x, "matrix"), mode)
 
+if(FALSE) { ## still does not work for c(1, Matrix(2))
+## For the same reason (and just in case) also do both S3 and S4 here:
+c.Matrix <- function(...) unlist(lapply(list(...), as.vector))
+## NB: Must use   signature  '(x, ..., recursive = FALSE)' :
+setMethod("c", "Matrix", function(x, ..., recursive) c.Matrix(x, ...))
+## The above is not sufficient for  c(NA, 3:2, <Matrix>, <matrix>)
+setMethod("c", "numMatrixLike", function(x, ..., recursive) c.Matrix(x, ...))
+}# not yet
+
 setAs("Matrix", "vector",  function(from) as.vector (as(from, "matrix")))
 setAs("Matrix", "numeric", function(from) as.numeric(as(from, "matrix")))
 setAs("Matrix", "logical", function(from) as.logical(as(from, "matrix")))
@@ -110,18 +140,17 @@ setMethod("isTriangular", signature(object = "matrix"), isTriMat)
 setMethod("isDiagonal", signature(object = "matrix"), .is.diagonal)
 
 ## The "catch all" methods -- far from optimal:
-setMethod("symmpart", signature(x = "Matrix"),
-	  function(x) as((x + t(x))/2, "symmetricMatrix"))
-setMethod("skewpart", signature(x = "Matrix"),
-	  function(x) (x - t(x))/2)
+setMethod("symmpart", signature(x = "Matrix"), function(x)
+    as(symmetrizeDimnames(x + t(x))/2, "symmetricMatrix"))
+setMethod("skewpart", signature(x = "Matrix"), function(x) symmetrizeDimnames(x - t(x))/2)
 
 ## FIXME: do this (similarly as for "ddense.." in C
-setMethod("symmpart", signature(x = "matrix"), function(x) (x + t(x))/2)
-setMethod("skewpart", signature(x = "matrix"), function(x) (x - t(x))/2)
+setMethod("symmpart", signature(x = "matrix"), function(x) symmetrizeDimnames(x + t(x))/2)
+setMethod("skewpart", signature(x = "matrix"), function(x) symmetrizeDimnames(x - t(x))/2)
 
 
-if(getRversion() > "3.1.0")
-## NB: "nsparseMatrix" has extra method
+if(getRversion() >= "3.1.0")
+## NB: ./nsparseMatrix.R and ./sparseVector.R have extra methods
 setMethod("anyNA", signature(x = "xMatrix"),
 	  function(x) anyNA(x@x))
 
@@ -143,19 +172,30 @@ dimnamesGets <- function (x, value) {
 	!(is.null(v2 <- value[[2]]) || length(v2) == d[2]))
 	stop(gettextf("invalid dimnames given for %s object", dQuote(class(x))),
 	     domain=NA)
-    x@Dimnames <- list(if(!is.null(v1)) as.character(v1),
-		       if(!is.null(v2)) as.character(v2))
+    x@Dimnames <- .fixupDimnames(value)
     x
 }
-setMethod("dimnames<-", signature(x = "Matrix", value = "list"),
-	  dimnamesGets)
+dimnamesGetsNULL <- function(x) {
+    message("dimnames(.) <- NULL:  translated to \ndimnames(.) <- list(NULL,NULL)  <==>  unname(.)")
+    x@Dimnames <- list(NULL,NULL)
+    x
+}
+setMethod("dimnames<-", signature(x = "compMatrix", value = "list"),
+          function(x, value) { ## "compMatrix" have 'factors' slot
+              if(length(x@factors)) x@factors <- list()
+              dimnamesGets(x, value)
+          })
+setMethod("dimnames<-", signature(x = "Matrix", value = "list"), dimnamesGets)
+
+setMethod("dimnames<-", signature(x = "compMatrix", value = "NULL"),
+          function(x, value) { ## "compMatrix" have 'factors' slot
+              if(length(x@factors)) x@factors <- list()
+              dimnamesGetsNULL(x)
+          })
 
 setMethod("dimnames<-", signature(x = "Matrix", value = "NULL"),
-	  function(x, value) {
-	      message("dimnames(.) <- NULL:  translated to \ndimnames(.) <- list(NULL,NULL)  <==>  unname(.)")
-	      x@Dimnames <- list(NULL,NULL)
-	      x
-	  })
+	  function(x, value) dimnamesGetsNULL(x))
+
 
 setMethod("unname", signature("Matrix", force="missing"),
 	  function(obj) { obj@Dimnames <- list(NULL,NULL); obj})
@@ -165,8 +205,6 @@ Matrix <- function (data = NA, nrow = 1, ncol = 1, byrow = FALSE,
                     dimnames = NULL, sparse = NULL,
                     doDiag = TRUE, forceCheck = FALSE)
 {
-    sparseDefault <- function(m) prod(dim(m)) > 2*sum(isN0(as(m, "matrix")))
-
     i.M <- is(data, "Matrix")
     sM <- FALSE
     if(i.M) {
@@ -180,7 +218,7 @@ Matrix <- function (data = NA, nrow = 1, ncol = 1, byrow = FALSE,
     }
     if(is.null(sparse1 <- sparse) && (i.M || is(data, "matrix")))
 	sparse <- sparseDefault(data)
-    doDN <- TRUE
+    doDN <- TRUE # by default
     if (i.M) {
 	if (!sV) {
 	    if(!missing(nrow) || !missing(ncol)|| !missing(byrow))
@@ -191,7 +229,7 @@ Matrix <- function (data = NA, nrow = 1, ncol = 1, byrow = FALSE,
 	    ## else : convert  dense <-> sparse -> at end
 	}
     }
-    else if (!is.matrix(data)) { ## cut & paste from "base::matrix" :
+    else if(!is.matrix(data)) { ## cut & paste from "base::matrix" :
 	## avoid copying to strip attributes in simple cases
 	if (is.object(data) || !is.atomic(data)) data <- as.vector(data)
 	if(length(data) == 1 && is0(data) && !identical(sparse, FALSE)) {
@@ -211,17 +249,14 @@ Matrix <- function (data = NA, nrow = 1, ncol = 1, byrow = FALSE,
 			Dimnames = if(is.null.DN(dimnames)) list(NULL,NULL)
 			else dimnames)
 	} else { ## normal case
-	    ## Now 'forbidden' :
-	    ## data <- .Internal(matrix(data, nrow, ncol, byrow, dimnames,
-	    ##				missing(nrow), missing(ncol)))
 	    data <- .External(Mmatrix,
 			      data, nrow, ncol, byrow, dimnames,
 			      missing(nrow), missing(ncol))
 	    if(is.null(sparse))
 		sparse <- sparseDefault(data)
 	}
-        doDN <- FALSE
-    } else if(!missing(nrow) || !missing(ncol)|| !missing(byrow))
+        doDN <- FALSE # .. set above
+    } else if(!missing(nrow) || !missing(ncol)|| !missing(byrow)) ## i.m == is.matrix(.)
 	warning("'nrow', 'ncol', etc, are disregarded for matrix 'data'")
 
     ## 'data' is now a "matrix" or "Matrix"
@@ -260,12 +295,11 @@ Matrix <- function (data = NA, nrow = 1, ncol = 1, byrow = FALSE,
     }
 
     if(isTri && !is(data, "triangularMatrix")) {
-	data <- if(attr(isTri,"kind") == "L") tril(data) else triu(data)
-					#was as(data, "triangularMatrix")
+	if(attr(isTri,"kind") == "L") tril(data) else triu(data)
     } else if(isSym && !is(data, "symmetricMatrix"))
-	data <- forceSymmetric(data) #was as(data, "symmetricMatrix")
-
-    data
+	forceSymmetric(data)
+    else
+	data
 }
 
 ## Methods for operations where one argument is numeric
@@ -294,12 +328,12 @@ setMethod("chol2inv", signature(x = "denseMatrix"),
 	  function (x, ...) chol2inv(as(as(x, "dMatrix"), "dtrMatrix"), ...))
 setMethod("chol2inv", signature(x = "diagonalMatrix"),
 	  function (x, ...) {
-	      chk.s(...)
+	      chk.s(..., which.call=-2)
 	      tcrossprod(solve(x))
 	  })
 setMethod("chol2inv", signature(x = "sparseMatrix"),
 	  function (x, ...) {
-	      chk.s(...)
+	      chk.s(..., which.call=-2)
 	      ## for now:
 	      tcrossprod(solve(as(x,"triangularMatrix")))
 	  })
@@ -350,6 +384,9 @@ setMethod("diag<-", signature(x = "Matrix"),
 setMethod("t", signature(x = "Matrix"),
 	  function(x) .bail.out.1(.Generic, class(x)))
 
+## NB: "sparseMatrix" works via "sparseVector"
+setMethod("rep", "Matrix", function(x, ...) rep(as(x, "matrix"), ...))
+
 setMethod("norm", signature(x = "Matrix", type = "character"),
 	  function(x, type, ...) .bail.out.1("norm", class(x)))
 setMethod("rcond", signature(x = "Matrix", norm = "character"),
@@ -363,7 +400,7 @@ setMethod("rcond", signature(x = "ANY", norm = "missing"),
 	  function(x, norm, ...) rcond(x, norm = "O", ...))
 
 setMethod("lu", "matrix", function(x, warnSing = TRUE, ...)
-	  lu(as(x, "dgeMatrix"), warnSing=warnSing, ...))
+	  lu(..2dge(x), warnSing=warnSing, ...))
 
 
 
@@ -422,92 +459,6 @@ setMethod("image", "Matrix",
 
 ## Group Methods
 
-## --- "Summary" ------- have "ddense*" and "dsparse*" ones in ---> ./dMatrix.R <---
-##      -------          "diagMatrix" --> ./diagMatrix.R            ~~~~~~~~~~~
-## For all other Matrix objects {and note that  "all" and "any" have their own}:
-
-setMethod("all", signature(x = "Matrix"),
-	  function(x, ..., na.rm)
-	  callGeneric(as(x, "lMatrix"), ..., na.rm=na.rm))
-
-setMethod("any", signature(x = "Matrix"),
-	  function(x, ..., na.rm)
-	  callGeneric(as(x, "lMatrix"), ..., na.rm=na.rm))
-
-setMethod("Summary", signature(x = "Matrix", na.rm = "ANY"),
-	  function(x, ..., na.rm)
-	  callGeneric(as(x,"dMatrix"), ..., na.rm = na.rm))
-
-## Try to make   min(1, <Matrix>)  work, i.e., not dispatch on first arg to .Primitive
-## This for(..) gives {during installation}
-## Error in setGeneric(F, signature = "...") :
-##   ‘max’ is a primitive function;  methods can be defined, but the generic function is implicit, and cannot be changed.
-if(FALSE)
-for(F in c("max", "min", "range", "prod", "sum", "any", "all")) {
-    setGeneric(F, signature = "...")
-}
-### try on "min" for now --- ~/R/Pkgs/Rmpfr/R/mpfr.R is the example (for "pmin")
-if(FALSE)### This gives error message that the "ANY" is method is sealed ...
-setMethod("min", "ANY",
-	  function(..., na.rm = FALSE) {
-	      args <- list(...)
-	      if(all(isAtm <- vapply(args, is.atomic, NA)))
-		  return( base::min(..., na.rm = na.rm) )
-              ## else try to dispatch on an argument which is a Matrix.. or in a
-              if(any(isM <- vapply(args, is, NA, class2="Matrix"))) {
-                  ## swap the Matrix with the first argument
-                  i <- which.max(isM)# the first "Matrix"
-                  if(i == 1)
-                      stop("programming error: min() should have dispatched w/ 1st arg much earlier")
-              } else { ## if no "Matrix", take the first non-atomic argument
-                  ## (FIXME: should take the first for which there is a method !)
-                  i <- which.max(!isAtm)
-              }
-              ii <- seq_along(args)
-              ii[c(1,i)] <- c(i,1)
-              do.call(min, c(args[ii], list(na.rm=na.rm)))
-          })
-
-if(FALSE) { ## FIXME: it does *not* solve the problem anyway ..
-##
-##  (m <- Matrix(c(0,0,2:0), 3,5))
-##   min(1,m)
-##-> error, as it calls the .Primitive min() and that does *not* dispatch on 2nd arg
-##
-setMethod("Summary", signature(x = "ANY", na.rm = "ANY"),
-	  function(x, ..., na.rm) {
-          if(!length(a <- list(...))) (get(.Generic, envir=baseenv()))(x, na.rm=na.rm)
-          else {
-                 if(!is.null(v <- getOption("Matrix.verbose")) && v >= 1)
-		     message(sprintf("in Summary(<ANY>, .): %s(<%s>, <%s>%s)\n",
-				     .Generic, class(x), class(a[[1]]), if(length(a) > 1)", ..." else ""),
-			     domain = NA)
-                 do.call(.Generic, c(x, a, list(na.rm=na.rm)))
-             }})
-}
-
-Summary.l <- function(x, ..., na.rm) { ## must be method directly
-    if(.Generic %in% c("all", "any"))
-	callGeneric(x@x, ..., na.rm = na.rm)
-    else {
-	r <- callGeneric(as(x,"dMatrix"), ..., na.rm = na.rm)
-	if(!is.infinite(r) && .Generic != "prod") as.integer(r) else r
-    }
-}
-## almost identical:
-Summary.np <- function(x, ..., na.rm) {
-    if(.Generic %in% c("all", "any"))
-	callGeneric(as(x, "lMatrix"), ..., na.rm = na.rm)
-    else {
-	r <- callGeneric(as(x,"dMatrix"), ..., na.rm = na.rm)
-	if(!is.infinite(r) && .Generic != "prod") as.integer(r) else r
-    }
-}
-##
-setMethod("Summary", signature(x = "lMatrix", na.rm = "ANY"), Summary.l)
-setMethod("Summary", signature(x = "nMatrix", na.rm = "ANY"), Summary.np)
-setMethod("Summary", signature(x = "indMatrix", na.rm = "ANY"), Summary.np)
-
 ## NOTE:  "&" and "|"  are now in group "Logic" c "Ops" --> ./Ops.R
 ##        "!" is in ./not.R
 
@@ -535,7 +486,7 @@ setMethod("[", signature(x = "Matrix", i = "index", j = "missing",
 			 drop = "missing"),
 	  function(x,i,j, ..., drop) {
 	      Matrix.msg("M[i,m,m] : nargs()=",nargs(), .M.level = 2)
-	      if(nargs() == 2) { ## e.g. M[0] , M[TRUE],  M[1:2]
+	      if(nargs() == 2) { ## e.g. M[0] , M[TRUE], M[1:2], M[-7]
                   .M.vectorSub(x,i)
 	      } else {
 		  callGeneric(x, i=i, , drop=TRUE)
@@ -548,8 +499,9 @@ setMethod("[", signature(x = "Matrix", i = "missing", j = "index",
 			 drop = "missing"),
 	  function(x,i,j, ..., drop) {
 	      Matrix.msg("M[m,i,m] : nargs()=",nargs(), .M.level = 2)
-	      callGeneric(x, j=j, drop= TRUE)
+	      callGeneric(x, , j=j, drop= TRUE)
 	  })
+## select both rows *and* columns
 setMethod("[", signature(x = "Matrix", i = "index", j = "index",
 			 drop = "missing"),
 	  function(x,i,j, ..., drop) {
@@ -581,7 +533,7 @@ setMethod("[", signature(x = "Matrix", i = "ANY", j = "ANY", drop = "ANY"),
 
 	## Note: current method dispatch seems not to call this ever
 
-	if(!any(is.na(i)) && all(i)) ## select everything
+	if(length(i) && !anyNA(i) && all(i)) ## select everything
 	    x
 	else ## not selecting all -> result is *NOT* diagonal/triangular/symmetric/..
 	    ## keep j missing, but  drop = "logical"
@@ -599,8 +551,9 @@ for(ii in c("lMatrix", "logical"))
 rm(ii)
 
 
+##' x[ ij ]  where ij is (i,j) 2-column matrix
+##' @note only called from  .M.sub.i.2col(x, i) below
 subset.ij <- function(x, ij) {
-    ## x[ ij ]  where ij is (i,j) 2-column matrix
     m <- nrow(ij)
     if(m > 3) {
         cld <- getClassDef(class(x))
@@ -628,22 +581,21 @@ subset.ij <- function(x, ij) {
 		ij.x <- non0.i(x, cld)
 	    }
 
-	    m1 <- .Call(m_encodeInd, ij.x, di, checkBounds = FALSE)
-            m2 <- .Call(m_encodeInd, ij -1L, di, checkBounds = TRUE)
-	    mi <- match(m1, m2, nomatch=0)
-	    mmi <- mi != 0
-	    ## Result:
+	    m1 <- .Call(m_encodeInd, ij.x, di, orig1=FALSE, checkBounds=FALSE)
+            m2 <- .Call(m_encodeInd, ij,   di, orig1= TRUE, checkBounds= TRUE)
+	    mi <- match(m2, m1, nomatch=0)
+	    mmi <- mi != 0L ## == (m2 %in% m1)
+	    ## Result: all FALSE or 0  apart from where we match non-zero entries
 	    ans <- vector(mode = .type.kind[.M.kindC(cld)], length = m)
 	    ## those that are *not* zero:
-	    ans[mi[mmi]] <-
-		if(extends(cld, "nsparseMatrix")) TRUE else x@x[mmi]
+	    ans[mmi] <- if(extends(cld, "nsparseMatrix")) TRUE else x@x[mi[mmi]]
 	    if(any(ina <- is.na(m2))) # has one or two NA in that (i,j) row
 		is.na(ans) <- ina
 	    ans
 
         } else { ## non-sparse : dense
             ##---- NEVER happens:  'denseMatrix' has its own setMethod(.) !
-            message("m[ <ij-matrix> ]: inefficiently indexing single elements")
+            message("m[<ij-matrix>]: inefficiently indexing single elements - should not happen, please report!")
             i1 <- ij[,1]
             i2 <- ij[,2]
             ## very inefficient for large m
@@ -660,21 +612,20 @@ subset.ij <- function(x, ij) {
 .M.sub.i.2col <- function (x, i, j, ..., drop)
 {
     nA <- nargs()
-    if(nA == 2) { ##  M [ cbind(ii,jj) ] or M [ <logical matrix> ]
-	if(!is.integer(nc <- ncol(i)))
-	    stop(".M.sub.i.2col(): 'i' has no integer column number;\n should never happen; please report")
-	if(is.logical(i))
-	    return(.M.sub.i.logical(x, i=i)) # call with 2 args!
-	else if(!is.numeric(i) || nc != 2)
-	    stop("such indexing must be by logical or 2-column numeric matrix")
-	m <- nrow(i)
-        if(m == 0) return(vector(mode = .type.kind[.M.kind(x)]))
-        ## else
-        subset.ij(x, i)
+    if(nA != 2)
+        stop(domain=NA, gettextf(
+            "nargs() = %d.  Extraneous illegal arguments inside '[ .. ]' (i.2col)?", nA))
+    ## else: (nA == 2):	 M [ cbind(ii,jj) ] or M [ <logical matrix> ]
+    if(!is.integer(nc <- ncol(i)))
+        stop(".M.sub.i.2col(): 'i' has no integer column number;\n should never happen; please report")
+    if(is.logical(i))
+        return(.M.sub.i.logical(x, i=i)) # call with 2 args!
+    else if(!is.numeric(i) || nc != 2)
+        stop("such indexing must be by logical or 2-column numeric matrix")
+    if(!nrow(i)) return(vector(mode = .type.kind[.M.kind(x)]))
+    ## else
+    subset.ij(x, i)
 
-    } else stop(gettextf(
-		"nargs() = %d.  Extraneous illegal arguments inside '[ .. ]' (i.2col)?",
-			 nA), domain=NA)
 }
 setMethod("[", signature(x = "Matrix", i = "matrix", j = "missing"),# drop="ANY"
 	  .M.sub.i.2col)
@@ -687,7 +638,8 @@ setMethod("[", signature(x = "Matrix", i = "matrix", j = "missing", drop="missin
 
 ## A[ ij ] <- value,  where ij is (i,j) 2-column matrix :
 ## ----------------
-## The cheap general method --- FIXME: provide special ones; done for Tsparse..
+## The cheap general method, now only used for "pMatrix","indMatrix"
+## sparse all use  .TM.repl.i.mat()
 ## NOTE:  need '...' below such that setMethod() does
 ##	  not use .local() such that nargs() will work correctly:
 .M.repl.i.2col <- function (x, i, j, ..., value)
@@ -706,7 +658,7 @@ setMethod("[", signature(x = "Matrix", i = "matrix", j = "missing", drop="missin
 	if(!is.integer(i)) storage.mode(i) <- "integer"
 	if(any(i < 0))
 	    stop("negative values are not allowed in a matrix subscript")
-	if(any(is.na(i)))
+	if(anyNA(i))
 	    stop("NAs are not allowed in subscripted assignments")
 	if(any(i0 <- (i == 0))) # remove them
             i <- i[ - which(i0, arr.ind = TRUE)[,"row"], ]
@@ -716,7 +668,7 @@ setMethod("[", signature(x = "Matrix", i = "matrix", j = "missing", drop="missin
 	if(length(value) > 0 && m %% length(value) != 0)
 	    warning("number of items to replace is not a multiple of replacement length")
 	## recycle:
-	value <- rep(value, length = m)
+	value <- rep_len(value, m)
 	i1 <- i[,1]
 	i2 <- i[,2]
 	if(m > 2)
@@ -745,7 +697,10 @@ setReplaceMethod("[", signature(x = "Matrix", i = "missing", j = "ANY",
 setReplaceMethod("[", signature(x = "Matrix", i = "ANY", j = "missing",
 				value = "Matrix"),
 		 function (x, i, j, ..., value)
-		 callGeneric(x=x, i=i, , value = as.vector(value)))
+		     if(nargs() == 3)
+			 callGeneric(x=x, i=i, value = as.vector(value))
+		     else
+			 callGeneric(x=x, i=i, , value = as.vector(value)))
 
 setReplaceMethod("[", signature(x = "Matrix", i = "ANY", j = "ANY",
 				value = "Matrix"),
@@ -761,7 +716,10 @@ setReplaceMethod("[", signature(x = "Matrix", i = "missing", j = "ANY",
 setReplaceMethod("[", signature(x = "Matrix", i = "ANY", j = "missing",
 				value = "matrix"),
 		 function (x, i, j, ..., value)
-		 callGeneric(x=x, i=i, , value = c(value)))
+		     if(nargs() == 3)
+			 callGeneric(x=x, i=i, value = c(value))
+		     else
+			 callGeneric(x=x, i=i, , value = c(value)))
 
 setReplaceMethod("[", signature(x = "Matrix", i = "ANY", j = "ANY",
 				value = "matrix"),
