@@ -1,5 +1,7 @@
-#### ------- All "Ops"  group methods for all Matrix classes ------------
-###               ===
+####--- All "Ops" group methods for all Matrix classes (incl sparseVector) ------
+####         ===   but diagonalMatrix  -> ./diagMatrix.R and  abIndex.R
+####                                        ~~~~~~~~~~~~      ~~~~~~~~~
+
 ### Note that the "Ops" group consists of
 ### sub-groups   "Arith", "Compare", and "Logic"
 ###               -----    -------        -----
@@ -8,25 +10,24 @@
 ###       'Logic'   :=  '"&"', '"|"'  (( but *not* '"!"' since that has
 ###			                 only one argument ))
 
-## in shell, find them with
-##    egrep 'Method\("(Ops|Compare|Arith|Logic)"' R/*R
+## cache them [rather in package 'methods' ??]
+.ArithGenerics   <- getGroupMembers("Arith")
+.CompareGenerics <- getGroupMembers("Compare")
+.LogicGenerics   <- getGroupMembers("Logic")
 
-### "Ops" ---- remember Ops = {Arith, Compare, Logic}  [Logic: since R 2.4.1]
-### -----
-
-
-### Note: diagonalMatrix are handled by special methods -> ./diagMatrix.R
-###       --------------                                   ~~~~~~~~~~~~~~
+## find them with M-x grep -E 'Method\("(Ops|Compare|Arith|Logic)"' *.R
+##                --------
 
 ### Design decision for *sparseMatrix*:
 ### work via Csparse  since Tsparse are not-unique (<-> slots not compatible)
 
+### Dimnames: (partly) via  dimNamesCheck()  [ ./Auxiliaries.R ]
 
 ### --  0 -- (not dense *or* sparse) -----------------------------------
 
 ##-------- originally from ./Matrix.R --------------------
 
-## Some ``Univariate'' "Arith":
+## Some ``Univariate'' "Arith" (univariate := 2nd argument 'e2' is missing)
 setMethod("+", signature(e1 = "Matrix", e2 = "missing"), function(e1,e2) e1)
 
 ## "fallback":
@@ -67,14 +68,36 @@ setMethod("Ops", signature(e1 = "matrix", e2 = "Matrix"),
 ## Note: things like  callGeneric(Matrix(e1, sparse=is(e2,"sparseMatrix")), e2))
 ##   may *not* be better: e.g. Matrix(.) can give *diagonal* instead of sparse
 
+##  NULL  should be treated as logical(0) {which often will be coerced to numeric(0)}:
+setMethod("Ops", signature(e1 = "Matrix", e2 = "NULL"),
+	  function(e1, e2) callGeneric(e1, logical()))
+setMethod("Ops", signature(e1 = "NULL", e2 = "Matrix"),
+	  function(e1, e2) callGeneric(logical(), e2))
+
 ## bail-outs -- on highest possible level, hence "Ops", not "Compare"/"Arith" :
 .bail.out.Ops <- function(e1, e2) {
-    if(is(e1, "Matrix") && is(e2, "Matrix"))
-        dimCheck(e1,e2)
+    if(is(e1, "mMatrix") && is(e2, "mMatrix"))
+	dimCheck(e1,e2)
     .bail.out.2(.Generic, class(e1), class(e2))
-          }
-setMethod("Ops", signature(e1 = "Matrix", e2 = "ANY"), .bail.out.Ops)
-setMethod("Ops", signature(e1 = "ANY", e2 = "Matrix"), .bail.out.Ops)
+}
+setMethod("Ops", signature(e1 = "Matrix", e2 = "ANY"),
+	  function(e1, e2) {
+	      if(is(e1, "mMatrix") && is(e2, "mMatrix")) dimCheck(e1,e2)
+	      if(is.matrix(e2) && identical(e2, as.matrix(e2)) &&
+		 is.object(e2) && !isS4(e2)) # e.g. for "table"
+		  callGeneric(e1, unclass(e2))
+	      else
+		  .bail.out.2(.Generic, class(e1), class(e2))
+	  })
+setMethod("Ops", signature(e1 = "ANY", e2 = "Matrix"),
+	  function(e1, e2) {
+	      if(is(e1, "mMatrix") && is(e2, "mMatrix")) dimCheck(e1,e2)
+	      if(is.matrix(e1) && identical(e1, as.matrix(e1)) &&
+		 is.object(e1) && !isS4(e1)) # e.g. for "table"
+		  callGeneric(unclass(e1), e2)
+	      else
+		  .bail.out.2(.Generic, class(e1), class(e2))
+	  })
 rm(.bail.out.Ops)
 
 ## "General principle"
@@ -104,7 +127,7 @@ setMethod("Compare", signature(e1 = "Matrix", e2 = "Matrix"),
 ## can be done for matching-dim "*geMatrix", and also
 ## matching-{dim + uplo} for *packed* (only!) symmetric+triangular
 .Ops.via.x <- function(e1,e2) {
-    d <- dimCheck(e1, e2)
+    dimCheck(e1, e2)
     e1@x <- callGeneric(e1@x, e2@x)
     e1
 }
@@ -125,20 +148,27 @@ setMethod("Compare", signature(e1 = "logical", e2 = "nMatrix"), .Cmp.swap)
 
 ## This is parallel to Logic.Mat.atomic() below --->  __keep parallel__ !
 Cmp.Mat.atomic <- function(e1, e2) { ## result will inherit from "lMatrix"
-    d <- e1@Dim
+    n1 <- prod(d <- e1@Dim)
     cl <- class(e1)
-    if((l2 <- length(e2)) == 0) {
-	if(prod(d) == 0)
-	    return(new(class2(cl, "l"), Dim= d))
-	else
-	    stop(gettextf("<Matrix> %s %s is undefined",
-			  .Generic, paste0(class(e2),"(0)")), domain=NA)
-    }
+    if((l2 <- length(e2)) == 0)
+	return(if(n1 == 0)
+		   as(e1, class2(cl, "l"))# more expensive than (but working for "dgC*"):
+		   ## new(class2(cl, "l"), Dim = d, Dimnames = e1@Dimnames)
+               else
+		   as.logical(e2))
+    ## else
+    if(n1 && n1 < l2)
+	stop(sprintf(
+	    "dim [product %d] do not match the length of object [%d]", n1, l2))
     cl1 <- getClassDef(cl)
     slots1 <- names(cl1@slots)
     has.x <- any("x" == slots1)# *fast* check for "x" slot presence
-    if(l2 > 1 && has.x) ## e2 cannot simply be compared with e1@x --> use another method
-        return(callGeneric(e1, Matrix(e2, nrow=d[1], ncol=d[2])))
+    if(l2 > 1 && has.x)
+	return(if(n1 == 0)
+		   new(class2(cl, "l"), x = callGeneric(e1@x, e2),
+		       Dim = d, Dimnames = e1@Dimnames)
+	       else ## e2 cannot simply be compared with e1@x --> use another method
+		   callGeneric(e1, Matrix(e2, nrow=d[1], ncol=d[2])))
     ## else
     Udg <- extends(cl1, "triangularMatrix") && e1@diag == "U"
     r0 <- callGeneric(0, e2)
@@ -149,7 +179,7 @@ Cmp.Mat.atomic <- function(e1, e2) { ## result will inherit from "lMatrix"
         r <- new(if(d[1] == d[2]) "lsyMatrix" else "lgeMatrix")
         r@Dim <- d
         r@Dimnames <- e1@Dimnames
-        r@x <- rep.int(TRUE, prod(d))
+        r@x <- rep.int(TRUE, n1)
     }
     else if(extends(cl1, "denseMatrix")) {
 	full <- !.isPacked(e1)	   # << both "dtr" and "dsy" are 'full'
@@ -160,7 +190,7 @@ Cmp.Mat.atomic <- function(e1, e2) { ## result will inherit from "lMatrix"
                     cl1 <- getClassDef(cl <- class(e1 <- as(e1, "dtrMatrix")))
             }
             ## FIXME? using copyClass() to copy "relevant" slots
-            r <- new(class2(cl, "l"), x = r, Dim = d, Dimnames = dimnames(e1))
+            r <- new(class2(cl, "l"), x = r, Dim = d, Dimnames = e1@Dimnames)
             if(extends(cl1, "symmetricMatrix")) {
                 r@uplo <- e1@uplo
             } else if(isTri) {
@@ -173,9 +203,9 @@ Cmp.Mat.atomic <- function(e1, e2) { ## result will inherit from "lMatrix"
             ## [dense & packed & not symmetric ] ==> must be "dtp*" :
             if(!extends(cl1, "dtpMatrix"))
                 stop("internal bug in \"Compare\" method (Cmp.Mat.atomic); please report")
-	    rx <- rep_len(r0, prod(d))
+	    rx <- rep_len(r0, n1)
 	    rx[indTri(d[1], upper = (e1@uplo == "U"), diag=TRUE)] <- r
-	    r <- new("lgeMatrix", x = rx, Dim = d, Dimnames = dimnames(e1))
+	    r <- new("lgeMatrix", x = rx, Dim = d, Dimnames = e1@Dimnames)
 	}
 
     }
@@ -199,11 +229,11 @@ Cmp.Mat.atomic <- function(e1, e2) { ## result will inherit from "lMatrix"
         }
 
         if(remainSparse) {
-            if(!any(is.na(r)) && ((Ar <- all(r)) || !any(r))) {
+            if(!anyNA(r) && ((Ar <- all(r)) || !any(r))) {
                 lClass <- class2(cl, "l") # is "lsparse*"
                 r <- new(lClass)
                 r@Dim <- d
-                r@Dimnames <- dimnames(e1)
+                r@Dimnames <- e1@Dimnames
                 if(Ar) {       # 'TRUE' instead of 'x': same sparsity:
 		    for(n in intersect(c("i","j","p","uplo","diag"), slots1))
 			slot(r, n) <- slot(e1, n)
@@ -235,14 +265,14 @@ Cmp.Mat.atomic <- function(e1, e2) { ## result will inherit from "lMatrix"
             lClass <- if(extends(cl1, "symmetricMatrix")) "lsyMatrix" else "lgeMatrix"
             Matrix.msg(sprintf("sparse to dense (%s) coercion in '%s' -> %s",
                                lClass, .Generic, "Cmp.Mat.atomic"), .M.level = 2)
-	    rx <- rep_len(r0, prod(d))
+	    rx <- rep_len(r0, n1)
 
             ## Here, we assume that 'r' and the indices align (!)
             encI <- .Call(m_encodeInd,
                           non0ind(e1, cl1, uniqT=FALSE, xtendSymm=FALSE),
                           di = d, orig1=FALSE, checkBounds=FALSE)
             rx[1L + encI] <- r
-            r <- new(lClass, x = rx, Dim = d, Dimnames = dimnames(e1))
+            r <- new(lClass, x = rx, Dim = d, Dimnames = e1@Dimnames)
         }
     }
     r
@@ -272,8 +302,8 @@ Ops.x.x <- function(e1, e2)
 		le <- prod(d)
 		isPacked <- function(x) length(x@x) < le
 		Mclass <-
-		    if(sym <- extends(c1, "symmetricMatrix") &&
-			      extends(c2, "symmetricMatrix")) {
+		    if(extends(c1, "symmetricMatrix") &&
+		       extends(c2, "symmetricMatrix")) {
 			if(e1@uplo != e2@uplo)
 			    ## one is upper, one is lower
 			    e2 <- t(e2)
@@ -290,8 +320,8 @@ Ops.x.x <- function(e1, e2)
 			} else
 			    "syMatrix"
 		    }
-		    else if(tri <- extends(c1, "triangularMatrix") &&
-				   extends(c2, "triangularMatrix")) {
+		    else if(extends(c1, "triangularMatrix") &&
+			    extends(c2, "triangularMatrix")) {
 			if(!(geM <- e1@uplo != e2@uplo || isN0(callGeneric(0,0)))) {
 			    p1 <- isPacked(e1)
 			    p2 <- isPacked(e2)
@@ -310,7 +340,7 @@ Ops.x.x <- function(e1, e2)
 				"trMatrix"
 			}
 		    }
-		    else {
+		    else { ## not symmetric, not triangular  ==> "general"
 			geM <- TRUE
 		    }
 		if(geM)
@@ -323,10 +353,13 @@ Ops.x.x <- function(e1, e2)
 	}
 	## now, in all cases @x should be matching & correct {only "uplo" part is used}
 	r <- callGeneric(e1@x, e2@x)
+	kr <- .M.kind(r)
+	if(kr == "d" && !is.double(r)) ## as "igeMatrix" does not yet exist!
+	    r <- as.double(r)
 	if(geM)
-	    new(paste0(.M.kind(r), "geMatrix"), x = r, Dim = d, Dimnames = dimnames(e1))
+	    new(paste0(kr, "geMatrix"), x = r, Dim = d, Dimnames = e1@Dimnames)
 	else
-	    new(paste0(.M.kind(r), Mclass), x = r, Dim = d, Dimnames = dimnames(e1), uplo = e1@uplo)
+	    new(paste0(kr, Mclass), x = r, Dim = d, Dimnames = e1@Dimnames, uplo = e1@uplo)
     }
     else {
 	r <- if(!dens1 && !dens2)
@@ -376,22 +409,26 @@ setMethod("Ops", signature(e1="numeric", e2="nMatrix"),
 for(cl in c("numeric", "logical")) { # "complex", "raw" : basically "replValue"
 
 setMethod("Arith", signature(e1 = cl, e2 = "dpoMatrix"),
-	  function(e1, e2) if(length(e1) <= 1 && any(.Generic == c("*","/","+")) && (e1 > 0)) {
+	  function(e1, e2) if(!(l1 <- length(e1))) numeric()
+			   else if(l1 == 1 && any(.Generic == c("*","/","+")) && (e1 > 0)) {
 	      e2@x <- callGeneric(e1, e2@x) ; e2 # remains "dpo"
 	  } else ## in all other cases
 	  callGeneric(e1, as(e2, "dsyMatrix")))
 setMethod("Arith", signature(e1 = cl, e2 = "dppMatrix"),
-	  function(e1, e2) if(length(e1) <= 1 && any(.Generic == c("*","/","+")) && (e1 > 0)) {
+	  function(e1, e2) if(!(l1 <- length(e1))) numeric()
+			   else if(l1 == 1 && any(.Generic == c("*","/","+")) && (e1 > 0)) {
 	      e2@x <- callGeneric(e1, e2@x) ; e2 # remains "dpp"
 	  } else ## in all other cases
 	  callGeneric(e1, as(e2, "dspMatrix")))
 setMethod("Arith", signature(e1 = "dpoMatrix", e2 = cl),
-	  function(e1, e2) if(length(e2) <= 1 && any(.Generic == c("*","/","+")) && (e2 > 0)) {
+	  function(e1, e2) if(!(l2 <- length(e2))) numeric()
+			   else if(l2 == 1 && any(.Generic == c("*","/","+")) && (e2 > 0)) {
 	      e1@x <- callGeneric(e1@x, e2) ; e1 # remains "dpo"
 	  } else ## in all other cases
 	  callGeneric(as(e1, "dsyMatrix"), e2))
 setMethod("Arith", signature(e1 = "dppMatrix", e2 = cl),
-	  function(e1, e2) if(length(e2) <= 1 && any(.Generic == c("*","/","+")) && (e2 > 0)) {
+	  function(e1, e2) if(!(l2 <- length(e2))) numeric()
+			   else if(l2 == 1 && any(.Generic == c("*","/","+")) && (e2 > 0)) {
 	      e1@x <- callGeneric(e1@x, e2) ; e1 # remains "dpp"
 	  } else ## in all other cases
 	  callGeneric(as(e1, "dspMatrix"), e2))
@@ -446,7 +483,12 @@ setMethod("Arith", signature(e1 = "dgeMatrix", e2 = "dgeMatrix"),
 A.M.n <- function(e1, e2) {
     d <- e1@Dim
     le <- length(e2)
-    if(le == 1 || le == d[1] || prod(d) == le) { # matching dim
+    if(le == 0)
+	if(prod(d) == 0)
+	    new(class2(class(e1), "d"), Dim = d, Dimnames = e1@Dimnames)
+	else
+	    as.numeric(e2)
+    else if(le == 1 || le == d[1] || any(prod(d) == c(le, 0L))) { # matching dim
         e1@x <- callGeneric(e1@x, as.vector(e2))
         e1
     } else stop ("length of 2nd arg does not match dimension of first")
@@ -458,7 +500,12 @@ setMethod("Arith", signature(e1 = "dgeMatrix", e2 = "sparseVector"), A.M.n)
 A.n.M <- function(e1, e2) {
     d <- e2@Dim
     le <- length(e1)
-    if(le == 1 || le == d[1] || prod(d) == le) { # matching dim
+    if(le == 0)
+	if(prod(d) == 0)
+	    new(class2(class(e2), "d"), Dim = d, Dimnames = e2@Dimnames)
+	else
+	    as.numeric(e1)
+    else if(le == 1 || le == d[1] || any(prod(d) == c(le, 0L))) { # matching dim
         e2@x <- callGeneric(as.vector(e1), e2@x)
         e2
     } else stop ("length of 1st arg does not match dimension of 2nd")
@@ -480,9 +527,17 @@ setMethod("Arith", signature(e1 = "ddenseMatrix", e2 = "ddenseMatrix"),
 .Arith.denseM.atom <- function(e1, e2) {
     ## since e1 = "dgeMatrix" has its own method, we have
     ## either symmetric or triangular !
-    d <- e1@Dim
+    n1 <- prod(d <- e1@Dim)
     le <- length(e2 <- as.vector(e2))
-    if(le == 1 || le == d[1] || prod(d) == le) { # matching dim
+    if(n1 && n1 < le)
+	stop(sprintf(
+	    "dim [product %d] do not match the length of object [%d]", n1, le))
+    if(le == 0)
+	if(prod(d) == 0)
+	    new(class2(class(e1), "d"), Dim = d, Dimnames = e1@Dimnames)
+	else
+	    as.numeric(e2)
+    else if(le == 1 || le == d[1] || any(prod(d) == c(le, 0L))) { # matching dim
         if(is(e1, "triangularMatrix")) {
             r0 <- callGeneric(0, e2)
             if(all0(r0)) {              # result remains triangular
@@ -513,7 +568,12 @@ setMethod("Arith", signature(e1 = "ddenseMatrix", e2 = "sparseVector"), .Arith.d
     d <- e2@Dim
     ## note that e2 is either symmetric or triangular here
     le <- length(e1 <- as.vector(e1))
-    if(le == 1 || le == d[1] || prod(d) == le) { # matching dim
+    if(le == 0)
+	if(prod(d) == 0)
+	    new(class2(class(e2), "d"), Dim = d, Dimnames = e2@Dimnames)
+	else
+	    as.numeric(e1)
+    else if(le == 1 || le == d[1] || any(prod(d) == c(le, 0L))) { # matching dim
 	if(is(e2, "triangularMatrix")) {
 	    r0 <- callGeneric(e1, 0)
 	    if(all0(r0)) {		# result remains triangular
@@ -625,23 +685,30 @@ for(Mcl in c("lMatrix","nMatrix","dMatrix"))
 
 ## This is parallel to Cmp.Mat.atomic() above --->  __keep parallel__ !
 Logic.Mat.atomic <- function(e1, e2) { ## result will typically be "like" e1:
-    e2 <- as.logical(e2)
-    if(.Generic == "&" && allTrue (e2)) return(as(e1, "lMatrix"))
-    if(.Generic == "|" && allFalse(e2)) return(as(e1, "lMatrix"))
-    d <- e1@Dim
+    l2 <- length(e2 <- as.logical(e2))
+    n1 <- prod(d <- e1@Dim)
+    if(n1 && n1 < l2)
+	stop(sprintf(
+	    "dim [product %d] do not match the length of object [%d]", n1, l2))
+    if(.Generic == "&" && l2 && allTrue (e2)) return(as(e1, "lMatrix"))
+    if(.Generic == "|" && l2 && allFalse(e2)) return(as(e1, "lMatrix"))
     cl <- class(e1)
-    if((l2 <- length(e2)) == 0) {
-	if(prod(d) == 0)
-	    return(new(class2(cl, "l"), Dim= d))
-	else
-	    stop(gettextf("<Matrix> %s %s is undefined",
-			  .Generic, paste0(class(e2),"(0)")), domain=NA)
-    }
+    if(l2 == 0)
+	return(if(n1 == 0)
+		   as(e1, class2(cl, "l"))# more expensive than (but working for "dgC*"):
+		   ## new(class2(cl, "l"), Dim = d, Dimnames = e1@Dimnames)
+	       else
+		   as.logical(e2))
+    ## else
     cl1 <- getClassDef(cl)
     slots1 <- names(cl1@slots)
     has.x <- any("x" == slots1)# *fast* check for "x" slot presence
-    if(l2 > 1 && has.x) ## e2 cannot simply be compared with e1@x --> use another method
-        return(callGeneric(e1, Matrix(e2, nrow=d[1], ncol=d[2])))
+    if(l2 > 1 && has.x)
+	return(if(prod(d) == 0)
+		   new(class2(cl, "l"), x = callGeneric(e1@x, e2),
+		       Dim = d, Dimnames = e1@Dimnames)
+	       else ## e2 cannot simply be compared with e1@x --> use another method
+		   callGeneric(e1, Matrix(e2, nrow=d[1], ncol=d[2])))
     ## else
     Udg <- extends(cl1, "triangularMatrix") && e1@diag == "U"
     r0 <- callGeneric(0, e2)
@@ -663,7 +730,7 @@ Logic.Mat.atomic <- function(e1, e2) { ## result will typically be "like" e1:
                     cl1 <- getClassDef(cl <- class(e1 <- as(e1, "dtrMatrix")))
             }
             ## FIXME? using copyClass() to copy "relevant" slots
-            r <- new(class2(cl, "l"), x = r, Dim = d, Dimnames = dimnames(e1))
+            r <- new(class2(cl, "l"), x = r, Dim = d, Dimnames = e1@Dimnames)
             if(extends(cl1, "symmetricMatrix")) {
                 r@uplo <- e1@uplo
             } else if(isTri) {
@@ -678,13 +745,14 @@ Logic.Mat.atomic <- function(e1, e2) { ## result will typically be "like" e1:
                 stop("internal bug in \"Logic\" method (Logic.Mat.atomic); please report")
 	    rx <- rep_len(r0, prod(d))
 	    rx[indTri(d[1], upper = (e1@uplo == "U"), diag=TRUE)] <- r
-	    r <- new("lgeMatrix", x = rx, Dim = d, Dimnames = dimnames(e1))
+	    r <- new("lgeMatrix", x = rx, Dim = d, Dimnames = e1@Dimnames)
 	}
 
     }
     else { ##---- e1 is(. , sparseMatrix) -----------------
         ## FIXME: remove this test eventually
-        if(extends(cl1, "diagonalMatrix")) stop("Logic.Mat.atomic() should not be called for diagonalMatrix")
+        if(extends(cl1, "diagonalMatrix"))
+            stop("Logic.Mat.atomic() should not be called for diagonalMatrix")
         remainSparse <- allFalse(r0) ## <==> things remain sparse
         if(Udg) {          # e1 *is* unit-diagonal (triangular sparse)
             r1 <- callGeneric(1, e2)
@@ -702,11 +770,11 @@ Logic.Mat.atomic <- function(e1, e2) { ## result will typically be "like" e1:
         }
 
         if(remainSparse) {
-            if(!any(is.na(r)) && ((Ar <- all(r)) || !any(r))) {
+            if(!anyNA(r) && ((Ar <- all(r)) || !any(r))) {
                 lClass <- class2(cl, "l") # is "lsparse*"
                 r <- new(lClass)
                 r@Dim <- d
-                r@Dimnames <- dimnames(e1)
+                r@Dimnames <- e1@Dimnames
                 if(Ar) {       # 'TRUE' instead of 'x': same sparsity:
 		    for(n in intersect(c("i","j","p","uplo","diag"), slots1))
 			slot(r, n) <- slot(e1, n)
@@ -746,7 +814,7 @@ Logic.Mat.atomic <- function(e1, e2) { ## result will typically be "like" e1:
                           non0ind(e1, cl1, uniqT=FALSE, xtendSymm=FALSE),
                           di = d, orig1=FALSE, checkBounds=FALSE)
             rx[1L + encI] <- r
-            r <- new(lClass, x = rx, Dim = d, Dimnames = dimnames(e1))
+            r <- new(lClass, x = rx, Dim = d, Dimnames = e1@Dimnames)
         }
     }
     r
@@ -970,13 +1038,13 @@ setMethod("Arith", signature(e1 = "dsCMatrix", e2 = "dsCMatrix"),
 
 ##-------- originally from ./dgCMatrix.R --------------------
 
-.Arith.Csparse <- function(e1, e2, Generic, class., triangular = FALSE)
+.Arith.Csparse <- function(e1, e2, Generic, class., triangular = FALSE, check.dimnames = TRUE)
 {
     ## Generic is one of  "+", "-", "*", "^", "%%", "%/%", "/"
 
     ## triangular:  TRUE  iff e1,e2 are triangular  _and_  e1@uplo == e2@uplo
     d <- dimCheck(e1, e2)
-    dn <- dimNamesCheck(e1, e2)
+    dn <- dimNamesCheck(e1, e2, check = check.dimnames)
     if(triangular) {
 	## need these for the 'x' slots in any case
 	if (e1@diag == "U") e1 <- .Call(Csparse_diagU2N, e1)
@@ -1118,7 +1186,7 @@ setMethod("Arith", signature(e1 = "numeric", e2 = "CsparseMatrix"), .Arith.atom.
 ##' compute indices for recycling <numeric> of length 'len' to match sparseMatrix 'spM'
 .Ops.recycle.ind <- function(spM, len) {
     n <- prod(d <- dim(spM))
-    if(n < len) stop("vector too long in Matrix - vector operation")
+    if(n && n < len) stop("vector too long in Matrix - vector operation")
     if(n %% len != 0) ## identical warning as in main/arithmetic.c
         warning("longer object length\n\tis not a multiple of shorter object length")
     ## TODO(speedup!): construction of [1L + in0 %%len] via one .Call()
@@ -1128,10 +1196,9 @@ setMethod("Arith", signature(e1 = "numeric", e2 = "CsparseMatrix"), .Arith.atom.
 }
 
 A.M.n <- function(e1, e2) {
-    if((l2 <- length(e2)) == 0)
-	stop(gettextf("<Matrix> %s %s is undefined",
-		      .Generic, paste0(class(e2),"(0)")), domain=NA)
-    is0f <- is0(f0 <- callGeneric(0, e2))
+    if((l2 <- length(e2)) == 0) # return 0-vector of e1's kind, as matrix()+<0>
+	return(if(length(e1)) vector(.type.kind[.M.kind(e1)]) else e1)
+    is0f <- is0(f0 <- callGeneric(0, e2)) #
     if(all(is0f)) { ## result keeps sparseness structure of e1
 	if(l2 > 1) {  #	 "recycle" e2 "carefully"
 	    e2 <- e2[.Ops.recycle.ind(e1, len = l2)]
@@ -1152,7 +1219,7 @@ A.M.n <- function(e1, e2) {
     }
     else { ## non-sparse, since '0 o e2' is not (all) 0
 	r <- as(e1, "matrix")
-	if(length(e2) == 1) {
+	if(l2 == 1) {
 	    r[] <- f0
 	    r[non0ind(e1, getClassDef("dgCMatrix")) + 1L] <- callGeneric(e1@x, e2)
 	    ..2dge(r)
@@ -1168,9 +1235,8 @@ setMethod("Arith", signature(e1 = "dsparseMatrix", e2 = "numeric"), .Arith.CM.at
 setMethod("Arith", signature(e1 = "dsparseMatrix", e2 = "logical"), .Arith.CM.atom)
 
 A.n.M <- function(e1, e2) {
-    if((l1 <- length(e1)) == 0)
-	stop(gettextf("%s %s <Matrix> is undefined",
-		      paste0(class(e2),"(0)"), .Generic), domain=NA)
+    if((l1 <- length(e1)) == 0) # return 0-vector of e2's kind, as <0> + matrix()
+	return(if(length(e2)) vector(.type.kind[.M.kind(e2)]) else e2)
     is0f <- is0(f0 <- callGeneric(e1, 0))
     if(all(is0f)) { ## result keeps sparseness structure of e2
 	if(l1 > 1) {  #	 "recycle" e1 "carefully"
@@ -1192,7 +1258,7 @@ A.n.M <- function(e1, e2) {
     }
     else { ## non-sparse, since '0 o e2' is not (all) 0
 	r <- as(e2, "matrix")
-	if(length(e1) == 1) {
+	if(l1 == 1) {
 	    r[] <- f0
 	    r[non0ind(e2, getClassDef("dgCMatrix")) + 1L] <-
 		callGeneric(e1, e2@x)
@@ -1286,7 +1352,7 @@ setMethod("Compare", signature(e1 = "CsparseMatrix", e2 = "CsparseMatrix"),
 		  shape <- if(T) "t" else if(S) "s" else "g"
 	      }
 
-	      dn <- dimNamesCheck(e1, e2)
+	      dn <- dimNamesCheck(e1, e2) ## <- FIXME: for 'S'; allow staying
               ## the result object:
 	      newC <- sub("^.", "l", MatrixClass(class(e1)))
               ## FIXME: "n" result when e1 & e2 are "n", or even whenever possible
@@ -1380,7 +1446,12 @@ setMethod("Compare", signature(e1 = "CsparseMatrix", e2 = "CsparseMatrix"),
 
 ## "Arith" short cuts / exceptions
 setMethod("-", signature(e1 = "sparseMatrix", e2 = "missing"),
-	  function(e1, e2) { e1 <- diagU2N(e1); e1@x <- -e1@x; e1 })
+	  function(e1, e2) {
+	      e1 <- diagU2N(e1)
+	      e1@x <- -e1@x
+	      e1@factors <- list()# Drop Cholesky factors; TODO: Consider to modify & keep LU
+	      e1
+	  })
 ## with the following exceptions:
 setMethod("-", signature(e1 = "nsparseMatrix", e2 = "missing"),
 	  function(e1,e2) callGeneric(as(as(as(e1, "CsparseMatrix"), "dMatrix"), "dgCMatrix")))
@@ -1442,11 +1513,11 @@ setMethod("Ops", signature(e1 = "ANY", e2 = "sparseVector"),
 setMethod("Ops", signature(e1 = "sparseVector", e2 = "atomicVector"),
 	  function(e1, e2) {
 	      if(length(e2) == 1) { ## scalar ------ special case - "fast"
-		  if(all0(rf <- callGeneric(FALSE, e2))) { # result remains sparse
+		  if(all0(callGeneric(FALSE, e2))) { # result remains sparse
 		      if(is(e1, "nsparseVector")) { # no 'x' slot, i.e. all TRUE
 			  r <- callGeneric(TRUE, e2)
 			  if(is.logical(r)) {
-			      if(isTRUE(at <- all(r))) # (could be NA)
+			      if(isTRUE(all(r))) # (could be NA)
 				  e1	# result unchanged
 			      else
 				  newSpVec("lsparseVector", x = r, e1)
@@ -1474,11 +1545,11 @@ setMethod("Ops", signature(e1 = "sparseVector", e2 = "atomicVector"),
 setMethod("Ops", signature(e1 = "atomicVector", e2 = "sparseVector"),
 	  function(e1, e2) {
 	      if(length(e1) == 1) { ## scalar ------ special case - "fast"
-		  if(all0(rf <- callGeneric(e1, FALSE))) { # result remains sparse
+		  if(all0(callGeneric(e1, FALSE))) { # result remains sparse
 		      if(is(e2, "nsparseVector")) { # no 'x' slot, i.e. all TRUE
 			  r <- callGeneric(e1, TRUE)
 			  if(is.logical(r)) {
-			      if(isTRUE(at <- all(r))) # (could be NA)
+			      if(isTRUE(all(r))) # (could be NA)
 				  e2	# result unchanged
 			      else
 				  newSpVec("lsparseVector", x = r, e2)
@@ -1507,13 +1578,16 @@ setMethod("Ops", signature(e1 = "atomicVector", e2 = "sparseVector"),
 Ops.spV.spV <- function(e1, e2) {
     n1 <- e1@length
     n2 <- e2@length
+    if(!n1 || !n2) ## return 0-length :
+	return(if(is.na(match(.Generic, .ArithGenerics))) logical() else numeric())
+    ## else  n1, n2 >= 1 :
     if(n1 != n2) {
 	if(n1 < n2) {
 	    n <- n1 ; N <- n2
 	} else {
 	    n <- n2 ; N <- n1
 	}
-	if(n == 1) {		  # simple case, do not really recycle
+        if(n == 1L) { # simple case, do not really recycle
 	    if(n1 < n2) return(callGeneric(sp2vec(e1), e2))
 	    else	return(callGeneric(e1, sp2vec(e2)))
 	}
@@ -1601,7 +1675,7 @@ Ops.M.spV <- function(e1, e2) {
     n1 <- prod(d)
     n2 <- e2@length
     if(n1 != n2) {
-	if(n1 < n2) {
+	if(n1 && n1 < n2) { # 0-extent matrix + vector is fine
 	    stop(sprintf(
 		"dim [product %d] do not match the length of object [%d]",
 			 n1, n2))
@@ -1627,7 +1701,7 @@ Ops.spV.M <- function(e1, e2) {
     d <- e2@Dim
     n2 <- prod(d)
     if(n2 != n1) {
-	if(n2 < n1) {
+	if(n2 && n2 < n1) { # vector + 0-extent matrix  is fine
 	    stop(sprintf(
 		"dim [product %d] do not match the length of object [%d]",
 			 n2, n1))
