@@ -1,3 +1,4 @@
+##  R routines for mgcv::gamm (c) Simon Wood 2002-2019
 
 ### the following two functions are for use in place of log and exp
 ### in positivity ensuring re-parameterization.... they have `better' 
@@ -348,12 +349,18 @@ smooth2random.fs.interaction <- function(object,vnames,type=1) {
 ## penalties are not interwoven, but blocked (i.e. this ordering is 
 ## as for gamm case). 
   if (object$fixed) return(list(fixed=TRUE,Xf=object$X))
-  ##if (type == 2) require(Matrix)
+  ## If smooth constructor was not called with "gamm" attribute set,
+  ## then we need to reset model matrix to base model matrix.
+  if (!is.null(object$Xb)) {
+    object$X <- object$Xb
+    object$S <- object$base$S
+    if (!is.null(object$S.scale)&&length(object$S)>0) for (i in 1:length(object$S)) object$S[[i]] <- object$S[[i]]/object$S.scale[i] 
+  }  
   colx <- ncol(object$X) 
   diagU <- rep(1,colx)
   ind <- 1:colx 
-  flev <- levels(object$fac)
-  n.lev <- length(flev)
+  ## flev <- levels(object$fac)
+  n.lev <- length(object$flev)
   if (type==2) {
     ## index which params in fit parameterization are penalized by each penalty.
     ## e.g. pen.ind==1 is TRUE for each param penalized by first penalty and
@@ -383,19 +390,15 @@ smooth2random.fs.interaction <- function(object,vnames,type=1) {
       attr(random[[i]],"group") <- object$fac  ## factor supplied as part of term 
       attr(random[[i]],"Xr.name") <- term.name
       attr(random[[i]],"Xr") <- X
-     # rind <- c(rind,k:(k+ncol(X)-1))
-     # rinc <- c(rinc,rep(ncol(X),ncol(X)))
-     # k <- k + n.lev * ncol(X)
     } else { ## gamm4 form --- whole sparse matrices
+      
       Xr <- as(matrix(0,nrow(X),0),"dgCMatrix")
       ii <- 0
       for (j in 1:n.lev) { ## assemble full sparse model matrix
-        Xr <- cbind2(Xr,as(X*as.numeric(object$fac==flev[j]),"dgCMatrix"))
+        Xr <- cbind2(Xr,as(X*as.numeric(object$fac==object$flev[j]),"dgCMatrix"))
         pen.ind[indi+ii] <- i;ii <- ii + colx
       }
-    #  rind <- c(rind,k:(k+ncol(Xr)-1))
-    #  rinc <- c(rinc,rep(ncol(Xr),ncol(Xr)))
-      random[[i]] <- Xr 
+      random[[i]] <- if (is.null(object$Xb)) Xr else as(Xr,"matrix") 
       names(random)[i] <- term.name
       attr(random[[i]],"s.label") <- object$label
     }
@@ -784,7 +787,7 @@ varWeights.dfo <- function(b,data)
    w
 }
 
-extract.lme.cov2<-function(b,data,start.level=1)
+extract.lme.cov2<-function(b,data=NULL,start.level=1)
 # function to extract the response data covariance matrix from an lme fitted
 # model object b, fitted to the data in data. "inner" == "finest" grouping 
 # start.level is the r.e. grouping level at which to start the construction, 
@@ -799,6 +802,10 @@ extract.lme.cov2<-function(b,data,start.level=1)
 # V is either returned as an array, if it's diagonal, a matrix if it is
 # a full matrix or a list of matrices if it is block diagonal.
 { if (!inherits(b,"lme")) stop("object does not appear to be of class lme")
+  if (is.null(data)) {
+    na.act <- na.action(b)
+    data <- if (is.null(na.act)) b$data else b$data[-na.act,] 
+  }
   grps <- nlme::getGroups(b) # labels of the innermost groupings - in data frame order
   n <- length(grps)    # number of data
   n.levels <- length(b$groups) # number of levels of grouping (random effects only)
@@ -933,9 +940,10 @@ extract.lme.cov2<-function(b,data,start.level=1)
     { # get matrix with columns that are indicator variables for ith set of groups...
       # groups has outer levels first 
       if(length(levels(b$groups[[n.levels-i+1]]))==1) { ## model.matrix needs >1 level 
-        X[[1]] <- matrix(rep(1,nrow(b$groups))) } else { 
-        X[[1]] <- model.matrix(~b$groups[[n.levels-i+1]]-1,
-                  contrasts.arg=c("contr.treatment","contr.treatment")) }
+        X[[1]] <- matrix(rep(1,nrow(b$groups))) } else {
+	clist <- list('b$groups[[n.levels - i + 1]]'=c("contr.treatment","contr.treatment"))
+        X[[1]] <- model.matrix(~b$groups[[n.levels - i + 1]]-1,
+                  contrasts.arg=clist) }
       # Get `model matrix' columns relevant to current grouping level...
       X[[2]] <- Zt[,i.col:(i.col+grp.dims[i]-1),drop=FALSE]
       i.col <- i.col+grp.dims[i]
@@ -987,15 +995,19 @@ extract.lme.cov2<-function(b,data,start.level=1)
     }
   }
   list(V=V,ind=Cind)
-}
+} ## extract.lme.cov2
 
-extract.lme.cov<-function(b,data,start.level=1)
+extract.lme.cov<-function(b,data=NULL,start.level=1)
 # function to extract the response data covariance matrix from an lme fitted
 # model object b, fitted to the data in data. "inner" == "finest" grouping 
 # start.level is the r.e. grouping level at which to start the construction, 
 # levels outer to this will not be included in the calculation - this is useful
 # for gamm calculations
 { if (!inherits(b,"lme")) stop("object does not appear to be of class lme")
+  if (is.null(data)) {
+    na.act <- na.action(b)
+    data <- if (is.null(na.act)) b$data else b$data[-na.act,] 
+  }
   grps<-nlme::getGroups(b) # labels of the innermost groupings - in data frame order
   n<-length(grps)    # number of data
   if (is.null(b$modelStruct$varStruct)) w<-rep(b$sigma,n) ### 
@@ -1047,9 +1059,10 @@ extract.lme.cov<-function(b,data,start.level=1)
     { # get matrix with columns that are indicator variables for ith set of groups...
       # groups has outer levels first 
       if(length(levels(b$groups[[n.levels-i+1]]))==1) { ## model.matrix needs >1 level 
-        X[[1]] <- matrix(rep(1,nrow(b$groups))) } else { 
-        X[[1]] <- model.matrix(~b$groups[[n.levels-i+1]]-1,
-                  contrasts.arg=c("contr.treatment","contr.treatment")) }
+        X[[1]] <- matrix(rep(1,nrow(b$groups))) } else {
+	clist <- list('b$groups[[n.levels - i + 1]]'=c("contr.treatment","contr.treatment"))
+        X[[1]] <- model.matrix(~b$groups[[n.levels - i + 1]]-1,
+                  contrasts.arg=clist) }
       # Get `model matrix' columns relevant to current grouping level...
       X[[2]] <- Zt[,i.col:(i.col+grp.dims[i]-1),drop=FALSE]
       i.col <- i.col+grp.dims[i]
@@ -1071,7 +1084,7 @@ extract.lme.cov<-function(b,data,start.level=1)
     V <- V+Z%*%Vr%*%t(Z)
   }
   V
-}
+} ## extract.lme.cov
 
 formXtViX <- function(V,X)
 ## forms X'V^{-1}X as efficiently as possible given the structure of
@@ -1119,35 +1132,51 @@ new.name <- function(proposed,old.names)
 }
 
 gammPQL <- function (fixed, random, family, data, correlation, weights,
-    control, niter = 30, verbose = TRUE, ...)
-
+    control, niter = 30, verbose = TRUE, mustart=NULL, etastart=NULL, ...) {
 ## service routine for `gamm' to do PQL fitting. Based on glmmPQL
-## from the MASS library (Venables & Ripley). In particular, for back 
-## compatibility the numerical results should be identical with gamm 
-## fits by glmmPQL calls. Because `gamm' already does some of the 
-## preliminary stuff that glmmPQL does, gammPQL can be simpler. It also 
-## deals with the possibility of the original data frame containing 
-## variables called `zz' `wts' or `invwt'
-
-{ off <- model.offset(data)
+## from the MASS library (Venables & Ripley). Because `gamm' already
+## does some of the preliminary stuff that glmmPQL does, gammPQL can
+## be simpler. It also deals with the possibility of the original
+## data frame containing variables called `zz' `wts' or `invwt'.
+## Modified 2019 to use standard GLM initialization to imporove convergence.
+  off <- model.offset(data)
   if (is.null(off)) off <- 0
+  y <- model.response(data) ## NEW
+  nobs <- nrow(data) ## NEW
+  if (is.null(weights)) weights <- rep(1, nrow(data))
+  start <-  NULL ## never used
+  if (is.null(mustart)) {
+    eval(family$initialize)
+  } else {
+    mukeep <- mustart
+    eval(family$initialize)
+    mustart <- mukeep
+  }
+  #eval(family$initialize) ## NEW
 
   wts <- weights
-  if (is.null(wts)) wts <- rep(1, nrow(data))
+  #if (is.null(wts)) wts <- rep(1, nrow(data))
   wts.name <- new.name("wts",names(data)) ## avoid overwriting what's already in `data'
   data[[wts.name]] <- wts 
-
-  fit0 <- NULL ## keep checking tools happy 
+ 
+#  fit0 <- NULL ## keep checking tools happy 
   ## initial fit (might be better replaced with `gam' call)
-  eval(parse(text=paste("fit0 <- glm(formula = fixed, family = family, data = data,",
-                        "weights =",wts.name,",...)")))
-  w <- fit0$prior.weights
-  eta <- fit0$linear.predictors
-    
-  zz <- eta + fit0$residuals - off
-  wz <- fit0$weights
+#  eval(parse(text=paste("fit0 <- glm(formula = fixed, family = family, data = data,",
+#                        "weights =",wts.name,",...)")))
+#  w <- fit0$prior.weights
+#  eta <- fit0$linear.predictors
   fam <- family
-
+  eta <- if (!is.null(etastart)) 
+            etastart
+	 else fam$linkfun(mustart)
+  mu <- fam$linkinv(eta)	 
+  w <- wts;  
+#  zz <- eta + fit0$residuals - off
+  mu.eta.val <- fam$mu.eta(eta)
+  zz <- eta + (y - mu)/mu.eta.val - off
+#  wz <- fit0$weights
+  wz <- w * mu.eta.val^2/fam$variance(mustart)
+  
   ## find non clashing name for pseudodata and insert in formula
   zz.name <- new.name("zz",names(data))
   eval(parse(text=paste("fixed[[2]] <- quote(",zz.name,")")))
@@ -1181,21 +1210,23 @@ gammPQL <- function (fixed, random, family, data, correlation, weights,
     mu <- fam$linkinv(eta)
     mu.eta.val <- fam$mu.eta(eta)
     ## get pseudodata and insert in `data' 
-    data[[zz.name]] <- eta + (fit0$y - mu)/mu.eta.val - off
+#    data[[zz.name]] <- eta + (fit0$y - mu)/mu.eta.val - off
+    data[[zz.name]] <- eta + (y - mu)/mu.eta.val - off
     wz <- w * mu.eta.val^2/fam$variance(mu)
     data[[invwt.name]] <- 1/wz
   } ## end i in 1:niter
   if (!converged) warning("gamm not converged, try increasing niterPQL")
-  fit$y <- fit0$y
+# fit$y <- fit0$y
+  fit$y <- y 
   fit$w <- w ## prior weights
   fit
-}
+} ## gammPQL
 
 
 
 gamm <- function(formula,random=NULL,correlation=NULL,family=gaussian(),data=list(),weights=NULL,
-      subset=NULL,na.action,knots=NULL,control=list(niterEM=0,optimMethod="L-BFGS-B"),
-      niterPQL=20,verbosePQL=TRUE,method="ML",drop.unused.levels=TRUE,...)
+      subset=NULL,na.action,knots=NULL,control=list(niterEM=0,optimMethod="L-BFGS-B",returnObject=TRUE),
+      niterPQL=20,verbosePQL=TRUE,method="ML",drop.unused.levels=TRUE,mustart=NULL, etastart=NULL,...)
 # Routine to fit a GAMM to some data. Fixed and smooth terms are defined in the formula, but the wiggly 
 # parts of the smooth terms are treated as random effects. The onesided formula random defines additional 
 # random terms. correlation describes the correlation structure. This routine is basically an interface
@@ -1350,24 +1381,26 @@ gamm <- function(formula,random=NULL,correlation=NULL,family=gaussian(),data=lis
     length(offset.name)==0) lme.used <- TRUE else lme.used <- FALSE
     if (lme.used&&!is.null(weights)&&!wisvf) lme.used <- FALSE   
 
-    if (lme.used)
-    { ## following construction is a work-around for problem in nlme 3-1.52 
+    if (lme.used) { ## following construction is a work-around for problem in nlme 3-1.52 
       eval(parse(text=paste("ret$lme<-lme(",deparse(fixed.formula),
           ",random=rand,data=strip.offset(mf),correlation=correlation,",
           "control=control,weights=weights,method=method)"
-            ,sep=""    ))) 
+            ,sep=""    )))
+      ## need to be able to work out full edf for following to work...	    
+      # if (is.null(correlation)) { ## compute conditional aic precursor
+      #  dev <- sum(family$dev.resids(G$y,fitted(ret$lme),weights))
+      #	ret$lme$aic <- family$aic(G$y,1,fitted(ret$lme),weights,dev)
+      # }
       ##ret$lme<-lme(fixed.formula,random=rand,data=mf,correlation=correlation,control=control)
-    } else
-    { ## Again, construction is a work around for nlme 3-1.52
+    } else { ## Again, construction is a work around for nlme 3-1.52
       if (wisvf) stop("weights must be like glm weights for generalized case")
       if (verbosePQL) cat("\n Maximum number of PQL iterations: ",niterPQL,"\n")
       eval(parse(text=paste("ret$lme<-gammPQL(",deparse(fixed.formula),
           ",random=rand,data=strip.offset(mf),family=family,",
           "correlation=correlation,control=control,",
-            "weights=weights,niter=niterPQL,verbose=verbosePQL)",sep=""))) 
+            "weights=weights,niter=niterPQL,verbose=verbosePQL,mustart=mustart,etastart=etastart,...)",sep=""))) 
       G$y <- ret$lme$y ## makes sure that binomial response is returned as a vector!
-      ##ret$lme<-glmmPQL(fixed.formula,random=rand,data=mf,family=family,correlation=correlation,
-      ##                 control=control,niter=niterPQL,verbose=verbosePQL)
+     
     }
 
     ### .... fitting finished
@@ -1532,6 +1565,11 @@ gamm <- function(formula,random=NULL,correlation=NULL,family=gaussian(),data=lis
     object$edf<-rowSums(Vb*t(XVX))   
     object$df.residual <- length(object$y) - sum(object$edf)    
 
+    #if (!is.null(ret$lme$aic)) { ## finish the conditional AIC (only happens if no correlation) 
+    #  object$aic <- ret$lme$aic + sum(object$edf) ## requires edf for r.e. as well!
+    #  ret$lme$aic<- NULL
+    #}
+
     object$sig2 <- ret$lme$sigma^2
     if (lme.used) { object$method <- paste("lme.",method,sep="")} 
     else { object$method <- "PQL"}
@@ -1583,7 +1621,8 @@ gamm <- function(formula,random=NULL,correlation=NULL,family=gaussian(),data=lis
     ## set environments to global to avoid enormous saved object files
     environment(attr(object$model,"terms")) <- 
     environment(object$terms) <- environment(object$pterms) <- 
-    environment(object$formula) <-environment(object$pred.formula) <-  .GlobalEnv
+    environment(object$formula) <- .GlobalEnv
+    if (!is.null(object$pred.formula)) environment(object$pred.formula) <-  .GlobalEnv
     ret$gam <- object
     environment(attr(ret$lme$data,"terms")) <- environment(ret$lme$terms) <- .GlobalEnv
     if (!is.null(ret$lme$modelStruct$varStruct)) {
