@@ -151,20 +151,24 @@ draw.key <- function(key, draw = FALSE, vp = NULL, ...)
     ## if it is)
     
     process.key <-
-        function(align = TRUE,
+        function(reverse.rows = FALSE, ## invert rows (e.g. for barchart, stack = FALSE)
+                 between = 2,
+                 align = TRUE,
                  title = NULL,
                  rep = TRUE,
                  background = trellis.par.get("background")$col,
                  alpha.background = 1,
                  border = FALSE,
                  transparent = FALSE, 
-                 divide = 3,
                  col = "black",
                  alpha = 1,
                  lty = 1,
                  lwd = 1,
                  font = 1, 
+                 fontface = NULL, 
+                 fontfamily = NULL,
                  pch = 8,
+                 cex = 1,
                  fill = "transparent",
                  adj = 0,
                  type = "l", 
@@ -172,20 +176,16 @@ draw.key <- function(key, draw = FALSE, vp = NULL, ...)
                  height = 1,
                  angle = 0, 
                  density = -1,
-                 reverse.rows = FALSE, ## invert rows (e.g. for barchart, stack = FALSE)
-                 between = 2,
-                 between.columns = 3,
-                 columns = 1,
-                 cex = 1,
                  cex.title = 1.5 * max(cex),                 
                  padding.text = 1,
                  lineheight = 1,
-                 fontface = NULL, 
-                 fontfamily = NULL,
-                 
+                 columns = 1,
+                 divide = 3,
+                 between.columns = 3,
+
                  ## to avoid partial matching, anything starting with
                  ## lines, points, rect, text must come after ...
-
+                 
                  ...,
                  lines.title = 2)
         {
@@ -198,33 +198,34 @@ draw.key <- function(key, draw = FALSE, vp = NULL, ...)
                  alpha.background = alpha.background,
                  border = border,
                  transparent = transparent, 
-                 columns = columns,
-                 divide = divide,
-                 between.columns = between.columns,
-                 cex = cex,
-                 cex.title = cex.title,
-                 padding.text = padding.text,
                  col = col,
                  alpha = alpha,
                  lty = lty,
                  lwd = lwd,
-                 lineheight = lineheight,
                  font = font,
                  fontface = fontface,
                  fontfamily = fontfamily,
                  pch = pch,
+                 cex = cex,
+                 fill = fill,
                  adj = adj,
                  type = type, 
                  size = size,
                  height = height,
                  angle = angle, 
                  density = density,
+                 cex.title = cex.title,
+                 padding.text = padding.text,
+                 lineheight = lineheight,
+                 columns = columns,
+                 divide = divide,
+                 between.columns = between.columns,
                  lines.title = lines.title,
                  ...)
         }
 
     fontsize.points <- trellis.par.get("fontsize")$points
-    key <- do.call("process.key", key)
+    key <- do.call(process.key, key, quote = TRUE)
 
     key.length <- length(key)
     key.names <- names(key)    # Need to update
@@ -257,6 +258,7 @@ draw.key <- function(key, draw = FALSE, vp = NULL, ...)
                      font = key$font,
                      fontface = key$fontface,
                      fontfamily = key$fontfamily)
+            pars <- pars[!sapply(pars, is.null)] # remove NULL components
             key[[i]][[1]] <- NULL
             key[[i]] <- complete_names(key[[i]], pars, allow.invalid = TRUE)
             pars[names(key[[i]])] <- key[[i]]
@@ -278,6 +280,7 @@ draw.key <- function(key, draw = FALSE, vp = NULL, ...)
                      height = key$height,
                      angle = key$angle,
                      density = key$density)
+            pars <- pars[!sapply(pars, is.null)] # remove NULL components
             key[[i]] <- complete_names(key[[i]], pars, allow.invalid = TRUE)
             pars[names(key[[i]])] <- key[[i]]
             tmplen <- max(unlist(lapply(pars,length)))
@@ -297,6 +300,7 @@ draw.key <- function(key, draw = FALSE, vp = NULL, ...)
                      fill = key$fill,
                      lwd = key$lwd,
                      type = key$type)
+            pars <- pars[!sapply(pars, is.null)] # remove NULL components
             key[[i]] <- complete_names(key[[i]], pars, allow.invalid = TRUE)
             pars[names(key[[i]])] <- key[[i]]
             tmplen <- max(unlist(lapply(pars,length)))
@@ -306,15 +310,17 @@ draw.key <- function(key, draw = FALSE, vp = NULL, ...)
         }
         else if (curname == 4) # "points"
         {
-            pars <- list(col = key$col,
-                         alpha = key$alpha,
-                         cex = key$cex,
-                         pch = key$pch,
-                         lwd = key$lwd,
-                         fill = key$fill,
-                         font = key$font,
-                         fontface = key$fontface,
-                         fontfamily = key$fontfamily)
+            pars <-
+                list(col = key$col,
+                     alpha = key$alpha,
+                     cex = key$cex,
+                     pch = key$pch,
+                     lwd = key$lwd,
+                     fill = key$fill,
+                     font = key$font,
+                     fontface = key$fontface,
+                     fontfamily = key$fontfamily)
+            pars <- pars[!sapply(pars, is.null)] # remove NULL components
             key[[i]] <- complete_names(key[[i]], pars, allow.invalid = TRUE)
             pars[names(key[[i]])] <- key[[i]]
             tmplen <- max(unlist(lapply(pars,length)))
@@ -756,6 +762,7 @@ draw.colorkey <- function(key, draw = FALSE, vp = NULL)
                  alpha = regions$alpha,
                  at,
                  tick.number = 7,
+                 tck = 1,
                  width = 2,
                  height = 1,
                  space = "right",
@@ -770,6 +777,7 @@ draw.colorkey <- function(key, draw = FALSE, vp = NULL)
                  alpha = alpha,
                  at = at,
                  tick.number = tick.number,
+                 tck = tck,
                  width = width,
                  height = height,
                  space = space,
@@ -785,8 +793,6 @@ draw.colorkey <- function(key, draw = FALSE, vp = NULL)
     axis.line <- updateList(trellis.par.get("axis.line"), key$axis.line)
     axis.text <- updateList(trellis.par.get("axis.text"), key$axis.text)
 
-## FIXME: delete later
-#str(key)
     ## made FALSE later if labels explicitly specified
     check.overlap <- TRUE
     
@@ -795,7 +801,7 @@ draw.colorkey <- function(key, draw = FALSE, vp = NULL)
     ## the breakpoints of the rectangles, and the other is key$lab$at
     ## (optional) which is the positions of the ticks. We will use the
     ## 'at' variable for the latter, 'atrange' for the range of the
-    ## former, and keyat explicitly when needed
+    ## former, and key$at explicitly when needed
 
 
 
@@ -837,6 +843,23 @@ draw.colorkey <- function(key, draw = FALSE, vp = NULL)
     lineheight <- axis.text$lineheight
     rot <- 0
 
+    ## The following code assumes names key$lab and key$lab$lab (which
+    ## may have been used in user code), whereas documentation says
+    ## key$labels and key$labels$labels.  To make both work without
+    ## 'partial matching' warnings, we rename key$labels to key$lab
+    ## etc.
+
+    if (!is.null(key[["labels"]])) 
+    {
+        key[["lab"]] <- key[["labels"]]
+        key[["labels"]] <- NULL
+        if (is.list(key[["lab"]]) && !is.null(key[["lab"]][["labels"]])) 
+        {
+            key[["lab"]][["lab"]] <- key[["lab"]][["labels"]]
+            key[["lab"]][["labels"]] <- NULL
+        }
+    }
+    
     if (is.null(key$lab))
     {
         at <- lpretty(atrange, key$tick.number)
@@ -895,7 +918,7 @@ draw.colorkey <- function(key, draw = FALSE, vp = NULL)
         heights.x <- c((1 - key$height) / 2, key$height, (1 - key$height) / 2)
         heights.units <- rep("null", 3)
 
-        widths.x <- c(0.6 * key$width, do.labels * 0.6, do.labels * 1)
+        widths.x <- c(0.6 * key$width, do.labels * (0.3 + key$tck * 0.3), do.labels * 1)
         widths.units <- c("lines", "lines", "grobwidth")
         widths.data <- list(NULL, NULL, labelsGrob)
         
@@ -915,7 +938,7 @@ draw.colorkey <- function(key, draw = FALSE, vp = NULL)
                                            width = 1, height = 1,
                                            vp = viewport(clip = "on"),
                                            name = trellis.grobname("raster",
-                                             type="colorkey"),
+                                                                   type="colorkey"),
                                            interpolate = key$interpolate),
                                 row = 2, col = 1)
         }
@@ -928,7 +951,7 @@ draw.colorkey <- function(key, draw = FALSE, vp = NULL)
                                          vp = viewport(yscale = atrange),
                                          height = recdim, 
                                          name = trellis.grobname("image",
-                                           type="colorkey"),
+                                                                 type="colorkey"),
                                          gp =
                                          gpar(fill = key$col,
                                               col = "transparent",
@@ -947,20 +970,21 @@ draw.colorkey <- function(key, draw = FALSE, vp = NULL)
                             row = 2, col = 1)
         if (do.labels)
         {
-            key.gf <- placeGrob(frame = key.gf, 
-                                segmentsGrob(x0 = rep(0, length(labscat)),
-                                             y0 = labscat,
-                                             x1 = rep(.4, length(labscat)),
-                                             y1 = labscat,
-                                             vp = viewport(yscale = atrange),
-                                             default.units = "native",
-                                             name = trellis.grobname("ticks",
-                                               type="colorkey"),
-                                             gp =
-                                             gpar(col = axis.line$col,
-                                                  lty = axis.line$lty,
-                                                  lwd = axis.line$lwd)),
-                                row = 2, col = 2)
+            if (key$tck != 0)
+                key.gf <- placeGrob(frame = key.gf, 
+                                    segmentsGrob(x0 = rep(0, length(labscat)),
+                                                 y0 = labscat,
+                                                 x1 = rep(key$tck / (1 + key$tck), length(labscat)),
+                                                 y1 = labscat,
+                                                 vp = viewport(yscale = atrange),
+                                                 default.units = "native",
+                                                 name = trellis.grobname("ticks",
+                                                                         type="colorkey"),
+                                                 gp =
+                                                 gpar(col = axis.line$col,
+                                                      lty = axis.line$lty,
+                                                      lwd = axis.line$lwd)),
+                                    row = 2, col = 2)
             key.gf <- placeGrob(key.gf,
                                 labelsGrob, 
                                 row = 2, col = 3)
@@ -990,7 +1014,8 @@ draw.colorkey <- function(key, draw = FALSE, vp = NULL)
         heights.x <- c((1 - key$height) / 2, key$height, (1 - key$height) / 2)
         heights.units <- rep("null", 3)
 
-        widths.x <- c(do.labels * 1, do.labels * 0.6, 0.6 * key$width)
+
+        widths.x <- c(do.labels * 1, do.labels * (0.3 + key$tck * 0.3), 0.6 * key$width)
         widths.units <- c("grobwidth", "lines", "lines")
         widths.data <- list(labelsGrob, NULL, NULL)
         
@@ -1042,20 +1067,21 @@ draw.colorkey <- function(key, draw = FALSE, vp = NULL)
                             row = 2, col = 3)
         if (do.labels)
         {
-            key.gf <- placeGrob(frame = key.gf, 
-                                segmentsGrob(x0 = rep(1, length(labscat)),
-                                             y0 = labscat,
-                                             x1 = rep(.6, length(labscat)),
-                                             y1 = labscat,
-                                             vp = viewport(yscale = atrange),
-                                             default.units = "native",
-                                             name = trellis.grobname("ticks",
-                                               type="colorkey"),
-                                             gp =
-                                             gpar(col = axis.line$col,
-                                                  lty = axis.line$lty,
-                                                  lwd = axis.line$lwd)),
-                                row = 2, col = 2)
+            if (key$tck != 0)
+                key.gf <- placeGrob(frame = key.gf, 
+                                    segmentsGrob(x0 = rep(1, length(labscat)),
+                                                 y0 = labscat,
+                                                 x1 = rep(1 - key$tck / (1 + key$tck), length(labscat)),
+                                                 y1 = labscat,
+                                                 vp = viewport(yscale = atrange),
+                                                 default.units = "native",
+                                                 name = trellis.grobname("ticks",
+                                                                         type="colorkey"),
+                                                 gp =
+                                                 gpar(col = axis.line$col,
+                                                      lty = axis.line$lty,
+                                                      lwd = axis.line$lwd)),
+                                    row = 2, col = 2)
             key.gf <- placeGrob(key.gf,
                                 labelsGrob, 
                                 row = 2, col = 1)
@@ -1085,7 +1111,7 @@ draw.colorkey <- function(key, draw = FALSE, vp = NULL)
         widths.x <- c((1 - key$height) / 2, key$height, (1 - key$height) / 2)
         widths.units <- rep("null", 3)
 
-        heights.x <- c(do.labels * 1, do.labels * 0.6, 0.6 * key$width)
+        heights.x <- c(do.labels * 1, do.labels * (0.3 + key$tck * 0.3), 0.6 * key$width)
         heights.units <- c("grobheight", "lines", "lines")
         heights.data <- list(labelsGrob, NULL, NULL)
         
@@ -1137,20 +1163,21 @@ draw.colorkey <- function(key, draw = FALSE, vp = NULL)
                             row = 3, col = 2)
         if (do.labels)
         {
-            key.gf <- placeGrob(frame = key.gf, 
-                                segmentsGrob(y0 = rep(0, length(labscat)),
-                                             x0 = labscat,
-                                             y1 = rep(.4, length(labscat)),
-                                             x1 = labscat,
-                                             vp = viewport(xscale = atrange),
-                                             default.units = "native",
-                                             name = trellis.grobname("ticks",
-                                               type="colorkey"),
-                                             gp =
-                                             gpar(col = axis.line$col,
-                                                  lty = axis.line$lty,
-                                                  lwd = axis.line$lwd)),
-                                row = 2, col = 2)
+            if (key$tck != 0)
+                key.gf <- placeGrob(frame = key.gf, 
+                                    segmentsGrob(y0 = rep(0, length(labscat)),
+                                                 x0 = labscat,
+                                                 y1 = rep(key$tck / (1 + key$tck), length(labscat)),
+                                                 x1 = labscat,
+                                                 vp = viewport(xscale = atrange),
+                                                 default.units = "native",
+                                                 name = trellis.grobname("ticks",
+                                                                         type="colorkey"),
+                                                 gp =
+                                                 gpar(col = axis.line$col,
+                                                      lty = axis.line$lty,
+                                                      lwd = axis.line$lwd)),
+                                    row = 2, col = 2)
             key.gf <- placeGrob(key.gf,
                                 labelsGrob, 
                                 row = 1, col = 2)
@@ -1180,7 +1207,7 @@ draw.colorkey <- function(key, draw = FALSE, vp = NULL)
         widths.x <- c((1 - key$height) / 2, key$height, (1 - key$height) / 2)
         widths.units <- rep("null", 3)
 
-        heights.x <- c(0.6 * key$width, do.labels * 0.6, do.labels * 1)
+        heights.x <- c(0.6 * key$width, do.labels * (0.3 + key$tck * 0.3), do.labels * 1)
         heights.units <- c("lines", "lines", "grobheight")
         heights.data <- list(NULL, NULL, labelsGrob)
         
@@ -1232,20 +1259,21 @@ draw.colorkey <- function(key, draw = FALSE, vp = NULL)
                             row = 1, col = 2)
         if (do.labels)
         {
-            key.gf <- placeGrob(frame = key.gf, 
-                                segmentsGrob(y0 = rep(1, length(labscat)),
-                                             x0 = labscat,
-                                             y1 = rep(.6, length(labscat)),
-                                             x1 = labscat,
-                                             vp = viewport(xscale = atrange),
-                                             default.units = "native",
-                                             name = trellis.grobname("ticks",
-                                               type="colorkey"),
-                                             gp =
-                                             gpar(col = axis.line$col,
-                                                  lty = axis.line$lty,
-                                                  lwd = axis.line$lwd)),
-                                row = 2, col = 2)
+            if (key$tck != 0)
+                key.gf <- placeGrob(frame = key.gf, 
+                                    segmentsGrob(y0 = rep(1, length(labscat)),
+                                                 x0 = labscat,
+                                                 y1 = rep(1 - key$tck / (1 + key$tck), length(labscat)),
+                                                 x1 = labscat,
+                                                 vp = viewport(xscale = atrange),
+                                                 default.units = "native",
+                                                 name = trellis.grobname("ticks",
+                                                                         type="colorkey"),
+                                                 gp =
+                                                 gpar(col = axis.line$col,
+                                                      lty = axis.line$lty,
+                                                      lwd = axis.line$lwd)),
+                                    row = 2, col = 2)
             key.gf <- placeGrob(key.gf,
                                 labelsGrob, 
                                 row = 3, col = 2)
