@@ -49,7 +49,7 @@ setAs("xsparseVector", "zsparseVector",
 
 setAs("xsparseVector", "nsparseVector",
       function(from) {
-          if(any(is.na(from@x)))
+	  if(anyNA(from@x))
               stop("cannot coerce 'NA's to \"nsparseVector\"")
           new("nsparseVector", i = from@i, length = from@length)
       })
@@ -82,40 +82,70 @@ setMethod("is.finite", signature(x = "sparseVector"),
 	      r
 	  })
 
+##' Uniquify sparceVectors, i.e., bring them in "regularized" from,
+##' --- similar in spirit (and action!) as  uniqTsparse(.) for "TsparseMatrix"
+##' __FIXME__ better name ??  , then export and document!  __TODO__
+uniqSpVec <- function(x) {
+    ii <- sort.list(x@i, method = "radix")
+    x@i <- x@i[ii]
+    x@x <- x@x[ii]
+    x
+}
 
-sp2vec <- function(x, mode = .type.kind[substr(cl, 1,1)]) {
+sp2vec <- function(x, mode = .type.kind[.M.kindC(class(x))]) {
     ## sparseVector  ->  vector
-    cl <- class(x)
+    has.x <- .hasSlot(x, "x")## has "x" slot
+    m.any <- (mode == "any")
+    if(m.any)
+	mode <- if(has.x) mode(x@x) else "logical"
+    else if(has.x) # is.<mode>() is much faster than inherits() | is():
+        xxOk <- switch(mode,
+		       "double" = is.double(x@x),
+		       "logical" = is.logical(x@x),
+		       "integer" = is.integer(x@x),
+		       "complex" = is.complex(x@x),
+		       ## otherwise (does not happen with default 'mode'):
+		       inherits(x@x, mode))
     r <- vector(mode, x@length)
     r[x@i] <-
-	if(cl != "nsparseVector") { # cheap test for 'has x slot'
-	    if(is(x@x, mode)) x@x else as(x@x, mode)
+	if(has.x) {
+	    if(m.any || xxOk) x@x else as(x@x, mode)
 	} else TRUE
     r
 }
 
+## so base functions calling as.vector() work too:
+## S3 dispatch works for base::as.vector(), but S4 dispatch does not:
+as.vector.sparseVector <- sp2vec
+as.array.sparseVector <- as.matrix.sparseVector <- function(x, ...) .sparseV2Mat(x)
+
 ##' Construct new sparse vector , *dropping* zeros
 
 ##' @param class  character, the sparseVector class
-##' @param x      numeric/logical/...:  the 'x' slot
+##' @param x      numeric/logical/...:  the 'x' slot -- if missing ==> "nsparseVector"
 ##' @param i      integer: index of non-zero entries
 ##' @param length integer: the 'length' slot
 
 ##' @return a sparseVector, with 0-dropped 'x' (and 'i')
 newSpV <- function(class, x, i, length, drop0 = TRUE, checkSort = TRUE) {
-    if(length(x) == 1 && (li <- length(i)) != 1) ## recycle x :
-	x <- rep.int(x, li)
-    if(drop0 && isTRUE(any(x0 <- x == 0))) {
-	keep <- is.na(x) | !x0
-	x <- x[keep]
-	i <- i[keep]
+    if(has.x <- !missing(x)) {
+	if(length(x) == 1 && (li <- length(i)) != 1) ## recycle x :
+	    x <- rep.int(x, li)
+	if(drop0 && isTRUE(any(x0 <- x == 0))) {
+	    keep <- is.na(x) | !x0
+	    x <- x[keep]
+	    i <- i[keep]
+	}
     }
     if(checkSort && is.unsorted(i)) {
 	ii <- sort.list(i)
-	x <- x[ii]
+	if(has.x) x <- x[ii]
 	i <- i[ii]
     }
-    new(class, x = x, i = i, length = length)
+    if(has.x)
+	new(class, x = x, i = i, length = length)
+    else
+	new(class,        i = i, length = length)
 }
 ## a "version" of 'prev' with changed contents:
 newSpVec <- function(class, x, prev)
@@ -123,17 +153,14 @@ newSpVec <- function(class, x, prev)
 
 ## Exported:
 sparseVector <- function(x, i, length) {
-    newSpV(class = paste0(.V.kind(x), "sparseVector"),
+    newSpV(class = paste0(if(missing(x)) "n" else .V.kind(x), "sparseVector"),
            x=x, i=i, length=length)
 }
 
 
 setAs("sparseVector", "vector", function(from) sp2vec(from))
 
-setMethod("as.vector", signature(x = "sparseVector", mode = "missing"),
-	  sp2vec)
-setMethod("as.vector", signature(x = "sparseVector", mode = "character"),
-	  sp2vec)
+setMethod("as.vector", "sparseVector", sp2vec)
 
 setMethod("as.numeric", "sparseVector", function(x) sp2vec(x, mode = "double"))
 setMethod("as.logical", "sparseVector", function(x) sp2vec(x, mode = "logical"))
@@ -173,6 +200,7 @@ setAs("CsparseMatrix", "sparseVector", ## could go via TsparseMatrix, but this i
       function(from) {
 	  d <- dim(from)
 	  n <- prod(d) # -> numeric, no integer overflow
+	  if((int.n <- n <= .Machine$integer.max)) n <- as.integer(n)
           cld <- getClassDef(class(from))
 	  kind <- .M.kind(from, cld)
 	  if(extends(cld, "symmetricMatrix"))
@@ -180,7 +208,7 @@ setAs("CsparseMatrix", "sparseVector", ## could go via TsparseMatrix, but this i
 	  else if(extends(cld, "triangularMatrix") && from@diag == "U")
 	      from <- .Call(Csparse_diagU2N, from)
           xj <- .Call(Matrix_expand_pointers, from@p)
-	  ii <- if(n < .Machine$integer.max)
+	  ii <- if(int.n)
 	      1L + from@i + d[1] * xj
 	  else
 	      1 + from@i + as.double(d[1]) * xj
@@ -195,15 +223,16 @@ setAs("TsparseMatrix", "sparseVector",
       function(from) {
 	  d <- dim(from)
 	  n <- prod(d) # -> numeric, no integer overflow
+	  if((int.n <- n <= .Machine$integer.max)) n <- as.integer(n)
           cld <- getClassDef(class(from))
 	  kind <- .M.kind(from, cld)
 	  if(extends(cld, "symmetricMatrix"))
 	      from <- as(from, "generalMatrix")
 	  else if(extends(cld, "triangularMatrix") && from@diag == "U")
 	      from <- .Call(Tsparse_diagU2N, from)
-	  if(is_duplicatedT(from, di = d))
+	  if(anyDuplicatedT(from, di = d))
 	      from <- uniqTsparse(from)
-	  ii <- if(n < .Machine$integer.max)
+	  ii <- if(int.n)
 	      1L + from@i + d[1] * from@j
 	  else
 	      1 + from@i + as.double(d[1]) * from@j
@@ -304,7 +333,7 @@ spV2M <- function (x, nrow, ncol, byrow = FALSE, check = TRUE, symmetric = FALSE
     r
 }## {spV2M}
 
-.sparseV2Mat <- function(from) spV2M(from, nrow=length(from), ncol=1L, check=FALSE)
+.sparseV2Mat <- function(from) spV2M(from, nrow=from@length, ncol=1L, check=FALSE)
 setAs("sparseVector","Matrix", .sparseV2Mat)
 setAs("sparseVector","sparseMatrix", .sparseV2Mat)
 setAs("sparseVector","TsparseMatrix", .sparseV2Mat)
@@ -324,6 +353,8 @@ setMethod("dim<-", signature(x = "sparseVector", value = "ANY"),
 
 setMethod("length", "sparseVector", function(x) x@length)
 
+setMethod("t", "sparseVector", function(x) spV2M(x, nrow=1L, ncol=x@length, check=FALSE))
+
 setMethod("show", signature(object = "sparseVector"),
    function(object) {
        n <- object@length
@@ -391,7 +422,7 @@ intIv <- function(i, n, cl.i = getClass(class(i)))
     ## else :
     if(extends(cl.i, "numeric")) {
         storage.mode(i) <- "integer"
-        int2i(i,n) ##-> ./Tsparse.R 
+        int2i(i,n) ##-> ./Tsparse.R
     }
     else if (extends(cl.i, "logical")) {
 	seq_len(n)[i]
@@ -418,7 +449,7 @@ setMethod("head", signature(x = "sparseVector"),
 	      ## As we *know* that '@i' is sorted increasingly: [x@i <= n] <==> [1:kk]
 	      x@length <- n
 	      x@i <- x@i[ii <- seq_len(which.max(x@i > n) - 1L)]
-	      if(substr(class(x), 1,1) != "n") ## be fast, ...
+	      if(.hasSlot(x, "x")) ## has.x: has "x" slot
 		  x@x <- x@x[ii]
 	      x
 	  })
@@ -436,7 +467,7 @@ setMethod("tail", signature(x = "sparseVector"),
 	      N <- length(x@i)
 	      ii <- if(any(G <- x@i > n)) which.max(G):N else FALSE
 	      x@i <- x@i[ii] - n
-	      if(substr(class(x), 1,1) != "n") ## be fast, ...
+	      if(.hasSlot(x, "x")) ## has.x: has "x" slot
 		  x@x <- x@x[ii]
 	      x
 	  })
@@ -444,29 +475,25 @@ setMethod("tail", signature(x = "sparseVector"),
 
 setMethod("[", signature(x = "sparseVector", i = "index"),
 	  function (x, i, j, ..., drop) {
-	      cld <- getClassDef(class(x))
-	      has.x <- !extends(cld, "nsparseVector")
+	      has.x <- .hasSlot(x, "x")## has "x" slot
 	      n <- x@length
 	      if(extends(cl.i <- getClass(class(i)), "numeric") && any(i < 0)) {
-		  ## negative indices - remain sparse --> *not* using intIv()
 		  if(any(i > 0))
 		      stop("you cannot mix negative and positive indices")
 		  if(any(z <- i == 0)) i <- i[!z]
-
-		  ## all (i < 0) :
-
-		  ## FIXME: an efficient solution would use C here
+		  ## all (i < 0), negative indices:
+		  ## want to remain sparse --> *not* using intIv()
+		  ##
+		  ## TODO: more efficient solution would use C ..
 		  i <- unique(sort(-i)) # so we need to drop the 'i's
-		  if(any(nom <- is.na(m <- match(x@i, i)))) {
-		      ## eliminate those with non-0 match
-		      x@i <- x@i[nom]
-		      if(has.x) x@x <- x@x[nom]
-		  }
-		  ii <- findInterval(x@i, i)	## subtract that :
-		  x@i <- x@i - ii
-		  x@length <- x@length - length(i)
-
-              } else {
+		  nom <- is.na(m <- match(x@i, i))
+		  ## eliminate those non-0 which do match:
+		  x@i <- x@i[nom]
+		  if(has.x) x@x <- x@x[nom]
+		  ## now all x@i "appear in 'i' but must be adjusted for the removals:
+		  x@i <- x@i - findInterval(x@i, i)
+		  x@length <- n - length(i)
+              } else { ## i >= 0  or  non-numeric 'i'
                   ii <- intIv(i, n, cl.i=cl.i)
                   m <- match(x@i, ii, nomatch = 0)
                   sel <- m > 0L
@@ -475,8 +502,8 @@ setMethod("[", signature(x = "sparseVector", i = "index"),
 		  if(any(iDup <- duplicated(ii))) {
                       i.i <- match(ii[iDup], ii)
                       jm <- lapply(i.i, function(.) which(. == m))
-                      sel <- c(which(sel), unlist(jm))
-                      x@i <- c(x@i, rep.int(which(iDup), sapply(jm, length)))
+                      if (has.x) sel <- c(which(sel), unlist(jm))
+                      x@i <- c(x@i, rep.int(which(iDup), lengths(jm)))
                   }
                   if (has.x)
                       x@x <- x@x[sel]
@@ -510,12 +537,11 @@ replSPvec <- function (x, i, value)
     n <- x@length
     ii <- intIv(i, n)
     lenRepl <- length(ii)
+    if(!lenRepl) return(x)
+    ## else:  lenRepl = length(ii) > 0
     lenV <- length(value)
-    if(lenV == 0) {
-	if(lenRepl != 0)
-	    stop("nothing to replace with")
-	else return(x)
-    }
+    if(lenV == 0)
+        stop("nothing to replace with")
     ## else: lenV := length(value) > 0
     if(lenRepl %% lenV != 0)
 	stop("number of items to replace is not a multiple of replacement length")
@@ -528,8 +554,7 @@ replSPvec <- function (x, i, value)
 	lenRepl <- length(ii)
     }
 
-    cld <- getClassDef(class(x))
-    has.x <- !extends(cld, "nsparseVector")
+    has.x <- .hasSlot(x, "x")## has "x" slot
     m <- match(x@i, ii, nomatch = 0)
     sel <- m > 0L
 
@@ -541,17 +566,17 @@ replSPvec <- function (x, i, value)
 		x@x <- x@x[!sel]
 	}
 	return(x)
-
     }
     ## else --	some( value != 0 ) --
     if(lenV > lenRepl)
 	stop("too many replacement values")
     else if(lenV < lenRepl)
 	value <- rep(value, length = lenRepl)
-    ## now:  length(value) == lenRepl
+    ## now:  length(value) == lenRepl > 0
 
     v0 <- is0(value)
     ## value[1:lenRepl]:  which are structural 0 now, which not?
+    v.sp <- inherits(value, "sparseVector")
 
     if(any(sel)) {
 	## indices of non-zero entries -- WRT to subvector
@@ -559,9 +584,10 @@ replSPvec <- function (x, i, value)
 
 	## 1a) replace those that are already non-zero with new val.
 	vN0 <- !v0[iN0]
-	if(any(vN0) && has.x)
-	    x@x[sel][vN0] <- value[iN0[vN0]]
-
+	if(any(vN0) && has.x) {
+	    vs <- value[iN0[vN0]]
+	    x@x[sel][vN0] <- if(v.sp) sp2vec(vs, mode=typeof(x@x)) else vs
+	}
 	## 1b) replace non-zeros with 0 --> drop entries
 	if(any(!vN0)) {
 	    i <- which(sel)[!vN0]
@@ -569,8 +595,7 @@ replSPvec <- function (x, i, value)
 		x@x <- x@x[-i]
 	    x@i <- x@i[-i]
 	}
-	iI0 <- if(length(iN0) < lenRepl)
-	    seq_len(lenRepl)[-iN0]
+	iI0 <- if(length(iN0) < lenRepl) seq_len(lenRepl)[-iN0] # else NULL
     } else iI0 <- seq_len(lenRepl)
 
     if(length(iI0) && any(vN0 <- !v0[iI0])) {
@@ -578,18 +603,18 @@ replSPvec <- function (x, i, value)
 	ij0 <- iI0[vN0]
 	x@i <- c(x@i, ii[ij0])
 	if(has.x)
-	    x@x <- c(x@x, value[ij0])
+	    x@x <- c(x@x, if(v.sp) sp2vec(value[ij0], mode=typeof(x@x)) else value[ij0])
     }
     x
 }
 
 setReplaceMethod("[", signature(x = "sparseVector", i = "index", j = "missing",
-				value = "replValue"),
+				value = "replValueSp"),
 		 replSPvec)
 
 setReplaceMethod("[", signature(x = "sparseVector",
                                 i = "sparseVector", j = "missing",
-				value = "replValue"),
+				value = "replValueSp"),
                  ## BTW, the important case: 'i' a *logical* sparseVector
 		 replSPvec)
 
@@ -662,8 +687,8 @@ all.equal.sparseV <- function(target, current, ...)
 	return(paste0("sparseVector", ": lengths (", lt, ", ", lc, ") differ"))
     }
 
-    t.has.x <- class(target)  != "nsparseVector"
-    c.has.x <- class(current) != "nsparseVector"
+    t.has.x <- .hasSlot(target,  "x")## has "x" slot
+    c.has.x <- .hasSlot(current, "x")## has "x" slot
     nz.t <- length(i.t <- target @i)
     nz.c <- length(i.c <- current@i)
     t.x <- if(t.has.x)	target@x else rep.int(TRUE, nz.t)
@@ -717,6 +742,39 @@ setMethod("all.equal", c(target = "sparseVector", current = "ANY"),
 	  all.equal(as.vector(target), current, ...))
 
 
+## S3 method for 'c' [but only for dispatch on 1st arg, hence also exported as fn]
+c.sparseVector <- function(...) {
+    svl <- lapply(list(...), as, Class = "sparseVector")
+    ## cls <- unique(unlist(lapply(svl, is)))
+    ns <- vapply(svl, slot, 1, "length")
+    if((N <- sum(ns)) < .Machine$integer.max) { # some purism ..
+	ns <- as.integer(ns)
+	N <- as.integer(N)
+    }
+    narg <- length(ns)
+    iss <- lapply(svl, slot, "i")
+    ## new 'i' slot:
+    ii <- unlist(iss) + rep(cumsum(c(0L, ns[-narg])), lengths(iss))
+    ## result must have 'x' slot if we have any
+    has.x <- any(have.x <- vapply(svl, .hasSlot, logical(1L), name = "x"))
+    if(has.x) {
+	cls <- if     (any(vapply(svl, is, NA, "zsparseVector"))) "zsparseVector"
+	       else if(any(vapply(svl, is, NA, "dsparseVector"))) "dsparseVector"
+	       else if(any(vapply(svl, is, NA, "isparseVector"))) "isparseVector"
+	       else "lsparseVector"
+	if(!(all.x <- all(have.x)))
+	    one <- if     (identical(cls, "lsparseVector")) TRUE
+		   else if(identical(cls, "isparseVector")) 1L else 1.
+	xx <- unlist(if(all.x) lapply(svl, slot, "x")
+		     else lapply(seq_len(narg), function(i) {
+			 if(have.x[[i]]) svl[[i]]@x
+			 else rep_len(one, length(iss[[i]]))
+		     }))
+	new(cls, x = xx,     i = ii, length = N)
+    } else ## no "x" slot
+	new("nsparseVector", i = ii, length = N)
+}
+
 ### rep(x, ...) -- rep() is primitive with internal default method with these args:
 ### -----------
 ### till R 2.3.1, it had  rep.default()  which we use as 'model' here.
@@ -725,7 +783,7 @@ repSpV <- function(x, times) {
     ## == rep.int(<sparseVector>, times)"
     times <- as.integer(times)# truncating as rep.default()
     n <- x@length
-    has.x <- substr(class(x), 1,1) != "n" ## fast, but hackish
+    has.x <- .hasSlot(x, "x")## has "x" slot
     ## just assign new correct slots:
     if(times <= 1) { ## be quick for {0, 1} times
         if(times < 0) stop("'times >= 0' is required")
@@ -768,43 +826,11 @@ setMethod("rep", "sparseVector",
 
 ### Group Methods (!)
 ## "Ops" : ["Arith", "Compare", "Logic"]:  ---> in ./Ops.R
-##                                                 -------
-
-## "Summary" group : "max"   "min"   "range" "prod"  "sum"   "any"   "all"
-
-setMethod("Summary", signature(x = "nsparseVector", na.rm = "ANY"),
-	  function(x, ..., na.rm) { ## no 'x' slot, no NA's ..
-	      n <- x@length
-	      l.x <- length(x@i)
-	      if(l.x == n)
-		  callGeneric(rep.int(TRUE, n), ..., na.rm = na.rm)
-	      else ## l.x < n :	 has some FALSE entries
-		  switch(.Generic,
-			 "prod" = 0,
-			 "min"	= 0L,
-			 "all" = FALSE,
-			 "any" = l.x > 0,
-			 "sum" = l.x,
-			 "max" = as.integer(l.x > 0),
-			 "range" = c(0L, as.integer(l.x > 0)))
-	  })
-
-## The "other" "sparseVector"s ("d", "l", "i" ..): all have an	'x' slot :
-setMethod("Summary", signature(x = "sparseVector", na.rm = "ANY"),
-	  function(x, ..., na.rm) {
-	      n <- x@length
-	      l.x <- length(x@x)
-	      if(l.x == n) ## fully non-zero (and "general") - very rare but quick
-		  callGeneric(x@x, ..., na.rm = na.rm)
-	      else if(.Generic != "prod") {
-		  logicF <- .Generic %in% c("any","all")
-		  ## we rely on	 <generic>(x, NULL, y, ..) :==	<generic>(x, y, ..):
-		  callGeneric(x@x, if(logicF) FALSE else 0, ..., na.rm = na.rm)
-	      }
-	      else { ## prod()
-		  if(any(is.na(x@x))) NaN else 0
-	      }
-	  })
+##						     -----
+## "Summary"  ---> ./Summary.R
+##		     ---------
+## "Math", "Math2": ./Math.R
+##		     -------
 
 
 setMethod("solve", signature(a = "Matrix", b = "sparseVector"),
@@ -812,8 +838,8 @@ setMethod("solve", signature(a = "Matrix", b = "sparseVector"),
 
 ## the 'i' slot is 1-based *and* has no NA's:
 
-setMethod("which", "nsparseVector", function(x, arr.ind) x@i)
-setMethod("which", "lsparseVector", function(x, arr.ind) x@i[is1(x@x)])
+setMethod("which", "nsparseVector", function(x, arr.ind, useNames) x@i)
+setMethod("which", "lsparseVector", function(x, arr.ind, useNames) x@i[is1(x@x)])
 ## and *error* for "dsparseVector", "i*", ...
 
 ##' indices of vector x[] to construct  Toeplitz matrix
