@@ -12,6 +12,23 @@ extern "C" {
 #include <R.h>  /* includes Rconfig.h */
 #include <Rversion.h>
 #include <Rdefines.h> /* Rinternals.h + GET_SLOT etc */
+#include <R_ext/RS.h> /* for Memzero() */
+
+#define imax2(x, y) ((x < y) ? y : x)
+#define imin2(x, y) ((x < y) ? x : y)
+
+
+// must come after <R.h> above, for clang (2015-08-05)
+#ifdef __GNUC__
+# undef alloca
+# define alloca(x) __builtin_alloca((x))
+#elif defined(__sun) || defined(_AIX)
+/* this is necessary (and sufficient) for Solaris 10 and AIX 6: */
+# include <alloca.h>
+#endif
+/* For R >= 3.2.2, the 'elif' above shall be replaced by
+#elif defined(HAVE_ALLOCA_H)
+*/
 
 #ifdef ENABLE_NLS
 #include <libintl.h>
@@ -22,19 +39,40 @@ extern "C" {
 #define dngettext(pkg, String, StringP, N) (N > 1 ? StringP : String)
 #endif
 
-#ifdef __GNUC__
-# undef alloca
-# define alloca(x) __builtin_alloca((x))
-#elif defined(__sun) || defined(_AIX)
-/* this is necessary (and sufficient) for Solaris 10 and AIX 6: */
-# include <alloca.h>
+#ifndef LONG_VECTOR_SUPPORT
+// notably for  R <= 2.15.x :
+# define XLENGTH(x) LENGTH(x)
+# if R_VERSION < R_Version(2,16,0)
+  typedef int R_xlen_t;
+# endif
 #endif
 
 #define Alloca(n, t)   (t *) alloca( (size_t) ( (n) * sizeof(t) ) )
 
+#define SMALL_4_Alloca 10000
+//			==== R uses the same cutoff in several places
+
+#define C_or_Alloca_TO(_VAR_, _N_, _TYPE_)			\
+	if(_N_ < SMALL_4_Alloca) {				\
+	    _VAR_ = Alloca(_N_, _TYPE_);  R_CheckStack();	\
+	} else {						\
+	    _VAR_ = Calloc(_N_, _TYPE_);			\
+	}
+// and user needs to   if(_N_ >= SMALL_4_Alloca)  Free(_VAR_);
+
 SEXP triangularMatrix_validate(SEXP obj);
 SEXP symmetricMatrix_validate(SEXP obj);
 SEXP dense_nonpacked_validate(SEXP obj);
+SEXP dim_validate(SEXP Dim, const char* name);
+SEXP Dim_validate(SEXP obj, SEXP name);
+SEXP dimNames_validate(SEXP obj);
+SEXP dimNames_validate__(SEXP dmNms, int dims[], const char* obj_name);
+
+
+// La_norm_type() & La_rcond_type()  have been in R_ext/Lapack.h
+//  but have still not been available to package writers ...
+char La_norm_type (const char *typstr);
+char La_rcond_type(const char *typstr);
 
 /* enum constants from cblas.h and some short forms */
 enum CBLAS_ORDER {CblasRowMajor=101, CblasColMajor=102};
@@ -59,6 +97,7 @@ SEXP set_double_by_name(SEXP obj, double val, char *nm);
 SEXP as_det_obj(double val, int log, int sign);
 SEXP get_factors(SEXP obj, char *nm);
 SEXP set_factors(SEXP obj, SEXP val, char *nm);
+SEXP R_set_factors(SEXP obj, SEXP val, SEXP name, SEXP warn);
 
 #if 0
 SEXP dgCMatrix_set_Dim(SEXP x, int nrow);
@@ -73,8 +112,16 @@ Rboolean equal_string_vectors(SEXP s1, SEXP s2);
 
 void d_packed_getDiag(double *dest, SEXP x, int n);
 void l_packed_getDiag(   int *dest, SEXP x, int n);
-void tr_d_packed_getDiag(double *dest, SEXP x);
-void tr_l_packed_getDiag(   int *dest, SEXP x);
+SEXP d_packed_setDiag(double *diag, int l_d, SEXP x, int n);
+SEXP l_packed_setDiag(   int *diag, int l_d, SEXP x, int n);
+SEXP d_packed_addDiag(double *diag, int l_d, SEXP x, int n);
+
+void tr_d_packed_getDiag(double *dest, SEXP x, int n);
+void tr_l_packed_getDiag(   int *dest, SEXP x, int n);
+
+SEXP tr_d_packed_setDiag(double *diag, int l_d, SEXP x, int n);
+SEXP tr_l_packed_setDiag(   int *diag, int l_d, SEXP x, int n);
+SEXP tr_d_packed_addDiag(double *diag, int l_d, SEXP x, int n);
 
 SEXP Matrix_getElement(SEXP list, char *nm);
 
@@ -115,29 +162,46 @@ extern	 /* stored pointers to symbols initialized in R_init_Matrix */
     if(R_has_slot(src, sym))					\
 	SET_SLOT(dest, sym, duplicate(GET_SLOT(src, sym)))
 
-/* TODO: Make this faster for the case where dimnames = list(NULL,NULL)
- *       and hence don't have to be set ! */
-#define SET_DimNames(dest, src) slot_dup(dest, src, Matrix_DimNamesSym)
+static R_INLINE
+void SET_DimNames(SEXP dest, SEXP src) {
+    SEXP dn = GET_SLOT(src, Matrix_DimNamesSym);
+    // Be fast (do nothing!) for the case where dimnames = list(NULL,NULL) :
+    if (!(isNull(VECTOR_ELT(dn,0)) && isNull(VECTOR_ELT(dn,1))))
+	SET_SLOT(dest, Matrix_DimNamesSym, duplicate(dn));
+}
+
+// code in ./Mutils.c :
+SEXP symmetric_DimNames(SEXP dn);
+SEXP R_symmetric_Dimnames(SEXP x);
+void SET_DimNames_symm(SEXP dest, SEXP src);
 
 
 #define uplo_P(_x_) CHAR(STRING_ELT(GET_SLOT(_x_, Matrix_uploSym), 0))
 #define diag_P(_x_) CHAR(STRING_ELT(GET_SLOT(_x_, Matrix_diagSym), 0))
+#define Diag_P(_x_) (R_has_slot(x, Matrix_diagSym) ?			\
+		     CHAR(STRING_ELT(GET_SLOT(_x_, Matrix_diagSym), 0)) : " ")
 #define class_P(_x_) CHAR(asChar(getAttrib(_x_, R_ClassSymbol)))
 
+
+enum dense_enum { ddense, ldense, ndense };
+
 // Define this "Cholmod compatible" to some degree
-enum x_slot_kind {x_pattern=-1, x_double=0, x_logical=1, x_integer=2, x_complex=3};
-//		    n		  d	      l		   i		z
+    enum x_slot_kind {
+	x_unknown=-2, x_pattern=-1, x_double=0, x_logical=1, x_integer=2, x_complex=3};
+//	  NA	              n		  d	      l		   i		z
+
+// FIXME: use 'x_slot_kind' instead of 'int' everywhere  Real_(KIND2?|kind)  is used
 
 /* should also work for "matrix" matrices: */
 #define Real_KIND(_x_)	(IS_S4_OBJECT(_x_) ? Real_kind(_x_) : \
-			 (isReal(_x_) ? x_double : (isLogical(_x_) ? x_logical : -1)))
-/* This one gives '0' also for integer "matrix" :*/
+			 (isReal(_x_) ? x_double : (isLogical(_x_) ? x_logical : x_pattern)))
+/* This one gives 'x_double' also for integer "matrix" :*/
 #define Real_KIND2(_x_)	(IS_S4_OBJECT(_x_) ? Real_kind(_x_) : \
-			 (isLogical(_x_) ? x_logical : 0))
+			 (isLogical(_x_) ? x_logical : x_double))
 
-/* requires 'x' slot: */
-#define Real_kind(_x_)	(isReal(GET_SLOT(_x_, Matrix_xSym)) ? 0	:	\
-			 (isLogical(GET_SLOT(_x_, Matrix_xSym)) ? 1 : -1))
+/* requires 'x' slot, i.e., not for ..nMatrix.  FIXME ? via R_has_slot(obj, name) */
+#define Real_kind(_x_)	(isReal(GET_SLOT(_x_, Matrix_xSym)) ? x_double	: \
+			 (isLogical(GET_SLOT(_x_, Matrix_xSym)) ? x_logical : x_pattern))
 
 #define DECLARE_AND_GET_X_SLOT(__C_TYPE, __SEXP)	\
     __C_TYPE *xx = __SEXP(GET_SLOT(x, Matrix_xSym))
@@ -181,7 +245,7 @@ int packed_ncol(int len)
  * @return SEXP of given type and length assigned as slot nm in obj
  */
 static R_INLINE
-SEXP ALLOC_SLOT(SEXP obj, SEXP nm, SEXPTYPE type, int length)
+SEXP ALLOC_SLOT(SEXP obj, SEXP nm, SEXPTYPE type, R_xlen_t length)
 {
     SEXP val = allocVector(type, length);
 
@@ -229,6 +293,30 @@ Rboolean any_NA_in_x(SEXP obj)
 }
 
 
+
+/** Inverse Permutation
+ * C version of   .inv.perm.R <- function(p) { p[p] <- seq_along(p) ; p }
+ */
+static R_INLINE
+SEXP inv_permutation(SEXP p_, SEXP zero_p, SEXP zero_res)
+{
+    int np = 1;
+    if(!isInteger(p_)) {p_ = PROTECT(coerceVector(p_, INTSXP)); np++; }
+    int *p = INTEGER(p_), n = LENGTH(p_);
+    SEXP val = PROTECT(allocVector(INTSXP, n));
+    int *v = INTEGER(val), p_0 = asLogical(zero_p), r_0 = asLogical(zero_res);
+    if(!p_0) v--; // ==> use 1-based indices
+    // shorter (but not 100% sure if ok: is LHS always eval'ed *before* RHS ?) :
+    // for(int i=0; i < n; ) v[p[i]] = ++i;
+    for(int i=0; i < n; ) {
+	int j = p[i]; v[j] = (r_0) ? i++ : ++i;
+    }
+    UNPROTECT(np);
+    return val;
+}
+
+SEXP Mmatrix(SEXP args);
+
 void make_d_matrix_triangular(double *x, SEXP from);
 void make_i_matrix_triangular(   int *x, SEXP from);
 
@@ -237,19 +325,27 @@ void make_i_matrix_symmetric(   int *to, SEXP from);
 
 SEXP Matrix_expand_pointers(SEXP pP);
 
-SEXP dup_mMatrix_as_dgeMatrix(SEXP A);
-SEXP dup_mMatrix_as_geMatrix (SEXP A);
+SEXP dup_mMatrix_as_dgeMatrix2(SEXP A, Rboolean tr_if_vec);
+SEXP dup_mMatrix_as_dgeMatrix (SEXP A);
+SEXP dup_mMatrix_as_geMatrix  (SEXP A);
 
 SEXP new_dgeMatrix(int nrow, int ncol);
-SEXP m_encodeInd (SEXP ij, SEXP di, SEXP chk_bnds);
-SEXP m_encodeInd2(SEXP i, SEXP j, SEXP di, SEXP chk_bnds);
+SEXP m_encodeInd (SEXP ij,        SEXP di, SEXP orig_1, SEXP chk_bnds);
+SEXP m_encodeInd2(SEXP i, SEXP j, SEXP di, SEXP orig_1, SEXP chk_bnds);
 
+SEXP R_rbind2_vector(SEXP a, SEXP b);
+
+SEXP R_all0(SEXP x);
+SEXP R_any0(SEXP x);
 
 static R_INLINE SEXP
-mMatrix_as_dgeMatrix(SEXP A)
-{
+mMatrix_as_dgeMatrix(SEXP A) {
     return strcmp(class_P(A), "dgeMatrix") ? dup_mMatrix_as_dgeMatrix(A) : A;
 }
+static R_INLINE SEXP
+mMatrix_as_dgeMatrix2(SEXP A, Rboolean tr_if_vec) {
+    return strcmp(class_P(A), "dgeMatrix") ? dup_mMatrix_as_dgeMatrix2(A, tr_if_vec) : A;
+}
 
 static R_INLINE SEXP
 mMatrix_as_geMatrix(SEXP A)
@@ -258,16 +354,45 @@ mMatrix_as_geMatrix(SEXP A)
 }
 
 // Keep centralized --- *and* in sync with ../inst/include/Matrix.h :
-#define MATRIX_VALID_dense			\
+#define MATRIX_VALID_ge_dense			\
         "dmatrix", "dgeMatrix",			\
 	"lmatrix", "lgeMatrix",			\
 	"nmatrix", "ngeMatrix",			\
 	"zmatrix", "zgeMatrix"
 
+/* NB:  ddiMatrix & ldiMatrix are part of VALID_ddense / VALID_ldense
+ * --   even though they are no longer "denseMatrix" formally.
+ * CARE: dup_mMatrix_as_geMatrix() code depends on  14 ddense and 6 ldense
+ * ----  entries here :
+*/
+#define MATRIX_VALID_ddense					\
+		    "dgeMatrix", "dtrMatrix",			\
+		    "dsyMatrix", "dpoMatrix", "ddiMatrix",	\
+		    "dtpMatrix", "dspMatrix", "dppMatrix",	\
+		    /* sub classes of those above:*/		\
+		    /* dtr */ "Cholesky", "LDL", "BunchKaufman",\
+		    /* dtp */ "pCholesky", "pBunchKaufman",	\
+		    /* dpo */ "corMatrix"
+
+#define MATRIX_VALID_ldense			\
+		    "lgeMatrix", "ltrMatrix",	\
+		    "lsyMatrix", "ldiMatrix",	\
+		    "ltpMatrix", "lspMatrix"
+
+#define MATRIX_VALID_ndense			\
+		    "ngeMatrix", "ntrMatrix",	\
+		    "nsyMatrix",		\
+		    "ntpMatrix", "nspMatrix"
+
+#define MATRIX_VALID_dCsparse			\
+ "dgCMatrix", "dsCMatrix", "dtCMatrix"
+#define MATRIX_VALID_nCsparse			\
+ "ngCMatrix", "nsCMatrix", "ntCMatrix"
+
 #define MATRIX_VALID_Csparse			\
- "dgCMatrix", "dsCMatrix", "dtCMatrix",		\
+    MATRIX_VALID_dCsparse,			\
  "lgCMatrix", "lsCMatrix", "ltCMatrix",		\
- "ngCMatrix", "nsCMatrix", "ntCMatrix",		\
+    MATRIX_VALID_nCsparse,			\
  "zgCMatrix", "zsCMatrix", "ztCMatrix"
 
 #define MATRIX_VALID_Tsparse			\
@@ -282,11 +407,29 @@ mMatrix_as_geMatrix(SEXP A)
  "ngRMatrix", "nsRMatrix", "ntRMatrix",		\
  "zgRMatrix", "zsRMatrix", "ztRMatrix"
 
+#define MATRIX_VALID_tri_Csparse		\
+   "dtCMatrix", "ltCMatrix", "ntCMatrix", "ztCMatrix"
+
+#ifdef __UN_USED__
+#define MATRIX_VALID_tri_sparse			\
+ "dtCMatrix",  "dtTMatrix", "dtRMatrix",	\
+ "ltCMatrix",  "ltTMatrix", "ltRMatrix",	\
+ "ntCMatrix",  "ntTMatrix", "ntRMatrix",	\
+ "ztCMatrix",  "ztTMatrix", "ztRMatrix"
+
+#define MATRIX_VALID_tri_dense			\
+ "dtrMatrix",  "dtpMatrix"			\
+ "ltrMatrix",  "ltpMatrix"			\
+ "ntrMatrix",  "ntpMatrix"			\
+ "ztrMatrix",  "ztpMatrix"
+#endif
+
 #define MATRIX_VALID_CHMfactor "dCHMsuper", "dCHMsimpl", "nCHMsuper", "nCHMsimpl"
 
 /**
  * Return the 0-based index of a string match in a vector of strings
  * terminated by an empty string.  Returns -1 for no match.
+ * Is  __cheap__ :  __not__ looking at superclasses --> better use  R_check_class_etc(obj, *)
  *
  * @param class string to match
  * @param valid vector of possible matches terminated by an empty string
@@ -306,19 +449,21 @@ Matrix_check_class(char *class, const char **valid)
 /**
  * These are the ones "everyone" should use -- is() versions, also looking
  * at super classes:
- */
-#if R_VERSION < R_Version(2, 15, 0)
-int Matrix_check_class_etc(SEXP x, const char **valid);
-#else
-# define Matrix_check_class_etc R_check_class_etc
-#endif
 
-#if R_VERSION < R_Version(2, 13, 0)
-int Matrix_check_class_and_super(SEXP x, const char **valid, SEXP rho);
-#else
+ * They now use R(semi_API) from  Rinternals.h :
+ * int R_check_class_and_super(SEXP x, const char **valid, SEXP rho);
+ * int R_check_class_etc      (SEXP x, const char **valid);
+
+ * R_check_class_etc      (x, v)      basically does  rho <- .classEnv(x)  and then calls
+ * R_check_class_and_super(x, v, rho)
+ */
+// No longer:
+#ifdef DEPRECATED_Matrix_check_class_
+# define Matrix_check_class_etc R_check_class_etc
 # define Matrix_check_class_and_super R_check_class_and_super
 #endif
 
+SEXP NEW_OBJECT_OF_CLASS(const char* cls);
 
 /** Accessing  *sparseVectors :  fast (and recycling)  v[i] for v = ?sparseVector:
  * -> ./sparseVector.c  -> ./t_sparseVector.c :
