@@ -551,7 +551,7 @@ csi *cs_counts (const cs *A, const csi *parent, const csi *post, csi ata)
         if (parent [j] != -1) colcount [parent [j]] += colcount [j] ;
     }
     return (cs_idone (colcount, AT, w, 1)) ;    /* success: free workspace */
-}
+} 
 /* p [0..n] = cumulative sum of c [0..n-1], and then copy p [0..n-1] into c */
 double cs_cumsum (csi *p, csi *c, csi n)
 {
@@ -761,7 +761,7 @@ static csi cs_nonzero (csi i, csi j, double aij, void *other)
 csi cs_dropzeros (cs *A)
 {
     return (cs_fkeep (A, &cs_nonzero, NULL)) ;  /* keep all nonzero entries */
-}
+} 
 /* remove duplicate entries from A */
 csi cs_dupl (cs *A)
 {
@@ -1271,7 +1271,7 @@ cs *cs_multiply (const cs *A, const cs *B)
         if (nz + m > C->nzmax && !cs_sprealloc (C, 2*(C->nzmax)+m))
         {
             return (cs_done (C, w, x, 0)) ;             /* out of memory */
-        }
+        } 
         Ci = C->i ; Cx = C->x ;         /* C->i and C->x may be reallocated */
         Cp [j] = nz ;                   /* column j of C starts here */
         for (p = Bp [j] ; p < Bp [j+1] ; p++)
@@ -1371,7 +1371,7 @@ csi cs_print (const cs *A, csi brief)
             (double) n, (double) nzmax, (double) (Ap [n]), cs_norm (A)) ;
         for (j = 0 ; j < n ; j++)
         {
-            Rprintf ("    col %g : locations %g to %g\n", (double) j,
+            Rprintf ("    col %g : locations %g to %g\n", (double) j, 
                 (double) (Ap [j]), (double) (Ap [j+1]-1)) ;
             for (p = Ap [j] ; p < Ap [j+1] ; p++)
             {
@@ -1408,7 +1408,7 @@ csn *cs_qr (const cs *A, const css *S)
     csi i, k, p, m, n, vnz, p1, top, m2, len, col, rnz, *s, *leftmost, *Ap, *Ai,
         *parent, *Rp, *Ri, *Vp, *Vi, *w, *pinv, *q ;
     cs *R, *V ;
-    csn *N ;
+    csn *N ; // the result
     if (!CS_CSC (A) || !S) return (NULL) ;
     m = A->m ; n = A->n ; Ap = A->p ; Ai = A->i ; Ax = A->x ;
     q = S->q ; parent = S->parent ; pinv = S->pinv ; m2 = S->m2 ;
@@ -1536,14 +1536,16 @@ csi *cs_randperm (csi n, csi seed)
     if (!p) return (NULL) ;             /* out of memory */
     for (k = 0 ; k < n ; k++) p [k] = n-k-1 ;
     if (seed == -1) return (p) ;        /* return reverse permutation */
-    srand (seed) ;                      /* get new random number seed */
+    GetRNGstate();/* <- for R package Matrix
+    srand (seed) ;                      .* get new random number seed */
     for (k = 0 ; k < n ; k++)
     {
-        j = k + (rand ( ) % (n-k)) ;    /* j = rand integer in range k to n-1 */
+        j = k + (int)(unif_rand() * (n-k)); // j = rand integer in range k to n-1
         t = p [j] ;                     /* swap p[k] and p[j] */
         p [j] = p [k] ;
         p [k] = t ;
     }
+    PutRNGstate(); // <- R package Matrix
     return (p) ;
 }
 /* xi [top...n-1] = nodes reachable from graph of G*P' via nodes in B(:,k).
@@ -1752,10 +1754,8 @@ css *cs_sqr (csi order, const cs *A, csi qr)
         post = cs_post (S->parent, n) ;
         S->cp = cs_counts (C, S->parent, post, 1) ;  /* col counts chol(C'*C) */
         cs_free (post) ;
-        ok = C && S->parent && S->cp &&
-	    cs_vcount (C, S) ; /* sets S->pinv et al */
+        ok = C && S->parent && S->cp && cs_vcount (C, S) ;
         if (ok) for (S->unz = 0, k = 0 ; k < n ; k++) S->unz += S->cp [k] ;
-        ok = ok && S->lnz >= 0 && S->unz >= 0 ;     /* csi overflow guard */
         if (order) cs_spfree (C) ;
     }
     else
@@ -1895,8 +1895,14 @@ csi cs_usolve (const cs *U, double *x)
     n = U->n ; Up = U->p ; Ui = U->i ; Ux = U->x ;
     for (j = n-1 ; j >= 0 ; j--)
     {
-        x [j] /= Ux [Up [j+1]-1] ;
-        for (p = Up [j] ; p < Up [j+1]-1 ; p++)
+        csi Upj_1 = Up [j+1]-1;
+        if (Upj_1 < 0) {
+            warning("cs_usolve(U, x): U is not invertible (j=%d)", j);
+	    x [j] = NA_REAL;
+	} else {
+	    x [j] /= Ux [Upj_1] ;
+	}
+        for (p = Up [j] ; p < Upj_1 ; p++)
         {
             x [Ui [p]] -= Ux [p] * x [j] ;
         }
@@ -2030,11 +2036,17 @@ csi cs_utsolve (const cs *U, double *x)
     n = U->n ; Up = U->p ; Ui = U->i ; Ux = U->x ;
     for (j = 0 ; j < n ; j++)
     {
-        for (p = Up [j] ; p < Up [j+1]-1 ; p++)
+        csi Upj_1 = Up [j+1]-1;
+        for (p = Up [j] ; p < Upj_1 ; p++)
         {
             x [j] -= Ux [p] * x [Ui [p]] ;
         }
-        x [j] /= Ux [Up [j+1]-1] ;
+        if (Upj_1 < 0) {
+            warning("cs_utsolve(U, x): U' is not invertible (j=%d)", j);
+	    x [j] = NA_REAL;
+	} else {
+	    x [j] /= Ux [Upj_1] ;
+	}
     }
     return (1) ;
 }
