@@ -162,10 +162,15 @@ tre_match(const tre_tnfa_t *tnfa, const void *string, size_t len,
       if (type == STR_USER)
 	{
 	  const tre_str_source *source = string;
-	  if (source->rewind == NULL || source->compare == NULL)
+	  if (source->rewind == NULL || source->compare == NULL) {
 	    /* The backtracking matcher requires rewind and compare
 	       capabilities from the input stream. */
+#ifndef TRE_USE_ALLOCA
+	    if (tags)
+		xfree(tags);
+#endif /* !TRE_USE_ALLOCA */
 	    return REG_BADPAT;
+	  }
 	}
       status = tre_tnfa_run_backtrack(tnfa, string, (int)len, type,
 				      tags, eflags, &eo);
@@ -217,7 +222,6 @@ tre_regexec(const regex_t *preg, const char *str,
   return tre_regnexec(preg, str, (unsigned)-1, nmatch, pmatch, eflags);
 }
 
-/* R addition */
 int
 tre_regexecb(const regex_t *preg, const char *str,
 	     size_t nmatch, regmatch_t pmatch[], int eflags)
@@ -329,7 +333,6 @@ tre_regaexec(const regex_t *preg, const char *str,
   return tre_reganexec(preg, str, (unsigned)-1, match, params, eflags);
 }
 
-/* R addition */
 int
 tre_regaexecb(const regex_t *preg, const char *str,
 	  regamatch_t *match, regaparams_t params, int eflags)
