@@ -196,7 +196,18 @@ int tre_iscntrl_func(tre_cint_t c) { return tre_iscntrl(c); }
 int tre_isdigit_func(tre_cint_t c) { return tre_isdigit(c); }
 int tre_isgraph_func(tre_cint_t c) { return tre_isgraph(c); }
 int tre_islower_func(tre_cint_t c) { return tre_islower(c); }
-int tre_isprint_func(tre_cint_t c) { return tre_isprint(c); }
+int tre_isprint_func(tre_cint_t c)
+{
+#ifdef TRE_WCHAR
+  /* Windows has \t as printable via iswprint in all locales. By POSIX
+     and ?regex, we need \t to be non-printable in the C locale, so we
+     cannot use iswprint. By C99, iswprint(L'\t') should be the same as
+     isprint('\t'). In Windows, in C locale, isprint('\t') is false, 
+     hence this workaround. */
+    if (c == L'\t') return isprint('\t');
+#endif
+  return tre_isprint(c); /* Windows has \t as printable */
+}
 int tre_ispunct_func(tre_cint_t c) { return tre_ispunct(c); }
 int tre_isspace_func(tre_cint_t c) { return tre_isspace(c); }
 int tre_isupper_func(tre_cint_t c) { return tre_isupper(c); }
