@@ -1,6 +1,5 @@
-#ifndef lint
-static char    *copyright = "Copyright (C) 1991, 1992, 1993, Chris Thewalt";
-#endif
+#include <R_ext/Boolean.h>
+#include <R_ext/Error.h>
 
 /*
  * Copyright (C) 1991, 1992, 1993 by Chris Thewalt (thewalt@ce.berkeley.edu)
@@ -19,38 +18,26 @@ static char    *copyright = "Copyright (C) 1991, 1992, 1993, Chris Thewalt";
  */
 
 #include       "getline.h"
+
 static int      gl_tab();  /* forward reference needed for gl_tab_hook */
-
-/********************* exported interface ********************************/
-
-int             getline();		/* read a line of input */
-void            gl_setwidth();		/* specify width of screen */
-void            gl_histadd();		/* adds entries to hist */
-void		gl_strwidth();		/* to bind gl_strlen */
-
 int 		(*gl_in_hook)() = 0;
 int 		(*gl_out_hook)() = 0;
 int 		(*gl_tab_hook)() = gl_tab;
 
+#include <rlocale.h>
+extern Rboolean mbcslocale;
+#define mbs_init(x) memset(x, 0, sizeof(mbstate_t))
 
-/******************** imported interface *********************************/
+/* NB:  this define must match the one in src/main/scan.c */
+#define CONSOLE_PROMPT_SIZE	256
 
 #include <string.h>
 #include <ctype.h>
-#include <errno.h>
 #include <setjmp.h>
-#ifdef __unix__
-#include <signal.h>
-#endif
 #include <stdlib.h>
 #include <stdio.h>
 #include <io.h>
 
-/*extern int     isatty();	
-extern void    *malloc();
-extern void    *calloc();
-extern void     free();*/
-
 /******************** internal interface *********************************/
 
 
@@ -64,11 +51,7 @@ static int      gl_overwrite = 0;	/* overwrite mode */
 static int      gl_pos, gl_cnt = 0;     /* position and size of input */
 static char    *gl_buf;                 /* input buffer */
 static char    *gl_killbuf;             /* killed text */
-static char    *gl_prompt;		/* to save the prompt string */
-static char     gl_intrc = 0;		/* keyboard SIGINT char */
-static char     gl_quitc = 0;		/* keyboard SIGQUIT char */
-static char     gl_suspc = 0;		/* keyboard SIGTSTP char */
-static char     gl_dsuspc = 0;		/* delayed SIGTSTP char */
+static const char    *gl_prompt;	/* to save the prompt string */
 static int      gl_search_mode = 0;	/* search mode flag */
 
 static jmp_buf  gl_jmp;
@@ -111,254 +94,54 @@ static void     gl_beep(void);          /* try to play a system beep sound */
 /*extern int      write();
  extern void     exit();*/
 
-#ifdef unix
-#ifndef __unix__
-#define __unix__
-#endif /* not __unix__ */
-#endif /* unix */
-
-#ifdef _IBMR2
-#ifndef __unix__
-#define __unix__
-#endif
-#endif
-
-#ifdef __GO32__
-#include <pc.h>
-#undef MSDOS
-#undef __unix__
-#endif
-
-#ifdef MSDOS
-#include <bios.h>
-#endif
-
-#ifdef Win32
-/* guido masarotto (3/12/98)*/
+#define WIN32_LEAN_AND_MEAN 1
 #include <windows.h>
 static HANDLE Win32OutputStream, Win32InputStream = NULL;
 static DWORD OldWin32Mode, AltIsDown;
-#endif
-
-#ifdef __unix__
-#ifndef __convexc__
-extern int      read();
-extern int      kill();
-extern int      ioctl();
-#endif /* not __convexc__ */
-#ifdef POSIX		/* use POSIX interface */
-#include <termios.h>
-struct termios  new_termios, old_termios;
-#else /* not POSIX */
-#include <sys/ioctl.h>
-#ifdef M_XENIX	/* does not really use bsd terminal interface */
-#undef TIOCSETN
-#endif /* M_XENIX */
-#ifdef TIOCSETN		/* use BSD interface */
-#include <sgtty.h>
-struct sgttyb   new_tty, old_tty;
-struct tchars   tch;
-struct ltchars  ltch;
-#else			/* use SYSV interface */
-#include <termio.h>
-struct termio   new_termio, old_termio;
-#endif /* TIOCSETN */
-#endif /* POSIX */
-#endif	/* __unix__ */
-
-#ifdef vms
-#include <descrip.h>
-#include <ttdef.h>
-#include <iodef.h>
-#include unixio
-   
-static int   setbuff[2];             /* buffer to set terminal attributes */
-static short chan = -1;              /* channel to terminal */
-struct dsc$descriptor_s descrip;     /* VMS descriptor */
-#endif
 
 static void
 gl_char_init()			/* turn off input echo */
 {
-#ifdef __unix__
-#ifdef POSIX
-    tcgetattr(0, &old_termios);
-    gl_intrc = old_termios.c_cc[VINTR];
-    gl_quitc = old_termios.c_cc[VQUIT];
-#ifdef VSUSP
-    gl_suspc = old_termios.c_cc[VSUSP];
-#endif
-#ifdef VDSUSP
-    gl_dsuspc = old_termios.c_cc[VDSUSP];
-#endif
-    new_termios = old_termios;
-    new_termios.c_iflag &= ~(BRKINT|ISTRIP|IXON|IXOFF);
-    new_termios.c_iflag |= (IGNBRK|IGNPAR);
-    new_termios.c_lflag &= ~(ICANON|ISIG|IEXTEN|ECHO);
-    new_termios.c_cc[VMIN] = 1;
-    new_termios.c_cc[VTIME] = 0;
-    tcsetattr(0, TCSANOW, &new_termios);
-#else				/* not POSIX */
-#ifdef TIOCSETN			/* BSD */
-    ioctl(0, TIOCGETC, &tch);
-    ioctl(0, TIOCGLTC, &ltch);
-    gl_intrc = tch.t_intrc;
-    gl_quitc = tch.t_quitc;
-    gl_suspc = ltch.t_suspc;
-    gl_dsuspc = ltch.t_dsuspc;
-    ioctl(0, TIOCGETP, &old_tty);
-    new_tty = old_tty;
-    new_tty.sg_flags |= RAW;
-    new_tty.sg_flags &= ~ECHO;
-    ioctl(0, TIOCSETN, &new_tty);
-#else				/* SYSV */
-    ioctl(0, TCGETA, &old_termio);
-    gl_intrc = old_termio.c_cc[VINTR];
-    gl_quitc = old_termio.c_cc[VQUIT];
-    new_termio = old_termio;
-    new_termio.c_iflag &= ~(BRKINT|ISTRIP|IXON|IXOFF);
-    new_termio.c_iflag |= (IGNBRK|IGNPAR);
-    new_termio.c_lflag &= ~(ICANON|ISIG|ECHO);
-    new_termio.c_cc[VMIN] = 1;
-    new_termio.c_cc[VTIME] = 0;
-    ioctl(0, TCSETA, &new_termio);
-#endif /* TIOCSETN */
-#endif /* POSIX */
-#endif /* __unix__ */
-
-#ifdef vms
-    descrip.dsc$w_length  = strlen("tt:");
-    descrip.dsc$b_dtype   = DSC$K_DTYPE_T;
-    descrip.dsc$b_class   = DSC$K_CLASS_S;
-    descrip.dsc$a_pointer = "tt:";
-    (void)sys$assign(&descrip,&chan,0,0);
-    (void)sys$qiow(0,chan,IO$_SENSEMODE,0,0,0,setbuff,8,0,0,0,0);
-    setbuff[1] |= TT$M_NOECHO;
-    (void)sys$qiow(0,chan,IO$_SETMODE,0,0,0,setbuff,8,0,0,0,0);
-#endif /* vms */
-
-#ifdef Win32
-/* guido masarotto (3/12/98)*/   
    if (!Win32InputStream) {
        Win32InputStream = GetStdHandle(STD_INPUT_HANDLE);
        Win32OutputStream = GetStdHandle(STD_OUTPUT_HANDLE);	
    }
    GetConsoleMode(Win32InputStream,&OldWin32Mode);
-   SetConsoleMode(Win32InputStream, 0);
+   SetConsoleMode(Win32InputStream, ENABLE_PROCESSED_INPUT); /* So ^C works */
    AltIsDown = 0;
-#endif      
-   
 }
 
 static void
 gl_char_cleanup(void)		/* undo effects of gl_char_init */
 {
-#ifdef __unix__
-#ifdef POSIX 
-    tcsetattr(0, TCSANOW, &old_termios);
-#else 			/* not POSIX */
-#ifdef TIOCSETN		/* BSD */
-    ioctl(0, TIOCSETN, &old_tty);
-#else			/* SYSV */
-    ioctl(0, TCSETA, &old_termio);
-#endif /* TIOCSETN */
-#endif /* POSIX */
-#endif /* __unix__ */
-
-#ifdef vms
-    setbuff[1] &= ~TT$M_NOECHO;
-    (void)sys$qiow(0,chan,IO$_SETMODE,0,0,0,setbuff,8,0,0,0,0);
-    sys$dassgn(chan);
-    chan = -1;
-#endif
-   
-#ifdef Win32
-/* guido masarotto (3/12/98)*/
    SetConsoleMode(Win32InputStream,OldWin32Mode);
    AltIsDown = 0;
-#endif         
 }
 
-#if MSDOS || __EMX__ || __GO32__
-int pc_keymap(c)
-int c;
-{
-    switch (c) {
-    case 72: c = 16;   /* up -> ^P */
-        break;
-    case 80: c = 14;   /* down -> ^N */
-        break;
-    case 75: c = 2;    /* left -> ^B */
-        break;
-    case 77: c = 6;    /* right -> ^F */
-        break;
-    default: c = 0;    /* make it garbage */
-    }
-    return c;
-}
-#endif /* MSDOS || __EMX__ || __GO32__ */
-
 static int
 gl_getc(void)
 /* get a character without echoing it to screen */
 {
     int             c;
-#ifdef __unix__
-    char            ch;
-#endif
+    static char buf[9] = "";
+    static int bufavail = 0;
+    static int bufpos = 0;
 
-#ifdef __unix__
-    while ((c = read(0, &ch, 1)) == -1) {
-	if (errno != EINTR)
-	    break;
+    if (bufavail > 0) {
+	bufavail--;
+	return buf[bufpos++];
     }
-    c = (ch <= 0)? -1 : ch;
-#endif	/* __unix__ */
-#ifdef MSDOS
-    c = _bios_keybrd(_NKEYBRD_READ);
-#endif  /* MSDOS */
-#ifdef __GO32__
-    c = getkey () ;
-    if (c > 255) c = pc_keymap(c & 0377);
-#endif /* __GO32__ */
-#ifdef __TURBOC__
-    while(!bioskey(1))
-	;
-    c = bioskey(0);
-#endif
-#if MSDOS || __TURBOC__
-    if ((c & 0377) == 224) {
-	c = pc_keymap((c >> 8) & 0377);
-    } else {
-	c &= 0377;
-    }
-#endif /* MSDOS || __TURBOC__ */
-#ifdef __EMX__
-    c = _read_kbd(0, 1, 0);
-    if (c == 224 || c == 0) {
-        c = pc_keymap(_read_kbd(0, 1, 0));
-    } else {
-        c &= 0377;
-    }
-#endif
-#ifdef vms
-    if(chan < 0) {
-       c='\0';
-    }
-    (void)sys$qiow(0,chan,IO$_TTYREADALL,0,0,0,&c,1,0,0,0,0);
-    c &= 0177;                        /* get a char */
-#endif
+    bufpos = 0;
 
-#ifdef Win32
 /* guido masarotto (3/12/98)
  * get Ansi char code from a Win32 console
-*/
-{   DWORD a;
+ */
+    DWORD a;
     INPUT_RECORD r;
     DWORD st;
     WORD vk;
     CONSOLE_SCREEN_BUFFER_INFO csb;
-    int bbb = 0, nAlt=0, n, d;
+    int bbb = 0, nAlt=0, n;
 
     c = 0; 
     while (!c) {
@@ -368,14 +151,14 @@ gl_getc(void)
       */
       GetConsoleScreenBufferInfo(Win32OutputStream, &csb);
       SetConsoleCursorPosition(Win32OutputStream, csb.dwCursorPosition);
-      ReadConsoleInput(Win32InputStream, &r, 1, &a);
+      ReadConsoleInputW(Win32InputStream, &r, 1, &a);
       if (!(r.EventType == KEY_EVENT)) break;
       st = r.Event.KeyEvent.dwControlKeyState;
       vk = r.Event.KeyEvent.wVirtualKeyCode;
       if (r.Event.KeyEvent.bKeyDown) {
-	if (vk == VK_MENU && (st & LEFT_ALT_PRESSED)) { /* VK_MENU is
+        AltIsDown = (st & LEFT_ALT_PRESSED);
+	if (vk == VK_MENU && AltIsDown) { /* VK_MENU is
 							   Alt or AltGr */
-	  AltIsDown = 1;
 	  nAlt = 0;
 	  bbb  = 0;
 	} 
@@ -412,8 +195,28 @@ gl_getc(void)
 	    nAlt = 0;
 	  } 
 	} 
-	else 
-	  c = r.Event.KeyEvent.uChar.AsciiChar;
+	else {
+	  /* Originally uChar.AsciiChar was used here and for MBCS characters
+	     GetConsoleInput returned as many events as bytes in the character.
+	     As of Windows 8 this reportedly no longer works, GetConsoleInput
+	     would only generate one event with the first byte in AsciiChar.
+	     The bug still exists in Windows 10, and thus we now call
+	     GetConsoleInputW to get uchar.UnicodeChar. Ideally (at least for
+	     Windows) all of getline code would be refactored to work with wide
+	     characters, but for now we just convert the character back to bytes
+	     in current native locale to recover the old behavior of gl_getc. */
+	  wchar_t wc = r.Event.KeyEvent.uChar.UnicodeChar;
+	  mbstate_t mb_st;
+	  mbs_init(&mb_st);
+	  if (wc != L'\0') {
+	    size_t cres = wcrtomb(buf, wc, &mb_st);
+	    if (cres != (size_t)-1) {
+	      bufavail = (int) cres - 1;
+	      bufpos = 1;
+	      c = buf[0];
+	    }
+	  }
+	}
       }
       else if (vk == VK_MENU && AltIsDown) { 
            /* Alt key up event: could be AltGr, but let's hope users 
@@ -426,20 +229,14 @@ gl_getc(void)
       if ((c < -127) || (c > 255)) c = 0; 
       if (c < 0) c = 256 + c;    
     }
-}
-#endif         
     return c;
 }
 
 static void
 gl_putc(int c)
 {
-#ifdef Win32
-/* guido masarotto (3/12/98)*/   
    int ch = c;
-#else   
-   char   ch = c;
-#endif    
+
     write(1, &ch, 1);
     if (ch == '\n') {
 	ch = '\r';
@@ -460,8 +257,7 @@ gl_puts(const char *const buf)
     }
 }
 
-static void
-gl_error(const char *const buf)
+void gl_error(const char *const buf)
 {
     int len = strlen(buf);
 
@@ -507,13 +303,14 @@ gl_setwidth(int w)
 }
 
 int
-getline(char *prompt, char *buf, int buflen)
+getline(const char *prompt, char *buf, int buflen)
 {
     int             c, loc, tmp;
+    int mb_len;
+    mbstate_t mb_st;
+    int i;
+    wchar_t wc;
 
-#ifdef __unix__
-    int	            sig;
-#endif
     BUF_SIZE = buflen;
     gl_buf = buf;
     gl_buf[0] = '\0';
@@ -556,13 +353,24 @@ getline(char *prompt, char *buf, int buflen)
 		break; 
 	      case '\001': gl_fixup(gl_prompt, -1, 0);		/* ^A */
 		break;
-	      case '\002': gl_fixup(gl_prompt, -1, gl_pos-1);	/* ^B */
+	      case '\002': 	/* ^B */
+		if(mbcslocale) {
+		    mb_len = 0;
+		    mbs_init(&mb_st);
+		    for(i = 0; i < gl_pos ;) {
+			mbrtowc(&wc, gl_buf+i, MB_CUR_MAX, &mb_st);
+			mb_len = Ri18n_wcwidth(wc);
+			i += (wc==0) ? 0 : mb_len;
+		    }
+		    gl_fixup(gl_prompt, -1, gl_pos - mb_len);
+		} else
+		    gl_fixup(gl_prompt, -1, gl_pos-1);
 		break;
 	      case '\003':                                      /* ^C */
-                gl_fixup(gl_prompt,-1,gl_cnt);
-		gl_puts("^C\n");
-                gl_kill(0);
-                gl_fixup(gl_prompt,-2,BUF_SIZE);
+		  gl_fixup(gl_prompt, -1, gl_cnt);
+		  gl_puts("^C\n");
+		  gl_kill(0);
+		  gl_fixup(gl_prompt, -2, BUF_SIZE);
 		break;
 	      case '\004':					/* ^D */
 		if (gl_cnt == 0) {
@@ -576,7 +384,20 @@ getline(char *prompt, char *buf, int buflen)
 		break;
 	      case '\005': gl_fixup(gl_prompt, -1, gl_cnt);	/* ^E */
 		break;
-	      case '\006': gl_fixup(gl_prompt, -1, gl_pos+1);	/* ^F */
+		case '\006': /* ^F */
+		  if(mbcslocale) { 
+		      if(gl_pos >= gl_cnt) break;
+		      mb_len = 0;
+		      mbs_init(&mb_st);
+		      for(i = 0; i<= gl_pos ;){
+			  mbrtowc(&wc, gl_buf+i, MB_CUR_MAX, &mb_st);
+			  mb_len = Ri18n_wcwidth(wc);
+			  i += (wc==0) ? 0 : mb_len;
+		      }
+		      gl_fixup(gl_prompt, -1, gl_pos + mb_len);
+		  }
+		else
+		  gl_fixup(gl_prompt, -1, gl_pos+1);
 		break;
 	      case '\010': case '\177': gl_del(-1);	/* ^H and DEL */
 		break;
@@ -584,7 +405,7 @@ getline(char *prompt, char *buf, int buflen)
                 if (gl_tab_hook) {
 		    tmp = gl_pos;
 	            loc = gl_tab_hook(gl_buf, gl_strlen(gl_prompt), &tmp);
-	            if (loc >= 0 || tmp != gl_pos)
+	            if (loc != -1 || tmp != gl_pos)
 	                gl_fixup(gl_prompt, loc, tmp);
                 }
 		break;
@@ -644,10 +465,32 @@ getline(char *prompt, char *buf, int buflen)
 	                    gl_in_hook(gl_buf);
 		        gl_fixup(gl_prompt, 0, BUF_SIZE);
 		        break;
-		      case 'C': gl_fixup(gl_prompt, -1, gl_pos+1); /* right */
-		        break;
-		      case 'D': gl_fixup(gl_prompt, -1, gl_pos-1); /* left */
+		    case 'C': /* right */
+			if(mbcslocale) { 
+			    mb_len = 0;
+			    mbs_init(&mb_st);
+			    for(i = 0; i <= gl_pos ;) {
+				mbrtowc(&wc, gl_buf+i, MB_CUR_MAX, &mb_st);
+				mb_len = Ri18n_wcwidth(wc);
+				i += (wc==0) ? 0 : mb_len;
+			    }
+			    gl_fixup(gl_prompt, -1, gl_pos + mb_len);
+			} else
+			    gl_fixup(gl_prompt, -1, gl_pos+1);
 		        break;
+		    case 'D': /* left */
+		       if(mbcslocale) {
+			   mb_len = 0;
+			   mbs_init(&mb_st);
+			   for(i = 0; i <= gl_pos ;) {
+			       mbrtowc(&wc, gl_buf+i, MB_CUR_MAX, &mb_st);
+			       mb_len = Ri18n_wcwidth(wc);
+			       i += (wc==0) ? 0 :mb_len;
+			   }
+			   gl_fixup(gl_prompt, -1, gl_pos - mb_len);
+		       } else
+			 gl_fixup(gl_prompt, -1, gl_pos-1);
+			break;
 		      default: gl_putc('\007');         /* who knows */
 		        break;
 		    }
@@ -659,30 +502,6 @@ getline(char *prompt, char *buf, int buflen)
 		    gl_putc('\007');
 		break;
 	      default:		/* check for a terminal signal */
-#ifdef __unix__
-	        if (c > 0) {	/* ignore 0 (reset above) */
-	            sig = 0;
-#ifdef SIGINT
-	            if (c == gl_intrc)
-	                sig = SIGINT;
-#endif
-#ifdef SIGQUIT
-	            if (c == gl_quitc)
-	                sig = SIGQUIT;
-#endif
-#ifdef SIGTSTP
-	            if (c == gl_suspc || c == gl_dsuspc)
-	                sig = SIGTSTP;
-#endif
-                    if (sig != 0) {
-	                gl_cleanup();
-	                kill(0, sig);
-	                gl_init();
-	                gl_redraw();
-			c = 0;
-		    } 
-		}
-#endif /* __unix__ */
                 if (c > 0)
 		    gl_putc('\007');
 		break;
@@ -705,16 +524,54 @@ gl_addchar(int c)
             gl_putc('\a');
             return; 
     }
-    if (gl_overwrite == 0 || gl_pos == gl_cnt) {
-        for (i=gl_cnt; i >= gl_pos; i--)
-            gl_buf[i+1] = gl_buf[i];
-        gl_buf[gl_pos] = (char) c;
-        gl_fixup(gl_prompt, gl_pos, gl_pos+1);
-    } else {
-	gl_buf[gl_pos] = (char) c;
-	gl_extent = 1;
-        gl_fixup(gl_prompt, gl_pos, gl_pos+1);
-    }
+    if(mbcslocale) {
+	mbstate_t mb_st;
+	wchar_t wc;
+	char s[9]; /* only 3 needed */
+	int res;
+	int clen ;
+      
+	s[0] = c;
+	clen = 1;
+	res = 0;
+	/* This is a DBCS locale, so input is 1 or 2 bytes.
+	   This loop should not be necessary.
+	 */
+	if((unsigned int) c >= (unsigned int) 0x80) {
+            while(clen <= MB_CUR_MAX) {
+	        mbs_init(&mb_st);
+	        res = mbrtowc(&wc, s, clen, &mb_st);
+	        if(res >= 0) break;
+	        if(res == -1) 
+		    gl_error("invalid multibyte character in mbcs_get_next");
+  	        /* so res == -2 */
+	        c = gl_getc();
+	        if(c == EOF) 
+		    gl_error("EOF whilst reading MBCS char");
+	        s[clen++] = c;
+	    } /* we've tried enough, so must be complete or invalid by now */
+	}
+	if( res >= 0 ) {
+	    if (!(gl_overwrite == 0 || gl_pos == gl_cnt))  
+		gl_del(0); 
+	    for (i = gl_cnt; i >= gl_pos; i--)
+                gl_buf[i+clen] = gl_buf[i];
+	    for (i = 0; i < clen; i++)
+                gl_buf[gl_pos + i] = s[i];
+	    gl_fixup(gl_prompt, gl_pos, gl_pos+clen);
+	}
+       
+    } else
+	if (gl_overwrite == 0 || gl_pos == gl_cnt) {
+	    for (i = gl_cnt; i >= gl_pos; i--)
+		gl_buf[i+1] = gl_buf[i];
+	    gl_buf[gl_pos] = (char) c;
+	    gl_fixup(gl_prompt, gl_pos, gl_pos+1);
+	} else {
+	    gl_buf[gl_pos] = (char) c;
+	    gl_extent = 1;
+	    gl_fixup(gl_prompt, gl_pos, gl_pos+1);
+	}
 }
 
 static void
@@ -755,11 +612,36 @@ gl_transpose(void)
     int    c;
 
     if (gl_pos > 0 && gl_cnt > gl_pos) {
-	c = gl_buf[gl_pos-1];
-	gl_buf[gl_pos-1] = gl_buf[gl_pos];
-	gl_buf[gl_pos] = (char) c;
-	gl_extent = 2;
-	gl_fixup(gl_prompt, gl_pos-1, gl_pos);
+	if(mbcslocale) {
+	    int l_len = 0;
+	    int r_len = 0;
+	    int i = 0;
+	    int j = 0;
+	    mbstate_t mb_st;
+
+	    mbs_init(&mb_st);
+	    for (i = 0; i < gl_pos;) {
+		l_len = mbrlen(gl_buf+i, MB_CUR_MAX, &mb_st);
+		i += l_len;
+	    }
+	    mbs_init(&mb_st);
+	    r_len = mbrlen(gl_buf+gl_pos, MB_CUR_MAX, &mb_st);
+	    for (i = 0; i < r_len; i++) {
+		for(j = 0; j < l_len; j++) {
+		    c = gl_buf[gl_pos+i-j];
+		    gl_buf[gl_pos+i-j] = gl_buf[gl_pos+i-j-1];
+		    gl_buf[gl_pos+i-j-1] = (char)c;
+		}
+	    }
+	    gl_extent = l_len + r_len;
+	    gl_fixup(gl_prompt, gl_pos - l_len, gl_pos + (r_len - l_len)); 
+	} else {
+	    c = gl_buf[gl_pos-1];
+	    gl_buf[gl_pos-1] = gl_buf[gl_pos];
+	    gl_buf[gl_pos] = (char) c;
+	    gl_extent = 2;
+	    gl_fixup(gl_prompt, gl_pos-1, gl_pos);
+	}
     } else
 	gl_beep();
 }
@@ -797,14 +679,34 @@ gl_del(int loc)
  *     0 : delete character under cursor
  */
 {
-    int i;
+   int i;
 
-    if ((loc == -1 && gl_pos > 0) || (loc == 0 && gl_pos < gl_cnt)) {
-        for (i=gl_pos+loc; i < gl_cnt; i++)
-	    gl_buf[i] = gl_buf[i+1];
-	gl_fixup(gl_prompt, gl_pos+loc, gl_pos+loc);
-    } else
-	gl_beep();
+   if(mbcslocale) {
+       int mb_len;
+       mbstate_t mb_st;
+       wchar_t wc;
+
+       mb_len=0;
+       mbs_init(&mb_st);
+   
+       if ((loc == -1 && gl_pos > 0) || (loc == 0 && gl_pos < gl_cnt)) {
+	   for(i = 0; i<= gl_pos + loc;) {
+	       mbrtowc(&wc,gl_buf+i, MB_CUR_MAX, &mb_st);
+	       mb_len = Ri18n_wcwidth(wc);
+	       i += (wc==0) ? 0 : mb_len;
+	   }
+	   for (i = gl_pos+(loc*mb_len); i <= gl_cnt - mb_len; i++)
+	       gl_buf[i] = gl_buf[i + mb_len];
+	   gl_fixup(gl_prompt,gl_pos+(loc * mb_len) , gl_pos+(loc * mb_len));
+       } else
+	   gl_beep();
+   } else   
+       if ((loc == -1 && gl_pos > 0) || (loc == 0 && gl_pos < gl_cnt)) {
+	   for (i = gl_pos+loc; i < gl_cnt; i++)
+	       gl_buf[i] = gl_buf[i+1];
+	   gl_fixup(gl_prompt, gl_pos+loc, gl_pos+loc);
+       } else
+	   gl_beep();
 }
 
 static void
@@ -911,7 +813,7 @@ gl_fixup(const char *prompt, int change, int cursor)
     static int   gl_shift;	/* index of first on screen character */
     static int   off_right;	/* true if more text right of screen */
     static int   off_left;	/* true if more text left of screen */
-    static char  last_prompt[80] = "";
+    static char  last_prompt[CONSOLE_PROMPT_SIZE] = "";
     int          left = 0, right = -1;		/* bounds for redraw */
     int          pad;		/* how much to erase at end of line */
     int          backup;        /* how far to backup before fixing */
@@ -925,7 +827,7 @@ gl_fixup(const char *prompt, int change, int cursor)
 	while (gl_pos--) gl_putc('\b');
         gl_pos = gl_cnt = gl_shift = off_right = off_left = 0;
 	gl_puts(prompt);
-	strcpy(last_prompt, prompt);
+	strncpy(last_prompt, prompt, CONSOLE_PROMPT_SIZE-1);
 	change = 0;
         gl_width = gl_termw - gl_strlen(prompt);
     } else if (strcmp(prompt, last_prompt) != 0) {
@@ -933,7 +835,7 @@ gl_fixup(const char *prompt, int change, int cursor)
 	l2 = gl_strlen(prompt);
         ll = gl_pos + l1;
 	gl_cnt = gl_cnt + l1 - l2;
-	strcpy(last_prompt, prompt);
+	strncpy(last_prompt, prompt, CONSOLE_PROMPT_SIZE-1);
 	while (ll--) gl_putc('\b');	
 	gl_puts(prompt);
 	gl_pos = gl_shift;
@@ -1067,10 +969,9 @@ gl_hist_init(int size, int beep)
 }
 
 void
-gl_histadd(char *buf)
+gl_histadd(const char *buf)
 {
-    static char *prev = 0;
-    char *p = buf;
+    const char *p = buf;
 
     /* in case we call gl_histadd() before we call getline() */
     if (gl_init_done < 0) {		/* -1 only on startup */
@@ -1081,7 +982,6 @@ gl_histadd(char *buf)
 	p++;
     if (*p) {
 	hist_buf[hist_last] = hist_save(buf);
-	prev = hist_buf[hist_last];
 	hist_last = hist_last + 1;
 	if(hist_last > HIST_SIZE - 1) {
 	    int i, size = HIST_SIZE + 512;
@@ -1133,7 +1033,7 @@ gl_hist_next(void)
 }
 
 static char *
-hist_save(char *p)
+hist_save(const char *p)
         
 /* makes a copy of the string */
 {
@@ -1156,7 +1056,7 @@ hist_save(char *p)
     return s;
 }
 
-void gl_savehistory(char *file, int size)
+void gl_savehistory(const char *file, int size)
 {
     FILE *fp;
     int i, init;
@@ -1176,7 +1076,7 @@ void gl_savehistory(char *file, int size)
     fclose(fp); 
 }
 
-void gl_loadhistory(char *file)
+void gl_loadhistory(const char *file)
 {
     FILE *fp;
     int i;
@@ -1345,9 +1245,5 @@ search_forw(int new_search)
 static void
 gl_beep(void)
 {
-#ifdef Win32
 	if(gl_beep_on) MessageBeep(MB_OK);
-#else
-	gl_putc('\007');
-#endif
 }	/* gl_beep */
