@@ -1,11 +1,15 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1998-2012    The R Core Team
+ *  Copyright (C) 1998-2017    The R Core Team
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  This header file is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2.1 of the License, or
  *  (at your option) any later version.
+
+ *  This file is part of R. R is distributed under the terms of the
+ *  GNU General Public License, either Version 2, June 1991 or Version 3,
+ *  June 2007. See doc/COPYRIGHTS for details of the copyright status of R.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +31,14 @@
 
 #include <R_ext/Boolean.h>
 #include <R_ext/Complex.h>
-#include <stddef.h>
+
+#if defined(__cplusplus) && !defined(DO_NOT_USE_CXX_HEADERS)
+# include <cstddef>
+# define R_SIZE_T std::size_t
+#else
+# include <stddef.h>
+# define R_SIZE_T size_t
+#endif
 
 #define revsort       Rf_revsort
 #define iPsort        Rf_iPsort
@@ -56,9 +67,9 @@ void	cPsort(Rcomplex*, int, int);
 
 /* ../../main/qsort.c : */
 /* dummy renamed to II to avoid problems with g++ on Solaris */
-void R_qsort    (double *v,         size_t i, size_t j);
+void R_qsort    (double *v,         R_SIZE_T i, R_SIZE_T j);
 void R_qsort_I  (double *v, int *II, int i, int j);
-void R_qsort_int  (int *iv,         size_t i, size_t j);
+void R_qsort_int  (int *iv,         R_SIZE_T i, R_SIZE_T j);
 void R_qsort_int_I(int *iv, int *II, int i, int j);
 #ifdef R_RS_H
 void F77_NAME(qsort4)(double *v, int *indx, int *ii, int *jj);
@@ -67,6 +78,9 @@ void F77_NAME(qsort3)(double *v,            int *ii, int *jj);
 
 /* ../../main/util.c  and others : */
 const char *R_ExpandFileName(const char *);
+#ifdef Win32
+const char *R_ExpandFileNameUTF8(const char *);
+#endif
 void	setIVector(int*, int, int);
 void	setRVector(double*, int, double);
 Rboolean StringFalse(const char *);
@@ -84,13 +98,16 @@ char *R_tmpnam2(const char *prefix, const char *tempdir, const char *fileext);
 
 void R_CheckUserInterrupt(void);
 void R_CheckStack(void);
-void R_CheckStack2(size_t);
+void R_CheckStack2(R_SIZE_T);
 
 
 /* ../../appl/interv.c: also in Applic.h */
 int findInterval(double *xt, int n, double x,
 		 Rboolean rightmost_closed,  Rboolean all_inside, int ilo,
 		 int *mflag);
+int findInterval2(double *xt, int n, double x,
+		  Rboolean rightmost_closed,  Rboolean all_inside, Rboolean left_open,
+		  int ilo, int *mflag);
 #ifdef R_RS_H
 int F77_SUB(interv)(double *xt, int *n, double *x,
 		    Rboolean *rightmost_closed, Rboolean *all_inside,
