@@ -433,11 +433,11 @@ INLINE_FUN Rbyte RAW_ELT(SEXP x, R_xlen_t i)
     return ALTREP(x) ? ALTRAW_ELT(x, i) : RAW0(x)[i];
 }
 
-INLINE_FUN void SET_RAW_ELT(SEXP x, R_xlen_t i, int v)
+INLINE_FUN void SET_RAW_ELT(SEXP x, R_xlen_t i, Rbyte v)
 {
-    CHECK_VECTOR_LGL_ELT(x, i);
-    if (ALTREP(x)) ALTLOGICAL_SET_ELT(x, i, v);
-    else RAW0(x)[i] = (Rbyte) v;
+    CHECK_VECTOR_RAW_ELT(x, i);
+    if (ALTREP(x)) ALTRAW_SET_ELT(x, i, v);
+    else RAW0(x)[i] = v;
 }
 
 #if !defined(COMPILING_R) && !defined(COMPILING_MEMORY_C) &&	\
@@ -462,6 +462,7 @@ extern SEXP* R_PPStack;
 
 INLINE_FUN SEXP protect(SEXP s)
 {
+    R_CHECK_THREAD;
     if (R_PPStackTop < R_PPStackSize)
 	R_PPStack[R_PPStackTop++] = s;
     else R_signal_protect_error();
@@ -470,6 +471,7 @@ INLINE_FUN SEXP protect(SEXP s)
 
 INLINE_FUN void unprotect(int l)
 {
+    R_CHECK_THREAD;
 #ifdef PROTECT_PARANOID
     if (R_PPStackTop >=  l)
 	R_PPStackTop -= l;
@@ -487,6 +489,7 @@ INLINE_FUN void R_ProtectWithIndex(SEXP s, PROTECT_INDEX *pi)
 
 INLINE_FUN void R_Reprotect(SEXP s, PROTECT_INDEX i)
 {
+    R_CHECK_THREAD;
     if (i >= R_PPStackTop || i < 0)
 	R_signal_reprotect_error(i);
     R_PPStack[i] = s;
