@@ -32,7 +32,11 @@
 	  else gettext("See help(\"Deprecated\")"))
     }
     else as.character(msg)
-    warning(paste(msg, collapse=""), call. = FALSE, domain = NA)
+    msg <- paste(msg, collapse = "")
+
+    if (missing(new)) new <- NULL
+    warning(warningCondition(msg, old = old, new = new, package = package,
+                             class = "deprecatedWarning"))
 }
 
 ## consider keeping one (commented) entry here, for easier additions
