@@ -1,7 +1,7 @@
 #  File src/library/base/R/character.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2016 The R Core Team
+#  Copyright (C) 1995-2018 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -122,11 +122,10 @@ toupper <- function(x)
 casefold <- function(x, upper = FALSE)
     if(upper) toupper(x) else tolower(x)
 
-sQuote <- function(x)
+sQuote <- function(x, q = getOption("useFancyQuotes"))
 {
     if (!length(x)) return(character())
     before <- after <- "'"
-    q <- getOption("useFancyQuotes")
     if(!is.null(q)) {
         if(isTRUE(q)) {
             li <- l10n_info()
@@ -158,11 +157,10 @@ sQuote <- function(x)
     paste0(before, x, after)
 }
 
-dQuote <- function(x)
+dQuote <- function(x, q = getOption("useFancyQuotes"))
 {
     if (!length(x)) return(character())
     before <- after <- "\""
-    q <- getOption("useFancyQuotes")
     if(!is.null(q)) {
         if(isTRUE(q)) {
             li <- l10n_info()
