@@ -120,3 +120,29 @@ local({
         lapply(routines[[i]],
                function(sym) assign(paste0(".C_", sym$name), sym, envir = .BaseNamespaceEnv))
 })
+
+## make sure these two promises are forced to avoid recursive invocation
+## of "args" and consequent "promise already under evaluation" error
+
+invisible(force(.ArgsEnv))
+invisible(force(.GenericArgsEnv))
+
+## also force these condition system callback promises to avoid
+## recursive invocation in some rare situations at start-up
+invisible(force(.signalSimpleWarning))
+invisible(force(.handleSimpleError))
+invisible(force(.tryResumeInterrupt))
+
+local({
+    assignWrapped <- function(x, method, home, envir) {
+        method <- method                # force evaluation
+        home <- home                    # force evaluation
+        delayedAssign(x, get(method, envir = home), assign.env = envir)
+    }          
+    methods <- paste0(.S3_methods_table[, 1L], ".",
+                      .S3_methods_table[, 2L])
+    env <- .BaseNamespaceEnv
+    table <- env[[".__S3MethodsTable__."]]
+    for(m in methods)
+        assignWrapped(m, m, env, table)
+})
