@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2007-8  The R Foundation
+ *  Copyright (C) 2007-11  The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,9 +14,9 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, a copy is available at
- *  http://www.r-project.org/Licenses/
+ *  https://www.R-project.org/Licenses/
  *
- *  Modular Quartz device for Mac OS X
+ *  Modular Quartz device for macOS
  *
  *  Partially based on code by Byron Ellis
  */
@@ -31,6 +31,8 @@
 #include <Rinternals.h>
 #define R_USE_PROTOTYPES 1
 #include <R_ext/GraphicsEngine.h>
+/* This sets ptr_QuartzBackend as a symbol in this file */
+#define IN_AQUA_C 1
 #include <R_ext/QuartzDevice.h>
 
 #include "grDevices.h"
@@ -39,7 +41,7 @@
 
 #define DEVQUARTZ_VERSION 1 /* first public Quartz API version */
 
-#define QBE_NATIVE   1  /* either Cocoa or Carbon depending on the OS X version */
+#define QBE_NATIVE   1  /* either Cocoa or Carbon depending on the macOS version */
 #define QBE_COCOA    2  /* internal Cocoa */
 #define QBE_CARBON   3  /* internal Carbon */
 #define QBE_BITMAP   4  /* bitmap file creating */
@@ -73,7 +75,7 @@ const quartz_module_t quartz_modules[] = {
 
 
 
-/* for compatibility with OS X <10.5 */
+/* for compatibility with macOS <10.5 */
 #ifndef CGFLOAT_DEFINED
 typedef float CGFloat;
 #define CGFLOAT_MIN FLT_MIN
@@ -99,6 +101,7 @@ typedef struct QuartzSpecific_s {
     int           canvas;          /* background color */
     int           antialias,smooth;/* smoothing flags (only aa makes any sense) */
     int           flags;           /* additional QDFLAGs */
+    int           holdlevel;       /* hold level */
     int           redraw;          /* redraw flag is set when replaying
 		                              and inhibits syncs on Mode */
     CGRect        clipRect;        /* clipping rectangle */
@@ -324,6 +327,12 @@ static void* QuartzDevice_SetParameter(QuartzDesc_t desc, const char *key, void
     return NULL;
 }
 
+void setup_RdotApp(void)
+{
+    int eflags = QP_Flags_CFLoop | QP_Flags_Cocoa | QP_Flags_Front;
+    QuartzDevice_SetParameter(NULL, QuartzParam_EmbeddingFlags, &eflags);
+}
+
 static void*  QuartzDevice_GetParameter(QuartzDesc_t desc, const char *key)
 {
     if (desc) { /* backend-specific? pass it on */
@@ -344,6 +353,7 @@ static void     RQuartz_Activate(pDevDesc);
 static void     RQuartz_Deactivate(pDevDesc);
 static void     RQuartz_Size(double*, double*, double*, double*, pDevDesc);
 static void     RQuartz_NewPage(const pGEcontext, pDevDesc);
+static int      RQuartz_HoldFlush(pDevDesc, int);
 static void     RQuartz_Clip(double, double, double, double, pDevDesc);
 static double   RQuartz_StrWidth(const char*, const pGEcontext, pDevDesc);
 static void     RQuartz_Text(double, double, const char*, double, double, const pGEcontext, pDevDesc);
@@ -357,6 +367,7 @@ static void     RQuartz_Circle(double, double, double, const pGEcontext, pDevDes
 static void     RQuartz_Line(double, double, double, double, const pGEcontext, pDevDesc);
 static void     RQuartz_Polyline(int, double*, double*, const pGEcontext, pDevDesc);
 static void     RQuartz_Polygon(int, double*, double*, const pGEcontext, pDevDesc);
+static void     RQuartz_Path(double*, double*, int, int*, Rboolean, const pGEcontext, pDevDesc);
 static Rboolean RQuartz_Locator(double*, double*, pDevDesc);
 static void     RQuartz_Mode(int mode, pDevDesc);
 static void     RQuartz_MetricInfo(int, const pGEcontext , double*, double*, double*, pDevDesc);
@@ -390,9 +401,11 @@ void* QuartzDevice_Create(void *_dev, QuartzBackend_t *def)
     dev->line         = RQuartz_Line;
     dev->polyline     = RQuartz_Polyline;
     dev->polygon      = RQuartz_Polygon;
+    dev->path         = RQuartz_Path;
     dev->locator      = RQuartz_Locator;
     dev->mode         = RQuartz_Mode;
     dev->metricInfo   = RQuartz_MetricInfo;
+    dev->holdflush    = RQuartz_HoldFlush;
     dev->hasTextUTF8  = TRUE;
     dev->textUTF8     = RQuartz_Text;
     dev->strWidthUTF8 = RQuartz_StrWidth;
@@ -411,6 +424,12 @@ void* QuartzDevice_Create(void *_dev, QuartzBackend_t *def)
     dev->canChangeGamma= FALSE;
     dev->displayListOn = (def->flags & QDFLAG_DISPLAY_LIST) ? TRUE : FALSE;
 
+    dev->haveTransparency = 2;
+    dev->haveTransparentBg = 3; /* FIXME: depends on underlying device */
+    dev->haveRaster = 2;
+    dev->haveCapture = (def->cap) ? 2 : 1;
+    dev->haveLocator = (def->locatePoint) ? 2 : 1;
+
     QuartzDesc *qd = calloc(1, sizeof(QuartzDesc));
     qd->width      = def->width;
     qd->height     = def->height;
@@ -447,6 +466,7 @@ void* QuartzDevice_Create(void *_dev, QuartzBackend_t *def)
     qd->dirty = 0;
     qd->redraw= 0;
     qd->async = 0;
+    qd->holdlevel = 0;
     return (QuartzDesc_t)qd;
 }
 
@@ -481,11 +501,12 @@ static QuartzFunctions_t qfn = {
     QuartzDevice_GetParameter
 };
 
+/* currrently unused: was used by R.app via aqua.c */
 QuartzFunctions_t *getQuartzAPI() {
     return &qfn;
 }
 
-/* old OS X versions has different names for some of the CGFont stuff */
+/* old macOS versions has different names for some of the CGFont stuff */
 #if MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_4
 #define CGFontCreateWithFontName CGFontCreateWithName
 #define CGFontGetGlyphBBoxes CGFontGetGlyphBoundingBoxes
@@ -640,7 +661,7 @@ CGFontRef RQuartz_Font(CTXDESC)
         if (!atsFont) { /* not in the cache? Then we need to find the 
 			   proper font name from the family name and face */
             /* as it turns out kATSFontFilterSelectorFontFamily is not 
-	       implemented in OS X (!!) so there is no way to query for a 
+	       implemented in macOS (!!) so there is no way to query for a 
 	       font from a specific family. Therefore we have to use 
 	       text-matching heuristics ... very nasty ... */
             char compositeFontName[256];
@@ -710,20 +731,38 @@ static void RQuartz_SetFont(CGContextRef ctx, const pGEcontext gc, QuartzDesc *x
     CGContextSetFontSize(ctx, gc->cex * gc->ps);
 }
 
+/* pre-10.5 doesn't have kCGColorSpaceGenericRGB so fall back to kCGColorSpaceGenericRGB */
+#if MAC_OS_X_VERSION_10_4 >= MAC_OS_X_VERSION_MAX_ALLOWED
+#define kCGColorSpaceSRGB kCGColorSpaceGenericRGB
+#endif
+
 void RQuartz_Set(CGContextRef ctx,const pGEcontext gc,int flags) {
+    CGColorSpaceRef cs = CGColorSpaceCreateWithName(kCGColorSpaceSRGB);
     if(flags & RQUARTZ_FILL) {
         int fill = gc->fill;
-        CGContextSetRGBFillColor(ctx, R_RED(fill)/255.0, R_GREEN(fill)/255.0, R_BLUE(fill)/255.0, R_ALPHA(fill)/255.0);
+        CGFloat fillColor[] = { R_RED(fill)/255.0, 
+                                R_GREEN(fill)/255.0, 
+                                R_BLUE(fill)/255.0, 
+                                R_ALPHA(fill)/255.0 };
+        CGColorRef fillColorRef = CGColorCreate(cs, fillColor);
+        CGContextSetFillColorWithColor(ctx, fillColorRef);
+        CGColorRelease(fillColorRef);
     }
     if(flags & RQUARTZ_STROKE) {
         int stroke = gc->col;
-        CGContextSetRGBStrokeColor(ctx, R_RED(stroke)/255.0, R_GREEN(stroke)/255.0, R_BLUE(stroke)/255.0, R_ALPHA(stroke)/255.0);
+        CGFloat strokeColor[] = { R_RED(stroke)/255.0, 
+                                  R_GREEN(stroke)/255.0, 
+                                  R_BLUE(stroke)/255.0, 
+                                  R_ALPHA(stroke)/255.0 };
+        CGColorRef strokeColorRef = CGColorCreate(cs, strokeColor);
+        CGContextSetStrokeColorWithColor(ctx, strokeColorRef);
+        CGColorRelease(strokeColorRef);
     }
     if(flags & RQUARTZ_LINE) {
         CGFloat dashlist[8];
         int   i, ndash = 0;
         int   lty = gc->lty;
-	float lwd = gc->lwd * 0.75;
+	float lwd = (float)(gc->lwd * 0.75);
         CGContextSetLineWidth(ctx, lwd);
 
         for(i = 0; i < 8 && lty; i++) {
@@ -747,6 +786,7 @@ void RQuartz_Set(CGContextRef ctx,const pGEcontext gc,int flags) {
         CGContextSetLineJoin(ctx, join);
         CGContextSetMiterLimit(ctx, gc->lmitre);
     }
+    CGColorSpaceRelease(cs);
 }
 
 #define SET(X) RQuartz_Set(ctx, gc, (X))
@@ -819,6 +859,29 @@ static void RQuartz_NewPage(CTXDESC)
     }
 }
 
+static int RQuartz_HoldFlush(DEVDESC, int level)
+{
+    int ol;
+    XD;
+    /* FIXME: should we check for interactive? */
+    ol = xd->holdlevel;
+    xd->holdlevel += level;
+    if (xd->holdlevel < 0) xd->holdlevel = 0;
+    if (xd->holdlevel == 0) { /* flush */
+	/* trigger flush */
+        if (xd->sync)
+            xd->sync(xd, xd->userInfo);
+        else {
+	    CGContextRef ctx = xd->getCGContext(xd, xd->userInfo);
+	    if (ctx)
+		CGContextSynchronize(ctx);
+	}
+    } else if (ol == 0) { /* first hold */
+	/* could display a wait cursor or something ... */
+    }
+    return xd->holdlevel;
+}
+
 static void RQuartz_Clip(double x0, double x1, double y0, double y1, DEVDESC)
 {
     DRAWSPEC;
@@ -865,31 +928,29 @@ static double RQuartz_StrWidth(const char *text, CTXDESC)
     DEVSPEC;
     if (!ctx) NOCTXR(strlen(text) * 10.0); /* for sanity reasons */
     RQuartz_SetFont(ctx, gc, xd);
-    {
-        CGFontRef font = CGContextGetFont(ctx);
-        float aScale   = (gc->cex * gc->ps * xd->tscale) / CGFontGetUnitsPerEm(font);
-        UniChar *buffer;
-        CGGlyph *glyphs;
-        int     *advances;
-        int Free = 0, len,i;
-        CFStringRef str = text2unichar(gc, dd, text, &buffer, &Free);
-	if (!str) return 0.0; /* invalid text contents */
-        len = CFStringGetLength(str);
-	/* FIXME: check allocations */
-        glyphs = malloc(sizeof(CGGlyph) * len);
-        advances = malloc(sizeof(int) * len);
-        CGFontGetGlyphsForUnichars(font, buffer, glyphs,len);
-        CGFontGetGlyphAdvances(font, glyphs, len, advances);
-        {
-            float width = 0.0; /* aScale*CGFontGetLeading(CGContextGetFont(ctx)); */
-            for(i = 0; i < len; i++) width += aScale * advances[i];
-            free(advances);
-            free(glyphs);
-            if(Free) free(buffer);
-            CFRelease(str);
-            return width;
-        }
-    }
+
+    CGFontRef font = CGContextGetFont(ctx);
+    float aScale   = (float)((gc->cex * gc->ps * xd->tscale) /
+			     CGFontGetUnitsPerEm(font));
+    UniChar *buffer;
+    CGGlyph *glyphs;
+    int     *advances;
+    int Free = 0, len;
+    CFStringRef str = text2unichar(gc, dd, text, &buffer, &Free);
+    if (!str) return 0.0; /* invalid text contents */
+    len = (int) CFStringGetLength(str);
+    /* FIXME: check allocations */
+    glyphs = malloc(sizeof(CGGlyph) * len);
+    advances = malloc(sizeof(int) * len);
+    CGFontGetGlyphsForUnichars(font, buffer, glyphs, len);
+    CGFontGetGlyphAdvances(font, glyphs, len, advances);
+    float width = 0.0; /* aScale*CGFontGetLeading(CGContextGetFont(ctx)); */
+    for(int i = 0; i < len; i++) width += aScale * advances[i];
+    free(advances);
+    free(glyphs);
+    if(Free) free(buffer);
+    CFRelease(str);
+    return width;
 }
 
 static void RQuartz_Text(double x, double y, const char *text, double rot, double hadj, CTXDESC)
@@ -903,7 +964,8 @@ static void RQuartz_Text(double x, double y, const char *text, double rot, doubl
     RQuartz_SetFont(ctx, gc, xd);
     gc->fill = fill;
     CGFontRef font = CGContextGetFont(ctx);
-    float aScale   = (gc->cex * gc->ps * xd->tscale) / CGFontGetUnitsPerEm(font);
+    float aScale   = (float) ((gc->cex * gc->ps * xd->tscale) /
+			      CGFontGetUnitsPerEm(font));
     UniChar *buffer;
     CGGlyph   *glyphs;
 
@@ -911,7 +973,7 @@ static void RQuartz_Text(double x, double y, const char *text, double rot, doubl
     float width = 0.0;
     CFStringRef str = text2unichar(gc, dd, text, &buffer, &Free);
     if (!str) return; /* invalid text contents */
-    len = CFStringGetLength(str);
+    len = (int) CFStringGetLength(str);
     /* FIXME: check allocations */
     glyphs = malloc(sizeof(CGGlyph) * len);
     CGFontGetGlyphsForUnichars(font, buffer, glyphs, len);
@@ -968,11 +1030,6 @@ static void RQuartz_Rect(double x0, double y0, double x1, double y1, CTXDESC)
     CGContextDrawPath(ctx, kCGPathFillStroke);
 }
 
-/* pre-10.5 doesn't have kCGColorSpaceGenericRGB so fall back to kCGColorSpaceGenericRGB */
-#if MAC_OS_X_VERSION_10_4 >= MAC_OS_X_VERSION_MAX_ALLOWED
-#define kCGColorSpaceSRGB kCGColorSpaceGenericRGB
-#endif
-
 static void RQuartz_Raster(unsigned int *raster, int w, int h,
                            double x, double y, 
                            double width, double height,
@@ -991,7 +1048,7 @@ static void RQuartz_Raster(unsigned int *raster, int w, int h,
 
     cs = CGColorSpaceCreateWithName(kCGColorSpaceSRGB);
 
- /* Create a quartz image from the data provider */
+    /* Create a quartz image from the data provider */
     img = CGImageCreate(w, h, 
                         8,   /* bits per channel */
                         32,  /* bits per pixel */
@@ -1023,11 +1080,10 @@ static void RQuartz_Raster(unsigned int *raster, int w, int h,
     /* Rotate */
     CGContextRotateCTM(ctx, rot*M_PI/180.0);
     /* Determine interpolation method */
-    if (interpolate) {
+    if (interpolate)
         CGContextSetInterpolationQuality(ctx, kCGInterpolationDefault);
-    } else {
+    else
         CGContextSetInterpolationQuality(ctx, kCGInterpolationNone);
-    }
     /* Draw the quartz image */
     CGContextDrawImage(ctx, CGRectMake(0, 0, width, height), img);
     CGContextRestoreGState(ctx);
@@ -1082,7 +1138,15 @@ static void RQuartz_Polyline(int n, double *x, double *y, CTXDESC)
     if (!ctx) NOCTX;
     SET(RQUARTZ_STROKE | RQUARTZ_LINE);
 
-    /* CGContextStrokeLineSegments turned out to be a bad idea due to Leopard restarting dashes for each segment. CGContextAddLineToPoint is fast enough. The only remaining porblem is that Quartz seems to restart dashes at segment breakup points. We should make the segments break-up an optional feature and possibly fix the underlying problem (software rendering). */
+    /* CGContextStrokeLineSegments turned out to be a bad idea due to
+       Leopard restarting dashes for each segment.
+       CGContextAddLineToPoint is fast enough. 
+       The only remaining problem is that Quartz seems to restart
+       dashes at segment breakup points. We should make the segments
+       break-up an optional feature and possibly fix the underlying
+       problem (software rendering).  
+    */
+
     while (i < n) {
         int j = i + max_segments;
         if (j > n) j = n;
@@ -1110,6 +1174,33 @@ static void RQuartz_Polygon(int n, double *x, double *y, CTXDESC)
     CGContextDrawPath(ctx, kCGPathFillStroke);
 }
 
+static void RQuartz_Path(double *x, double *y, 
+                         int npoly, int* nper,
+                         Rboolean winding,
+                         CTXDESC)
+{
+    int i, j, index;
+    DRAWSPEC;
+    if (!ctx) NOCTX;
+    SET(RQUARTZ_FILL | RQUARTZ_STROKE | RQUARTZ_LINE);
+    index = 0;
+    CGContextBeginPath(ctx);
+    for (i=0; i < npoly; i++) {
+        CGContextMoveToPoint(ctx, x[index], y[index]);
+        index++;
+        for(j=1; j < nper[i]; j++) {
+            CGContextAddLineToPoint(ctx, x[index], y[index]);
+            index++;
+        }
+        CGContextClosePath(ctx);
+    }
+    if (winding) {
+        CGContextDrawPath(ctx, kCGPathFillStroke);
+    } else {
+        CGContextDrawPath(ctx, kCGPathEOFillStroke);
+    }
+}
+
 static void RQuartz_Mode(int mode, DEVDESC)
 {
     DEVSPEC;
@@ -1117,7 +1208,7 @@ static void RQuartz_Mode(int mode, DEVDESC)
     /* don't do anything in redraw as we can signal the end */
     if (xd->redraw) return;
     /* mode=0 -> drawing complete, signal sync */
-    if (mode == 0) {
+    if (mode == 0 && xd->holdlevel == 0) {
         if (xd->sync)
             xd->sync(xd, xd->userInfo);
         else
@@ -1140,17 +1231,18 @@ RQuartz_MetricInfo(int c, const pGEcontext gc,
     RQuartz_SetFont(ctx, gc, xd);
     {
 	CGFontRef font = CGContextGetFont(ctx);
-        float aScale   = (gc->cex * gc->ps * xd->tscale) / CGFontGetUnitsPerEm(font);
+        float aScale   = (float)((gc->cex * gc->ps * xd->tscale) /
+				 CGFontGetUnitsPerEm(font));
 	UniChar  *buffer, single;
         CGGlyph  glyphs[8];
 	CFStringRef str = NULL;
         int free_buffer = 0, len;
 	*width = *ascent = *descent = 0.0; /* data for bail-out cases */
 	if (c >= 0 && c <= ((mbcslocale && gc->fontface != 5) ? 127 : 255)) {
-	    char    text[2] = { c, 0 };
+	    char    text[2] = { (char)c, 0 };
 	    str = text2unichar(gc, dd, text, &buffer, &free_buffer);
 	    if(!str) return;
-	    len = CFStringGetLength(str);
+	    len = (int) CFStringGetLength(str);
 	    if (len > 7) return; /* this is basically impossible,
 				    but you never know */
 	} else {
@@ -1198,7 +1290,8 @@ static Rboolean RQuartz_Locator(double *x, double *y, DEVDESC)
 /* disabled for now until we get to test in on 10.3 #include "qdCarbon.h" */
 
 /* current fake */
-QuartzDesc_t QuartzCarbon_DeviceCreate(pDevDesc dd, QuartzFunctions_t *fn, QuartzParameters_t *par)
+QuartzDesc_t 
+QuartzCarbon_DeviceCreate(pDevDesc dd, QuartzFunctions_t *fn, QuartzParameters_t *par)
 {
     return NULL;
 }
@@ -1206,8 +1299,10 @@ QuartzDesc_t QuartzCarbon_DeviceCreate(pDevDesc dd, QuartzFunctions_t *fn, Quart
 #define ARG(HOW,WHAT) HOW(CAR(WHAT));WHAT = CDR(WHAT)
 
 /* C version of the Quartz call (experimental)
-   Quartz descriptor on success, NULL on failure. if errorCode is not NULL, it will contain the error code on exit */
-QuartzDesc_t Quartz_C(QuartzParameters_t *par, quartz_create_fn_t q_create, int *errorCode)
+   Quartz descriptor on success, NULL on failure. 
+   If errorCode is not NULL, it will contain the error code on exit */
+QuartzDesc_t 
+Quartz_C(QuartzParameters_t *par, quartz_create_fn_t q_create, int *errorCode)
 {
     if (!q_create || !par) {
 	if (errorCode) errorCode[0] = -4;
@@ -1246,16 +1341,17 @@ QuartzDesc_t Quartz_C(QuartzParameters_t *par, quartz_create_fn_t q_create, int
     }
 }
 
-/* ARGS: type, file, width, height, ps, family, antialias, fontsm,
+/* ARGS: type, file, width, height, ps, family, antialias,
    title, bg, canvas, dpi */
 SEXP Quartz(SEXP args)
 {
     SEXP tmps, bgs, canvass;
     double   width, height, ps;
-    Rboolean antialias, smooth;
+    Rboolean antialias;
     int      quartzpos, bg, canvas, module = 0;
     double   mydpi[2], *dpi = 0;
-    const char *type, *mtype = 0, *file = 0, *family, *title;
+    const char *type, *mtype = 0, *family, *title;
+    char *file = NULL;
     QuartzDesc_t qd = NULL;
 
     const void *vmax = vmaxget();
@@ -1270,16 +1366,17 @@ SEXP Quartz(SEXP args)
     tmps = CAR(args);    args = CDR(args);
     if (isNull(tmps)) 
 	file = NULL;
-    else if (isString(tmps) && LENGTH(tmps) >= 1)
-        file  = CHAR(STRING_ELT(tmps, 0));
-    else
+    else if (isString(tmps) && LENGTH(tmps) >= 1) {
+        const char *tmp = R_ExpandFileName(CHAR(STRING_ELT(tmps, 0)));
+	file = R_alloc(strlen(tmp) + 1, sizeof(char));
+	strcpy(file, tmp);
+    } else
         error(_("invalid 'file' argument"));
     width     = ARG(asReal,args);
     height    = ARG(asReal,args);
     ps        = ARG(asReal,args);
     family    = CHAR(STRING_ELT(CAR(args), 0)); args = CDR(args);
     antialias = ARG(asLogical,args);
-    smooth    = ARG(asLogical,args);
     title     = CHAR(STRING_ELT(CAR(args), 0)); args = CDR(args);
     bgs       = CAR(args); args = CDR(args);
     bg        = RGBpar(bgs, 0);
@@ -1301,7 +1398,7 @@ SEXP Quartz(SEXP args)
     if (dpi && (ISNAN(dpi[0]) || ISNAN(dpi[1]))) dpi=0;
 
     if (ISNAN(width) || ISNAN(height) || width <= 0.0 || height <= 0.0)
-        error(_("invalid Quartz device size"));
+        error(_("invalid quartz() device size"));
 
     if (type) {
         const quartz_module_t *m = quartz_modules;
@@ -1329,7 +1426,7 @@ SEXP Quartz(SEXP args)
 	pDevDesc dev = calloc(1, sizeof(DevDesc));
 
 	if (!dev)
-	    error(_("Unable to create device description."));
+	    error(_("unable to create device description"));
 
 	QuartzParameters_t qpar = {
 	    sizeof(qpar),
@@ -1379,12 +1476,15 @@ SEXP Quartz(SEXP args)
 	if (qd == NULL) {
 	    vmaxset(vmax);
 	    free(dev);
-	    error(_("Unable to create Quartz device target, given type may not be supported."));
+	    error(_("unable to create quartz() device target, given type may not be supported"));
 	}
 	const char *devname = "quartz_off_screen";
 	if(streql(type, "") || streql(type, "native") || streql(type, "cocoa") 
 	   || streql(type, "carbon")) devname = "quartz";
- 	gsetVar(R_DeviceSymbol, mkString(devname), R_BaseEnv);
+	SEXP f = PROTECT(mkString(devname));
+	if(file) setAttrib(f, install("filepath"), mkString(file));
+ 	gsetVar(R_DeviceSymbol, f, R_BaseEnv);
+	UNPROTECT(1);
 	pGEDevDesc dd = GEcreateDevDesc(dev);
 	GEaddDevice(dd);
 	GEinitDisplayList(dd);
@@ -1393,6 +1493,21 @@ SEXP Quartz(SEXP args)
     return R_NilValue;
 }
 
+#include <sys/sysctl.h>
+
+static double cached_darwin_version = 0.0;
+
+/* Darwin version X.Y maps to macOS version 10.(X - 4).Y */
+static double darwin_version() {
+    char ver[32];
+    size_t len = sizeof(ver) - 1;
+    int mib[2] = { CTL_KERN, KERN_OSRELEASE };
+    if (cached_darwin_version > 0.0)
+	return cached_darwin_version;
+    sysctl(mib, 2, &ver, &len, 0, 0);
+    return (cached_darwin_version = atof(ver));
+}
+
 #include <mach/mach.h>
 #include <servers/bootstrap.h>
 
@@ -1403,53 +1518,79 @@ extern kern_return_t bootstrap_info(mach_port_t , /* bootstrap port */
                                     bool_array_t*, mach_msg_type_number_t*); /* active */
 
 /* returns 1 if window server session service
- (com.apple.windowserver.session) is present in the boostrap
- namespace. returns 0 if an error occurred or the service is not
- present. For all practical purposes this returns 1 only if run
- interactively via LS. Although ssh to a machine that has a running
- session for the same user will allow a WS connection, this function
- will still return 0 in that case.
- NOTE: on Mac OS X 10.5 we are currently NOT searching the parent
- namespaces. This is currently OK, because the session service will
- be registered in the session namespace which is the last in the
- chain. However, this could change in the future.
+   (com.apple.windowserver.session) is present in the boostrap
+   namespace (pre-Lion) or when a current session is present, active
+   and there is no SSH_CONNECTION (Lion and later).
+   returns 0 if an error occurred or the service is not
+   present. For all practical purposes this returns 1 only if run
+   interactively via LS. Although ssh to a machine that has a running
+   session for the same user will allow a WS connection, this function
+   will still return 0 in that case.
+   NOTE: on macOS 10.5 we are currently NOT searching the parent
+   namespaces. This is currently OK, because the session service will
+   be registered in the session namespace which is the last in the
+   chain. However, this could change in the future.
  */
 static int has_wss() {
     int res = 0;
-    kern_return_t kr;
-    mach_port_t self = mach_task_self();
-    mach_port_t bport = MACH_PORT_NULL;
-    kr = task_get_bootstrap_port(self, &bport);
-    if (kr == KERN_SUCCESS) {
-	kern_return_t           kr;
-	name_array_t            serviceNames;
-	mach_msg_type_number_t  serviceNameCount;
-	name_array_t            serverNames;
-	mach_msg_type_number_t  serverNameCount;
-	bool_array_t            active;
-	mach_msg_type_number_t  activeCount;
-	
-	serviceNames  = NULL;
-	serverNames   = NULL;
-	active        = NULL;
-	
-	kr = bootstrap_info(bport, 
-			    &serviceNames, &serviceNameCount, 
-			    &serverNames, &serverNameCount, 
-			    &active, &activeCount);
+
+    if (darwin_version() < 11.0) { /* before Lion we get reliable information from the bootstrap info */
+	kern_return_t kr;
+	mach_port_t self = mach_task_self();
+	mach_port_t bport = MACH_PORT_NULL;
+	kr = task_get_bootstrap_port(self, &bport);
 	if (kr == KERN_SUCCESS) {
-	    unsigned int i = 0;
-	    while (i < serviceNameCount) {
-		if (!strcmp(serviceNames[i], "com.apple.windowserver.session")) {
-		    res = 1;
-		    break;
+	    kern_return_t           kr;
+	    name_array_t            serviceNames;
+	    mach_msg_type_number_t  serviceNameCount;
+	    name_array_t            serverNames;
+	    mach_msg_type_number_t  serverNameCount;
+	    bool_array_t            active;
+	    mach_msg_type_number_t  activeCount;
+	    
+	    serviceNames  = NULL;
+	    serverNames   = NULL;
+	    active        = NULL;
+	    
+	    kr = bootstrap_info(bport, 
+				&serviceNames, &serviceNameCount, 
+				&serverNames, &serverNameCount, 
+				&active, &activeCount);
+	    if (kr == KERN_SUCCESS) {
+		unsigned int i = 0;
+		while (i < serviceNameCount) {
+		    if (!strcmp(serviceNames[i], "com.apple.windowserver.session")) {
+			res = 1;
+			break;
+		    }
+		    i++;
 		}
-		i++;
 	    }
 	}
+	if (bport != MACH_PORT_NULL)
+	    mach_port_deallocate(mach_task_self(), bport);
+    } else {
+	/* On macOS 10.7 (Lion) and higher two things changed:
+	   a) there is no com.apple.windowserver.session anymore 
+	   so the above will fail
+	   b) every process has now the full bootstrap info, 
+	   so in fact even remote connections will be able to 
+	   run on-screen tasks if the user is logged in
+	   So we need to add some heuristics to decide when the user 
+	   actually wants Quartz ... */   
+	/* check user's session */
+	CFDictionaryRef dict = CGSessionCopyCurrentDictionary();
+	if (dict) { /* allright, let's see if the session is current */
+	    CFTypeRef obj = CFDictionaryGetValue(dict, CFSTR("kCGSSessionOnConsoleKey"));
+	    if (obj && CFGetTypeID(obj) == CFBooleanGetTypeID()) {
+		/* even if this session is active, we don't use Quartz for SSH connections */
+		if (CFBooleanGetValue(obj) && (!getenv("SSH_CONNECTION") || getenv("SSH_CONNECTION")[0] == 0))
+		    res = 1;
+	    }
+	    CFRelease(dict);
+	}
     }
-    if (bport != MACH_PORT_NULL)
-	mach_port_deallocate(mach_task_self(), bport);
+
     return res;
 }
 
@@ -1473,7 +1614,8 @@ SEXP makeQuartzDefault() {
     return ScalarLogical(FALSE);
 }
 
-QuartzDesc_t Quartz_C(QuartzParameters_t *par, quartz_create_fn_t q_create, int *errorCode)
+QuartzDesc_t 
+Quartz_C(QuartzParameters_t *par, quartz_create_fn_t q_create, int *errorCode)
 {
     if (errorCode) errorCode[0] = -1;
     return NULL;
