@@ -24,7 +24,8 @@ image.default <- function (x = seq(0, 1, length.out = nrow(z)),
 		   zlim = range(z[is.finite(z)]),
 		   xlim = range(x),
 		   ylim = range(y),
-		   col = heat.colors(12), add = FALSE,
+		   col = hcl.colors(12, "YlOrRd", rev = TRUE),
+                   add = FALSE,
 		   xaxs = "i", yaxs = "i", xlab, ylab,
                    breaks, oldstyle = FALSE,
                    useRaster, ...)
@@ -44,8 +45,8 @@ image.default <- function (x = seq(0, 1, length.out = nrow(z)),
 	} else stop("no 'z' matrix specified")
     } else if (is.list(x)) {
 	xn <- deparse(substitute(x))
-	if (missing(xlab)) xlab <- paste(xn, "x", sep = "$")
-	if (missing(ylab)) ylab <- paste(xn, "y", sep = "$")
+	if (missing(xlab)) xlab <- paste0(xn, "$x")
+	if (missing(ylab)) ylab <- paste0(xn, "$y")
 	y <- x$y
 	x <- x$x
     } else {
