@@ -140,9 +140,9 @@ int sumFunc(SEXP ua) {
 int unitLength(SEXP u) 
 {
     int result = 0;
-    if (isUnitList(u))
+    if (isUnitList(u)) {
 	result = LENGTH(u);
-    else if (isUnitArithmetic(u))
+    } else if (isUnitArithmetic(u)) {
 	if (fOp(u)) {
 	    if (timesOp(u)) {
 		/*
@@ -156,10 +156,14 @@ int unitLength(SEXP u)
 		int n2 = unitLength(arg2(u));
 		result = (n1 > n2) ? n1 : n2;
 	    }
-	} else /* must be "min" or "max" or "sum" */
-	  result = 1;  /* unitLength(arg1(u)); */
-    else /* Must be a unit object */
+	} else { /* must be "min" or "max" or "sum" */
+            result = 1;  /* unitLength(arg1(u)); */
+        }
+    } else if (inherits(u, "unit")) { /* a "plain" unit object */
 	result = LENGTH(u);
+    } else {
+	error(_("object is not a unit, unit.list, or unitArithmetic object"));
+    }
     return result;
 }
 
