@@ -1,7 +1,7 @@
 #  File src/library/methods/R/Methods.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2016 The R Core Team
+#  Copyright (C) 1995-2018 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -54,7 +54,7 @@ setGeneric <-
         }
         ## you can only conflict with a primitive if you supply
         ## useAsDefault to signal you really mean a different function
-        if(!is.function(useAsDefault) && !identical(useAsDefault, FALSE)) {
+        if(!is.function(useAsDefault) && !isFALSE(useAsDefault)) {
             msg <- gettextf("%s dispatches internally;  methods can be defined, but the generic function is implicit, and cannot be changed.", sQuote(name))
             stop(msg, domain = NA)
         }
@@ -136,7 +136,7 @@ setGeneric <-
         if(!(is.object(fdef) && is(fdef, "genericFunction"))) {
             fdeflt <-
                 if(is.function(useAsDefault)) useAsDefault
-                else if(identical(useAsDefault, FALSE)) NULL
+                else if(isFALSE(useAsDefault)) NULL
                 else if(is.function(prevDefault) &&
                         !identical(formalArgs(prevDefault), formalArgs(fdef)) &&
                         !is.primitive(prevDefault))
@@ -168,7 +168,7 @@ setGeneric <-
 	    cmp <- .identicalGeneric(fdef, implicit,
 				     allow.extra.dots =
 				     !nzchar(Sys.getenv("R_SETGENERIC_PICKY_DOTS")))
-            if(identical(cmp, TRUE)) {
+            if(isTRUE(cmp)) {
                 fdef <- implicit
             }  # go ahead silently
             else if(is.function(implicit)) {
@@ -186,8 +186,8 @@ setGeneric <-
                     message(gettextf(
                          "Creating a new generic function for %s in %s",
                                      sQuote(name), thisPName),
-                        domain = NA)
-                    fdef@package <- attr(fdef@generic, "package") <- thisPackage
+                         domain = NA)
+                    fdef@package <- packageSlot(fdef@generic) <- packageSlot(environment(fdef)$.Generic) <- thisPackage
                 }
             }
             else { # generic prohibited
@@ -196,7 +196,7 @@ setGeneric <-
                                  sQuote(name), sQuote(package),
                                  thisPName),
                         domain = NA)
-                fdef@package <- attr(fdef@generic, "package") <- thisPackage
+                fdef@package <- packageSlot(fdef@generic) <- packageSlot(environment(fdef)$.Generic) <- thisPackage
             }
         }
     }
@@ -596,7 +596,7 @@ setMethod <-
     whereMethods <-
 	## do.mlist <- is.not.base && (!.noMlists() || all(signature == "ANY"))
 	if(is.not.base && !.noMlists()) # do.mlist
-	    insertMethod(.getOrMakeMethodsList(f, where, fdef),
+	    insertMethod(getMethodsMetaData(f, where),
 			 signature, margs, definition) ## else NULL
     mtable <- getMethodsForDispatch(fdef)
     if(cacheOnAssign(where)) { # will be FALSE for sourceEnvironment's
@@ -610,7 +610,7 @@ setMethod <-
     ## assigns the methodslist object
     ## and deals with flags for primitives & for updating group members
     assignMethodsMetaData(f, whereMethods, fdef, where)
-    f
+    invisible(f)
 }
 
 removeMethod <- function(f, signature = character(), where = topenv(parent.frame())) {
@@ -926,13 +926,13 @@ showMethods <-
 {
     if(missing(showEmpty))
 	showEmpty <- !missing(f)
-    if(identical(printTo, FALSE))
+    if(isFALSE(printTo))
         con <- textConnection(NULL, "w")
     else
         con <- printTo
     ## must resolve showEmpty in line; using an equivalent default
     ## fails because R resets the "missing()" result for f later on (grumble)
-    if(is(f, "function")) {
+    if(is.function(f)) {
         fdef <- f ## note that this causes missing(fdef) to be FALSE below
         if(missing(where))
             where <- environment(f)
@@ -973,7 +973,7 @@ showMethods <-
                               classes = classes, showEmpty = showEmpty,
                               printTo = con)
     }
-    if(identical(printTo, FALSE)) {
+    if(isFALSE(printTo)) {
         txtOut <- textConnectionValue(con)
         close(con)
         txtOut
@@ -1493,7 +1493,7 @@ registerImplicitGenerics <- function(what = .ImplicitGenericsTable(where),
 	else
 	    "<none>"
     }
-    if(identical(f2, FALSE))
+    if(isFALSE(f2))
 	return(gettext("original function is prohibited as a generic function"))
     if(!(is.function(f2) && is.function(f1)))
 	return(gettext("not both functions!"))
@@ -1580,7 +1580,7 @@ findMethods <- function(f, where, classes = character(), inherited = FALSE, pack
                 fdef <- getGeneric(f, package = package)
         }
     }
-    else if(!is(f, "function"))
+    else if(!is.function(f))
         stop(gettextf("argument %s must be a generic function or a single character string; got an object of class %s",
                       sQuote("f"), dQuote(class(f))),
              domain = NA)
@@ -1598,7 +1598,7 @@ findMethods <- function(f, where, classes = character(), inherited = FALSE, pack
     if(missing(where))
       table <- get(if(inherited) ".AllMTable" else ".MTable", envir = environment(fdef))
     else {
-        if(!identical(inherited, FALSE))
+        if(!isFALSE(inherited))
           stop(gettextf("only FALSE is meaningful for 'inherited', when 'where' is supplied (got %s)", inherited), domain = NA)
         where <- as.environment(where)
         what <- .TableMetaName(f, fdef@package)
