@@ -1,7 +1,7 @@
 #  File src/library/methods/R/zzz.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2015 The R Core Team
+#  Copyright (C) 1995-2019 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -27,6 +27,11 @@
     assign(".methodsNamespace", new.env(), envir = where)
     .Call(C_R_set_method_dispatch, TRUE)
     cat("initializing class and method definitions ...")
+    op <- options(error = quote({ ## allow *some* debugging
+	utils::dump.frames("onLoad_init", to.file = TRUE, include.GlobalEnv=TRUE)
+	if(!interactive()) q(status = 1)
+    }))
+    on.exit(options(op))
     ## set up default prototype (uses .Call so has be at load time)
     assign(".defaultPrototype", .Call(C_Rf_allocS4Object), envir = where)
     assign(".SealedClasses", character(), envir = where)
@@ -87,6 +92,10 @@
     assign("envRefMethodNames",
 	   names(getClassDef("envRefClass")@refMethods), envir = where)
     assign(".onLoad", ..onLoad, envir = where)
+    if(isTRUE(as.logical(Sys.getenv("_R_METHODS_DONT_CHECKSUBCLASSES", "false"))))
+        ## change to a no-op -- later giving partly broken class hierarchies,
+        ## e.g. in tests/reg-packages.R  inherits(dd, "mM") became FALSE
+        assign(".checkSubclasses", function(...){}, envir = where)
     rm(...onLoad, ..onLoad, envir = where)
     dbbase <- file.path(libname, pkgname, "R", pkgname)
     ns <- asNamespace(pkgname)
@@ -109,15 +118,15 @@
     assign(".methodsNamespace", where, where)
     ## assign to baseenv also, signalling methods loaded
     assign(".methodsNamespace", where, baseenv())
-    if(Sys.getenv("R_S4_BIND") == "active")
-        bind_activation(TRUE)
+    ## if(Sys.getenv("R_S4_BIND") == "active")
+    ##     bind_activation(TRUE)
 }
 
 .onUnload <- function(libpath)
 {
     message("unloading 'methods' package ...") # see when this is called
     .isMethodsDispatchOn(FALSE)
-    bind_activation(FALSE)
+    ## bind_activation(FALSE)
     library.dynam.unload("methods", libpath)
 }
 
