@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2001-2016   The R Core Team.
+ *  Copyright (C) 2001-2018   The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -64,7 +64,7 @@ static SEXP f_x_i_skeleton, fgets_x_i_skeleton, f_x_skeleton, fgets_x_skeleton;
 SEXP R_quick_method_check(SEXP object, SEXP fsym, SEXP fdef);
 
 static SEXP R_target, R_defined, R_nextMethod, R_dot_nextMethod,
-    R_loadMethod_name;
+    R_loadMethod_name, R_methods_name, R_tripleColon_name;
 
 static SEXP Methods_Namespace = NULL;
 
@@ -80,6 +80,8 @@ static void init_loadMethod()
     R_nextMethod = install("nextMethod");
     R_loadMethod_name = install("loadMethod");
     R_dot_nextMethod = install(".nextMethod");
+    R_methods_name = install("methods");
+    R_tripleColon_name = install(":::");
 }
 
 
@@ -674,14 +676,16 @@ SEXP R_nextMethodCall(SEXP matched_call, SEXP ev)
     PROTECT(op = findVarInFrame3(ev, R_dot_nextMethod, TRUE));
     if(op == R_UnboundValue)
 	error("internal error in 'callNextMethod': '.nextMethod' was not assigned in the frame of the method call");
-    PROTECT(e = duplicate(matched_call));
+    PROTECT(e = shallow_duplicate(matched_call));
     prim_case = isPrimitive(op);
     if (!prim_case) {
         if (inherits(op, "internalDispatchMethod")) {
 	    SEXP generic = findVarInFrame3(ev, R_dot_Generic, TRUE);
 	    if(generic == R_UnboundValue)
 	        error("internal error in 'callNextMethod': '.Generic' was not assigned in the frame of the method call");
+	    PROTECT(generic);
 	    op = INTERNAL(installTrChar(asChar(generic)));
+	    UNPROTECT(1); /* generic */
 	    prim_case = TRUE;
 	}
     }
@@ -741,7 +745,7 @@ static SEXP R_loadMethod(SEXP def, SEXP fname, SEXP ev)
 	else if(t == R_nextMethod)  {
 	    defineVar(R_dot_nextMethod, CAR(s), ev); found++;
 	}
-	else if(t == R_SourceSymbol || t == s_generic)  {
+	else if(t == R_SrcrefSymbol || t == s_generic)  {
 	    /* ignore */ found++;
 	}
     }
@@ -756,7 +760,9 @@ static SEXP R_loadMethod(SEXP def, SEXP fname, SEXP ev)
 	}
 	SEXP e, val;
 	PROTECT(e = allocVector(LANGSXP, 4));
-	SETCAR(e, R_loadMethod_name); val = CDR(e);
+	SETCAR(e,
+	       lang3(R_tripleColon_name, R_methods_name, R_loadMethod_name));
+	val = CDR(e);
 	SETCAR(val, def); val = CDR(val);
 	SETCAR(val, fname); val = CDR(val);
 	SETCAR(val, ev);
@@ -1007,11 +1013,11 @@ SEXP R_dispatchGeneric(SEXP fname, SEXP ev, SEXP fdef)
 		thisClass = dots_class(ev, &check_err);
 	    }
 	    else {
-		arg = eval(arg_sym, ev);
+		PROTECT(arg = eval(arg_sym, ev));
 		/* PROTECT(arg = R_tryEvalSilent(arg_sym, ev, &check_err)); // <- related to bug PR#16111 */
 		/* if(!check_err) */
 		thisClass = R_data_class(arg, TRUE);
-		/* UNPROTECT(1); /\* for arg *\/ */
+		UNPROTECT(1); /* arg */
 	    }
 	    if(check_err)
 		error(_("error in evaluating the argument '%s' in selecting a method for function '%s': %s"),
