@@ -1,7 +1,7 @@
 #  File src/library/parallel/R/snow.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2017 The R Core Team
+#  Copyright (C) 1995-2018 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -35,6 +35,10 @@ setDefaultCluster <- function(cl = NULL)
     assign("default", cl, envir = .reg)
 }
 
+getDefaultCluster <-
+function()
+    get("default", envir = .reg)
+
 #
 # Checking and subsetting
 #
@@ -95,6 +99,7 @@ initDefaultClusterOptions <- function(libname)
     }
     Sys.i <- Sys.info()
     options <- list(port = as.integer(port),
+                    setup_timeout = 60 * 2,      # 2 minutes
                     timeout = 60 * 60 * 24 * 30, # 30 days
                     master = Sys.i[["nodename"]],
                     homogeneous = TRUE,
@@ -143,11 +148,11 @@ makeCluster <-
     function (spec, type = getClusterOption("type"), ...)
 {
     switch(type,
-           PSOCK = makePSOCKcluster(spec, ...),
-           FORK = makeForkCluster(spec, ...),
-           SOCK = snow::makeSOCKcluster(spec, ...),
-           MPI = snow::makeMPIcluster(spec, ...),
-           NWS = snow::makeNWScluster(spec, ...),
+           PSOCK = makePSOCKcluster(names = spec, ...),
+           FORK = makeForkCluster(nnodes = spec, ...),
+           SOCK = snow::makeSOCKcluster(names = spec, ...),
+           MPI = snow::makeMPIcluster(count = spec, ...),
+           NWS = snow::makeNWScluster(names = spec, ...),
            stop("unknown cluster type"))
 }
 
