@@ -29,7 +29,7 @@ function(x, g, ...)
         DNAME <- deparse(substitute(x))
         x <- lapply(x, function(u) u <- u[complete.cases(u)])
         k <- length(x)
-        l <- sapply(x, "length")
+        l <- lengths(x)
         if (any(l == 0))
             stop("all groups must contain data")
         g <- factor(rep(1 : k, l))
@@ -43,8 +43,6 @@ function(x, g, ...)
         OK <- complete.cases(x, g)
         x <- x[OK]
         g <- g[OK]
-        if (!all(is.finite(g)))
-            stop("all group levels must be finite")
         g <- factor(g)
         k <- nlevels(g)
         if (k < 2)
