@@ -31,7 +31,7 @@ function(x, g, ...)
         if (!all(sapply(x, is.numeric)))
             warning("some elements of 'x' are not numeric and will be coerced to numeric")
         k <- length(x)
-        l <- sapply(x, "length")
+        l <- lengths(x)
         if (any(l == 0L))
             stop("all groups must contain data")
         g <- factor(rep.int(seq_len(k), l))
@@ -45,8 +45,6 @@ function(x, g, ...)
         OK <- complete.cases(x, g)
         x <- x[OK]
         g <- g[OK]
-        if (!all(is.finite(g)))
-            stop("all group levels must be finite")
         g <- factor(g)
         k <- nlevels(g)
         if (k < 2L)
