@@ -142,7 +142,8 @@ simpleLoess <- function(y, x, weights, span = 0.75, degree = 2L,
     surf.stat <- paste(surface, statistics, sep = "/")
     do.rob <- (iterations > 1L) # will do robustness iter.
     if(!do.rob && iterTrace) {
-	warning("iterTrace = ", iterTrace," not obeyed as iterations = ", iterations)
+	warning(sprintf(gettext("iterTrace = %d is not obeyed since iterations = %d"),
+                        iterTrace, iterations))
 	iterTrace <- FALSE
     }
     no.st <- (statistics == "none")
