@@ -1,7 +1,7 @@
 #  File src/library/stats/R/lowess.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2016 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -14,12 +14,12 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 lowess <- function(x, y = NULL, f = 2/3, iter = 3L,
                    delta = 0.01 * diff(range(x)))
 {
-    xy <- xy.coords(x,y)
+    xy <- xy.coords(x,y, setLab = FALSE)
     o <- order(xy$x)
     x <- as.double(xy$x[o])
     list(x = x, y = .Call(C_lowess, x, as.double(xy$y[o]), f, iter, delta))
