@@ -1,5 +1,7 @@
 #  File src/library/stats/R/reorder.factor.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
+#
+#  Copyright (C) 1995-2012 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -12,13 +14,12 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
-reorder.character <- 
-reorder.factor <- function(x, X, FUN = mean, ..., order = is.ordered(x))
+reorder.default <- function(x, X, FUN = mean, ..., order = is.ordered(x))
 {
-    scores <- tapply(X, x, FUN, ...)
-    ans <- (if (order) ordered else factor)(x, levels = names(sort(scores)))
+    scores <- tapply(X = X, INDEX = x, FUN = FUN, ...)
+    ans <- (if (order) ordered else factor)(x, levels = names(sort(scores, na.last = TRUE)))
     attr(ans, "scores") <- scores
     ans
 }
