@@ -144,9 +144,8 @@ static void fftmx(double *a, double *b, int ntot, int n, int nspan, int isn,
     int lim, maxf, mm, nn, nt;
 
     // converted from Fortran, so 1-based indexing
-    a--; b--; at--; ck--; bt--; sk--;
-    np--;
-    nfac--;
+    a--; b--; at--; ck--; bt--; sk--; np--;
+    // nfac has had indexing converted to avoid compiler warnings
 
     inc = abs(isn);
     nt = inc*ntot;
@@ -164,7 +163,7 @@ static void fftmx(double *a, double *b, int ntot, int n, int nspan, int isn,
 #ifdef SCALING
 	/* scale by 1/n for isn > 0 */
 	ak = 1.0/n;
-	for(j=1 ; j<=nt ; j+=inc) {
+	for(j = 1 ; j <= nt ; j += inc) {
 	    a[j] *= ak;
 	    b[j] *= ak;
 	}
@@ -181,8 +180,8 @@ static void fftmx(double *a, double *b, int ntot, int n, int nspan, int isn,
     klim = lim*jc;
     i = 0;
     jf = 0;
-    maxf = nfac[m - kt];
-    if(kt > 0) maxf = imax2(nfac[kt],maxf);
+    maxf = nfac[m - kt - 1];
+    if(kt > 0) maxf = imax2(nfac[kt-1], maxf);
 
 	/* compute fourier transform */
 
@@ -193,7 +192,7 @@ L_start:
     sd = sin(dr*rad);
     kk = 1;
     i++;
-    if( nfac[i] != 2) goto L110;
+    if( nfac[i-1] != 2) goto L110;
 
 /* transform for factor of 2 (including rotation factor) */
 
@@ -292,7 +291,7 @@ L100:
 /* transform for factor of 4 */
 
 L110:
-    if( nfac[i] != 4) goto L_f_odd;
+    if( nfac[i-1] != 4) goto L_f_odd;
     kspnn = kspan;
     kspan /= 4;
 L120:
@@ -429,7 +428,7 @@ L220:
 /* transform for odd factors */
 
 L_f_odd:
-    k = nfac[i];
+    k = nfac[i-1];
     kspnn = kspan;
     kspan /= k;
     if(k == 3) goto L100;
@@ -485,7 +484,7 @@ L270:
     aj = 0.0;
     bj = 0.0;
     k = 1;
-    for(k=2; k < jf; k++) {
+    for(k = 2; k < jf; k++) {
 	ak += at[k]*ck[jj];
 	bk += bt[k]*ck[jj];
 	k++;
@@ -569,8 +568,8 @@ L_fin:
     if( m < k) k--;
     np[k+1] = jc;
     for(j = 1; j < k; j++, k--) {
-	np[j+1] = np[j]/nfac[j];
-	np[k] = np[k+1]*nfac[j];
+	np[j+1] = np[j]/nfac[j-1];
+	np[k] = np[k+1]*nfac[j-1];
     }
     k3 = np[k+1];
     kspan = np[2];
@@ -639,11 +638,11 @@ L440:
 
     /* Here, nfac[] is overwritten... -- now CUMULATIVE ("cumprod") factors */
     nn = m - kt;
-    nfac[nn+1] = 1;
+    nfac[nn] = 1;
     for(j = nn; j > kt; j--)
-	nfac[j] *= nfac[j+1];
+	nfac[j-1] *= nfac[j];
     kt++;
-    nn = nfac[kt] - 1;
+    nn = nfac[kt-1] - 1;
     jj = 0;
     j = 0;
     goto L480;
@@ -651,15 +650,15 @@ L460:
     jj -= k2;
     k2 = kk;
     k++;
-    kk = nfac[k];
+    kk = nfac[k-1];
 L470:
     jj += kk;
     if( jj >= k2) goto L460;
     np[j] = jj;
 L480:
-    k2 = nfac[kt];
+    k2 = nfac[kt-1];
     k = kt + 1;
-    kk = nfac[k];
+    kk = nfac[k-1];
     j++;
     if( j <= nn) goto L470;
 
@@ -691,7 +690,7 @@ L520:
     k = np[j];
     kk = jc*k + i + jj;
 
-    for(k1= kk + kspan, k2= 1; k1 != kk;
+    for(k1 = kk + kspan, k2 = 1; k1 != kk;
 	k1 -= inc, k2++) {
 	at[k2] = a[k1];
 	bt[k2] = b[k1];
@@ -710,7 +709,7 @@ L520:
 	kk = k2;
     } while(k != j);
 
-    for(k1= kk + kspan, k2= 1; k1 > kk;
+    for(k1 = kk + kspan, k2 = 1; k1 > kk;
 	k1 -= inc, k2++) {
 	a[k1] = at[k2];
 	b[k1] = bt[k2];
@@ -849,8 +848,6 @@ void fft_factor(int n, int *pmaxf, int *pmaxp)
 Rboolean fft_work(double *a, double *b, int nseg, int n, int nspn, int isn,
 		  double *work, int *iwork)
 {
-    int nf, nspan, ntot;
-
 	/* check that factorization was successful */
 
     if(old_n == 0) return FALSE;
@@ -862,12 +859,11 @@ Rboolean fft_work(double *a, double *b, int nseg, int n, int nspn, int isn,
 
 	/* perform the transform */
 
-    nf = n;
-    nspan = nf * nspn;
-    ntot = nspan * nseg;
+    size_t mf = maxf;
+    int nspan = n * nspn, ntot = nspan * nseg;
 
-    fftmx(a, b, ntot, nf, nspan, isn, m_fac, kt,
-	  &work[0], &work[maxf], &work[2*(size_t)maxf], &work[3*(size_t)maxf],
+    fftmx(a, b, ntot, n, nspan, isn, m_fac, kt,
+	  work, work+mf, work+2*mf, work+3*mf,
 	  iwork, nfac);
 
     return TRUE;
