@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1998-2012	The R Foundation
+ *  Copyright (C) 1998-2016	The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,9 +14,10 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, a copy is available at
- *  http://www.r-project.org/Licenses/
+ *  https://www.R-project.org/Licenses/
  */
 
+#include <math.h>
 #include <R.h>			/* for NA_REAL, includes math.h */
 #include <Rinternals.h>
 
@@ -62,11 +63,29 @@ static void BDRksmooth(double *x, double *y, R_xlen_t n,
 }
 
 
-void F77_SUB(bdrsplerr)(void)
+// called only from  spline()  in ./ppr.f
+void NORET F77_SUB(bdrsplerr)(void)
 {
     error(_("only 2500 rows are allowed for sm.method=\"spline\""));
 }
 
+void F77_SUB(splineprt)(double* df, double* gcvpen, int* ismethod,
+			      double* lambda, double *edf)
+{
+    Rprintf("spline(df=%5.3g, g.pen=%11.6g, ismeth.=%+2d) -> (lambda, edf) = (%.7g, %5.2f)\n",
+	    *df, *gcvpen, *ismethod, *lambda, *edf);
+    return;
+}
+
+// called only from smooth(..., trace=TRUE)  in ./ppr.f :
+void F77_SUB(smoothprt)(double* span, int* iper, double* var, double* cvar)
+{
+    Rprintf("smooth(span=%4g, iper=%+2d) -> (var, cvar) = (%g, %g)\n",
+	    *span, *iper, *var, *cvar);
+    return;
+}
+
+
 SEXP ksmooth(SEXP x, SEXP y, SEXP xp, SEXP skrn, SEXP sbw)
 {
     int krn = asInteger(skrn);
