@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2005-2007   The R Core Team.
+ *  Copyright (C) 2005-2015   The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, a copy is available at
- *  http://www.r-project.org/Licenses/
+ *  https://www.R-project.org/Licenses/
  */
 
 #include "port.h"
@@ -100,7 +100,7 @@ extern void F77_NAME(dv7dfl)(const int *Alg, const int *Lv, double v[]);
 /**
  * Supply default values for elements of the iv and v arrays
  *
- * @param alg algorithm specification (1 <= alg <= 4)
+ * @param alg algorithm specification (1 <= alg <= 2)  (was alg <= 4, but reduced to work around gcc bug; see PR#15914)
  * @param iv integer working vector
  * @param liv length of iv
  * @param lv length of v
@@ -146,7 +146,7 @@ void Rf_divset(int alg, int iv[], int liv, int lv, double v[])
     F77_CALL(dv7dfl)(&alg1, &lv, &v[1]);
     //       ------
     iv[1] = 12;
-    if (alg > 2) iv[DRADPR] = 1;
+    if (alg > 2) error(_("port algorithms 3 or higher are not supported"));
     iv[IVNEED] = 0;
     iv[LASTIV] = miv;
     iv[LASTV] = mv;
@@ -235,7 +235,8 @@ double F77_NAME(dv2nrm)(int *n, const double x[])
 /* dv7cpy.... copy src to dest */
 void F77_NAME(dv7cpy)(int *n, double dest[], const double src[])
 {
-    memcpy(dest, src, *n * sizeof(double));
+    /* Was memcpy, but overlaps seen */
+    memmove(dest, src, *n * sizeof(double));
 }
 
 /* dv7ipr... applies forward permutation to vector.  */
@@ -307,7 +308,7 @@ int F77_NAME(stopx)(void)
 static
 double* check_gv(SEXP gr, SEXP hs, SEXP rho, int n, double *gv, double *hv)
 {
-    SEXP gval = PROTECT(coerceVector(eval(gr, rho), REALSXP));
+    SEXP gval = PROTECT(coerceVector(PROTECT(eval(gr, rho)), REALSXP));
     if (LENGTH(gval) != n)
 	error(_("gradient function must return a numeric vector of length %d"), n);
     Memcpy(gv, REAL(gval), n);
@@ -331,7 +332,7 @@ double* check_gv(SEXP gr, SEXP hs, SEXP rho, int n, double *gv, double *hv)
 	    }
 	UNPROTECT(1);
     }
-    UNPROTECT(1);
+    UNPROTECT(2);
     return gv;
 }
 
@@ -368,6 +369,7 @@ SEXP port_nlminb(SEXP fn, SEXP gr, SEXP hs, SEXP rho,
 {
     int i, n = LENGTH(d);
     SEXP xpt;
+    SEXP dot_par_symbol = install(".par");
     double *b = (double *) NULL, *g = (double *) NULL,
 	*h = (double *) NULL, fx = R_PosInf;
     if (isNull(rho)) {
@@ -380,18 +382,18 @@ SEXP port_nlminb(SEXP fn, SEXP gr, SEXP hs, SEXP rho,
 	error(_("'d' must be a nonempty numeric vector"));
     if (hs != R_NilValue && gr == R_NilValue)
 	error(_("When Hessian defined must also have gradient defined"));
-    if (R_NilValue == (xpt = findVarInFrame(rho, install(".par"))) ||
+    if (R_NilValue == (xpt = findVarInFrame(rho, dot_par_symbol)) ||
 	!isReal(xpt) || LENGTH(xpt) != n)
 	error(_("environment 'rho' must contain a numeric vector '.par' of length %d"),
 	      n);
     /* We are going to alter .par, so must duplicate it */
-    defineVar(install(".par"), duplicate(xpt), rho);
-    PROTECT(xpt = findVarInFrame(rho, install(".par")));
+    defineVar(dot_par_symbol, duplicate(xpt), rho);
+    PROTECT(xpt = findVarInFrame(rho, dot_par_symbol));
 
     if ((LENGTH(lowerb) == n) && (LENGTH(upperb) == n)) {
 	if (isReal(lowerb) && isReal(upperb)) {
 	    double *rl=REAL(lowerb), *ru=REAL(upperb);
-	    b = Calloc(2*n, double);
+	    b = (double *)R_alloc(2*n, sizeof(double));
 	    for (i = 0; i < n; i++) {
 		b[2*i] = rl[i];
 		b[2*i + 1] = ru[i];
@@ -399,8 +401,9 @@ SEXP port_nlminb(SEXP fn, SEXP gr, SEXP hs, SEXP rho,
 	} else error(_("'lower' and 'upper' must be numeric vectors"));
     }
     if (gr != R_NilValue) {
-	g = Calloc(n, double);
-	if (hs != R_NilValue) h = Calloc((n * (n + 1))/2, double);
+	g = (double *)R_alloc(n, sizeof(double));
+	if (hs != R_NilValue)
+	    h = (double *)R_alloc((n * (n + 1))/2, sizeof(double));
     }
 
     do {
@@ -414,10 +417,16 @@ SEXP port_nlminb(SEXP fn, SEXP gr, SEXP hs, SEXP rho,
 		fx = R_PosInf;
 	    }
 	}
+
+	/* duplicate .par value again in case a callback has stored
+	   value (package varComp does this) */
+	defineVar(dot_par_symbol, duplicate(xpt), rho);
+	xpt = findVarInFrame(rho, dot_par_symbol);
+	UNPROTECT(1);
+	PROTECT(xpt);
     } while(INTEGER(iv)[0] < 3);
 
-    if (b) Free(b); if (g) Free(g); if (h) Free(h);
-    UNPROTECT(1);
+    UNPROTECT(1); /* xpt */
     return R_NilValue;
 }
 
@@ -530,8 +539,10 @@ SEXP port_nlsb(SEXP m, SEXP d, SEXP gg, SEXP iv, SEXP v,
     SEXP getPars, setPars, resid, gradient,
 	rr = PROTECT(allocVector(REALSXP, nd)),
 	x = PROTECT(allocVector(REALSXP, n));
+    // This used to use Calloc, but that will leak if 
+    // there is a premature return (and did in package drfit)
     double *b = (double *) NULL,
-	*rd = Calloc(nd, double);
+	*rd = (double *)R_alloc(nd, sizeof(double));
 
     if (!isReal(d) || n < 1)
 	error(_("'d' must be a nonempty numeric vector"));
@@ -549,8 +560,8 @@ SEXP port_nlsb(SEXP m, SEXP d, SEXP gg, SEXP iv, SEXP v,
 
     if ((LENGTH(lowerb) == n) && (LENGTH(upperb) == n)) {
 	if (isReal(lowerb) && isReal(upperb)) {
-	    double *rl=REAL(lowerb), *ru=REAL(upperb);
-	    b = Calloc(2*n, double);
+	    double *rl = REAL(lowerb), *ru = REAL(upperb);
+	    b = (double *)R_alloc(2*n, sizeof(double));
 	    for (i = 0; i < n; i++) {
 		b[2*i] = rl[i];
 		b[2*i + 1] = ru[i];
@@ -591,7 +602,6 @@ SEXP port_nlsb(SEXP m, SEXP d, SEXP gg, SEXP iv, SEXP v,
 	}
     } while(INTEGER(iv)[0] < 3);
 
-    Free(rd); if (b) Free(b);
     UNPROTECT(6);
     return R_NilValue;
 }
