@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2012   The R Core Team.
+ *  Copyright (C) 2012-2019   The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -165,8 +165,8 @@ SEXP call_dqagi(SEXP);
 SEXP call_dqags(SEXP);
 
 SEXP Rsm(SEXP x, SEXP stype, SEXP send);
-SEXP tukeyline(SEXP x, SEXP y, SEXP call);
-SEXP runmed(SEXP x, SEXP stype, SEXP sk, SEXP end, SEXP print_level);
+SEXP tukeyline(SEXP x, SEXP y, SEXP iter, SEXP call);
+SEXP runmed(SEXP sx, SEXP stype, SEXP sk, SEXP end, SEXP naAct, SEXP printLev);
 SEXP influence(SEXP mqr, SEXP do_coef, SEXP e, SEXP stol);
 
 SEXP pSmirnov2x(SEXP statistic, SEXP snx, SEXP sny);
