@@ -1,7 +1,7 @@
 #  File src/library/tools/R/RdConv2.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2016 The R Core Team
+#  Copyright (C) 1995-2018 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@ isBlankRd <- function(x)
     length(grep("^[[:blank:]]*\n?$", x, perl = TRUE)) == length(x) # newline optional
 
 isBlankLineRd <- function(x) {
-    attr(x, "srcref")[2L] == 1 &&
+    utils:::getSrcByte(x) == 1L &&
     length(grep("^[[:blank:]]*\n", x, perl = TRUE)) == length(x)   # newline required
 }
 
@@ -40,7 +40,10 @@ stopRd <- function(block, Rdfile, ...)
     	    Rdfile <- srcfile$filename
     }
     if (missing(Rdfile) || is.null(Rdfile)) Rdfile <- ""
-    else Rdfile <- paste0(Rdfile, ":")
+    else {
+        Rdfile <- sub("^man/", "", Rdfile) # for consistency with earlier reports
+        Rdfile <- paste0(Rdfile, ":")
+    }
 
     msg <- if (is.null(srcref))
         paste0(Rdfile, " ", ...)
@@ -60,9 +63,12 @@ warnRd <- function(block, Rdfile, ...)
     	if (is.environment(srcfile))
     	    Rdfile <- srcfile$filename
     }
-    if (missing(Rdfile) || is.null(Rdfile)) Rdfile <- ""
-    else Rdfile <- paste0(Rdfile, ":")
-
+    Rdfile <-
+        if(missing(Rdfile) || is.null(Rdfile))
+            ""
+        else
+            paste0(sub("^man/", "", Rdfile), # for consistency with earlier reports
+                   ":")
     msg <- if (is.null(srcref))
         paste0(Rdfile, " ", ...)
     else {
@@ -97,7 +103,7 @@ RweaveRdOptions <- function(options)
     NOLOGOPTS <- c(NUMOPTS, "results", "stage", "strip.white")
 
     for(opt in names(options)){
-        if(! (opt %in% NOLOGOPTS)){
+        if(opt %notin% NOLOGOPTS) {
             oldval <- options[[opt]]
             if(!is.logical(options[[opt]])){
                 options[[opt]] <- c2l(options[[opt]])
@@ -184,6 +190,15 @@ setDynamicFlags <- function(block, flags) {  # flags in format coming from getDy
     block
 }
 
+replaceRdSrcrefs <- function(Rd, srcref) {
+    if(!is.null(attr(Rd, "srcref")))
+	attr(Rd, "srcref") <- srcref
+    if(is.list(Rd)) # recurse
+	for(i in seq_along(Rd))
+	    Rd[[i]] <- replaceRdSrcrefs(Rd[[i]], srcref)
+    Rd
+}
+
 processRdChunk <- function(code, stage, options, env, Rdfile, macros)
 {
     if (is.null(opts <- attr(code, "Rd_option"))) opts <- ""
@@ -225,27 +240,27 @@ processRdChunk <- function(code, stage, options, env, Rdfile, macros)
 	    }
 	    if(options$echo && length(dce)) {
 		res <- c(res,"\n",
-                         paste(getOption("prompt"), dce[1L:leading],
-                               sep="", collapse="\n"))
+                         paste0(getOption("prompt"), dce[1L:leading],
+                                collapse="\n"))
 		if (length(dce) > leading)
 		    res <- c(res, "\n",
-                             paste(getOption("continue"), dce[-(1L:leading)],
-                                   sep="", collapse="\n"))
+                             paste0(getOption("continue"), dce[-(1L:leading)],
+                                    collapse="\n"))
 		thisline <- thisline + length(dce)
 	    }
 
 	    tmpcon <- file()
 	    sink(file = tmpcon)
 	    if(options$eval) err <- evalWithOpt(ce, options, env)
-	    res <- c(res, "\n") # make sure final line is complete
+	    res <- c(res, "\n") # attampt to  make sure final line is complete
 	    sink()
-	    output <- readLines(tmpcon)
+	    output <- readLines(tmpcon, warn = FALSE) # sometimes attempt fails.
 	    close(tmpcon)
 	    ## delete empty output
-	    if(length(output) == 1L & output[1L] == "") output <- NULL
+	    if(length(output) == 1L && output[1L] == "") output <- NULL
 
 	    if (inherits(err, "error")) {
-	    	attr(code, "srcref") <- codesrcref
+	    	code <- replaceRdSrcrefs(code, codesrcref)
 	    	stopRd(code, Rdfile, err$message)
 	    }
 
@@ -284,7 +299,7 @@ processRdChunk <- function(code, stage, options, env, Rdfile, macros)
 
 	    # First clear out the junk.
 	    tags <- RdTags(res)
-	    keep <- rep(TRUE, length(tags))
+	    keep <- rep.int(TRUE, length(tags))
 	    for (i in seq_along(tags)) {
 	        if (tags[i] == "TEXT" && res[[i]] == "")
 	            keep[i] <- FALSE
@@ -317,8 +332,8 @@ processRdChunk <- function(code, stage, options, env, Rdfile, macros)
 	    res <- tagged(res, "\\verb")
 	} else res <- tagged("", "COMMENT")
     } else res <- code
-    attr(res, "srcref") <- codesrcref
-    res
+    ## return :
+    replaceRdSrcrefs(res, codesrcref)
 }
 
 processRdIfdefs <- function(blocks, defines)
@@ -331,14 +346,15 @@ processRdIfdefs <- function(blocks, defines)
 		target <- block[[1L]][[1L]]
 		# The target will have picked up some whitespace and a newline
 		target <- psub("[[:blank:][:cntrl:]]*", "", target)
-		if ((target %in% defines) == (tag == "#ifdef")) {
-		    flag <- getDynamicFlags(block[[2L]])
-		    block <- tagged(block[[2L]], "#expanded")
-		    block <- setDynamicFlags(block, flag)
-		} else
-		    block <- structure(tagged(paste(tag, target, "not active"),
-                                              "COMMENT"),
-		    		       srcref = attr(block, "srcref"))
+		block <-
+                    if((target %in% defines) == (tag == "#ifdef")) {
+                        flag <- getDynamicFlags(block[[2L]])
+                        block <- tagged(block[[2L]], "#expanded")
+                        setDynamicFlags(block, flag)
+                    } else
+                        structure(tagged(paste(tag, target, "not active"),
+                                         "COMMENT"),
+                                  srcref = attr(block, "srcref"))
 	    }
 	}
 	if (is.list(block)) {
@@ -350,7 +366,7 @@ processRdIfdefs <- function(blocks, defines)
 	    	if (!is.null(newtag) && newtag == "#expanded") { # ifdef has expanded.
 	    	    all <- seq_along(block)
 	    	    before <- all[all < i]
-	    	    after <- all[all > i]
+	    	    after  <- all[all > i]
 	    	    block <- structure(tagged(c(block[before], newval, block[after]),
 	    	    			      tag), srcref = attr(block, "srcref"))
 	    	} else {
@@ -359,10 +375,10 @@ processRdIfdefs <- function(blocks, defines)
 		    i <- i+1L
 		}
 	    }
-	    block <- setDynamicFlags(block, flags)
-	}
-	block
-    }
+	    setDynamicFlags(block, flags)
+	} else
+	    block
+    } # end{recurse}
 
     recurse(blocks)
 }
@@ -489,7 +505,7 @@ prepare2_Rd <- function(Rd, Rdfile)
         	stopRd(docType, Rdfile, "'docType' must be plain text")
             ## Some people have \docType{ package } and similar.
             docTypes[i] <- sub("^ *", "", sub(" *$", "", docType[[1L]]))
-            if (! docTypes[i] %in%
+            if (docTypes[i] %notin%
                 c("data", "package", "methods", "class", "import"))
                 warnRd(dt[i], Rdfile, "docType ", sQuote(docTypes[i]),
                        " is unrecognized")
@@ -500,7 +516,7 @@ prepare2_Rd <- function(Rd, Rdfile)
     extras <- c("COMMENT", "TEXT", "\\docType", "\\Rdversion", "\\RdOpts",
                 "USERMACRO", "\\newcommand", "\\renewcommand")
     drop <- drop | (sections %in% extras)
-    bad <- ! sections %in% c(names(sectionOrder), extras)
+    bad <- sections %notin% c(names(sectionOrder), extras)
     if (any(bad)) {
         for(s in which(bad))
             warnRd(Rd[[s]], Rdfile, "Section ",
@@ -530,7 +546,7 @@ prepare2_Rd <- function(Rd, Rdfile)
 prepare3_Rd <- function(Rd, Rdfile, msglevel = 0)
 {
     ## Drop 'empty' sections: less rigorous than checkRd test
-    keep <- rep(TRUE, length(Rd))
+    keep <- rep.int(TRUE, length(Rd))
     checkEmpty <- function(x, this)
     {
         if(this) return(TRUE)
@@ -602,8 +618,9 @@ fsub1 <- function(pattern, replacement, x)
 checkRd <- function(Rd, defines=.Platform$OS.type, stages = "render",
                     unknownOK = TRUE, listOK = TRUE, ..., def_enc = FALSE)
 {
-    warnRd <- function(block, Rdfile, ..., level=0)
+    warnRd <- function(block, Rdfile, ..., level = 0L)
     {
+        Rdfile <- sub("^man/", "", Rdfile)
         srcref <- attr(block, "srcref")
         msg <- if (is.null(srcref))
             paste0("file '", Rdfile, "': ", ...)
@@ -637,12 +654,14 @@ checkRd <- function(Rd, defines=.Platform$OS.type, stages = "render",
                        ## check for encoding; this is UTF-8 if known
                        ## (but then def_enc = TRUE?)
                        msg2 <- if(inEnc2) "in second part of \\enc" else "without declared encoding"
+                       txt <- unclass(block); attributes(txt) <- NULL
+                       msg3 <- paste0(msg2, ":\n  ", sQuote(txt))
                        if(Encoding(block) == "UTF-8")
                            warnRd(block, Rdfile, level = -1,
-                                  "Non-ASCII contents ", msg2)
+                                  "Non-ASCII contents ", msg3)
                        if(grepl("<[0123456789abcdef][0123456789abcdef]>", block))
                            warnRd(block, Rdfile, level = -3,
-                                  "Apparent non-ASCII contents ", msg2)
+                                  "Apparent non-ASCII contents ", msg3)
                    }
                    ## check if this renders as non-whitespace
                    if(!grepl("^[[:space:]]*$", block)) has_text <<- TRUE
@@ -722,8 +741,8 @@ checkRd <- function(Rd, defines=.Platform$OS.type, stages = "render",
     		       stopRd(block, Rdfile, "Condition must be \\Sexpr or plain text")
     		   condition <- condition[tags == "TEXT"]
     		   allow <- trimws(strsplit(paste(condition, collapse=""), ",")[[1L]])
-    		   unknown <- allow[!(allow %in%
-    		          c("", "latex", "example", "text", "html", "TRUE", "FALSE"))]
+    		   unknown <- allow %w/o% c("", "latex", "example", "text",
+                                            "html", "TRUE", "FALSE")
     		   if (length(unknown))
     		       warnRd(block, Rdfile, "Unrecognized format: ", unknown)
                    checkContent(block[[2L]])
@@ -871,7 +890,7 @@ checkRd <- function(Rd, defines=.Platform$OS.type, stages = "render",
     		"\\itemize"= {})
     	    },
     	    { # default
-    	    	if (inlist && !(blocktag %in% c("\\itemize", "\\enumerate"))
+    	    	if (inlist && (blocktag %notin% c("\\itemize", "\\enumerate"))
     	    	           && !(tag == "TEXT" && isBlankRd(block))) {
     		    inlist <- FALSE
     		}
@@ -997,9 +1016,9 @@ toRd <- function(obj, ...)
     UseMethod("toRd")
 
 toRd.default <- function(obj, ...) {
-    obj <- as.character(obj)
-    obj <- gsub("\\", "\\\\", obj, fixed = TRUE)
-    obj <- gsub("{", "\\{", obj, fixed = TRUE)
-    obj <- gsub("}", "\\}", obj, fixed = TRUE)
-    gsub("%", "\\%", obj, fixed = TRUE)
+    fsub <- function(from, to, x) gsub(from, to, x, fixed=TRUE)
+    fsub("%", "\\%",
+     fsub("}", "\\}",
+      fsub("{", "\\{",
+       fsub("\\", "\\\\", as.character(obj)))))
 }
