@@ -16,17 +16,24 @@
 #  A copy of the GNU General Public License is available at
 #  https://www.R-project.org/Licenses/
 
-Rd_expr_PR <- function(x)
+Rd_expr_PR <-
+function(x)
 {
     baseurl <- "https://bugs.R-project.org/bugzilla3/show_bug.cgi?id"
     sprintf("\\href{%s=%s}{PR#%s}", baseurl, x, x)
 }
 
-# These following functions are to take information from the package DESCRIPTION file
-# at build time.  During a build, the current directory holds the DESCRIPTION file;
-# set dir to something else if used in a different context.
+## These following functions are to take information from the package
+## DESCRIPTION file at build time.  During a build, the current
+## directory holds the DESCRIPTION file; set dir to something else if
+## used in a different context.
 
-Rd_package_title <- function(pkg, dir = ".")
+Rd_macros_package_dir <-
+function()
+    Sys.getenv("_R_RD_MACROS_PACKAGE_DIR_", ".")
+
+Rd_package_title <-
+function(pkg, dir = Rd_macros_package_dir())
 {
     desc <- .read_description(file.path(dir, "DESCRIPTION"))
     if (pkg != desc["Package"])
@@ -34,7 +41,8 @@ Rd_package_title <- function(pkg, dir = ".")
     desc["Title"]
 }
 
-Rd_package_description <- function(pkg, dir = ".")
+Rd_package_description <-
+function(pkg, dir = Rd_macros_package_dir())
 {
     desc <- .read_description(file.path(dir, "DESCRIPTION"))
     if (pkg != desc["Package"])
@@ -42,7 +50,8 @@ Rd_package_description <- function(pkg, dir = ".")
     desc["Description"]
 }
 
-Rd_package_author <- function(pkg, dir = ".")
+Rd_package_author <-
+function(pkg, dir = Rd_macros_package_dir())
 {
     desc <- .read_description(file.path(dir, "DESCRIPTION"))
     if (pkg != desc["Package"])
@@ -50,7 +59,8 @@ Rd_package_author <- function(pkg, dir = ".")
     desc["Author"]
 }
 
-Rd_package_maintainer <- function(pkg, dir = ".")
+Rd_package_maintainer <-
+function(pkg, dir = Rd_macros_package_dir())
 {
     desc <- .read_description(file.path(dir, "DESCRIPTION"))
     if (pkg != desc["Package"])
@@ -58,7 +68,8 @@ Rd_package_maintainer <- function(pkg, dir = ".")
     desc["Maintainer"]
 }
 
-Rd_package_DESCRIPTION <- function(pkg, lib.loc = Sys.getenv("R_BUILD_TEMPLIB"))
+Rd_package_DESCRIPTION <-
+function(pkg, lib.loc = Sys.getenv("R_BUILD_TEMPLIB"))
 {
     if (!length(find.package(pkg, lib.loc = lib.loc, quiet=TRUE)))
         "This package was not yet installed at build time.\\cr"
@@ -74,7 +85,8 @@ Rd_package_DESCRIPTION <- function(pkg, lib.loc = Sys.getenv("R_BUILD_TEMPLIB"))
     }
 }
 
-Rd_package_indices <- function(pkg, lib.loc = Sys.getenv("R_BUILD_TEMPLIB"))
+Rd_package_indices <-
+function(pkg, lib.loc = Sys.getenv("R_BUILD_TEMPLIB"))
 {
     if (!length(find.package(pkg, lib.loc = lib.loc, quiet=TRUE)))
         result <- c("", "Index:  This package was not yet installed at build time.\\cr")
@@ -99,7 +111,8 @@ Rd_package_indices <- function(pkg, lib.loc = Sys.getenv("R_BUILD_TEMPLIB"))
     result
 }
 
-Rd_expr_doi <- function(x)
+Rd_expr_doi <-
+function(x)
 {
     ## Be nice ...
     x <- .canonicalize_doi(x)
@@ -109,11 +122,11 @@ Rd_expr_doi <- function(x)
     y <- gsub("/", "\\\\out{\\\\slash{}}", gsub("-", "\\\\out{\\\\-}", x))
 
     sprintf("\\ifelse{text}{%s}{\\ifelse{latex}{%s}{%s}}",
-            sprintf("doi: %s (URL: http://doi.org/%s)",
+            sprintf("doi: %s (URL: https://doi.org/%s)",
                     x, x),
-            sprintf("doi:\\out{\\nobreakspace{}}\\href{http://doi.org/%s}{%s}",
+            sprintf("doi:\\out{\\nobreakspace{}}\\href{https://doi.org/%s}{%s}",
                     x, y),
-            sprintf("doi: \\href{http://doi.org/%s}{%s}",
+            sprintf("doi: \\href{https://doi.org/%s}{%s}",
                     x, x)
             )
 }
