@@ -1,7 +1,7 @@
 #  File src/library/tools/R/assertCondition.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 2013-2014 The R Core Team
+#  Copyright (C) 2013-2020 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -23,15 +23,14 @@ assertCondition <-
 {
     getConds <- function(expr) {
 	conds <- list()
-	tryCatch(withCallingHandlers(expr,
-				     warning = function(w) {
-					 conds <<- c(conds, list(w))
-					 invokeRestart("muffleWarning")
-				     },
-				     condition = function(cond)
-					 conds <<- c(conds, list(cond))),
-		 error = function(e)
-		     conds <<- c(conds, list(e)))
+        withCallingHandlers(
+            tryCatch(expr, error = function(e) conds <<- c(conds, list(e))),
+            warning = function(w) {
+                conds <<- c(conds, list(w))
+                invokeRestart("muffleWarning")
+            },
+            condition = function(cond)
+                conds <<- c(conds, list(cond)))
 	conds
     }
     conds <- if(nargs() > 1) c(...) # else NULL
@@ -44,7 +43,9 @@ assertCondition <-
 	    invisible(res)
         }
 	else {
-	    ii <- sapply(res, function(cond) any(class(cond) %in% conds))
+	    ii <- vapply(res,
+                         function(cond) any(class(cond) %in% conds),
+                         NA)
 	    if(any(ii)) {
                 if(verbose) {
                     found <-
@@ -75,7 +76,9 @@ assertError <- function(expr, verbose = FALSE) {
                       call. = FALSE)
              )
     if(verbose) {
-        error <- res[ sapply(res, function(cond) "error" %in% class(cond)) ]
+        error <- res[vapply(res,
+                            function(cond) "error" %in% class(cond),
+                            NA)]
         message(sprintf("Asserted error: %s", error[[1]]$message))
     }
     invisible(res)
@@ -84,10 +87,14 @@ assertError <- function(expr, verbose = FALSE) {
 assertWarning <- function(expr, verbose = FALSE) {
     d.expr <- .deparseTrim(substitute(expr), cutoff = 30L)
     res <- assertCondition(expr, "warning", .exprString = d.expr)
-    if(any(sapply(res, function(cond) "error" %in% class(cond))))
+    if(any(vapply(res,
+                  function(cond) "error" %in% class(cond),
+                  NA)))
         stop(gettextf("Got warning in evaluating %s, but also an error", d.expr))
     if(verbose) {
-        warning <- res[ sapply(res, function(cond) "warning" %in% class(cond)) ]
+        warning <- res[vapply(res,
+                              function(cond) "warning" %in% class(cond),
+                              NA)]
         message(sprintf("Asserted warning: %s", warning[[1]]$message))
     }
     invisible(res)
