@@ -16,8 +16,10 @@
 #  A copy of the GNU General Public License is available at
 #  https://www.R-project.org/Licenses/
 
-md5sum <- function(files)
+md5sum <- function(files) {
+    files <- path.expand(files)
     structure(.Call(C_Rmd5, files), names=files)
+}
 
 .installMD5sums <- function(pkgDir, outDir = pkgDir)
 {
