@@ -1,7 +1,7 @@
 #  File src/library/tools/R/sotools.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 2011-2017 The R Core Team
+#  Copyright (C) 2011-2020 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -17,14 +17,29 @@
 #  https://www.R-project.org/Licenses/
 
 if(.Platform$OS.type == "windows") {
+    read_symbols_from_dll_state <- new.env(hash = FALSE) # small
     read_symbols_from_dll <- function(f, rarch)
     {
-        ## reasonable to assume this on the path
-        DLL_nm <- "objdump.exe"
-        if(!nzchar(Sys.which(DLL_nm))) {
-            warning("this requires 'objdump.exe' to be on the PATH")
-            return()
-        }
+	DLL_nm <- read_symbols_from_dll_state$DLL_nm
+	if (is.null(DLL_nm)) {
+	    ## R CMD config will fail when 'sh' (from Rtools) is not on PATH
+	    DLL_nm <- tryCatch(
+	        Rcmd(c("config", "OBJDUMP"), stdout = TRUE, stderr = FALSE),
+		error = function(x) NULL,
+		warning = function(x) NULL)
+
+	    if (is.null(DLL_nm) || !nzchar(DLL_nm) ||
+	        !file.exists(paste0(DLL_nm, ".exe"))) {
+
+		## fall back to the old behavior: take OBJDUMP from PATH
+		DLL_nm <- "objdump.exe"
+		if(!nzchar(Sys.which(DLL_nm))) {
+		    warning("this requires 'objdump.exe' to be on the PATH")
+		    return()
+		}
+	    }
+	    read_symbols_from_dll_state$DLL_nm <- DLL_nm
+	}
         f <- file_path_as_absolute(f)
         s0 <- suppressWarnings(system2(DLL_nm, c("-x", shQuote(f)),
                                        stdout = TRUE, stderr = TRUE))
@@ -88,6 +103,7 @@ so_symbol_names_table <-
       "linux, C, gcc, _exit, _exit",
       "linux, C, gcc, _Exit, _Exit",
       "linux, C, gcc, printf, printf",
+      "linux, C, gcc, printf, __printf_chk",
       "linux, C, gcc, printf, puts",
       "linux, C, gcc, puts, puts",
       "linux, C, gcc, putchar, putchar",
@@ -114,6 +130,17 @@ so_symbol_names_table <-
       "linux, Fortran, gfortran, stop, _gfortran_stop_numeric_f08",
       "linux, Fortran, gfortran, stop, _gfortran_stop_string",
       "linux, Fortran, gfortran, rand, _gfortran_rand",
+      "linux, Fortran, flang, open, f90io_open03",
+      "linux, Fortran, flang, open, f90io_open2003",
+      "linux, Fortran, flang, close, f90io_close",
+      "linux, Fortran, flang, rewind, f90io_rewind",
+      "linux, Fortran, flang, write, f90io_print_init",
+      "linux, Fortran, flang, print, f90io_print_init",
+      "linux, Fortran, flang, read, f90io_fmt_read",
+      "linux, Fortran, flang, write, f90io_fmt_write",
+      "linux, Fortran, flang, stop, f90_stop",
+      "linux, Fortran, flang, stop, f90_stop08",
+      "linux, Fortran, flang, rand, rand",
 
       "osx, C, gcc, abort, _abort",
       "osx, C, gcc, assert, ___assert_rtn",
@@ -211,6 +238,34 @@ so_symbol_names_table <-
       "solaris, Fortran, solf95, runtime, abort",
       "solaris, Fortran, solf95, rand, rand_",
 
+      "solaris, C, gcc, abort, abort",
+      "solaris, C, gcc, assert, __assert_c99",
+      "solaris, C, gcc, exit, exit",
+      "solaris, C, gcc, _exit, _exit",
+      "solaris, C, gcc, _Exit, _Exit",
+      "solaris, C, gcc, printf, printf",
+      "solaris, C, gcc, printf, puts",
+      "solaris, C, gcc, puts, puts",
+      "solaris, C, gcc, putchar, putchar",
+      "solaris, C, gcc, vprintf, vprintf",
+      "solaris, C, gcc, rand, rand",
+      "solaris, C, gcc, random, random",
+      "solaris, C, gcc, rand_r, rand_r",
+      "solaris, C, gcc, srand, srand",
+      "solaris, C, gcc, srandom, srandom",
+      "solaris, C, gcc, srand48, srand48",
+      "solaris, C++, gxx, std::cout, _ZSt4cout",
+      "solaris, C++, gxx, std::cerr, _ZSt4cerr",
+      "solaris, Fortran, gfortran, open, _gfortran_st_open",
+      "solaris, Fortran, gfortran, close, _gfortran_st_close",
+      "solaris, Fortran, gfortran, rewind, _gfortran_st_rewind",
+      "solaris, Fortran, gfortran, read, _gfortran_st_read",
+      "solaris, Fortran, gfortran, write, _gfortran_st_write",
+      "solaris, Fortran, gfortran, print, _gfortran_st_write",
+      "solaris, Fortran, gfortran, stop, _gfortran_stop_numeric_f08",
+      "solaris, Fortran, gfortran, stop, _gfortran_stop_string",
+      "solaris, Fortran, gfortran, rand, _gfortran_rand",
+
       ## Windows statically links libstdc++, libgfortran
       ## only in .o, positions hard-coded in check_so_symbols
       "windows, C++, g++, std::cout, _ZSt4cout",
@@ -716,7 +771,7 @@ function(file = "symbols.rds")
     objects <- commandArgs(trailingOnly = TRUE)
     tables <- lapply(objects, read_symbols_from_object_file)
     names(tables) <- objects
-    saveRDS(tables, file = file)
+    saveRDS(tables, file = file, version = 2)
 }
 
 
@@ -827,8 +882,10 @@ function(calls, dir = NULL, character_only = TRUE)
     nrdb <- do.call(rbind, nrdb)
     nrdb <- as.data.frame(unique(nrdb), stringsAsFactors = FALSE)
 
-    if(NROW(nrdb) == 0L || length(nrdb) != 3L)
-        stop("no native symbols were extracted")
+    if(NROW(nrdb) == 0L || length(nrdb) != 3L) {
+        message("no native symbols were extracted")
+        return(NULL)
+    }
     nrdb[, 3L] <- as.numeric(nrdb[, 3L])
     nrdb <- nrdb[order(nrdb[, 1L], nrdb[, 2L], nrdb[, 3L]), ]
     nms <- nrdb[, "s"]
@@ -932,21 +989,21 @@ function(nrdb, align = TRUE, include_declarations = FALSE)
             "*/",
             if(NROW(y <- nrdb$.C)) {
                  args <- sapply(y$n, function(n) if(n >= 0)
-                                paste(rep("void *", n), collapse=", ")
+                                paste(rep.int("void *", n), collapse=", ")
                                 else "/* FIXME */")
                 c("", "/* .C calls */",
                   paste0("extern void ", y$s, "(", args, ");"))
            },
             if(NROW(y <- nrdb$.Call)) {
                 args <- sapply(y$n, function(n) if(n >= 0)
-                               paste(rep("SEXP", n), collapse=", ")
+                               paste(rep.int("SEXP", n), collapse=", ")
                                else "/* FIXME */")
                c("", "/* .Call calls */",
                   paste0("extern SEXP ", y$s, "(", args, ");"))
             },
             if(NROW(y <- nrdb$.Fortran)) {
                  args <- sapply(y$n, function(n) if(n >= 0)
-                                paste(rep("void *", n), collapse=", ")
+                                paste(rep.int("void *", n), collapse=", ")
                                 else "/* FIXME */")
                 c("", "/* .Fortran calls */",
                   paste0("extern void F77_NAME(", y$s, ")(", args, ");"))
