@@ -78,12 +78,14 @@ massageExamples <-
 
     cat("base::assign(\".oldSearch\", base::search(), pos = 'CheckExEnv')\n", file = out)
     ## cat("assign(\".oldNS\", loadedNamespaces(), pos = 'CheckExEnv')\n", file = out)
+    cat("base::assign(\".old_wd\", base::getwd(), pos = 'CheckExEnv')\n",
+        file = out)
     for(file in files) {
         nm <- sub("\\.R$", "", basename(file))
         ## make a syntactic name out of the filename
         nm <- gsub("[^- .a-zA-Z0-9_]", ".", nm, perl = TRUE, useBytes = TRUE)
         if (pkg == "grDevices" && nm == "postscript") next
-        ## Latin-1 examples are treated separat
+        ## Latin-1 examples are treated separately
         if (pkg == "graphics" && nm == "text") next
         if(!file.exists(file))
             stop("file ", file, " cannot be opened", domain = NA)
@@ -151,16 +153,22 @@ Rdiff <- function(from, to, useDiff = FALSE, forEx = FALSE,
                               txt, perl = TRUE, useBytes = TRUE)) &&
            length(bot <- grep("quit R.$", txt, perl = TRUE, useBytes = TRUE)))
             txt <- txt[-(top[1L]:bot[1L])]
-        ## for massageExamples()
+        ## for massageExamples(), used for timings
         ll <- grep("</HEADER>", txt, fixed = TRUE, useBytes = TRUE)
         if(length(ll)) txt <- txt[-seq_len(max(ll))]
         ll <- grep("<FOOTER>", txt, fixed = TRUE, useBytes = TRUE)
         if(length(ll)) txt <- txt[seq_len(max(ll) - 1L)]
+        ## remove header change in R 3.5.0
+        if(forEx) {
+            ll <- grep('".old_wd"', txt, fixed = TRUE, useBytes = TRUE)
+            if(length(ll)) txt <- txt[-ll]
+        }
         ## remove BATCH footer
         nl <- length(txt)
         if(nl > 3L && startsWith(txt[nl-2L], "> proc.time()"))
             txt <- txt[1:(nl-3L)]
         ## remove text between IGNORE_RDIFF markers.
+        ## maybe this should only be done for forEx = TRUE?
         txt <- txt[(cumsum(txt == "> ## IGNORE_RDIFF_BEGIN") <=
                     cumsum(txt == "> ## IGNORE_RDIFF_END"))]
         ## (Keeps the end markers, but that's ok.)
@@ -171,8 +179,10 @@ Rdiff <- function(from, to, useDiff = FALSE, forEx = FALSE,
         txt <- .canonicalize_quotes(txt)
         if(.Platform$OS.type == "windows") {
             ## not entirely safe ...
-            txt <- gsub("(\x91|\x92)", "'", txt, perl = TRUE, useBytes = TRUE)
-            txt <- gsub("(\x93|\x94)", '"', txt, perl = TRUE, useBytes = TRUE)
+            txt <- gsub(paste0("(",rawToChar(as.raw(0x91)),"|",rawToChar(as.raw(0x92)),")"),
+                        "'", txt, perl = TRUE, useBytes = TRUE)
+            txt <- gsub(paste0("(",rawToChar(as.raw(0x93)),"|",rawToChar(as.raw(0x94)),")"),
+                        '"', txt, perl = TRUE, useBytes = TRUE)
         }
         ## massageExamples() adds options(pager = "console") only for
         ## Windows, but we should ignore a corresponding diff on all
@@ -193,8 +203,7 @@ Rdiff <- function(from, to, useDiff = FALSE, forEx = FALSE,
     if (forEx) {
         left <- clean2(left)
         ## remove lines from R CMD check --timings
-        left <- grep("[.](format_|)ptime", left, value = TRUE,
-                     invert = TRUE, useBytes = TRUE)
+        left <- filtergrep("[.](format_|)ptime", left, useBytes = TRUE)
         right <- clean2(right)
     }
     if (!useDiff && (length(left) == length(right))) {
@@ -431,11 +440,19 @@ testInstalledPackage <-
 }
 
 .runPackageTests <-
-    function(use_gct = FALSE, use_valgrind = FALSE, Log = NULL, stop_on_error = TRUE, ...)
+    function(use_gct = FALSE, use_valgrind = FALSE, Log = NULL,
+             stop_on_error = TRUE, ...)
 {
+    tlim <- Sys.getenv("_R_CHECK_ONE_TEST_ELAPSED_TIMEOUT_",
+            Sys.getenv("_R_CHECK_TESTS_ELAPSED_TIMEOUT_",
+            Sys.getenv("_R_CHECK_ELAPSED_TIMEOUT_")))
+    tlim <- get_timeout(tlim)
     if (!is.null(Log)) Log <- file(Log, "wt")
     WINDOWS <- .Platform$OS.type == "windows"
     td0 <- as.numeric(Sys.getenv("_R_CHECK_TIMINGS_"))
+    theta <-
+        as.numeric(Sys.getenv("_R_CHECK_TEST_TIMING_CPU_TO_ELAPSED_THRESHOLD_",
+                              NA_character_))
     if (is.na(td0)) td0 <- Inf
     print_time <- function(t1, t2, Log)
     {
@@ -471,10 +488,22 @@ testInstalledPackage <-
         } else
             cmd <- paste("LANGUAGE=C", "R_TESTS=startup.Rs", cmd)
         t1 <- proc.time()
-        res <- system(cmd)
+        res <- system(cmd, timeout = tlim)
         t2 <- proc.time()
         print_time(t1, t2, Log)
+        if (!WINDOWS && !is.na(theta)) {
+            td <- t2 - t1
+            cpu <- sum(td[-3L])
+            if(cpu >= pmax(theta * td[3L], 1)) {
+                ratio <- round(cpu/td[3L], 1L)
+                msg <- sprintf("Running R code in %s had CPU time %g times elapsed time\n",
+                               sQuote(f), ratio)
+                cat(msg)
+                if (!is.null(Log)) cat(msg, file = Log)
+            }
+        }
         if (res) {
+            if(identical(res, 124L)) report_timeout(tlim)
             file.rename(outfile, paste0(outfile, ".fail"))
             return(1L)
         }
@@ -733,14 +762,14 @@ detachPackages <- function(pkgs, verbose = TRUE)
         ## hopefully force = TRUE is never needed, but it does ensure
         ## that progress gets made
         try(detach(this, character.only = TRUE,
-                   unload = unload && !(this %in% exclusions),
+                   unload = unload && (this %notin% exclusions),
                    force = TRUE))
         deps <- deps[-i]
     }
 }
 
 ## Usage: Rscript --vanilla --default-packages=NULL args
-.Rdiff <- function()
+.Rdiff <- function(no.q = FALSE)
 {
     options(showErrorCalls=FALSE)
 
@@ -758,8 +787,12 @@ detachPackages <- function(pkgs, verbose = TRUE)
             sep = "\n")
     }
 
-    do_exit <- function(status = 0L)
-        q("no", status = status, runLast = FALSE)
+    do_exit <-
+	if(no.q)
+	    function(status = 0L) (if(status) stop else message)(
+		".Rdiff() exit status ", status)
+	else
+	    function(status = 0L) q("no", status = status, runLast = FALSE)
 
     args <- commandArgs(TRUE)
     if (!length(args)) {
@@ -769,7 +802,7 @@ detachPackages <- function(pkgs, verbose = TRUE)
     args <- paste(args, collapse=" ")
     args <- strsplit(args,'nextArg', fixed = TRUE)[[1L]][-1L]
     if (length(args) == 1L) {
-        if(args[1L] %in% c("-h", "--help")) { Usage(); do_exit() }
+        if(args[1L] %in% c("-h", "--help")) { Usage(); do_exit(0) }
         if(args[1L] %in% c("-v", "--version")) {
             cat("R output diff: ",
                 R.version[["major"]], ".",  R.version[["minor"]],
@@ -779,7 +812,7 @@ detachPackages <- function(pkgs, verbose = TRUE)
                 "This is free software; see the GNU General Public License version 2",
                 "or later for copying conditions.  There is NO warranty.",
                 sep = "\n")
-            do_exit()
+            do_exit(0)
         }
         Usage()
         do_exit(1L)
