@@ -1,7 +1,7 @@
 #  File src/library/tools/R/urltools.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 2015-2016 The R Core Team
+#  Copyright (C) 2015-2019 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@ function()
     ## <http://www.iana.org/assignments/uri-schemes/uri-schemes.xhtml>.
     baseurl <- "http://www.iana.org/assignments/uri-schemes/"
     db <- utils::read.csv(url(paste0(baseurl, "uri-schemes-1.csv")),
-                          stringsAsFactors = FALSE)
+                          stringsAsFactors = FALSE, encoding = "UTF-8")
     names(db) <- chartr(".", "_", names(db))
     db
 }
@@ -69,11 +69,35 @@ function(x)
 .get_urls_from_HTML_file <-
 function(f)
 {
-    nodes <- xml2::xml_find_all(xml2::read_html(f), "//a")
+    doc <- xml2::read_html(f)
+    if(!inherits(doc, "xml_node")) return(character())
+    nodes <- xml2::xml_find_all(doc, "//a")
     hrefs <- xml2::xml_attr(nodes, "href")
     unique(hrefs[!is.na(hrefs) & !startsWith(hrefs, "#")])
 }
 
+.get_urls_from_PDF_file <-
+function(f)    
+{
+    ## Seems there is no straightforward way to extract hyperrefs from a
+    ## PDF, hence first convert to HTML.
+    ## Note that pdftohtml always outputs in cwd ...
+    owd <- getwd()
+    dir.create(d <- tempfile())
+    on.exit({ unlink(d, recursive = TRUE); setwd(owd) })
+    file.copy(normalizePath(f), d)
+    setwd(d)
+    g <- tempfile(tmpdir = d, fileext = ".xml")
+    system2("pdftohtml",
+            c("-s -q -i -c -xml", basename(f), basename(g)))
+    ## Oh dear: seems that pdftohtml can fail without a non-zero exit
+    ## status.
+    if(file.exists(g))
+        .get_urls_from_HTML_file(g)
+    else
+        character()
+}
+
 url_db <-
 function(urls, parents)
 {
@@ -85,6 +109,60 @@ function(urls, parents)
     db
 }
 
+url_db_from_HTML_files <-
+function(dir, recursive = FALSE, files = NULL, verbose = FALSE)
+{
+    urls <- parents <- character()
+    if(is.null(files)) 
+        files <- list.files(dir, pattern = "[.]html$",
+                            full.names = TRUE,
+                            recursive = recursive)
+    urls <-
+        lapply(files,
+               function(f) {
+                   if(verbose)
+                       message(sprintf("processing %s",
+                                       .file_path_relative_to_dir(f, dir)))
+                   .get_urls_from_HTML_file(f)
+               })
+    names(urls) <- files
+    urls <- Filter(length, urls)
+    if(length(urls)) {
+        parents <- rep.int(.file_path_relative_to_dir(names(urls), dir),
+                           lengths(urls))
+        urls <- unlist(urls, use.names = FALSE)
+    }
+    url_db(urls, parents)
+}
+
+url_db_from_PDF_files <-
+function(dir, recursive = FALSE, files = NULL, verbose = FALSE)
+{
+    urls <- parents <- character()
+    if(is.null(files))
+        files <- list.files(dir, pattern = "[.]pdf$",
+                            full.names = TRUE,
+                            recursive = recursive)
+    ## FIXME: this is simpler to do with full.names = FALSE and without
+    ## tools:::.file_path_relative_to_dir().
+    urls <-
+        lapply(files,
+               function(f) {
+                   if(verbose)
+                       message(sprintf("processing %s",
+                                       .file_path_relative_to_dir(f, dir)))
+                   .get_urls_from_PDF_file(f)
+               })
+    names(urls) <- files
+    urls <- Filter(length, urls)
+    if(length(urls)) {
+        parents <- rep.int(.file_path_relative_to_dir(names(urls), dir),
+                           lengths(urls))
+        urls <- unlist(urls, use.names = FALSE)
+    }
+    url_db(urls, parents)
+}
+
 url_db_from_package_Rd_db <-
 function(db)
 {
@@ -131,38 +209,25 @@ function(dir, meta, installed = FALSE)
 url_db_from_package_news <-
 function(dir, installed = FALSE)
 {
-    urls <- character()
     path <- if(installed) "NEWS.Rd" else file.path("inst", "NEWS.Rd")
     nfile <- file.path(dir, path)
-    if(file.exists(nfile)) {
-        macros <- initialRdMacros()
-        urls <- .get_urls_from_Rd(prepare_Rd(tools::parse_Rd(nfile,
-                                                             macros = macros),
-                                             stages = "install"))
-    }
+    urls <-
+        if(file.exists(nfile)) {
+            macros <- initialRdMacros()
+            .get_urls_from_Rd(prepare_Rd(parse_Rd(nfile, macros = macros),
+                                         stages = "install"))
+        } else character()
     url_db(urls, rep.int(path, length(urls)))
 }
 
 url_db_from_package_HTML_files <-
 function(dir, installed = FALSE)
 {
-    urls <- parents <- character()
     path <- if(installed) "doc" else file.path("inst", "doc")
     files <- Sys.glob(file.path(dir, path, "*.html"))
     if(installed && file.exists(rfile <- file.path(dir, "README.html")))
         files <- c(files, rfile)
-    if(length(files)) {
-        urls <- lapply(files, .get_urls_from_HTML_file)
-        names(urls) <- files
-        urls <- Filter(length, urls)
-        if(length(urls)) {
-            parents <- rep.int(.file_path_relative_to_dir(names(urls),
-                                                          dir),
-                               lengths(urls))
-            urls <- unlist(urls, use.names = FALSE)
-        }
-    }
-    url_db(urls, parents)
+    url_db_from_HTML_files(dir, files = files)
 }
 
 url_db_from_package_README_md <-
@@ -308,17 +373,24 @@ table_of_FTP_server_return_codes <-
       )
 
 check_url_db <-
-function(db, verbose = FALSE)
+function(db, remote = TRUE, verbose = FALSE)
 {
+    use_curl <-
+        config_val_to_logical(Sys.getenv("_R_CHECK_URLS_USE_CURL_",
+                                         "TRUE")) &&
+        requireNamespace("curl", quietly = TRUE)
+
     .gather <- function(u = character(),
                         p = list(),
                         s = rep.int("", length(u)),
                         m = rep.int("", length(u)),
+                        new = rep.int("", length(u)),
                         cran = rep.int("", length(u)),
-                        spaces = rep.int("", length(u))) {
+                        spaces = rep.int("", length(u)),
+                        R = rep.int("", length(u))) {
         y <- data.frame(URL = u, From = I(p), Status = s, Message = m,
-                        CRAN = cran, Spaces = spaces,
-                        stringsAsFactors = FALSE)
+                        New = new, CRAN = cran, Spaces = spaces, R = R,
+                        row.names = NULL, stringsAsFactors = FALSE)
         y$From <- p
         class(y) <- c("check_url_db", "data.frame")
         y
@@ -328,6 +400,9 @@ function(db, verbose = FALSE)
         if(verbose) message(sprintf("processing %s", u))
         h <- tryCatch(curlGetHeaders(u), error = identity)
         if(inherits(h, "error")) {
+            ## Currently, this info is only used in .check_http().
+            ## Might be useful for checking ftps too, so simply leave it
+            ## here instead of moving to .check_http().
             msg <- conditionMessage(h)
             if (grepl("libcurl error code (51|60)", msg)) {
                 h2 <- tryCatch(curlGetHeaders(u, verify = FALSE),
@@ -350,7 +425,12 @@ function(db, verbose = FALSE)
         c(s, msg, "", "")
     }
 
-    .check_http <- function(u) {
+    .check_http <- if(remote)
+        function(u) c(.check_http_A(u), .check_http_B(u))
+    else
+        function(u) c(rep.int("", 3L), .check_http_B(u))
+
+    .check_http_A <- function(u) {
         h <- .fetch(u)
         newLoc <- ""
         if(inherits(h, "error")) {
@@ -371,14 +451,32 @@ function(db, verbose = FALSE)
             if (length(ind))
                 newLoc <- sub("^[Ll]ocation: ([^\r]*)\r\n", "\\1", h[max(ind)])
         }
+        ##
+        if((s != "200") && use_curl) {
+            g <- .curl_GET_status(u)
+            if(g == "200") {
+                s <- g
+                msg <- "OK"
+            }
+        }
         ## A mis-configured site
         if (s == "503" && any(grepl("www.sciencedirect.com", c(u, newLoc))))
             s <- "405"
-        cran <- (grepl("https?://cran.r-project.org/web/packages/[.[:alnum:]]+(|/|/index.html)$",
-                       u, ignore.case = TRUE) ||
-                 any(substring(tolower(u), 1L, nchar(mirrors)) == mirrors))
+        c(s, msg, newLoc)
+    }
+
+    .check_http_B <- function(u) {
+        ul <- tolower(u)
+        cran <- ((grepl("^https?://cran.r-project.org/web/packages", ul) &&
+                  !grepl("^https?://cran.r-project.org/web/packages/[.[:alnum:]]+(html|pdf|rds)$",
+                         ul)) ||
+                 (grepl("^https?://cran.r-project.org/web/views/[[:alnum:]]+[.]html$",
+                        ul)) ||
+                 startsWith(ul, "http://cran.r-project.org") ||
+                 any(startsWith(ul, mirrors)))
+        R <- grepl("^http://(www|bugs|journal).r-project.org", ul)
         spaces <- grepl(" ", u)
-        c(s, msg, newLoc, if(cran) u else "", if(spaces) u else "")
+        c(if(cran) u else "", if(spaces) u else "", if(R) u else "")
     }
 
     bad <- .gather()
@@ -393,8 +491,15 @@ function(db, verbose = FALSE)
                  "https://cran.rstudio.com/")
     mirrors <- tolower(sub("/$", "", mirrors))
 
-    parents <- split(db$Parent, db$URL)
-    urls <- names(parents)
+    if(inherits(db, "check_url_db")) {
+        ## Allow re-checking check results.
+        parents <- db$From
+        urls <- db$URL
+    } else {
+        parents <- split(db$Parent, db$URL)
+        urls <- names(parents)
+    }
+
     parts <- parse_URI_reference(urls)
 
     ## Empty URLs.
@@ -421,14 +526,14 @@ function(db, verbose = FALSE)
         msg <- rep.int("Invalid URI scheme", len)
         doi <- schemes[ind] == "doi"
         if(any(doi))
-            msg[doi] <- paste(msg[doi], "(use \\doi for DOIs)")
+            msg[doi] <- paste(msg[doi], "(use \\doi for DOIs in Rd markup)")
         bad <- rbind(bad,
                      .gather(urls[ind], parents[ind], m = msg))
     }
 
     ## ftp.
     pos <- which(schemes == "ftp")
-    if(length(pos)) {
+    if(length(pos) && remote) {
         results <- do.call(rbind, lapply(urls[pos], .check_ftp))
         status <- as.numeric(results[, 1L])
         ind <- (status < 0L) | (status >= 400L)
@@ -450,21 +555,23 @@ function(db, verbose = FALSE)
         status <- as.numeric(results[, 1L])
         ## 405 is HTTP not allowing HEAD requests
         ## maybe also skip 500, 503, 504 as likely to be temporary issues
-        ind <- !(status %in% c(200L, 405L)) |
-            nzchar(results[, 4L]) | nzchar(results[, 5L])
+        ind <- is.na(match(status, c(200L, 405L, NA))) |
+            nzchar(results[, 4L]) |
+            nzchar(results[, 5L]) |
+            nzchar(results[, 6L])
         if(any(ind)) {
             pos <- pos[ind]
             s <- as.character(status[ind])
+            s[is.na(s)] <- ""
             s[s == "-1"] <- "Error"
             m <- results[ind, 2L]
             m[is.na(m)] <- ""
-            url <- urls[pos]; newLoc <- results[ind, 3L]
-            ind2 <- nzchar(newLoc)
-            url[ind2] <-
-                paste0(url[ind2], " (moved to ", newLoc[ind2], ")")
             bad <- rbind(bad,
-                         .gather(url, parents[pos], s, m,
-                                 results[ind, 4L], results[ind, 5L]))
+                         .gather(urls[pos], parents[pos], s, m,
+                                 results[ind, 3L],
+                                 results[ind, 4L],
+                                 results[ind, 5L],
+                                 results[ind, 6L]))
         }
     }
     bad
@@ -475,21 +582,29 @@ function(x, ...)
 {
     if(!NROW(x)) return(character())
 
-    paste0(sprintf("URL: %s", x$URL),
+    u <- x$URL
+    new <- x$New
+    ind <- nzchar(new)
+    u[ind] <- sprintf("%s (moved to %s)", u[ind], new[ind])
+
+    paste0(sprintf("URL: %s", u),
            sprintf("\nFrom: %s",
-                   sapply(x$From, paste, collapse = "\n      ")),
+                   vapply(x$From, paste, "", collapse = "\n      ")),
            ifelse((s <- x$Status) == "",
                   "",
                   sprintf("\nStatus: %s", s)),
            ifelse((m <- x$Message) == "",
                   "",
-                  sprintf("\nMessage: %s", m)),
+                  sprintf("\nMessage: %s", gsub("\n", "\n  ", m))),
            ifelse((m <- x$Spaces) == "",
                   "",
                   "\nURL contains spaces"),
            ifelse((m <- x$CRAN) == "",
                   "",
-                  "\nCRAN URL not in canonical form")
+                  "\nCRAN URL not in canonical form"),
+           ifelse((m <- x$R) == "",
+                  "",
+                  "\nR-project URL not in canonical form")
            )
 }
 
@@ -500,3 +615,38 @@ function(x, ...)
         writeLines(paste(format(x), collapse = "\n\n"))
     invisible(x)
 }
+
+as.matrix.check_url_db <-
+function(x, ...)
+{
+    n <- lengths(x[["From"]])
+    y <- do.call(cbind,
+                 c(list(URL = rep.int(x[["URL"]], n),
+                        Parent = unlist(x[["From"]])),
+                   lapply(x[-c(1L, 2L)], rep.int, n)))
+    rownames(y) <- NULL
+    y
+}
+
+.curl_GET_status <-
+function(u, verbose = FALSE)
+{
+    if(verbose)
+        message(sprintf("processing %s", u))
+    ## Configure curl handle for better luck with JSTOR URLs/DOIs.
+    ## Alternatively, special-case requests to
+    ##   https?://doi.org/10.2307
+    ##   https?://www.jstor.org
+    h <- curl::new_handle()
+    curl::handle_setopt(h,
+                        cookiesession = 1,
+                        followlocation = 1,
+                        http_version = 2L,
+                        ssl_enable_alpn = 0)
+    g <- tryCatch(curl::curl_fetch_memory(u, handle = h),
+                  error = identity)
+    if(inherits(g, "error"))
+        -1L
+    else
+        g$status_code
+}
