@@ -14,12 +14,11 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, a copy is available at
- *  http://www.r-project.org/Licenses/
+ *  https://www.R-project.org/Licenses/
  */
 
 /* <UTF8> OK since this is intended to treat chars as byte streams */
 
-#include <R.h>
 #include "tools.h"
 #define ROL_UNUSED
 #include "md5.h"
@@ -38,7 +37,7 @@ SEXP Rmd5(SEXP files)
     PROTECT(ans = allocVector(STRSXP, nfiles));
     for(i = 0; i < nfiles; i++) {
 	path = translateChar(STRING_ELT(files, i));
-#ifdef WIN32
+#ifdef _WIN32
 	fp = fopen(path, "rb");
 #else
 	fp = fopen(path, "r");
