@@ -67,7 +67,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996, 1997  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2018  The R Core Team
+ *  Copyright (C) 1997--2019  The R Core Team
  *  Copyright (C) 2010 Duncan Murdoch
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -89,6 +89,7 @@
 #include <config.h>
 #endif
 
+#define R_USE_SIGNALS 1
 #include <Defn.h>
 #include <Parse.h>
 #define STRICT_R_HEADERS
@@ -103,10 +104,16 @@
 #define _(String) (String)
 #endif
 
-/* bison creates a non-static symbol yylloc in both gramLatex.o and gramRd.o,
-   so remap */
+/* bison creates a non-static symbol yylloc (and other) in both gramLatex.o
+   and gramRd.o, so remap */
 
 #define yylloc yyllocL
+#undef yynerrs /* from Defn.h */
+#define yynerrs yynerrsL
+#undef yychar /* from Defn.h */
+#define yychar yycharL
+#undef yylval /* from Defn.h */
+#define yylval yylvalL
 
 #define DEBUGVALS 0		/* 1 causes detailed internal state output to R console */	
 #define DEBUGMODE 0		/* 1 causes Bison output of parse state, to stdout or stderr */
@@ -160,7 +167,7 @@ typedef struct yyltype
 
 /* Functions used in the parsing process */
 
-static SEXP	GrowList(SEXP, SEXP);
+static void	GrowList(SEXP, SEXP);
 static int	KeywordLookup(const char *);
 static SEXP	NewList(void);
 static SEXP     makeSrcref(YYLTYPE *, SEXP);
@@ -184,13 +191,16 @@ struct ParseState {
     SEXP	xxVerbatimList;/* A STRSXP containing all the verbatim environment names */
 
     SEXP     SrcFile;  /* parseLatex will *always* supply a srcfile */
-    
+    SEXP mset; /* precious mset for protecting parser semantic values */
     ParseState *prevState;
 };
 
 static Rboolean busy = FALSE;
 static ParseState parseState;
 
+#define PRESERVE_SV(x) R_PreserveInMSet((x), parseState.mset)
+#define RELEASE_SV(x)  R_ReleaseFromMSet((x), parseState.mset)
+
 /* Routines used to build the parse tree */
 
 static SEXP	xxnewlist(SEXP);
@@ -208,6 +218,8 @@ static int 	mkComment(int);
 static int      mkVerb(int);
 static int      mkVerbEnv();
 
+static SEXP R_LatexTagSymbol = NULL;
+
 #define YYSTYPE		SEXP
 
 
@@ -593,9 +605,9 @@ static const yytype_uint8 yytranslate[] =
   /* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
 static const yytype_uint8 yyrline[] =
 {
-       0,   166,   166,   167,   168,   171,   172,   173,   174,   176,
-     177,   179,   180,   181,   182,   183,   184,   186,   186,   190,
-     192,   193
+       0,   177,   177,   178,   179,   182,   183,   184,   185,   187,
+     188,   190,   191,   192,   193,   194,   195,   197,   197,   201,
+     203,   204
 };
 #endif
 
@@ -1210,31 +1222,31 @@ yydestruct (const char *yymsg, int yytype, YYSTYPE *yyvaluep, YYLTYPE *yylocatio
     {
           case 5: /* MACRO  */
 
-      { UNPROTECT_PTR(((*yyvaluep))); }
+      { RELEASE_SV(((*yyvaluep))); }
 
         break;
 
     case 6: /* TEXT  */
 
-      { UNPROTECT_PTR(((*yyvaluep))); }
+      { RELEASE_SV(((*yyvaluep))); }
 
         break;
 
     case 7: /* COMMENT  */
 
-      { UNPROTECT_PTR(((*yyvaluep))); }
+      { RELEASE_SV(((*yyvaluep))); }
 
         break;
 
     case 8: /* BEGIN  */
 
-      { UNPROTECT_PTR(((*yyvaluep))); }
+      { RELEASE_SV(((*yyvaluep))); }
 
         break;
 
     case 9: /* END  */
 
-      { UNPROTECT_PTR(((*yyvaluep))); }
+      { RELEASE_SV(((*yyvaluep))); }
 
         break;
 
@@ -1534,7 +1546,7 @@ yyreduce:
 
   case 4:
 
-    { PROTECT(parseState.Value = R_NilValue);  YYABORT; }
+    { PRESERVE_SV(parseState.Value = R_NilValue);  YYABORT; }
 
     break;
 
@@ -1619,7 +1631,7 @@ yyreduce:
   case 18:
 
     { (yyval) = xxenv((yyvsp[-7]), (yyvsp[-4]), (yyvsp[-1]), &(yyloc));
-                                                  UNPROTECT_PTR((yyvsp[-9])); UNPROTECT_PTR((yyvsp[-3])); }
+                                                  RELEASE_SV((yyvsp[-9])); RELEASE_SV((yyvsp[-3])); }
 
     break;
 
@@ -1882,35 +1894,32 @@ yyreturn:
 
 static SEXP xxnewlist(SEXP item)
 {
-    SEXP ans, tmp;
+    SEXP ans;
 #if DEBUGVALS
     Rprintf("xxnewlist(item=%p)", item);
 #endif    
-    PROTECT(tmp = NewList());
+    PRESERVE_SV(ans = NewList());
     if (item) {
-    	PROTECT(ans = GrowList(tmp, item));
-    	UNPROTECT_PTR(tmp);
-    	UNPROTECT_PTR(item);
-    } else ans = tmp;
+	GrowList(ans, item);
+	RELEASE_SV(item);
+    }
 #if DEBUGVALS
     Rprintf(" result: %p is length %d\n", ans, length(ans));
 #endif
     return ans;
 }
 
-static SEXP xxlist(SEXP oldlist, SEXP item)
+static SEXP xxlist(SEXP list, SEXP item)
 {
-    SEXP ans;
 #if DEBUGVALS
-    Rprintf("xxlist(oldlist=%p, item=%p)", oldlist, item);
+    Rprintf("xxlist(list=%p, item=%p)", list, item);
 #endif
-    PROTECT(ans = GrowList(oldlist, item));
-    UNPROTECT_PTR(item);
-    UNPROTECT_PTR(oldlist);
+    GrowList(list, item);
+    RELEASE_SV(item);
 #if DEBUGVALS
-    Rprintf(" result: %p is length %d\n", ans, length(ans));
+    Rprintf(" result: %p is length %d\n", list, length(list));
 #endif
-    return ans;
+    return list;
 }
 
 static SEXP xxenv(SEXP begin, SEXP body, SEXP end, YYLTYPE *lloc)
@@ -1919,19 +1928,18 @@ static SEXP xxenv(SEXP begin, SEXP body, SEXP end, YYLTYPE *lloc)
 #if DEBUGVALS
     Rprintf("xxenv(begin=%p, body=%p, end=%p)", begin, body, end);    
 #endif
-    PROTECT(ans = allocVector(VECSXP, 2));
+    PRESERVE_SV(ans = allocVector(VECSXP, 2));
     SET_VECTOR_ELT(ans, 0, begin);
-    UNPROTECT_PTR(begin);
+    RELEASE_SV(begin);
     if (!isNull(body)) {
 	SET_VECTOR_ELT(ans, 1, PairToVectorList(CDR(body)));
-    	UNPROTECT_PTR(body);	
+	RELEASE_SV(body);
     }
     /* FIXME:  check that begin and end match */
     setAttrib(ans, R_SrcrefSymbol, makeSrcref(lloc, parseState.SrcFile));
-    SEXP s_latex_tag = install("latex_tag");
-    setAttrib(ans, s_latex_tag, mkString("ENVIRONMENT"));
+    setAttrib(ans, R_LatexTagSymbol, mkString("ENVIRONMENT"));
     if (!isNull(end)) 
-    	UNPROTECT_PTR(end);
+	RELEASE_SV(end);
 #if DEBUGVALS
     Rprintf(" result: %p\n", ans);    
 #endif
@@ -1944,11 +1952,10 @@ static SEXP xxmath(SEXP body, YYLTYPE *lloc)
 #if DEBUGVALS
     Rprintf("xxmath(body=%p)", body);    
 #endif
-    PROTECT(ans = PairToVectorList(CDR(body)));
-    UNPROTECT_PTR(body);
+    PRESERVE_SV(ans = PairToVectorList(CDR(body)));
+    RELEASE_SV(body);
     setAttrib(ans, R_SrcrefSymbol, makeSrcref(lloc, parseState.SrcFile));
-    SEXP s_latex_tag = install("latex_tag");
-    setAttrib(ans, s_latex_tag, mkString("MATH"));
+    setAttrib(ans, R_LatexTagSymbol, mkString("MATH"));
 #if DEBUGVALS
     Rprintf(" result: %p\n", ans);    
 #endif
@@ -1962,14 +1969,13 @@ static SEXP xxblock(SEXP body, YYLTYPE *lloc)
     Rprintf("xxblock(body=%p)", body);    
 #endif
     if (!body) 
-        PROTECT(ans = allocVector(VECSXP, 0));
+        PRESERVE_SV(ans = allocVector(VECSXP, 0));
     else {
-	PROTECT(ans = PairToVectorList(CDR(body)));
-    	UNPROTECT_PTR(body);	
+	PRESERVE_SV(ans = PairToVectorList(CDR(body)));
+	RELEASE_SV(body);
     }
     setAttrib(ans, R_SrcrefSymbol, makeSrcref(lloc, parseState.SrcFile));
-    SEXP s_latex_tag = install("latex_tag");
-    setAttrib(ans, s_latex_tag, mkString("BLOCK"));
+    setAttrib(ans, R_LatexTagSymbol, mkString("BLOCK"));
 
 #if DEBUGVALS
     Rprintf(" result: %p\n", ans);    
@@ -1992,20 +1998,19 @@ static void xxSetInVerbEnv(SEXP envname)
     char buffer[256];
     if (VerbatimLookup(CHAR(STRING_ELT(envname, 0)))) {
     	snprintf(buffer, sizeof(buffer), "\\end{%s}", CHAR(STRING_ELT(envname, 0)));
-    	PROTECT(parseState.xxInVerbEnv = ScalarString(mkChar(buffer)));
+	PRESERVE_SV(parseState.xxInVerbEnv = ScalarString(mkChar(buffer)));
     } else parseState.xxInVerbEnv = NULL;
 }
 
 static void xxsavevalue(SEXP items, YYLTYPE *lloc)
 {
     if (items) {
-    	PROTECT(parseState.Value = PairToVectorList(CDR(items)));
-    	UNPROTECT_PTR(items);
+	PRESERVE_SV(parseState.Value = PairToVectorList(CDR(items)));
+	RELEASE_SV(items);
     } else {
-    	PROTECT(parseState.Value = allocVector(VECSXP, 1));
+	PRESERVE_SV(parseState.Value = allocVector(VECSXP, 1));
     	SET_VECTOR_ELT(parseState.Value, 0, ScalarString(mkChar("")));
-    	SEXP s_latex_tag = install("latex_tag");
-    	setAttrib(VECTOR_ELT(parseState.Value, 0), s_latex_tag, mkString("TEXT"));
+	setAttrib(VECTOR_ELT(parseState.Value, 0), R_LatexTagSymbol, mkString("TEXT"));
     }	
     if (!isNull(parseState.Value)) {
     	setAttrib(parseState.Value, R_ClassSymbol, mkString("LaTeX"));
@@ -2015,8 +2020,7 @@ static void xxsavevalue(SEXP items, YYLTYPE *lloc)
 
 static SEXP xxtag(SEXP item, int type, YYLTYPE *lloc)
 {
-    SEXP s_latex_tag = install("latex_tag");
-    setAttrib(item, s_latex_tag, mkString(yytname[YYTRANSLATE(type)]));
+    setAttrib(item, R_LatexTagSymbol, mkString(yytname[YYTRANSLATE(type)]));
     setAttrib(item, R_SrcrefSymbol, makeSrcref(lloc, parseState.SrcFile));
     return item;
 }
@@ -2109,7 +2113,7 @@ static SEXP makeSrcref(YYLTYPE *lloc, SEXP srcfile)
     INTEGER(val)[5] = lloc->last_column;
     setAttrib(val, R_SrcfileSymbol, srcfile);
     setAttrib(val, R_ClassSymbol, mkString("srcref"));
-    UNPROTECT(1);
+    UNPROTECT(1); /* val */
     return val;
 }
 
@@ -2120,7 +2124,7 @@ static SEXP mkString2(const char *s, size_t len)
 
     PROTECT(t = allocVector(STRSXP, 1));
     SET_STRING_ELT(t, 0, mkCharLenCE(s, (int) len, enc));
-    UNPROTECT(1);
+    UNPROTECT(1); /* t */
     return t;
 }
 
@@ -2143,15 +2147,12 @@ static SEXP NewList(void)
 
 /* Add a new element at the end of a stretchy list */
 
-static SEXP GrowList(SEXP l, SEXP s)
+static void GrowList(SEXP l, SEXP s)
 {
     SEXP tmp;
-    PROTECT(s);
     tmp = CONS(s, R_NilValue);
-    UNPROTECT(1);
     SETCDR(CAR(l), tmp);
     SETCAR(l, tmp);
-    return l;
 }
 
 /*--------------------------------------------------------------------------*/
@@ -2182,8 +2183,16 @@ static void UseState(ParseState *state) {
     parseState.prevState = state->prevState;
 }
 
+static void InitSymbols(void)
+{
+    if (!R_LatexTagSymbol)
+	R_LatexTagSymbol = install("latex_tag");
+}
+
 static SEXP ParseLatex(ParseStatus *status, SEXP srcfile)
 {
+    InitSymbols();
+
     R_ParseContextLast = 0;
     R_ParseContext[0] = '\0';
     	
@@ -2194,6 +2203,8 @@ static SEXP ParseLatex(ParseStatus *status, SEXP srcfile)
     parseState.xxbyteno = 1;
     
     parseState.SrcFile = srcfile;
+
+    PROTECT(parseState.mset = R_NewPreciousMSet(50));
     
     npush = 0;
     
@@ -2204,8 +2215,11 @@ static SEXP ParseLatex(ParseStatus *status, SEXP srcfile)
 
 #if DEBUGVALS
     Rprintf("ParseRd result: %p\n", parseState.Value);    
-#endif    
-    UNPROTECT_PTR(parseState.Value);
+#endif
+
+    RELEASE_SV(parseState.Value);
+    UNPROTECT(1); /* parseState.mset */
+
     return parseState.Value;
 }
 
@@ -2408,7 +2422,7 @@ static int token(void)
         yylloc.last_line = 0;
         yylloc.last_column = 0;
         yylloc.last_byte = 0;
-    	PROTECT(yylval = mkString(""));
+	PRESERVE_SV(yylval = mkString(""));
         c = parseState.xxinitvalue;
     	parseState.xxinitvalue = 0;
     	return(c);
@@ -2455,7 +2469,7 @@ static int mkText(int c)
     };
 stop:
     xxungetc(c);
-    PROTECT(yylval = mkString2(stext,  bp - stext));
+    PRESERVE_SV(yylval = mkString2(stext,  bp - stext));
     if(stext != st0) free(stext);
     return TEXT;
 }
@@ -2472,7 +2486,7 @@ static int mkComment(int c)
     if (c == R_EOF) xxungetc(c);
     else TEXT_PUSH(c);
     
-    PROTECT(yylval = mkString2(stext,  bp - stext));
+    PRESERVE_SV(yylval = mkString2(stext,  bp - stext));
     if(stext != st0) free(stext);    
     return COMMENT;
 }
@@ -2500,9 +2514,8 @@ static int mkMarkup(int c)
         else if (c != ' ') /* Eat a space, but keep other terminators */
     	    xxungetc(c);
     }
-    if (retval != VERB) {
-    	PROTECT(yylval = mkString(stext));
-    }
+    if (retval != VERB)
+	PRESERVE_SV(yylval = mkString(stext));
     if(stext != st0) free(stext);
     return retval;
 }
@@ -2519,7 +2532,7 @@ static int mkVerb(int c)
     while ((c = xxgetc()) != delim) TEXT_PUSH(c);
     TEXT_PUSH(c);
     
-    PROTECT(yylval = mkString2(stext, bp - stext));
+    PRESERVE_SV(yylval = mkString2(stext, bp - stext));
     if(stext != st0) free(stext);
     return VERB;  
 }
@@ -2542,11 +2555,11 @@ static int mkVerbEnv()
     if ( !CHAR(STRING_ELT(parseState.xxInVerbEnv, 0))[matched] ) {
     	for (i = matched-1; i >= 0; i--) 
     	    xxungetc(*(--bp));    	    
-    	UNPROTECT_PTR(parseState.xxInVerbEnv);
+	RELEASE_SV(parseState.xxInVerbEnv);
     	parseState.xxInVerbEnv = NULL;
     }
     	    
-    PROTECT(yylval = mkString2(stext, bp - stext));
+    PRESERVE_SV(yylval = mkString2(stext, bp - stext));
     if (stext != st0) free(stext);
     return VERB;
 }
