@@ -43,7 +43,7 @@ select.list <-
         nc <- length(choices)
         if (length(title) && nzchar(title[1L]))
             cat(title, "\n", sep = "")
-        def <- if(is.null(preselect)) rep(FALSE, nc)
+        def <- if(is.null(preselect)) rep.int(FALSE, nc)
         else choices %in% preselect
         op <- paste0(format(seq_len(nc)), ": ",
                      ifelse(def, "+", " "), " ", choices)
@@ -52,7 +52,7 @@ select.list <-
             nw <- nchar(fop[1L], "w") + 2L
             ncol <- getOption("width") %/% nw
             if(ncol > 1L)
-                op <- paste0(fop, c(rep("  ", ncol - 1L), "\n"),
+                op <- paste0(fop, c(rep.int("  ", ncol - 1L), "\n"),
                              collapse = "")
             cat("", op, sep = "\n")
         } else cat("", op, "", sep = "\n")
