@@ -1,7 +1,7 @@
 #  File src/library/utils/R/zip.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2018 The R Core Team
+#  Copyright (C) 1995-2019 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -37,14 +37,22 @@ unzip <-
             stop("'unzip' must be a single character string")
         zipfile <- path.expand(zipfile)
         if (list) {
+            ## -q to suppress per-file and per-archive comments (since 5.52)
+            ## it also suppresses the first line "Archive: filename"
             res <- if (WINDOWS)
-                    system2(unzip, c("-l", shQuote(zipfile)), stdout = TRUE)
+                    system2(unzip, c("-ql", shQuote(zipfile)), stdout = TRUE)
                 else
-                    system2(unzip, c("-l", shQuote(zipfile)), stdout = TRUE,
+                    system2(unzip, c("-ql", shQuote(zipfile)), stdout = TRUE,
                             env = c("TZ=UTC"))
             l <- length(res)
-            res2 <- res[-c(1,3, l-1, l)]
-            con <- textConnection(res2); on.exit(close(con))
+            res2 <- res[-c(2, l-1, l)]
+
+            ## this allows space in file name, but it would break with
+            ## double quotes (though those are discouraged both by Windows
+            ## documentation and POSIX)
+            res3 <- gsub(" *([^ ]+) +([^ ]+) +([^ ]+) +(.*)",
+                         "\\1 \\2 \\3 \"\\4\"", res2)
+            con <- textConnection(res3); on.exit(close(con))
             z <- read.table(con, header=TRUE, as.is=TRUE)
             dt <- paste(z$Date, z$Time)
             ## Unzip 6.00 always uses 4-digits years, but any order is
@@ -69,7 +77,13 @@ unzip <-
             z[, "Date"] <- zz
             z[c("Name", "Length", "Date")]
         } else {
-            args <- c("-oq", shQuote(zipfile))
+            ## -n -o -q -j are supported in Unzip 5.52 and 6.00
+            args <- character()
+            if (junkpaths) args <- c(args, "-j")
+            if (overwrite)
+                args <- c(args, "-oq", shQuote(zipfile))
+            else
+                args <- c(args, "-nq", shQuote(zipfile))
             if (length(files)) args <- c(args, shQuote(files))
             if (exdir != ".") args <- c(args, "-d", shQuote(exdir))
             ## there is an unzip clone about that does not respect -q
@@ -90,8 +104,20 @@ zip <- function(zipfile, files, flags = "-r9X", extras = "",
         stop("'files' must a character vector specifying one or more filepaths")
     args <- c(flags, shQuote(path.expand(zipfile)),
               shQuote(files), extras)
+    if (sum(nchar(c(args, Sys.getenv()))) + length(args) > 8000) {
+        # -@ is supported in Info-ZIP from version 2.3 (like -X), but not in
+        # old Mac OS builds (MACOS macro), so better not rely on it for
+        # common use
+        #
+        # 8191 is the maximum command line length on Windows (since 2000/NT)
+        # and 8096 is the internal buffer size used by system() on systems
+        # without readline
+        args <- c(flags, "-@", shQuote(path.expand(zipfile)), extras)
+        input <- files
+    } else input <- NULL
+
     if (.Platform$OS.type == "windows")
-        invisible(system2(zip, args, invisible = TRUE))
-    else invisible(system2(zip, args))
+        invisible(system2(zip, args, input = input, invisible = TRUE))
+    else invisible(system2(zip, args, input = input))
 }
 
