@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
+ *  Copyright (C) 1997-2018 The R Core Team
  *  Copyright (C) 1995-1996 Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997-2015 The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -38,8 +38,7 @@
  *
  *  This module provides support for run-time loading of shared objects
  *  access to symbols within such objects via .C and .Fortran.  This is
- *  done under Unix with dlopen, dlclose and dlsym (the exception is
- *  hpux, where we use compatibility code provided by Luke Tierney).
+ *  done under Unix with dlopen, dlclose and dlsym.
  *  There are two cases:
  *
  *
@@ -63,7 +62,7 @@
  *  more recently loaded objects are found first.
  *
  *
- *  Accessing native routines in base (the R executable).
+ *  2. Accessing native routines in base (the R executable).
  *
  *  In this case, we use the registration mechanism and the DllInfo array
  *  in ../main/Rdynload.c to locate functions in the executable. We do this
@@ -71,8 +70,6 @@
  *  Note that the base routines registered are listed in
  *               ../main/registration.c
  *  and are registered during the initialization of the R engine.
- *  (This replaces the previous mechanism that built a table
- *  from ../appl/ROUTINES using Perl/sed).
  *
  *
  *  If speed is ever an issue in the lookup of registered symbols, we can
@@ -97,18 +94,9 @@
 #include <Rdynpriv.h>
 
 #ifdef Unix
-/* HP-UX 11.0 has dlfcn.h, but according to libtool as of Dec 2001
-   this support is broken. So we force use of shlib even when dlfcn.h
-   is available */
-# ifdef __hpux
-#  ifdef HAVE_DL_H
-#   define HAVE_DYNAMIC_LOADING
-#  endif
-# else
-#  ifdef HAVE_DLFCN_H
-#   define HAVE_DYNAMIC_LOADING
-#  endif
-# endif /* __hpux */
+# ifdef HAVE_DLFCN_H
+#  define HAVE_DYNAMIC_LOADING
+# endif
 # ifndef HAVE_NO_SYMBOL_UNDERSCORE
 #  ifdef HAVE_ELF_H
 #   define HAVE_NO_SYMBOL_UNDERSCORE
@@ -128,17 +116,14 @@ R_CPFun CPFun[MAX_CACHE];
 int nCPFun = 0;
 #endif
 
-/* Note that it is likely that dlopen will use up at least one file
-   descriptor for each DLL loaded (it may load further dynamically
-   linked libraries), so we do not want to get close to the fd limit
-   (which may be as low as 256). */
-#define MAX_NUM_DLLS	100
+static int MaxNumDLLs = 0; /* initialized in initLoadedDLL */
 
 static int CountDLL = 0;
 
 #include <R_ext/Rdynload.h>
 
-static DllInfo LoadedDLL[MAX_NUM_DLLS];
+/* Allocated in initLoadedDLL at R session start. Never free'd */
+static DllInfo* LoadedDLL = NULL;
 
 static int addDLL(char *dpath, char *name, HINSTANCE handle);
 static SEXP Rf_MakeDLLInfo(DllInfo *info);
@@ -153,19 +138,101 @@ attribute_hidden OSDynSymbol Rf_osDynSymbol;
 attribute_hidden OSDynSymbol *R_osDynSymbol = &Rf_osDynSymbol;
 
 void R_init_base(DllInfo *); /* In Registration.c */
-DL_FUNC R_dlsym(DllInfo *dll, char const *name,
-		R_RegisteredNativeSymbol *symbol);
+
+static void initLoadedDLL();
 
 void attribute_hidden
 InitDynload()
 {
-    DllInfo *dll;
+    initLoadedDLL();
     int which = addDLL(strdup("base"), "base", NULL);
-    dll = &LoadedDLL[which];
+    DllInfo *dll = &LoadedDLL[which];
     R_init_base(dll);
     InitFunctionHashing();
 }
 
+/* Allocate LoadedDLL. Errors are reported via R_Suicide, because this is
+   called too early during startup to use error(.) */
+static void initLoadedDLL()
+{
+    if (CountDLL != 0 || LoadedDLL != NULL)
+	R_Suicide("DLL table corruption detected"); /* not translated */
+
+    /* Note that it is likely that dlopen will use up at least one file
+       descriptor for each DLL loaded (it may load further dynamically
+       linked libraries), so we do not want to get close to the fd limit
+       (which may be as low as 256).
+
+       When R_MAX_NUM_DLLS environment variable is set and is in range
+       [100,1000] and the fd limit is sufficient or can be increased,
+       this becomes the maximum number of DLLs. Otherwise, R fails to start.
+
+       When R_MAX_NUM_DLLS is not set, R uses a reasonable default value
+       that matches the fd limit. R attempts to increase the limit if it
+       is too small. The goal for maximum number of DLLs is currently 614.
+
+       The limit receives increased attention with 'workflow'
+       documents which load increasingly more packages, hitting the
+       default fd limit of 256 on macOS systems.
+    */
+
+    char *req = getenv("R_MAX_NUM_DLLS");
+    if (req != NULL) {
+	/* set exactly the requested limit, or fail */
+	int reqlimit = atoi(req);
+	if (reqlimit < 100) {
+	    char msg[128];
+	    snprintf(msg, 128,
+	      _("R_MAX_NUM_DLLS must be at least %d"), 100);
+	    R_Suicide(msg);
+	}
+	if (reqlimit > 1000) {
+	    char msg[128];
+	    snprintf(msg, 128,
+	      _("R_MAX_NUM_DLLS cannot be bigger than %d"), 1000);
+	    R_Suicide(msg);
+	}
+	int needed_fds = (int)ceil(reqlimit / 0.6);
+	int fdlimit = R_EnsureFDLimit(needed_fds);
+	if (fdlimit < 0 && reqlimit > 100) {
+	    /* this is very unlikely */
+	    char msg[128];
+	    snprintf(msg, 128,
+	      _("R_MAX_NUM_DLLS cannot be bigger than %d when fd limit is not known"),
+	      100);
+	    R_Suicide(msg);
+	} else if (fdlimit >= 0 && fdlimit < needed_fds) {
+	    int maxdlllimit = (int) (0.6 * fdlimit);
+	    if (maxdlllimit < 100)
+		R_Suicide(_("the limit on the number of open files is too low"));
+	    char msg[128];
+	    snprintf(msg, 128,
+	      _("R_MAX_NUM_DLLS bigger than %d may exhaust open files limit"),
+	      maxdlllimit);
+	    R_Suicide(msg);
+	}
+	/* when fdlimit == -1 (not known), currently only reqlimit of 100 is
+	   allowed */
+	MaxNumDLLs = reqlimit;
+    } else {
+	/* set a reasonable default limit */
+	int needed_fds = 1024;
+	int fdlimit = R_EnsureFDLimit(needed_fds);
+	if (fdlimit < 0)
+	    MaxNumDLLs = 100;
+	else {
+	    MaxNumDLLs = (int) (0.6 * fdlimit);
+	    if (MaxNumDLLs < 100)
+		R_Suicide(_("the limit on the number of open files is too low"));
+	}
+    }
+
+    /* memory is set to zero */
+    LoadedDLL = (DllInfo *) calloc(MaxNumDLLs, sizeof(DllInfo));
+    if (LoadedDLL == NULL)
+	R_Suicide(_("could not allocate space for DLL table"));
+}
+
 /* returns DllInfo used by the embedding application.
    the underlying "(embedding)" entry is created if not present */
 DllInfo *R_getEmbeddingDllInfo()
@@ -216,7 +283,7 @@ R_addExternalRoutine(DllInfo *info,
 /*
  Returns a reference to the DllInfo object associated with the shared object
  with the path name `path'. This ensures uniqueness rather than having the
- undesirable situation of two object with the same name but in different
+ undesirable situation of two objects with the same name but in different
  directories.
  This is available so that it can be called from arbitrary C routines
  that need to call R_registerRoutines(). The initialization routine
@@ -318,19 +385,6 @@ R_setPrimitiveArgTypes(const R_FortranMethodDef * const croutine,
 
 }
 
-static void
-R_setArgStyles(const R_FortranMethodDef * const croutine,
-	       Rf_DotFortranSymbol *sym)
-{
-    sym->styles = (R_NativeArgStyle *)
-	malloc(sizeof(R_NativeArgStyle) * (size_t) croutine->numArgs);
-    if(!sym->styles)
-	error("allocation failure in R_setArgStyles");
-    if(sym->styles)
-	memcpy(sym->styles, croutine->styles,
-	       sizeof(R_NativeArgStyle) * (size_t) croutine->numArgs);
-}
-
 static void
 R_addFortranRoutine(DllInfo *info,
 		    const R_FortranMethodDef * const croutine,
@@ -341,8 +395,6 @@ R_addFortranRoutine(DllInfo *info,
     sym->numArgs = croutine->numArgs > -1 ? croutine->numArgs : -1;
     if(croutine->types)
 	R_setPrimitiveArgTypes(croutine, sym);
-    if(croutine->styles)
-	R_setArgStyles(croutine, sym);
 }
 
 static void
@@ -355,8 +407,6 @@ R_addExternalRoutine(DllInfo *info,
     sym->numArgs = croutine->numArgs > -1 ? croutine->numArgs : -1;
 }
 
-
-
 static void
 R_addCRoutine(DllInfo *info, const R_CMethodDef * const croutine,
 	      Rf_DotCSymbol *sym)
@@ -366,9 +416,6 @@ R_addCRoutine(DllInfo *info, const R_CMethodDef * const croutine,
     sym->numArgs = croutine->numArgs > -1 ? croutine->numArgs : -1;
     if(croutine->types)
 	R_setPrimitiveArgTypes(croutine, sym);
-    if(croutine->styles)
-	R_setArgStyles(croutine, sym);
-
 }
 
 static void
@@ -471,6 +518,7 @@ found:
     if(R_osDynSymbol->deleteCachedSymbols)
 	R_osDynSymbol->deleteCachedSymbols(&LoadedDLL[loc]);
 #endif
+    R_reinit_altrep_classes(&LoadedDLL[loc]);
     R_callDLLUnload(&LoadedDLL[loc]);
     R_osDynSymbol->closeLibrary(LoadedDLL[loc].handle);
     Rf_freeDllInfo(LoadedDLL+loc);
@@ -533,7 +581,7 @@ static DllInfo* AddDLL(const char *path, int asLocal, int now,
     DllInfo *info = NULL;
 
     DeleteDLL(path);
-    if(CountDLL == MAX_NUM_DLLS) {
+    if(CountDLL == MaxNumDLLs) {
 	strcpy(DLLerror, _("`maximal number of DLLs reached..."));
 	return NULL;
     }
@@ -582,13 +630,6 @@ static DllInfo *R_RegisterDLL(HINSTANCE handle, const char *path)
     char *dpath,  DLLname[PATH_MAX], *p;
     DllInfo *info;
 
-    info = &LoadedDLL[CountDLL];
-    /* default is to use old-style dynamic lookup.  The object's
-       initialization routine can limit access by setting this to FALSE.
-    */
-    info->useDynamicLookup = TRUE;
-    info->forceSymbols = FALSE;
-
     dpath = (char *) malloc(strlen(path)+1);
     if(dpath == NULL) {
 	strcpy(DLLerror, _("could not allocate space for 'path'"));
@@ -613,9 +654,16 @@ static DllInfo *R_RegisterDLL(HINSTANCE handle, const char *path)
     if(p > DLLname && strcmp(p, SHLIB_EXT) == 0) *p = '\0';
 #endif
 
-    addDLL(dpath, DLLname, handle);
-
-    return(info);
+    if (addDLL(dpath, DLLname, handle)) {
+	info = &LoadedDLL[CountDLL-1];
+	/* default is to use old-style dynamic lookup.  The object's
+	   initialization routine can limit access by setting this to FALSE.
+	*/
+	info->useDynamicLookup = TRUE;
+	info->forceSymbols = FALSE;
+	return info;
+    } else
+	return NULL;
 }
 
 static int
@@ -1315,6 +1363,8 @@ do_getSymbolInfo(SEXP call, SEXP op, SEXP args, SEXP env)
     SEXP sname = CAR(args), spackage = CADR(args),
 	withRegistrationInfo = CADDR(args);
 
+    if (!isString(sname) || LENGTH(sname) != 1)
+	error(_("invalid '%s' argument"), "name");
     name = translateChar(STRING_ELT(sname, 0));
     if(length(spackage)) {
 	if(TYPEOF(spackage) == STRSXP)
