@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1998-2017   The R Core Team
+ *  Copyright (C) 1998-2018   The R Core Team
  *  Copyright (C) 2002-2015   The R Foundation
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
  *
@@ -242,7 +242,7 @@ SEXP alloc3DArray(SEXPTYPE mode, int nrow, int ncol, int nface)
 	error(_("negative extents to 3D array"));
 #ifndef LONG_VECTOR_SUPPORT
     if ((double)nrow * (double)ncol * (double)nface > INT_MAX)
-	error(_("'alloc3Darray': too many elements specified"));
+	error(_("'alloc3DArray': too many elements specified"));
 #endif
     n = ((R_xlen_t) nrow) * ncol * nface;
     PROTECT(s = allocVector(mode, n));
@@ -261,11 +261,13 @@ SEXP allocArray(SEXPTYPE mode, SEXP dims)
     SEXP array;
     int i;
     R_xlen_t n = 1;
+#ifndef LONG_VECTOR_SUPPORT
     double dn = 1;
+#endif
 
     for (i = 0; i < LENGTH(dims); i++) {
-	dn *= INTEGER(dims)[i];
 #ifndef LONG_VECTOR_SUPPORT
+	dn *= INTEGER(dims)[i];
 	if(dn > INT_MAX)
 	    error(_("'allocArray': too many elements specified by 'dims'"));
 #endif
@@ -416,7 +418,7 @@ SEXP attribute_hidden do_drop(SEXP call, SEXP op, SEXP args, SEXP rho)
 	for (i = 0; i < n; i++)
 	    if (INTEGER(xdims)[i] == 1) shorten = 1;
 	if (shorten) {
-	    if (MAYBE_REFERENCED(x)) x = duplicate(x);
+	    if (MAYBE_REFERENCED(x)) x = R_duplicate_attr(x);
 	    x = DropDims(x);
 	}
     }
@@ -569,31 +571,32 @@ SEXP attribute_hidden do_lengths(SEXP call, SEXP op, SEXP args, SEXP rho)
 
 SEXP attribute_hidden do_rowscols(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
-    SEXP x, ans;
-    int i, j, nr, nc;
-
     checkArity(op, args);
-    /* This is the dimensions vector */
-    x = CAR(args);
-    if (!isInteger(x) || LENGTH(x) != 2)
+    SEXP dim = CAR(args);
+    int nprot = 0;
+    if (!isInteger(dim)) {
+	PROTECT(dim = coerceVector(dim, INTSXP)); nprot++;
+    }
+    if (LENGTH(dim) != 2)
 	error(_("a matrix-like object is required as argument to '%s'"),
 	      (PRIMVAL(op) == 2) ? "col" : "row");
 
-    nr = INTEGER(x)[0];
-    nc = INTEGER(x)[1];
+    int nr = INTEGER(dim)[0],
+	nc = INTEGER(dim)[1];
+    if(nprot) UNPROTECT(nprot);
 
-    ans = allocMatrix(INTSXP, nr, nc);
+    SEXP ans = allocMatrix(INTSXP, nr, nc);
 
     R_xlen_t NR = nr;
     switch (PRIMVAL(op)) {
-    case 1:
-	for (i = 0; i < nr; i++)
-	    for (j = 0; j < nc; j++)
+    case 1: // row() & .row()
+	for (int i = 0; i < nr; i++)
+	    for (int j = 0; j < nc; j++)
 		INTEGER(ans)[i + j * NR] = i + 1;
 	break;
-    case 2:
-	for (i = 0; i < nr; i++)
-	    for (j = 0; j < nc; j++)
+    case 2: // col() & .col()
+	for (int i = 0; i < nr; i++)
+	    for (int j = 0; j < nc; j++)
 		INTEGER(ans)[i + j * NR] = j + 1;
 	break;
     }
@@ -798,15 +801,15 @@ static void matprod(double *x, int nrx, int ncx,
 
     if (ncy == 1) /* matrix-vector or dot product */
 	F77_CALL(dgemv)(transN, &nrx, &ncx, &one, x,
-			&nrx, y, &ione, &zero, z, &ione);
+			&nrx, y, &ione, &zero, z, &ione FCONE);
     else if (nrx == 1) /* vector-matrix */
 	/* Instead of xY, compute (xY)^T == (Y^T)(x^T)
 	   The result is a vector, so transposing its content is no-op */
 	F77_CALL(dgemv)(transT, &nry, &ncy, &one, y,
-			&nry, x, &ione, &zero, z, &ione);
+			&nry, x, &ione, &zero, z, &ione FCONE);
     else /* matrix-matrix or outer product */
-	F77_CALL(dgemm)(transN, transN, &nrx, &ncy, &ncx, &one,
-			x, &nrx, y, &nry, &zero, z, &nrx);
+	F77_CALL(dgemm)(transN, transN, &nrx, &ncy, &ncx, &one, x, 
+			&nrx, y, &nry, &zero, z, &nrx FCONE FCONE);
 }
 
 static void internal_cmatprod(Rcomplex *x, int nrx, int ncx,
@@ -944,7 +947,7 @@ static void cmatprod(Rcomplex *x, int nrx, int ncx,
     one.r = 1.0; one.i = zero.r = zero.i = 0.0;
 
     F77_CALL(zgemm)(transa, transb, &nrx, &ncy, &ncx, &one,
-                    x, &nrx, y, &nry, &zero, z, &nrx);
+                    x, &nrx, y, &nry, &zero, z, &nrx FCONE FCONE);
 #endif
 }
 
@@ -981,7 +984,8 @@ static void symcrossprod(double *x, int nr, int nc, double *z)
     char *trans = "T", *uplo = "U";
     double one = 1.0, zero = 0.0;
 
-    F77_CALL(dsyrk)(uplo, trans, &nc, &nr, &one, x, &nr, &zero, z, &nc);
+    F77_CALL(dsyrk)(uplo, trans, &nc, &nr, &one, x, &nr, &zero, z, &nc
+		    FCONE FCONE);
     for (int i = 1; i < nc; i++)
 	for (int j = 0; j < i; j++) z[i + NC *j] = z[j + NC * i];
 }
@@ -1025,15 +1029,15 @@ static void crossprod(double *x, int nrx, int ncx,
 
     if (ncy == 1) /* matrix-vector or dot product */
 	F77_CALL(dgemv)(transT, &nrx, &ncx, &one, x,
-			&nrx, y, &ione, &zero, z, &ione);
+			&nrx, y, &ione, &zero, z, &ione FCONE);
     else if (ncx == 1) /* vector-matrix */
 	/* Instead of (x^T)Y, compute ((x^T)Y)^T == (Y^T)x
 	   The result is a vector, so transposing its content is no-op */
 	F77_CALL(dgemv)(transT, &nry, &ncy, &one, y,
-			&nry, x, &ione, &zero, z, &ione);
+			&nry, x, &ione, &zero, z, &ione FCONE);
     else /* matrix-matrix  or outer product */
 	F77_CALL(dgemm)(transT, transN, &ncx, &ncy, &nrx, &one,
-		        x, &nrx, y, &nry, &zero, z, &ncx);
+		        x, &nrx, y, &nry, &zero, z, &ncx FCONE FCONE);
 }
 
 static void ccrossprod(Rcomplex *x, int nrx, int ncx,
@@ -1079,7 +1083,7 @@ static void ccrossprod(Rcomplex *x, int nrx, int ncx,
     one.r = 1.0; one.i = zero.r = zero.i = 0.0;
 
     F77_CALL(zgemm)(transa, transb, &ncx, &ncy, &nrx, &one,
-                    x, &nrx, y, &nry, &zero, z, &ncx);
+                    x, &nrx, y, &nry, &zero, z, &ncx FCONE FCONE);
 #endif
 }
 
@@ -1116,7 +1120,8 @@ static void symtcrossprod(double *x, int nr, int nc, double *z)
     char *trans = "N", *uplo = "U";
     double one = 1.0, zero = 0.0;
 
-    F77_CALL(dsyrk)(uplo, trans, &nr, &nc, &one, x, &nr, &zero, z, &nr);
+    F77_CALL(dsyrk)(uplo, trans, &nr, &nc, &one, x, &nr, &zero, z, &nr
+		    FCONE FCONE);
     for (int i = 1; i < nr; i++)
 	for (int j = 0; j < i; j++) z[i + nr *j] = z[j + nr * i];
 }
@@ -1158,15 +1163,15 @@ static void tcrossprod(double *x, int nrx, int ncx,
 
     if (nry == 1) /* matrix-vector or dot product */
 	F77_CALL(dgemv)(transN, &nrx, &ncx, &one, x,
-			&nrx, y, &ione, &zero, z, &ione);
+			&nrx, y, &ione, &zero, z, &ione FCONE);
     else if (nrx == 1) /* vector-matrix */
 	/* Instead of x(Y^T), compute (x(Y^T))^T == Y(x^T)
 	   The result is a vector, so transposing its content is no-op */
 	F77_CALL(dgemv)(transN, &nry, &ncy, &one, y,
-			&nry, x, &ione, &zero, z, &ione);
+			&nry, x, &ione, &zero, z, &ione FCONE);
     else /* matrix-matrix or outer product */
 	F77_CALL(dgemm)(transN, transT, &nrx, &nry, &ncx, &one,
-		    x, &nrx, y, &nry, &zero, z, &nrx);
+			x, &nrx, y, &nry, &zero, z, &nrx FCONE FCONE);
 }
 
 static void tccrossprod(Rcomplex *x, int nrx, int ncx,
@@ -1211,7 +1216,7 @@ static void tccrossprod(Rcomplex *x, int nrx, int ncx,
     one.r = 1.0; one.i = zero.r = zero.i = 0.0;
 
     F77_CALL(zgemm)(transa, transb, &nrx, &nry, &ncx, &one,
-                    x, &nrx, y, &nry, &zero, z, &nrx);
+                    x, &nrx, y, &nry, &zero, z, &nrx FCONE FCONE);
 #endif
 }
 
@@ -2280,7 +2285,8 @@ SEXP attribute_hidden do_backsolve(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    memcpy(REAL(ans) + j*k, REAL(b) + j*nrb, (size_t)k *sizeof(double));
 	double one = 1.0;
 	F77_CALL(dtrsm)("L", upper ? "U" : "L", trans ? "T" : "N", "N",
-			&k, &ncb, &one, rr, &nrr, REAL(ans), &k);
+			&k, &ncb, &one, rr, &nrr, REAL(ans), &k
+			FCONE FCONE FCONE FCONE);
     }
     UNPROTECT(nprot);
     return ans;
