@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1997--2017  The R Core Team
+ *  Copyright (C) 1997--2018  The R Core Team
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -565,7 +565,7 @@ SEXP classgets(SEXP vec, SEXP klass)
 SEXP attribute_hidden do_classgets(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     checkArity(op, args);
-    check1arg(args, call, "x");
+    // have 2 args: check1arg(args, call, "x");
 
     if (MAYBE_SHARED(CAR(args))) SETCAR(args, shallow_duplicate(CAR(args)));
     if (length(CADR(args)) == 0) SETCADR(args, R_NilValue);
@@ -872,7 +872,7 @@ SEXP attribute_hidden do_namesgets(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP ans;
     checkArity(op, args);
-    check1arg(args, call, "x");
+    // 2 args ("x", "value")
 
     if (DispatchOrEval(call, op, "names<-", args, env, &ans, 0, 1))
 	return(ans);
@@ -882,7 +882,7 @@ SEXP attribute_hidden do_namesgets(SEXP call, SEXP op, SEXP args, SEXP env)
 	return CAR(args);
     PROTECT(args = ans);
     if (MAYBE_SHARED(CAR(args)))
-	SETCAR(args, shallow_duplicate(CAR(args)));
+	SETCAR(args, R_shallow_duplicate_attr(CAR(args)));
     if (TYPEOF(CAR(args)) == S4SXP) {
 	const char *klass = CHAR(STRING_ELT(R_data_class(CAR(args), FALSE), 0));
 	error(_("invalid to use names()<- on an S4 object of class '%s'"),
@@ -1012,12 +1012,12 @@ SEXP attribute_hidden do_dimnamesgets(SEXP call, SEXP op, SEXP args, SEXP env)
     SEXP ans;
 
     checkArity(op, args);
-    check1arg(args, call, "x");
-
+    // 2 args ("x", "value")
     if (DispatchOrEval(call, op, "dimnames<-", args, env, &ans, 0, 1))
 	return(ans);
     PROTECT(args = ans);
-    if (MAYBE_SHARED(CAR(args))) SETCAR(args, shallow_duplicate(CAR(args)));
+    if (MAYBE_SHARED(CAR(args)))
+	SETCAR(args, R_shallow_duplicate_attr(CAR(args)));
     setAttrib(CAR(args), R_DimNamesSymbol, CADR(args));
     UNPROTECT(1);
     SETTER_CLEAR_NAMED(CAR(args));
@@ -1270,8 +1270,7 @@ SEXP attribute_hidden do_levelsgets(SEXP call, SEXP op, SEXP args, SEXP env)
     SEXP ans;
 
     checkArity(op, args);
-    check1arg(args, call, "x");
-
+    // 2 args ("x", "value")
     if (DispatchOrEval(call, op, "levels<-", args, env, &ans, 0, 1))
 	/* calls, e.g., levels<-.factor() */
 	return(ans);
@@ -1300,7 +1299,6 @@ SEXP attribute_hidden do_attributesgets(SEXP call, SEXP op, SEXP args, SEXP env)
     /* Extract the arguments from the argument list */
 
     checkArity(op, args);
-    check1arg(args, call, "x");
 
     object = CAR(args);
     attrs = CADR(args);
@@ -1332,7 +1330,7 @@ SEXP attribute_hidden do_attributesgets(SEXP call, SEXP op, SEXP args, SEXP env)
 	   setting any attributes as an error later on would leave
 	   'obj' changed */
 	if (MAYBE_SHARED(object) || (MAYBE_REFERENCED(object) && nattrs))
-	    object = shallow_duplicate(object);
+	    object = R_shallow_duplicate_attr(object);
 	PROTECT(object);
     }
 
