@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1997-2018  The R Core Team
- *  Copyright (C) 2003-2018  The R Foundation
+ *  Copyright (C) 1997-2019  The R Core Team
+ *  Copyright (C) 2003-2019  The R Foundation
  *  Copyright (C) 1995,1996  Robert Gentleman, Ross Ihaka
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -26,6 +26,7 @@
 /* interval at which to check interrupts */
 #define NINTERRUPT 10000000
 
+#include <Parse.h>
 #include <Defn.h> /*-- Maybe modularize into own Coerce.h ..*/
 #include <Internal.h>
 #include <float.h> /* for DBL_DIG */
@@ -965,17 +966,10 @@ static SEXP coerceToPairList(SEXP v)
     return (ans);
 }
 
-/* Coerce a pairlist to the given type */
+/* Coerce a LISTSXP ('pairlist') _or_ LANGSXP to the given type */
 static SEXP coercePairList(SEXP v, SEXPTYPE type)
 {
-    int i, n=0;
-    SEXP rval= R_NilValue, vp, names;
-
-    /* Hmm, this is also called to LANGSXP, and coerceVector already
-       did the check of TYPEOF(v) == type */
-    if(type == LISTSXP) return v;/* IS pairlist */
-
-    names = v;
+    SEXP rval= R_NilValue, vp;
     if (type == EXPRSXP) {
 	PROTECT(rval = allocVector(type, 1));
 	SET_VECTOR_ELT(rval, 0, v);
@@ -983,7 +977,7 @@ static SEXP coercePairList(SEXP v, SEXPTYPE type)
 	return rval;
     }
     else if (type == STRSXP) {
-	n = length(v);
+	int i, n = length(v);
 	PROTECT(rval = allocVector(type, n));
 	for (vp = v, i = 0; vp != R_NilValue; vp = CDR(vp), i++) {
 	    if (isString(CAR(vp)) && length(CAR(vp)) == 1)
@@ -997,7 +991,7 @@ static SEXP coercePairList(SEXP v, SEXPTYPE type)
 	return rval;
     }
     else if (isVectorizable(v)) {
-	n = length(v);
+	int i, n = length(v);
 	PROTECT(rval = allocVector(type, n));
 	switch (type) {
 	case LGLSXP:
@@ -1025,19 +1019,22 @@ static SEXP coercePairList(SEXP v, SEXPTYPE type)
 	}
     }
     else
-	error(_("'pairlist' object cannot be coerced to type '%s'"),
-	      type2char(type));
+	error(_("'%s' object cannot be coerced to type '%s'"),
+	      type2char(TYPEOF(v)), type2char(type));
 
     /* If any tags are non-null then we */
     /* need to add a names attribute. */
-    for (vp = v, i = 0; vp != R_NilValue; vp = CDR(vp))
-	if (TAG(vp) != R_NilValue)
-	    i = 1;
+    Rboolean has_nms = FALSE;
+    for (vp = v; vp != R_NilValue; vp = CDR(vp))
+	if (TAG(vp) != R_NilValue) {
+	    has_nms = TRUE;
+	    break;
+	}
 
-    if (i) {
-	i = 0;
-	names = allocVector(STRSXP, n);
-	for (vp = v; vp != R_NilValue; vp = CDR(vp), i++)
+    if (has_nms) {
+	SEXP names = allocVector(STRSXP, length(v));
+	int i;
+	for (vp = v, i = 0; vp != R_NilValue; vp = CDR(vp), i++)
 	    if (TAG(vp) != R_NilValue)
 		SET_STRING_ELT(names, i, PRINTNAME(TAG(vp)));
 	setAttrib(rval, R_NamesSymbol, names);
@@ -1144,8 +1141,8 @@ static SEXP coerceVectorList(SEXP v, SEXPTYPE type)
 	}
     }
     else
-	error(_("(list) object cannot be coerced to type '%s'"),
-	      type2char(type));
+	error(_("'%s' object cannot be coerced to type '%s'"),
+	      "list", type2char(type));
 
     if (warn) CoercionWarning(warn);
     names = getAttrib(v, R_NamesSymbol);
@@ -1167,19 +1164,17 @@ static SEXP coerceSymbol(SEXP v, SEXPTYPE type)
     else if (type == STRSXP)
 	rval = ScalarString(PRINTNAME(v));
     else
-	warning(_("(symbol) object cannot be coerced to type '%s'"),
-		type2char(type));
+	warning(_("'%s' object cannot be coerced to type '%s'"),
+		"symbol", type2char(type));
     return rval;
 }
 
 SEXP coerceVector(SEXP v, SEXPTYPE type)
 {
-    SEXP op, vp, ans = R_NilValue;	/* -Wall */
-    int i,n;
-
     if (TYPEOF(v) == type)
 	return v;
 
+    SEXP ans = R_NilValue;	/* -Wall */
     if (ALTREP(v)) {
 	ans = ALTREP_COERCE(v, type);
 	if (ans) return ans;
@@ -1206,9 +1201,10 @@ SEXP coerceVector(SEXP v, SEXPTYPE type)
 	break;
     case NILSXP:
     case LISTSXP:
+	if(type == LISTSXP) return v; // as coercePairList() is also used for LANGSXP
 	ans = coercePairList(v, type);
 	break;
-    case LANGSXP:
+    case LANGSXP: {
 	if (type != STRSXP) {
 	    ans = coercePairList(v, type);
 	    break;
@@ -1217,15 +1213,15 @@ SEXP coerceVector(SEXP v, SEXPTYPE type)
 	/* This is mostly copied from coercePairList, but we need to
 	 * special-case the first element so as not to get operators
 	 * put in backticks. */
-	n = length(v);
+	int n = length(v);
 	PROTECT(ans = allocVector(type, n));
 	if (n == 0) {
 	    /* Can this actually happen? */
 	    UNPROTECT(1);
 	    break;
 	}
-	i = 0;
-	op = CAR(v);
+	int i = 0;
+	SEXP op = CAR(v);
 	/* The case of practical relevance is "lhs ~ rhs", which
 	 * people tend to split using as.character(), modify, and
 	 * paste() back together. However, we might as well
@@ -1239,7 +1235,7 @@ SEXP coerceVector(SEXP v, SEXPTYPE type)
 	/* The distinction between strings and other elements was
 	 * here "always", but is really dubious since it makes x <- a
 	 * and x <- "a" come out identical. Won't fix just now. */
-	for (vp = v;  vp != R_NilValue; vp = CDR(vp), i++) {
+	for (SEXP vp = v;  vp != R_NilValue; vp = CDR(vp), i++) {
 	    if (isString(CAR(vp)) && length(CAR(vp)) == 1)
 		SET_STRING_ELT(ans, i, STRING_ELT(CAR(vp), 0));
 	    else
@@ -1247,6 +1243,7 @@ SEXP coerceVector(SEXP v, SEXPTYPE type)
 	}
 	UNPROTECT(1);
 	break;
+    }
     case VECSXP:
     case EXPRSXP:
 	ans = coerceVectorList(v, type);
@@ -1608,27 +1605,61 @@ SEXP attribute_hidden do_asfunction(SEXP call, SEXP op, SEXP args, SEXP rho)
 }
 
 
+/* primitive,
+ * op = 0 : str2lang(s)
+ * op = 1 : str2expression(text) */
+SEXP attribute_hidden do_str2lang(SEXP call, SEXP op, SEXP args, SEXP rho) {
+    checkArity(op, args);
+    // check1arg(args, call, "s");
+    args = CAR(args);
+    if(TYPEOF(args) != STRSXP)
+	errorcall(call, _("argument must be character"));
+
+    Rboolean to_lang = !PRIMVAL(op); // op = 0: character *string* to call-like
+    if(to_lang) {
+	if(LENGTH(args) != 1)
+	    errorcall(call, _("argument must be a character string"));
+    // basically parse(text = "...."), for str2lang() '[[1]]' :
+    } else // str2expression()
+	if(!LENGTH(args))
+	    return(allocVector(EXPRSXP, 0));
+
+    ParseStatus status;
+    SEXP srcfile = PROTECT(mkString("<text>"));
+    SEXP ans = PROTECT(R_ParseVector(args, -1, &status, srcfile));
+    if (status != PARSE_OK) parseError(call, R_ParseError);
+    if(to_lang) {
+	if(LENGTH(ans) != 1) // never? happens
+	    errorcall(call, _("parsing result not of length one, but %d"), LENGTH(ans));
+	ans = VECTOR_ELT(ans, 0);
+    }
+    UNPROTECT(2);
+    return ans;
+}
+
 /* primitive */
 SEXP attribute_hidden do_ascall(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
-    SEXP ap, ans, names;
-    int i, n;
-
     checkArity(op, args);
     check1arg(args, call, "x");
 
+    SEXP ans;
+    if (DispatchOrEval(call, op, "as.call", args, rho, &ans, 0, 1))
+	return(ans);
+
     args = CAR(args);
     switch (TYPEOF(args)) {
     case LANGSXP:
 	ans = args;
 	break;
     case VECSXP:
-    case EXPRSXP:
-	if(0 == (n = length(args)))
+    case EXPRSXP: {
+	int n = length(args);
+	if(n == 0)
 	    errorcall(call, _("invalid length 0 argument"));
-	PROTECT(names = getAttrib(args, R_NamesSymbol));
+	SEXP names = PROTECT(getAttrib(args, R_NamesSymbol)), ap;
 	PROTECT(ap = ans = allocList(n));
-	for (i = 0; i < n; i++) {
+	for (int i = 0; i < n; i++) {
 	    SETCAR(ap, VECTOR_ELT(args, i));
 	    if (names != R_NilValue && !StringBlank(STRING_ELT(names, i)))
 		SET_TAG(ap, installTrChar(STRING_ELT(names, i)));
@@ -1636,11 +1667,12 @@ SEXP attribute_hidden do_ascall(SEXP call, SEXP op, SEXP args, SEXP rho)
 	}
 	UNPROTECT(2); /* ap, names */
 	break;
+    }
     case LISTSXP:
 	ans = duplicate(args);
 	break;
     case STRSXP:
-	errorcall(call, _("as.call(<character string>)  not yet implemented"));
+	errorcall(call, _("as.call(<character>) not feasible; consider str2lang(<char.>)"));
 	break;
     default:
 	errorcall(call, _("invalid argument list"));
@@ -1652,14 +1684,27 @@ SEXP attribute_hidden do_ascall(SEXP call, SEXP op, SEXP args, SEXP rho)
 }
 
 
-/* int, not Rboolean, for NA_LOGICAL : */
-int asLogical(SEXP x)
+/* call and rho are only needed for _R_CHECK_LENGTH_1_LOGIC2_ checking
+       and diagnostics; to be removed if length>1 value is turned to error */
+/* return int, not Rboolean, for NA_LOGICAL : */
+int asLogical2(SEXP x, int checking, SEXP call, SEXP rho)
 {
     int warn = 0;
 
     if (isVectorAtomic(x)) {
 	if (XLENGTH(x) < 1)
 	    return NA_LOGICAL;
+	if (checking && XLENGTH(x) > 1) {
+	    char msg[128];
+	    snprintf(msg, 128, _("'length(x) = %lld > 1' in coercion to '%s'"),
+		    (long long) XLENGTH(x), "logical(1)");
+	    R_BadValueInRCode(x, call, rho,
+		"length > 1 in coercion to logical",
+		msg,
+		msg,
+		"_R_CHECK_LENGTH_1_LOGIC2_",
+		FALSE /* by default do nothing */);
+	}
 	switch (TYPEOF(x)) {
 	case LGLSXP:
 	    return LOGICAL_ELT(x, 0);
@@ -1682,6 +1727,12 @@ int asLogical(SEXP x)
     return NA_LOGICAL;
 }
 
+int asLogical(SEXP x)
+{
+    return asLogical2(x, /* checking = */ 0, R_NilValue, R_NilValue);
+}
+
+
 int asInteger(SEXP x)
 {
     int warn = 0, res;
@@ -1715,6 +1766,38 @@ int asInteger(SEXP x)
     return NA_INTEGER;
 }
 
+R_xlen_t asXLength(SEXP x)
+{
+    const R_xlen_t na = -999; /* any negative number should do */
+
+    if (isVectorAtomic(x) && XLENGTH(x) >= 1) {
+	switch (TYPEOF(x)) {
+	case INTSXP:
+	{
+	    int res = INTEGER_ELT(x, 0);
+	    if (res == NA_INTEGER)
+		return na;
+	    else
+		return (R_xlen_t) res;
+	}
+	case LGLSXP:
+	case REALSXP:
+	case CPLXSXP:
+	case STRSXP:
+	    break;
+	default:
+	    UNIMPLEMENTED_TYPE("asXLength", x);
+	}
+    } else if(TYPEOF(x) != CHARSXP)
+	return na;
+
+    double d = asReal(x);
+    if (!R_FINITE(d) || d > R_XLEN_T_MAX || d < 0)
+	return na;
+    else
+	return (R_xlen_t) d;
+}
+
 double asReal(SEXP x)
 {
     int warn = 0;
@@ -2517,7 +2600,7 @@ SEXP attribute_hidden do_docall(SEXP call, SEXP op, SEXP args, SEXP rho)
 
 #ifdef __maybe_in_the_future__
     if (!isNull(args) && !isVectorList(args))
-	error(_("'args' must be a list or expression"));
+	error(_("'%s' must be a list or expression"), "args");
 #else
     if (!isNull(args) && !isNewList(args))
 	error(_("'%s' must be a list"), "args");
@@ -2788,18 +2871,14 @@ static SEXP R_set_class(SEXP obj, SEXP value, SEXP call)
 	  do_unsetS4(obj, value);
     }
     else if(length(value) == 0) {
-	UNPROTECT(nProtect); nProtect = 0;
 	error(_("invalid replacement object to be a class string"));
     }
     else {
-	const char *valueString;
-	int whichType;
-
-	SEXP cur_class; SEXPTYPE valueType;
-	valueString = CHAR(asChar(value)); /* ASCII */
-	whichType = class2type(valueString);
-	valueType = (whichType == -1) ? (SEXPTYPE) -1 : classTable[whichType].sexp;
-	PROTECT(cur_class = R_data_class(obj, FALSE)); nProtect++;
+	const char *valueString = CHAR(asChar(value)); /* ASCII */
+	int whichType = class2type(valueString);
+	SEXPTYPE valueType = (whichType == -1) ? (SEXPTYPE) -1
+	    : classTable[whichType].sexp;
+	// SEXP cur_class = PROTECT(R_data_class(obj, FALSE)); nProtect++;
 	/*  assigning type as a class deletes an explicit class attribute. */
 	if(valueType != (SEXPTYPE)-1) {
 	    setAttrib(obj, R_ClassSymbol, R_NilValue);
