@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2016  The R Core Team.
+ *  Copyright (C) 1997--2018  The R Core Team.
  *  Copyright (C) 2003--2016  The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -47,13 +47,13 @@
 
 /* this is just for conformity with other types */
 attribute_hidden
-void formatRaw(Rbyte *x, R_xlen_t n, int *fieldwidth)
+void formatRaw(const Rbyte *x, R_xlen_t n, int *fieldwidth)
 {
     *fieldwidth = 2;
 }
 
 attribute_hidden
-void formatString(SEXP *x, R_xlen_t n, int *fieldwidth, int quote)
+void formatString(const SEXP *x, R_xlen_t n, int *fieldwidth, int quote)
 {
     int xmax = 0;
     int l;
@@ -67,7 +67,7 @@ void formatString(SEXP *x, R_xlen_t n, int *fieldwidth, int quote)
     *fieldwidth = xmax;
 }
 
-void formatLogical(int *x, R_xlen_t n, int *fieldwidth)
+void formatLogical(const int *x, R_xlen_t n, int *fieldwidth)
 {
     *fieldwidth = 1;
     for(R_xlen_t i = 0 ; i < n; i++) {
@@ -84,7 +84,7 @@ void formatLogical(int *x, R_xlen_t n, int *fieldwidth)
     }
 }
 
-void formatInteger(int *x, R_xlen_t n, int *fieldwidth)
+void formatInteger(const int *x, R_xlen_t n, int *fieldwidth)
 {
     int xmin = INT_MAX, xmax = INT_MIN, naflag = 0;
     int l;
@@ -122,11 +122,12 @@ void formatInteger(int *x, R_xlen_t n, int *fieldwidth)
  * Using GLOBAL	 R_print.digits	 -- had	 #define MAXDIG R_print.digits
 */
 
-/* long double is C99, so should always be defined but may be slow */
+/*  Very likely everyone has nearbyintl now (2018), but it took until
+    2012 for FreeBSD to get it, and longer for Cygwin.
+*/
 #if defined(HAVE_LONG_DOUBLE) && (SIZEOF_LONG_DOUBLE > SIZEOF_DOUBLE)
 # ifdef HAVE_NEARBYINTL
 # define R_nearbyintl nearbyintl
-/* Cygwin had rintl but not nearbyintl */
 # elif defined(HAVE_RINTL)
 # define R_nearbyintl rintl
 # else
@@ -144,15 +145,6 @@ LDOUBLE private_nearbyintl(LDOUBLE x)
     }
 }
 # endif
-# else /* no long double */
-# ifdef HAVE_NEARBYINT
-#  define R_nearbyint nearbyint
-# elif defined(HAVE_RINT)
-#  define R_nearbyint rint
-# else
-#  define R_nearbyint private_rint
-#  include "nmath2.h" // for private_rint
-# endif
 #endif
 
 #define NB 1000
@@ -190,7 +182,7 @@ static const double tbl[] =
 #endif
 
 static void
-scientific(double *x, int *neg, int *kpower, int *nsig, Rboolean *roundingwidens)
+scientific(const double *x, int *neg, int *kpower, int *nsig, Rboolean *roundingwidens)
 {
     /* for a number x , determine
      *	neg    = 1_{x < 0}  {0/1}
@@ -229,6 +221,7 @@ scientific(double *x, int *neg, int *kpower, int *nsig, Rboolean *roundingwidens
             if (kp > 0) r_prec /= tbl[kp+1]; else if (kp < 0) r_prec *= tbl[ -kp+1];
         }
 #ifdef HAVE_POWL
+	// powl is C99 but only added to FreeBSD in 2017.
 	else
             r_prec /= powl(10.0, (long double) kp);
 #else
@@ -245,13 +238,13 @@ scientific(double *x, int *neg, int *kpower, int *nsig, Rboolean *roundingwidens
 	   accuracy limited by double rounding problem,
 	   alpha already rounded to 64 bits */
         alpha = (double) R_nearbyintl(r_prec);
-#else
+#else /* not using long doubles */
 	double r_prec = r;
         /* use exact scaling factor in double precision, if possible */
         if (abs(kp) <= 22) {
             if (kp >= 0) r_prec /= tbl[kp+1]; else r_prec *= tbl[ -kp+1];
         }
-        /* on IEEE 1e-308 is not representable except by gradual underflow.
+        /* For IEC60559 1e-308 is not representable except by gradual underflow.
            Shifting by 303 allows for any potential denormalized numbers x,
            and makes the reasonable assumption that R_dec_min_exponent+303
            is in range. Representation of 1e+303 has low error.
@@ -267,7 +260,7 @@ scientific(double *x, int *neg, int *kpower, int *nsig, Rboolean *roundingwidens
         /* round alpha to integer, 10^(digits-1) <= alpha <= 10^digits */
         /* accuracy limited by double rounding problem,
 	   alpha already rounded to 53 bits */
-        alpha = R_nearbyint(r_prec);
+        alpha = nearbyint(r_prec);
 #endif
         *nsig = R_print.digits;
         for (j = 1; j <= R_print.digits; j++) {
@@ -309,7 +302,7 @@ scientific(double *x, int *neg, int *kpower, int *nsig, Rboolean *roundingwidens
    it is 0 except when called from do_format.
 */
 
-void formatReal(double *x, R_xlen_t n, int *w, int *d, int *e, int nsmall)
+void formatReal(const double *x, R_xlen_t n, int *w, int *d, int *e, int nsmall)
 {
     int left, right, sleft;
     int mnl, mxl, rgt, mxsl, mxns, wF;
@@ -394,11 +387,12 @@ void formatReal(double *x, R_xlen_t n, int *w, int *d, int *e, int nsmall)
     if (neginf && *w < 4) *w = 4;
 }
 
+/*   From complex.c. */
+void z_prec_r(Rcomplex *r, const Rcomplex *x, double digits);
+
 /* As from 2.2.0 the number of digits applies to real and imaginary parts
    together, not separately */
-void z_prec_r(Rcomplex *r, Rcomplex *x, double digits);
-
-void formatComplex(Rcomplex *x, R_xlen_t n, int *wr, int *dr, int *er,
+void formatComplex(const Rcomplex *x, R_xlen_t n, int *wr, int *dr, int *er,
 		   int *wi, int *di, int *ei, int nsmall)
 {
 /* format.info() for  x[1..n] for both Re & Im */
