@@ -1,7 +1,7 @@
 /*
- *  R : A Computer Langage for Statistical Data Analysis
+ *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998-2014   The R Core Team.
+ *  Copyright (C) 1998-2017   The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -221,19 +221,28 @@ SEXP attribute_hidden matchArgs(SEXP formals, SEXP supplied, SEXP call)
     a = actuals;
     arg_i = 0;
     while (f != R_NilValue) {
-	if (TAG(f) != R_DotsSymbol) {
+      SEXP ftag = TAG(f);
+      const char *ftag_name = CHAR(PRINTNAME(ftag));
+      if (ftag != R_DotsSymbol && ftag != R_NilValue) {
 	    for (b = supplied, i = 1; b != R_NilValue; b = CDR(b), i++) {
-		if (TAG(b) != R_NilValue && pmatch(TAG(f), TAG(b), /*exact*/ TRUE)) {
-		    if (fargused[arg_i] == 2)
-			error(_("formal argument \"%s\" matched by multiple actual arguments"),
-			      CHAR(PRINTNAME(TAG(f))));
-		    if (ARGUSED(b) == 2)
-			error(_("argument %d matches multiple formal arguments"), i);
-		    SETCAR(a, CAR(b));
-		    if(CAR(b) != R_MissingArg) SET_MISSING(a, 0);
-		    SET_ARGUSED(b, 2);
-		    fargused[arg_i] = 2;
-		}
+	      SEXP btag = TAG(b);
+	      if (btag != R_NilValue) {
+		  const char *btag_name = CHAR(PRINTNAME(btag));
+		  if (streql( ftag_name, btag_name )) {
+		      if (fargused[arg_i] == 2)
+			  errorcall(call,
+	                      _("formal argument \"%s\" matched by multiple actual arguments"),
+	                      CHAR(PRINTNAME(TAG(f))));
+		      if (ARGUSED(b) == 2)
+			  errorcall(call,
+	                      _("argument %d matches multiple formal arguments"),
+                              i);
+		      SETCAR(a, CAR(b));
+		      if(CAR(b) != R_MissingArg) SET_MISSING(a, 0);
+		      SET_ARGUSED(b, 2);
+		      fargused[arg_i] = 2;
+		  }
+	      }
 	    }
 	}
 	f = CDR(f);
@@ -261,10 +270,12 @@ SEXP attribute_hidden matchArgs(SEXP formals, SEXP supplied, SEXP call)
 		    if (ARGUSED(b) != 2 && TAG(b) != R_NilValue &&
 			pmatch(TAG(f), TAG(b), seendots)) {
 			if (ARGUSED(b))
-			    error(_("argument %d matches multiple formal arguments"), i);
+			    errorcall(call,
+				_("argument %d matches multiple formal arguments"), i);
 			if (fargused[arg_i] == 1)
-			    error(_("formal argument \"%s\" matched by multiple actual arguments"),
-				  CHAR(PRINTNAME(TAG(f))));
+			    errorcall(call,
+				_("formal argument \"%s\" matched by multiple actual arguments"),
+				CHAR(PRINTNAME(TAG(f))));
 			if (R_warn_partial_match_args) {
 			    warningcall(call,
 					_("partial argument match of '%s' to '%s'"),
@@ -387,6 +398,22 @@ SEXP attribute_hidden matchArgs(SEXP formals, SEXP supplied, SEXP call)
     return(actuals);
 }
 
+/* Use matchArgs_RC if the result might escape into R. */
+SEXP attribute_hidden matchArgs_RC(SEXP formals, SEXP supplied, SEXP call)
+{
+    SEXP args = matchArgs(formals, supplied, call);
+    /* it would be better not to build this arglist with CONS_NR in
+       the first place */
+    for (SEXP a = args; a  != R_NilValue; a = CDR(a)) {
+	if (! TRACKREFS(a)) {
+	    ENABLE_REFCNT(a);
+	    INCREMENT_REFCNT(CAR(a));
+	    INCREMENT_REFCNT(CDR(a));
+	}
+    }
+    return args;
+}
+
 
 /* patchArgsByActuals - patch promargs (given as 'supplied') to be promises
    for the respective actuals in the given environment 'cloenv'.  This is
