@@ -106,7 +106,7 @@ SEXP attribute_hidden do_paste(SEXP call, SEXP op, SEXP args, SEXP env)
 	    /* formerly in R code: moved to C for speed */
 	    SEXP call, xj = VECTOR_ELT(x, j);
 	    if(OBJECT(xj)) { /* method dispatch */
-		PROTECT(call = lang2(install("as.character"), xj));
+		PROTECT(call = lang2(R_AsCharacterSymbol, xj));
 		SET_VECTOR_ELT(x, j, eval(call, env));
 		UNPROTECT(1);
 	    } else if (isSymbol(xj))
@@ -309,7 +309,7 @@ SEXP attribute_hidden do_filepath(SEXP call, SEXP op, SEXP args, SEXP env)
 	    /* formerly in R code: moved to C for speed */
 	    SEXP call, xj = VECTOR_ELT(x, j);
 	    if(OBJECT(xj)) { /* method dispatch */
-		PROTECT(call = lang2(install("as.character"), xj));
+		PROTECT(call = lang2(R_AsCharacterSymbol, xj));
 		SET_VECTOR_ELT(x, j, eval(call, env));
 		UNPROTECT(1);
 	    } else if (isSymbol(xj))
