@@ -1,8 +1,8 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
+ *  Copyright (C) 1997--2018  The R Core Team
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2015  The R Core Team
- *  Copyright (C) 2003--2008  The R Foundation
+ *  Copyright (C) 2003--2018  The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -382,9 +382,16 @@ walker_ProbSampleReplace(int n, double *p, int *a, int nans, int *ans)
     for (i = 0; i < n; i++) q[i] += i;
 
     /* generate sample */
+    Sampletype Sample_kind = R_sample_kind();
     for (i = 0; i < nans; i++) {
-	rU = unif_rand() * n;
-	k = (int) rU;
+	if (Sample_kind == ROUNDING) {
+	    rU = unif_rand() * n;
+	    k = (int) rU;
+	}
+	else {
+	    k = (int) R_unif_index(n);
+	    rU = k + unif_rand();
+	}
 	ans[i] = (rU < q[k]) ? k+1 : a[k]+1;
     }
     if(n > SMALL) {
@@ -528,7 +535,7 @@ SEXP attribute_hidden do_sample(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    int *iy = INTEGER(y);
 	    /* avoid allocation for a single sample */
 	    if (replace || k < 2) {
-		for (int i = 0; i < k; i++) iy[i] = (int)(R_unif_index(dn) + 1);
+		for (int i = 0; i < k; i++) iy[i] = (int)(R_unif_index(n) + 1);
 	    } else {
 		int *x = (int *)R_alloc(n, sizeof(int));
 		for (int i = 0; i < n; i++) x[i] = i;
