@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2005-2015   The R Core Team
+ *  Copyright (C) 2005-2017   The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -152,7 +152,7 @@ static cjk_locale_name_t cjk_locale_name[] = {
 };
 
 // used in character.c, ../gnuwin32/console.c , ../library/grDevices/src/devP*.c :
-int Ri18n_wcwidth(wchar_t c)
+int Ri18n_wcwidth(Rwchar_t c)
 {
     char lc_str[128];
     unsigned int i, j;
@@ -161,7 +161,7 @@ int Ri18n_wcwidth(wchar_t c)
     static int lc = 0;
 
     if (0 != strcmp(setlocale(LC_CTYPE, NULL), lc_cache)) {
-	strncpy(lc_str, setlocale(LC_CTYPE, NULL), sizeof(lc_str));
+	strncpy(lc_str, setlocale(LC_CTYPE, NULL), sizeof(lc_str) - 1);
         lc_str[sizeof(lc_str) - 1] = '\0';
 	for (i = 0, j = (int) strlen(lc_str); i < j && i < sizeof(lc_str); i++)
 	    lc_str[i] = (char) toupper(lc_str[i]);
