@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1998-2017  The R Core Team.
+ *  Copyright (C) 1998-2018  The R Core Team.
  *  Copyright (C) 1995-1998  Robert Gentleman and Ross Ihaka
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -220,7 +220,6 @@ static SEXP rep2(SEXP s, SEXP ncopy)
 	    SEXP elt = lazy_duplicate(VECTOR_ELT(s, i)); \
 	    for (j = (R_xlen_t) it[i]; j > 0; j--) \
 		SET_VECTOR_ELT(a, n++, elt); \
-	    if (j > 1) ENSURE_NAMEDMAX(elt); \
 	} \
 	break; \
     case RAWSXP: \
@@ -347,6 +346,9 @@ SEXP attribute_hidden do_rep_int(SEXP call, SEXP op, SEXP args, SEXP rho)
     R_xlen_t nc;
     SEXP a;
 
+    if (DispatchOrEval(call, op, "rep.int", args, rho, &a, 0, 0))
+      return(a);
+
     if (!isVector(ncopy))
 	error(_("invalid type (%s) for '%s' (must be a vector)"),
 	      type2char(TYPEOF(ncopy)), "times");
@@ -403,6 +405,10 @@ SEXP attribute_hidden do_rep_len(SEXP call, SEXP op, SEXP args, SEXP rho)
     SEXP a, s, len;
 
     checkArity(op, args);
+
+    if (DispatchOrEval(call, op, "rep_len", args, rho, &a, 0, 0))
+      return(a);
+
     s = CAR(args);
 
     if (!isVector(s) && s != R_NilValue)
@@ -524,8 +530,9 @@ static SEXP rep4(SEXP x, SEXP times, R_xlen_t len, R_xlen_t each, R_xlen_t nt)
 	for(i = 0, k = 0, k2 = 0; i < lx; i++) {			\
 	    /*		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();*/ \
 	    for(j = 0, sum = 0; j < each; j++) sum += (R_xlen_t) itimes[k++]; \
+	    SEXP elt = lazy_duplicate(VECTOR_ELT(x, i));		\
 	    for(k3 = 0; k3 < sum; k3++) {				\
-		SET_VECTOR_ELT(a, k2++, VECTOR_ELT(x, i));		\
+		SET_VECTOR_ELT(a, k2++, elt);				\
 		if(k2 == len) goto done;				\
 	    }								\
 	}								\
@@ -580,7 +587,8 @@ static SEXP rep4(SEXP x, SEXP times, R_xlen_t len, R_xlen_t each, R_xlen_t nt)
 	case EXPRSXP:
 	    for(i = 0; i < len; i++) {
 //		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
-		SET_VECTOR_ELT(a, i, VECTOR_ELT(x, (i/each) % lx));
+		SEXP elt = lazy_duplicate(VECTOR_ELT(x, (i/each) % lx));
+		SET_VECTOR_ELT(a, i, elt);
 	    }
 	    break;
 	case RAWSXP:
@@ -889,8 +897,7 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
 		   reduced below 1/INT_MAX this is the same as the
 		   next, so this is future-proofing against longer integers.
 		*/
-		/* seq.default gives integer result from
-		   from + (0:n)*by
+		/* as seq.default also returns integer for from + (0:n)*by
 		*/
 		nn = (R_xlen_t) n;
 		ans = allocVector(INTSXP, nn+1);
