@@ -207,7 +207,7 @@ static int defaultSerializeVersion()
 	if (val == 2 || val == 3)
 	    dflt = val;
 	else
-	    dflt = 2; /* the default */
+	    dflt = 3; /* the default */
     }
     return dflt;
 }
@@ -575,6 +575,8 @@ static void OutFormat(R_outpstream_t stream)
     case R_pstream_ascii_format:
     case R_pstream_asciihex_format:
 	stream->OutBytes(stream, "A\n", 2); break;
+	/* on deserialization, asciihex_format is treated exactly the same
+	   way as ascii_format; the distinction is handled inside scanf %lg */
     case R_pstream_binary_format: stream->OutBytes(stream, "B\n", 2); break;
     case R_pstream_xdr_format:    stream->OutBytes(stream, "X\n", 2); break;
     case R_pstream_any_format:
@@ -589,7 +591,7 @@ static void InFormat(R_inpstream_t stream)
     R_pstream_format_t type;
     stream->InBytes(stream, buf, 2);
     switch (buf[0]) {
-    case 'A': type = R_pstream_ascii_format; break;
+    case 'A': type = R_pstream_ascii_format; break; /* also for asciihex */
     case 'B': type = R_pstream_binary_format; break;
     case 'X': type = R_pstream_xdr_format; break;
     case '\n':
@@ -1006,8 +1008,20 @@ static void WriteItem (SEXP s, SEXP ref_table, R_outpstream_t stream)
     SEXP t;
 
     if (R_compile_pkgs && TYPEOF(s) == CLOSXP && TYPEOF(BODY(s)) != BCODESXP &&
-        !R_disable_bytecode) {
+        !R_disable_bytecode &&
+        (!IS_S4_OBJECT(s) || (!inherits(s, "refMethodDef") &&
+                              !inherits(s, "defaultBindingFunction")))) {
 
+	/* Do not compile reference class methods in their generators, because
+	   the byte-code is dropped as soon as the method is installed into a
+	   new environment. This is a performance optimization but it also
+	   prevents byte-compiler warnings about no visible binding for super
+	   assignment to a class field.
+
+	   Do not compile default binding functions, because the byte-code is
+	   dropped as fields are set in constructors (just an optimization).
+	*/
+	    
 	SEXP new_s;
 	R_compile_pkgs = FALSE;
 	PROTECT(new_s = R_cmpfun1(s));
@@ -2135,7 +2149,7 @@ SEXP R_Unserialize(R_inpstream_t stream)
 	char nbuf[nelen + 1];
 	InString(stream, nbuf, nelen);
 	nbuf[nelen] = '\0';
-	nelen = nelen < (R_CODESET_MAX + 1) ? nelen : (R_CODESET_MAX + 1);
+	nelen = nelen < (R_CODESET_MAX + 1) ? nelen : R_CODESET_MAX;
 	strncpy(stream->native_encoding, nbuf, nelen);
 	stream->native_encoding[nelen] = '\0';
 	break;
