@@ -809,7 +809,7 @@ static SEXP
 stringSubscript(SEXP s, R_xlen_t ns, R_xlen_t nx, SEXP names,
 		R_xlen_t *stretch, SEXP call)
 {
-    SEXP indx, indexnames;
+    SEXP indx, indexnames = R_NilValue;
     R_xlen_t i, j, nnames, extra, sub;
     int canstretch = *stretch > 0;
     /* product may overflow, so check factors as well. */
@@ -817,7 +817,6 @@ stringSubscript(SEXP s, R_xlen_t ns, R_xlen_t nx, SEXP names,
 
     PROTECT(s);
     PROTECT(names);
-    PROTECT(indexnames = allocVector(VECSXP, ns));
     nnames = nx;
     extra = nnames;
 
@@ -837,8 +836,7 @@ stringSubscript(SEXP s, R_xlen_t ns, R_xlen_t nx, SEXP names,
 	int *pindx = INTEGER(indx);
 	for (i = 0; i < ns; i++)
 	    if(STRING_ELT(s, i) == NA_STRING || !CHAR(STRING_ELT(s, i))[0])
-		pindx[i] = 0;
-	for (i = 0; i < ns; i++) SET_VECTOR_ELT(indexnames, i, R_NilValue);
+		pindx[i] = 0;	
     } else {
 	PROTECT(indx = allocVector(INTSXP, ns));
 	int *pindx = INTEGER(indx);
@@ -849,7 +847,6 @@ stringSubscript(SEXP s, R_xlen_t ns, R_xlen_t nx, SEXP names,
 		    SEXP names_j = STRING_ELT(names, j);
 		    if (NonNullStringMatch(STRING_ELT(s, i), names_j)) {
 			sub = j + 1;
-			SET_VECTOR_ELT(indexnames, i, R_NilValue);
 			break;
 		    }
 		}
@@ -859,15 +856,21 @@ stringSubscript(SEXP s, R_xlen_t ns, R_xlen_t nx, SEXP names,
     }
 
     int *pindx = INTEGER(indx);
+    SEXP sindx = NULL;
     for (i = 0; i < ns; i++) {
 	sub = pindx[i];
 	if (sub == 0) {
-	    for (j = 0 ; j < i ; j++)
-		if (NonNullStringMatch(STRING_ELT(s, i), STRING_ELT(s, j))) {
-		    sub = pindx[j];
-		    SET_VECTOR_ELT(indexnames, i, STRING_ELT(s, j));
-		    break;
-		}
+	    if (sindx == NULL) {
+		sindx = PROTECT(match(s, s, 0));
+		indexnames = PROTECT(allocVector(VECSXP, ns));
+		for (int z = 0; z < ns; z++)
+		    SET_VECTOR_ELT(indexnames, z, R_NilValue);
+	    }
+	    int j = INTEGER(sindx)[i] - 1;
+	    if(STRING_ELT(s, i) != NA_STRING && CHAR(STRING_ELT(s, i))[0]) {
+		sub = pindx[j];
+		SET_VECTOR_ELT(indexnames, i, STRING_ELT(s, j));
+	    }
 	}
 	if (sub == 0) {
 	    if (!canstretch) {
@@ -883,9 +886,12 @@ stringSubscript(SEXP s, R_xlen_t ns, R_xlen_t nx, SEXP names,
        subscript vector. */
     if (extra != nnames)
 	setAttrib(indx, R_UseNamesSymbol, indexnames);
+    if (sindx != NULL) {
+	UNPROTECT(2);
+    }
     if (canstretch)
 	*stretch = extra;
-    UNPROTECT(4);
+    UNPROTECT(3);
     return indx;
 }
 
