@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1995, 1996, 2017  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2016  The R Core Team
+ *  Copyright (C) 1997--2019  The R Core Team
+ *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -879,14 +879,11 @@ static SEXP match_transform(SEXP s, SEXP env)
 // workhorse of R's match() and hence also  " ix %in% itable "
 SEXP match5(SEXP itable, SEXP ix, int nmatch, SEXP incomp, SEXP env)
 {
-    SEXP ans, x, table;
-    SEXPTYPE type;
-    HashData data;
-
     R_xlen_t n = xlength(ix);
-
     /* handle zero length arguments */
     if (n == 0) return allocVector(INTSXP, 0);
+
+    SEXP ans;
     if (length(itable) == 0) {
 	ans = allocVector(INTSXP, n);
 	int *pa = INTEGER0(ans);
@@ -895,10 +892,11 @@ SEXP match5(SEXP itable, SEXP ix, int nmatch, SEXP incomp, SEXP env)
     }
 
     int nprot = 0;
-    PROTECT(x	  = match_transform(ix,	    env)); nprot++;
-    PROTECT(table = match_transform(itable, env)); nprot++;
+    SEXP x     = PROTECT(match_transform(ix,     env)); nprot++;
+    SEXP table = PROTECT(match_transform(itable, env)); nprot++;
     /* or should we use PROTECT_WITH_INDEX and REPROTECT below ? */
 
+    SEXPTYPE type;
     /* Coerce to a common type; type == NILSXP is ok here.
      * Note that above we coerce factors and "POSIXlt", only to character.
      * Hence, coerce to character or to `higher' type
@@ -911,11 +909,6 @@ SEXP match5(SEXP itable, SEXP ix, int nmatch, SEXP incomp, SEXP env)
     // special case scalar x -- for speed only :
     if(XLENGTH(x) == 1 && !incomp) {
       int val = nmatch;
-      /* FIXME:
-	 The code for the special case uses LENGTH(table),
-	 the code for the regular case uses length(table):
-	 These should use the same.
-      */
       int ntable = LENGTH(table);
       switch (type) {
       case STRSXP: {
@@ -973,7 +966,7 @@ SEXP match5(SEXP itable, SEXP ix, int nmatch, SEXP incomp, SEXP env)
       PROTECT(ans = ScalarInteger(val)); nprot++;
     }
     else { // regular case
-
+	HashData data;
 	if (incomp) { PROTECT(incomp = coerceVector(incomp, type)); nprot++; }
 	data.nomatch = nmatch;
 	HashTableSetup(table, &data, NA_INTEGER);
@@ -981,7 +974,7 @@ SEXP match5(SEXP itable, SEXP ix, int nmatch, SEXP incomp, SEXP env)
 	    Rboolean useBytes = FALSE;
 	    Rboolean useUTF8 = FALSE;
 	    Rboolean useCache = TRUE;
-	    for(R_xlen_t i = 0; i < length(x); i++) {
+	    for(R_xlen_t i = 0; i < xlength(x); i++) {
 		SEXP s = STRING_ELT(x, i);
 		if(IS_BYTES(s)) {
 		    useBytes = TRUE;
@@ -997,7 +990,7 @@ SEXP match5(SEXP itable, SEXP ix, int nmatch, SEXP incomp, SEXP env)
 		}
 	    }
 	    if(!useBytes || useCache) {
-		for(int i = 0; i < length(table); i++) {
+		for(int i = 0; i < LENGTH(table); i++) {
 		    SEXP s = STRING_ELT(table, i);
 		    if(IS_BYTES(s)) {
 			useBytes = TRUE;
@@ -1414,7 +1407,7 @@ SEXP attribute_hidden do_matchcall(SEXP call, SEXP op, SEXP args, SEXP env)
     /* Get the formals and match the actual args */
 
     formals = FORMALS(b);
-    PROTECT(actuals = duplicate(CDR(funcall)));
+    PROTECT(actuals = shallow_duplicate(CDR(funcall)));
 
     /* If there is a ... symbol then expand it out in the sysp env
        We need to take some care since the ... might be in the middle
@@ -1479,7 +1472,7 @@ SEXP attribute_hidden do_matchcall(SEXP call, SEXP op, SEXP args, SEXP env)
     rlist = StripUnmatched(rlist);
 
     PROTECT(rval = allocSExp(LANGSXP));
-    SETCAR(rval, duplicate(CAR(funcall)));
+    SETCAR(rval, lazy_duplicate(CAR(funcall)));
     SETCDR(rval, rlist);
     UNPROTECT(3);
     return rval;
