@@ -48,8 +48,15 @@ static void R_wfixslash(wchar_t *s);
 
 extern "C" {
 #endif
+
+#if defined FC_LEN_T
+# include <stddef.h>
+void F77_SYMBOL(rwarnc)(char *msg, int *nchar, FC_LEN_T msg_len);
+void NORET F77_SYMBOL(rexitc)(char *msg, int *nchar, FC_LEN_T msg_len);
+#else
 void F77_SYMBOL(rwarnc)(char *msg, int *nchar);
 void NORET F77_SYMBOL(rexitc)(char *msg, int *nchar);
+#endif
 
 #ifdef __cplusplus
 }
@@ -1604,7 +1611,11 @@ void R_fixbackslash(char *s)
 }
 #endif
 
+#if defined FC_LEN_T
+void NORET F77_SYMBOL(rexitc)(char *msg, int *nchar, FC_LEN_T msg_len)
+#else
 void NORET F77_SYMBOL(rexitc)(char *msg, int *nchar)
+#endif
 {
     int nc = *nchar;
     char buf[256];
@@ -1617,7 +1628,11 @@ void NORET F77_SYMBOL(rexitc)(char *msg, int *nchar)
     error("%s", buf);
 }
 
+#if defined FC_LEN_T
+void F77_SYMBOL(rwarnc)(char *msg, int *nchar, FC_LEN_T msg_len)
+#else
 void F77_SYMBOL(rwarnc)(char *msg, int *nchar)
+#endif
 {
     int nc = *nchar;
     char buf[256];
