@@ -2257,7 +2257,7 @@ static void X11_Raster(unsigned int *raster, int w, int h,
         imageWidth = (int) -(width - .5);
         x = x - imageWidth*cos(angle);
         if (angle != 0)
-            y = y - imageWidth*sin(angle);
+            y = y + imageWidth*sin(angle);
         invertX = 1;
     } else {
         imageWidth = (int) (width + .5);
@@ -2273,6 +2273,16 @@ static void X11_Raster(unsigned int *raster, int w, int h,
                          rasterImage, imageWidth, imageHeight);
     }
     
+    if (invertX || invertY) {
+        unsigned int *flippedRaster;
+
+        flippedRaster = (unsigned int *) R_alloc(imageWidth * imageHeight,
+                                                 sizeof(unsigned int));
+        flipRaster(rasterImage, imageWidth, imageHeight, 
+                   invertX, invertY, flippedRaster);
+        rasterImage = flippedRaster;
+    }
+
     if (rot != 0) {
         
         int newW, newH;
@@ -2304,16 +2314,6 @@ static void X11_Raster(unsigned int *raster, int w, int h,
         imageHeight = newH;
     }
 
-    if (invertX || invertY) {
-        unsigned int *flippedRaster;
-
-        flippedRaster = (unsigned int *) R_alloc(imageWidth * imageHeight,
-                                                 sizeof(unsigned int));
-        flipRaster(rasterImage, imageWidth, imageHeight, 
-                   invertX, invertY, flippedRaster);
-        rasterImage = flippedRaster;
-    }
-
     image = XCreateImage(display, visual, depth,
                          ZPixmap,
                          0, /* offset */
