@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2000-2015   The R Core Team.
+ *  Copyright (C) 2000-2019   The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -32,7 +32,8 @@
 #include <errno.h>
 #include <R_ext/Print.h>
 
-static void *in_R_HTTPOpen(const char *url, const char *headers, const int cacheOK);
+static void *in_R_HTTPOpen(const char *url, const char *agent, const char *headers,
+			   int cacheOK);
 static int   in_R_HTTPRead(void *ctx, char *dest, int len);
 static void  in_R_HTTPClose(void *ctx);
 
@@ -43,18 +44,18 @@ static void  in_R_FTPClose(void *ctx);
 SEXP in_do_curlVersion(SEXP call, SEXP op, SEXP args, SEXP rho);
 SEXP in_do_curlGetHeaders(SEXP call, SEXP op, SEXP args, SEXP rho);
 SEXP in_do_curlDownload(SEXP call, SEXP op, SEXP args, SEXP rho);
-Rconnection 
-in_newCurlUrl(const char *description, const char * const mode, int type);
+Rconnection
+in_newCurlUrl(const char *description, const char * const mode, SEXP headers, int type);
 
 #ifdef Win32
-static void *in_R_HTTPOpen2(const char *url, const char *headers, const int cacheOK);
+static void *in_R_HTTPOpen2(const char *url, const char *agent, const char *headers, int cacheOK);
 static int   in_R_HTTPRead2(void *ctx, char *dest, int len);
 static void  in_R_HTTPClose2(void *ctx);
 static void *in_R_FTPOpen2(const char *url);
 
-#define Ri_HTTPOpen(url, headers, cacheOK) \
-    (meth ? in_R_HTTPOpen2(url, headers, cacheOK) : \
-	in_R_HTTPOpen(url, headers, cacheOK));
+#define Ri_HTTPOpen(url, agent, headers, cacheOK)	   \
+    (meth ? in_R_HTTPOpen2(url, agent, headers, cacheOK) : \
+       in_R_HTTPOpen(url, agent, headers, cacheOK));
 
 #define Ri_HTTPRead(ctx, dest, len) \
     (meth ? in_R_HTTPRead2(ctx, dest, len) : in_R_HTTPRead(ctx, dest, len))
@@ -110,24 +111,25 @@ static Rboolean url_open(Rconnection con)
     switch(type) {
 #ifdef Win32
     case HTTPSsh:
-	    warning(_("for https:// URLs use setInternet2(TRUE)"));
+	    warning(_("for https:// URLs use method = \"wininet\""));
 	    return FALSE;
 #endif
     case HTTPsh:
     {
-	SEXP sheaders, agentFun;
-	const char *headers;
+	SEXP sagent, agentFun;
+	const char *agent;
 	SEXP s_makeUserAgent = install("makeUserAgent");
 	agentFun = PROTECT(lang1(s_makeUserAgent)); // defaults to ,TRUE
 	SEXP utilsNS = PROTECT(R_FindNamespace(mkString("utils")));
-	sheaders = eval(agentFun, utilsNS);
+	struct urlconn *uc = con->private;
+	sagent = eval(agentFun, utilsNS);
 	UNPROTECT(1); /* utilsNS */
-	PROTECT(sheaders);
-	if(TYPEOF(sheaders) == NILSXP)
-	    headers = NULL;
+	PROTECT(sagent);
+	if(TYPEOF(sagent) == NILSXP)
+	    agent = NULL;
 	else
-	    headers = CHAR(STRING_ELT(sheaders, 0));
-	ctxt = in_R_HTTPOpen(url, headers, 0);
+	    agent = CHAR(STRING_ELT(sagent, 0));
+	ctxt = in_R_HTTPOpen(url, agent, uc->headers, 0);
 	UNPROTECT(2);
 	if(ctxt == NULL) {
 	  /* if we call error() we get a connection leak*/
@@ -150,7 +152,7 @@ static Rboolean url_open(Rconnection con)
 	break;
 
     default:
-	warning(_("URL scheme unsupported by this method"));
+	warning(_("scheme not supported in URL '%s'"), url);
 	return FALSE;
     }
 
@@ -167,13 +169,15 @@ static Rboolean url_open(Rconnection con)
 static void url_close(Rconnection con)
 {
     UrlScheme type = ((Rurlconn)(con->private))->type;
+    struct urlconn *uc = con->private;
     switch(type) {
     case HTTPsh:
     case HTTPSsh:
-	in_R_HTTPClose(((Rurlconn)(con->private))->ctxt);
+	if (uc && uc->headers) free(uc->headers);
+	in_R_HTTPClose(uc->ctxt);
 	break;
     case FTPsh:
-	in_R_FTPClose(((Rurlconn)(con->private))->ctxt);
+	in_R_FTPClose(uc->ctxt);
 	break;
     default:
 	break;
@@ -239,16 +243,17 @@ static Rboolean url_open2(Rconnection con)
     case HTTPSsh:
     case HTTPsh:
     {
-	SEXP sheaders, agentFun;
-	const char *headers;
+	SEXP sagent, agentFun;
+	const char *agent;
 	SEXP s_makeUserAgent = install("makeUserAgent");
+	struct urlconn * uc = con->private;
 	agentFun = PROTECT(lang2(s_makeUserAgent, ScalarLogical(0)));
-	sheaders = PROTECT(eval(agentFun, R_FindNamespace(mkString("utils"))));
-	if(TYPEOF(sheaders) == NILSXP)
-	    headers = NULL;
+	sagent = PROTECT(eval(agentFun, R_FindNamespace(mkString("utils"))));
+	if(TYPEOF(sagent) == NILSXP)
+	    agent = NULL;
 	else
-	    headers = CHAR(STRING_ELT(sheaders, 0));
-	ctxt = in_R_HTTPOpen2(url, headers, 0);
+	    agent = CHAR(STRING_ELT(sagent, 0));
+	ctxt = in_R_HTTPOpen2(url, agent, uc->headers, 0);
 	UNPROTECT(2);
 	if(ctxt == NULL) {
 	  /* if we call error() we get a connection leak*/
@@ -271,7 +276,7 @@ static Rboolean url_open2(Rconnection con)
 	break;
 
     default:
-	warning(_("URL scheme unsupported by this method"));
+	warning(_("scheme not supported in URL '%s'"), url);
 	return FALSE;
     }
 
@@ -339,22 +344,23 @@ static size_t url_read2(void *ptr, size_t size, size_t nitems,
 }
 #endif
 
-static Rconnection 
-in_R_newurl(const char *description, const char * const mode, int type)
+static Rconnection
+in_R_newurl(const char *description, const char * const mode, SEXP headers, int type)
 {
     Rconnection new;
-
     new = (Rconnection) malloc(sizeof(struct Rconn));
     if(!new) error(_("allocation of url connection failed"));
     new->class = (char *) malloc(strlen("url-wininet") + 1);
     if(!new->class) {
 	free(new);
 	error(_("allocation of url connection failed"));
+        /* for Solaris 12.5 */ new = NULL;
     }
     new->description = (char *) malloc(strlen(description) + 1);
     if(!new->description) {
 	free(new->class); free(new);
 	error(_("allocation of url connection failed"));
+        /* for Solaris 12.5 */ new = NULL;
     }
     init_con(new, description, CE_NATIVE, mode);
     new->canwrite = FALSE;
@@ -375,10 +381,20 @@ in_R_newurl(const char *description, const char * const mode, int type)
 	strcpy(new->class, "url");
     }
     new->fgetc = &dummy_fgetc;
-    new->private = (void *) malloc(sizeof(struct urlconn));
+    struct urlconn *uc = new->private = (void *) malloc(sizeof(struct urlconn));
     if(!new->private) {
 	free(new->description); free(new->class); free(new);
 	error(_("allocation of url connection failed"));
+	/* for Solaris 12.5 */ new = NULL;
+    }
+    uc->headers = NULL;
+    if(!isNull(headers)) {
+	uc->headers = strdup(CHAR(STRING_ELT(headers, 0)));
+	if(!uc->headers) {
+	    free(new->description); free(new->class); free(new->private); free(new);
+	    error(_("allocation of url connection failed"));
+	    /* for Solaris 12.5 */ new = NULL;
+	}
     }
 
     IDquiet = TRUE;
@@ -440,7 +456,7 @@ static void doneprogressbar(void *data)
 #define IBUFSIZE 4096
 static SEXP in_do_download(SEXP args)
 {
-    SEXP scmd, sfile, smode;
+    SEXP scmd, sfile, smode, sheaders;
     const char *url, *file, *mode;
     int quiet, status = 0, cacheOK;
 #ifdef Win32
@@ -467,15 +483,19 @@ static SEXP in_do_download(SEXP args)
     if(!isString(smode) || length(smode) != 1)
 	error(_("invalid '%s' argument"), "mode");
     mode = CHAR(STRING_ELT(smode, 0));
-    cacheOK = asLogical(CAR(args));
+    cacheOK = asLogical(CAR(args)); args = CDR(args);
     if(cacheOK == NA_LOGICAL)
 	error(_("invalid '%s' argument"), "cacheOK");
+    Rboolean file_URL = (strncmp(url, "file://", 7) == 0);
+    sheaders = CAR(args);
+    if(TYPEOF(sheaders) != NILSXP && !isString(sheaders))
+        error(_("invalid '%s' argument"), "headers");
 #ifdef Win32
     int meth = asLogical(CADR(args));
     if(meth == NA_LOGICAL)
 	error(_("invalid '%s' argument"), "method");
-    if(meth == 0) meth = UseInternet2;
-    if (R_Interactive && !quiet && !pbar.wprog) {
+//    if(meth == 0) meth = UseInternet2;
+    if (!file_URL && R_Interactive && !quiet && !pbar.wprog) {
 	pbar.wprog = newwindow(_("Download progress"), rect(0, 0, 540, 100),
 			       Titlebar | Centered);
 	setbackground(pbar.wprog, dialog_bg());
@@ -484,7 +504,7 @@ static SEXP in_do_download(SEXP args)
 	pbar.pc = 0;
     }
 #endif
-    if(strncmp(url, "file://", 7) == 0) {
+    if(file_URL) {
 	FILE *in, *out;
 	static char buf[CPBUFSIZE];
 	size_t n;
@@ -495,7 +515,7 @@ static SEXP in_do_download(SEXP args)
 	if (strlen(url) > 9 && url[7] == '/' && url[9] == ':') nh = 8;
 #endif
 
-	/* Use binary transfers */
+	/* Use binary transfers? */
 	in = R_fopen(R_ExpandFileName(url+nh), (mode[2] == 'b') ? "rb" : "r");
 	if(!in) {
 	    error(_("cannot open URL '%s', reason '%s'"),
@@ -514,7 +534,6 @@ static SEXP in_do_download(SEXP args)
 	}
 	fclose(out); fclose(in);
 
-#ifdef HAVE_INTERNET
     } else if (strncmp(url, "http://", 7) == 0
 #ifdef Win32
 	       || ((strncmp(url, "https://", 8) == 0) && meth)
@@ -539,7 +558,7 @@ static SEXP in_do_download(SEXP args)
 
 	R_Busy(1);
 	if(!quiet) REprintf(_("trying URL '%s'\n"), url);
-	SEXP agentFun, sheaders;
+	SEXP agentFun, sagent;
 #ifdef Win32
 	R_FlushConsole();
 	if(meth)
@@ -550,12 +569,15 @@ static SEXP in_do_download(SEXP args)
 	agentFun = PROTECT(lang1(install("makeUserAgent")));
 #endif
 	SEXP utilsNS = PROTECT(R_FindNamespace(mkString("utils")));
-	sheaders = eval(agentFun, utilsNS);
+	sagent = eval(agentFun, utilsNS);
 	UNPROTECT(1); /* utilsNS */
-	PROTECT(sheaders);
-	const char *headers = (TYPEOF(sheaders) == NILSXP) ?
+	PROTECT(sagent);
+	const char *cagent = (TYPEOF(sagent) == NILSXP) ?
+	    NULL : CHAR(STRING_ELT(sagent, 0));
+	/* TODO: flatten headers */
+	const char *cheaders = (TYPEOF(sheaders) == NILSXP) ?
 	    NULL : CHAR(STRING_ELT(sheaders, 0));
-	ctxt = Ri_HTTPOpen(url, headers, cacheOK);
+	ctxt = Ri_HTTPOpen(url, cagent, cheaders, cacheOK);
 	UNPROTECT(2);
 	if(ctxt == NULL) status = 1;
 	else {
@@ -641,6 +663,7 @@ static SEXP in_do_download(SEXP args)
 			(double)nbytes, (double)total);
 	}
 	fclose(out);
+	if (status == 1 && strchr(mode, 'w')) unlink(R_ExpandFileName(file));
 	R_Busy(0);
 	if (status == 1) error(_("cannot open URL '%s'"), url);
 
@@ -687,7 +710,7 @@ static SEXP in_do_download(SEXP args)
 		setprogressbar(pbar.pb, 0);
 		settext(pbar.wprog, "Download progress");
 		show(pbar.wprog);
-		
+
 		/* set up a context which will close progressbar on error. */
 		begincontext(&(pbar.cntxt), CTXT_CCODE, R_NilValue, R_NilValue,
 			     R_NilValue, R_NilValue, R_NilValue);
@@ -753,35 +776,46 @@ static SEXP in_do_download(SEXP args)
 	}
 	R_Busy(0);
 	fclose(out);
+	if (status == 1 && strchr(mode, 'w')) unlink(R_ExpandFileName(file));
 	if (status == 1) error(_("cannot open URL '%s'"), url);
-#endif
-
     } else
-	error(_("unsupported URL scheme"));
+	error(_("scheme not supported in URL '%s'"), url);
 
     return ScalarInteger(status);
 }
 
 
-#if defined(SUPPORT_LIBXML)
-
-void *in_R_HTTPOpen(const char *url, const char *headers, const int cacheOK)
+void *in_R_HTTPOpen(const char *url, const char *agent, const char *headers, int cacheOK)
 {
     inetconn *con;
     void *ctxt;
     int timeout = asInteger(GetOption1(install("timeout")));
     DLsize_t len = -1;
     char *type = NULL;
+    char *fullheaders = NULL;
 
     if(timeout == NA_INTEGER || timeout <= 0) timeout = 60;
 
     RxmlNanoHTTPTimeout(timeout);
-    ctxt = RxmlNanoHTTPOpen(url, NULL, headers, cacheOK);
+
+    if (agent || headers) {
+	fullheaders = malloc((agent ? strlen(agent) : 0) +
+			     (headers ? strlen(headers) : 0) + 1);
+	if(!fullheaders) error(_("could not allocate memory for http headers"));
+	fullheaders[0] = '\0';
+	if (agent) strcat(fullheaders, agent);
+	if (headers) strcat(fullheaders, headers);
+    }
+
+    ctxt = RxmlNanoHTTPOpen(url, NULL, fullheaders, cacheOK);
+    if (fullheaders) free(fullheaders);
+
     if(ctxt != NULL) {
 	int rc = RxmlNanoHTTPReturnCode(ctxt);
 	if(rc != 200) {
-	    warning(_("cannot open URL '%s': HTTP status was '%d %s'"), 
-		    url, rc, RxmlNanoHTTPStatusMsg(ctxt));
+	    // FIXME: should this be ctxt->location, after redirection?
+	    warning(_("cannot open URL '%s': %s status was '%d %s'"),
+		    url, "HTTP", rc, RxmlNanoHTTPStatusMsg(ctxt));
 	    RxmlNanoHTTPClose(ctxt);
 	    return NULL;
 	} else {
@@ -794,7 +828,7 @@ void *in_R_HTTPOpen(const char *url, const char *headers, const int cacheOK)
 		    REprintf(" length %0.0f bytes (%0.1f MB)\n", (double)len,
 			len/1024.0/1024.0);
 		else if(len > 10240)
-		    REprintf(" length %d bytes (%d KB)\n", 
+		    REprintf(" length %d bytes (%d KB)\n",
 			     (int)len, (int)(len/1024));
 		else if(len >= 0)
 		    REprintf(" length %d bytes\n", (int)len);
@@ -869,7 +903,6 @@ static void in_R_FTPClose(void *ctx)
 	free(ctx);
     }
 }
-#endif /* SUPPORT_LIBXML */
 
 
 #ifdef Win32
@@ -884,28 +917,29 @@ typedef struct wictxt {
     HINTERNET session;
 } wIctxt, *WIctxt;
 
-static void *in_R_HTTPOpen2(const char *url, const char *headers,
-			    const int cacheOK)
+static void *in_R_HTTPOpen2(const char *url, const char *agent, const char *headers,
+			    int cacheOK)
 {
     WIctxt  wictxt;
-    DWORD status, d1 = 4, d2 = 0, d3 = 100;
+    DWORD status = 0, len = 0, d1 = 4, d2 = 0, d3 = 100;
     char buf[101], *p;
 
     wictxt = (WIctxt) malloc(sizeof(wIctxt));
     wictxt->length = -1;
     wictxt->type = NULL;
     wictxt->hand =
-	InternetOpen(headers, INTERNET_OPEN_TYPE_PRECONFIG, NULL, NULL, 0);
+	InternetOpen(agent, INTERNET_OPEN_TYPE_PRECONFIG, NULL, NULL, 0);
     if(!wictxt->hand) {
 	free(wictxt);
 	/* error("cannot open Internet connection"); */
 	return NULL;
     }
 
-    // use keep-alive semantics, do not use local WinInet cache.
-    DWORD flags = INTERNET_FLAG_KEEP_CONNECTION | INTERNET_FLAG_NO_CACHE_WRITE;
+    // use keep-alive semantics, do not use local WinINet cache.
+    DWORD flags = INTERNET_FLAG_KEEP_CONNECTION | INTERNET_FLAG_NO_CACHE_WRITE |
+                  INTERNET_FLAG_IGNORE_REDIRECT_TO_HTTPS | INTERNET_FLAG_IGNORE_REDIRECT_TO_HTTP;
     if(!cacheOK) flags |= INTERNET_FLAG_PRAGMA_NOCACHE;
-    wictxt->session = InternetOpenUrl(wictxt->hand, url, NULL, 0, flags, 0);
+    wictxt->session = InternetOpenUrl(wictxt->hand, url, headers, headers ? -1 : 0, flags, 0);
     if(!wictxt->session) {
 	DWORD err1 = GetLastError(), err2, blen = 101;
 	InternetCloseHandle(wictxt->hand);
@@ -946,8 +980,8 @@ static void *in_R_HTTPOpen2(const char *url, const char *headers,
 	InternetCloseHandle(wictxt->session);
 	InternetCloseHandle(wictxt->hand);
 	free(wictxt);
-	warning(_("cannot open URL '%s': HTTP status was '%d %s'"), 
-		url, status, buf);
+	warning(_("cannot open URL '%s': %s status was '%d %s'"),
+		url, "HTTP", status, buf);
 	return NULL;
     }
 
@@ -955,20 +989,22 @@ static void *in_R_HTTPOpen2(const char *url, const char *headers,
 		  HTTP_QUERY_CONTENT_TYPE, &buf, &d3, &d2);
     d2 = 0;
     // NB: this can only retrieve in a DWORD, so up to 2GB or 4GB?
-    HttpQueryInfo(wictxt->session,
-		  HTTP_QUERY_CONTENT_LENGTH | HTTP_QUERY_FLAG_NUMBER,
-		  &status, &d1, &d2);
-    wictxt->length = status;
+    if (HttpQueryInfo(wictxt->session,
+		      HTTP_QUERY_CONTENT_LENGTH | HTTP_QUERY_FLAG_NUMBER,
+		      &len, &d1, &d2))
+	wictxt->length = len;
     wictxt->type = strdup(buf);
     if(!IDquiet) {
-	if(status > 1024*1024)
-	    REprintf("Content type '%s' length %0.0f bytes (%0.1f MB)\n",
-		     buf, (double) status, status/1024.0/1024.0);
-	else if(status > 10240)
-	    REprintf("Content type '%s' length %d bytes (%d KB)\n",
-		     buf, (int) status, (int) (status/1024));
-	else
-	    REprintf("Content type '%s' length %d bytes\n", buf, (int) status);
+	REprintf("Content type '%s'", buf);
+	if(len > 1024*1024)
+	    REprintf(" length %0.0f bytes (%0.1f MB)\n", (double)len,
+		     len/1024.0/1024.0);
+	else if(len > 10240)
+	    REprintf(" length %d bytes (%d KB)\n",
+		     (int)len, (int)(len/1024));
+	else if(wictxt->length >= 0) /* signed; len is not */
+	    REprintf(" length %d bytes\n", (int)len);
+	else REprintf(" length unknown\n", len);
 	R_FlushConsole();
     }
 
@@ -1039,37 +1075,6 @@ static void *in_R_FTPOpen2(const char *url)
 }
 #endif // Win32
 
-#ifndef HAVE_INTERNET
-static void *in_R_HTTPOpen(const char *url, const char *headers,
-			   const int cacheOK)
-{
-    return NULL;
-}
-
-static int in_R_HTTPRead(void *ctx, char *dest, int len)
-{
-    return -1;
-}
-
-static void in_R_HTTPClose(void *ctx)
-{
-}
-
-static void *in_R_FTPOpen(const char *url)
-{
-    return NULL;
-}
-
-static int in_R_FTPRead(void *ctx, char *dest, int len)
-{
-    return -1;
-}
-
-static void in_R_FTPClose(void *ctx)
-{
-}
-#endif
-
 
 #define MBUFSIZE 8192
 void RxmlMessage(int level, const char *format, ...)
