@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C)  2001-12   The R Core Team.
+ *  Copyright (C)  2001-2015   The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,8 +23,6 @@
 #include <config.h>
 #endif
 
-#ifdef HAVE_SOCKETS
-
 
 /* ------------------- socket connections  --------------------- */
 
@@ -104,7 +102,7 @@ static void sock_close(Rconnection con)
     con->isopen = FALSE;
 }
 
-static size_t sock_read_helper(Rconnection con, void *ptr, size_t size)
+static ssize_t sock_read_helper(Rconnection con, void *ptr, size_t size)
 {
     Rsockconn this = (Rsockconn)con->private;
     ssize_t res;
@@ -148,7 +146,7 @@ static size_t sock_read_helper(Rconnection con, void *ptr, size_t size)
 static int sock_fgetc_internal(Rconnection con)
 {
     unsigned char c;
-    size_t n;
+    ssize_t n;
 
     n = sock_read_helper(con, (char *)&c, 1);
     return (n == 1) ? c : R_EOF;
@@ -157,15 +155,17 @@ static int sock_fgetc_internal(Rconnection con)
 static size_t sock_read(void *ptr, size_t size, size_t nitems,
 			Rconnection con)
 {
-    return sock_read_helper(con, ptr, size * nitems)/size;
+    ssize_t n = sock_read_helper(con, ptr, size * nitems)/size;
+    return n > 0 ? n : 0;
 }
 
 static size_t sock_write(const void *ptr, size_t size, size_t nitems,
 			 Rconnection con)
 {
     Rsockconn this = (Rsockconn)con->private;
-
-    return R_SockWrite(this->fd, ptr, (int)(size * nitems), this->timeout)/size;
+    ssize_t n = R_SockWrite(this->fd, ptr, (int)(size * nitems),
+			    this->timeout)/size;
+    return n > 0 ? n : 0;
 }
 
 Rconnection in_R_newsock(const char *host, int port, int server,
@@ -179,12 +179,14 @@ Rconnection in_R_newsock(const char *host, int port, int server,
     if(!new->class) {
 	free(new);
 	error(_("allocation of socket connection failed"));
+        /* for Solaris 12.5 */ new = NULL;
     }
     strcpy(new->class, "sockconn");
     new->description = (char *) malloc(strlen(host) + 10);
     if(!new->description) {
 	free(new->class); free(new);
 	error(_("allocation of socket connection failed"));
+        /* for Solaris 12.5 */ new = NULL;
     }
     init_con(new, host, CE_NATIVE, mode);
     new->open = &sock_open;
@@ -198,11 +200,10 @@ Rconnection in_R_newsock(const char *host, int port, int server,
     if(!new->private) {
 	free(new->description); free(new->class); free(new);
 	error(_("allocation of socket connection failed"));
+	/* for Solaris 12.5 */ new = NULL;
     }
     ((Rsockconn)new->private)-> port = port;
     ((Rsockconn)new->private)-> server = server;
     ((Rsockconn)new->private)-> timeout = timeout;
     return new;
 }
-
-#endif /* HAVE_SOCKETS */
