@@ -1,6 +1,6 @@
 /*
  *  Mathlib : A C Library of Special Functions
- *  Copyright (C) 1998-2014 Ross Ihaka and the R Core team.
+ *  Copyright (C) 1998-2015 Ross Ihaka and the R Core team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, a copy is available at
- *  http://www.r-project.org/Licenses/
+ *  https://www.R-project.org/Licenses/
  */
 
 /*  DESCRIPTION --> see below */
@@ -35,6 +35,7 @@
 static void Y_bessel(double *x, double *alpha, int *nb,
 		     double *by, int *ncalc);
 
+// unused now from R
 double bessel_y(double x, double alpha)
 {
     int nb, ncalc;
@@ -55,9 +56,13 @@ double bessel_y(double x, double alpha)
     if (alpha < 0) {
 	/* Using Abramowitz & Stegun  9.1.2
 	 * this may not be quite optimal (CPU and accuracy wise) */
-	return(bessel_y(x, -alpha) * cospi(alpha) -
-	       ((alpha == na) ? 0 :
-		bessel_j(x, -alpha) * sinpi(alpha)));
+	return(((alpha - na == 0.5) ? 0 : bessel_y(x, -alpha) * cospi(alpha)) -
+	       ((alpha      == na ) ? 0 : bessel_j(x, -alpha) * sinpi(alpha)));
+    }
+    else if (alpha > 1e7) {
+	MATHLIB_WARNING(_("besselY(x, nu): nu=%g too large for bessel_y() algorithm"),
+			alpha);
+	return ML_NAN;
     }
     nb = 1+ (int)na;/* nb-1 <= alpha < nb */
     alpha -= (double)(nb-1);
@@ -79,7 +84,7 @@ double bessel_y(double x, double alpha)
 	    return ML_POSINF;
 	}
 	else if(ncalc < -1)
-	    MATHLIB_WARNING4(_("bessel_y(%g): ncalc (=%ld) != nb (=%ld); alpha=%g. Arg. out of range?\n"),
+	    MATHLIB_WARNING4(_("bessel_y(%g): ncalc (=%d) != nb (=%d); alpha=%g. Arg. out of range?\n"),
 			     x, ncalc, nb, alpha);
 	else /* ncalc >= 0 */
 	    MATHLIB_WARNING2(_("bessel_y(%g,nu=%g): precision lost in result\n"),
@@ -94,8 +99,8 @@ double bessel_y(double x, double alpha)
     return x;
 }
 
-/* modified version of bessel_y that accepts a work array instead of
-   allocating one. */
+/* Called from R: modified version of bessel_y(), accepting a work array
+ * instead of allocating one. */
 double bessel_y_ex(double x, double alpha, double *by)
 {
     int nb, ncalc;
@@ -113,9 +118,13 @@ double bessel_y_ex(double x, double alpha, double *by)
     if (alpha < 0) {
 	/* Using Abramowitz & Stegun  9.1.2
 	 * this may not be quite optimal (CPU and accuracy wise) */
-	return(bessel_y_ex(x, -alpha, by) * cospi(alpha) -
-	       ((alpha == na) ? 0 :
-		bessel_j_ex(x, -alpha, by) * sinpi(alpha)));
+	return(((alpha - na == 0.5) ? 0 : bessel_y_ex(x, -alpha, by) * cospi(alpha)) -
+	       ((alpha      == na ) ? 0 : bessel_j_ex(x, -alpha, by) * sinpi(alpha)));
+    }
+    else if (alpha > 1e7) {
+	MATHLIB_WARNING(_("besselY(x, nu): nu=%g too large for bessel_y() algorithm"),
+			alpha);
+	return ML_NAN;
     }
     nb = 1+ (int)na;/* nb-1 <= alpha < nb */
     alpha -= (double)(nb-1);
@@ -124,7 +133,7 @@ double bessel_y_ex(double x, double alpha, double *by)
 	if(ncalc == -1)
 	    return ML_POSINF;
 	else if(ncalc < -1)
-	    MATHLIB_WARNING4(_("bessel_y(%g): ncalc (=%ld) != nb (=%ld); alpha=%g. Arg. out of range?\n"),
+	    MATHLIB_WARNING4(_("bessel_y(%g): ncalc (=%d) != nb (=%d); alpha=%g. Arg. out of range?\n"),
 			     x, ncalc, nb, alpha);
 	else /* ncalc >= 0 */
 	    MATHLIB_WARNING2(_("bessel_y(%g,nu=%g): precision lost in result\n"),
