@@ -1,7 +1,7 @@
 /*
  *  Mathlib : A C Library of Special Functions
  *  Copyright (C) 1998 Ross Ihaka
- *  Copyright (C) 2000-2001 The R Core Team
+ *  Copyright (C) 2000-2014 The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, a copy is available at
- *  http://www.r-project.org/Licenses/
+ *  https://www.R-project.org/Licenses/
  *
  *  SYNOPSIS
  *
@@ -65,22 +65,29 @@ double beta(double a, double b)
 
     if (a < 0 || b < 0)
 	ML_ERR_return_NAN
-    else if (a == 0 || b == 0) {
+    else if (a == 0 || b == 0)
 	return ML_POSINF;
-    }
-    else if (!R_FINITE(a) || !R_FINITE(b)) {
+    else if (!R_FINITE(a) || !R_FINITE(b))
 	return 0;
-    }
 
-    if (a + b < xmax) /* ~= 171.61 for IEEE */
-	return gammafn(a) * gammafn(b) / gammafn(a+b);
-    else {
+    if (a + b < xmax) {/* ~= 171.61 for IEEE */
+//	return gammafn(a) * gammafn(b) / gammafn(a+b);
+	/* All the terms are positive, and all can be large for large
+	   or small arguments.  They are never much less than one.
+	   gammafn(x) can still overflow for x ~ 1e-308,
+	   but the result would too.
+	*/
+	return (1 / gammafn(a+b)) * gammafn(a) * gammafn(b);
+    } else {
 	double val = lbeta(a, b);
+// underflow to 0 is not harmful per se;  exp(-999) also gives no warning
+#ifndef IEEE_754
 	if (val < lnsml) {
 	    /* a and/or b so big that beta underflows */
 	    ML_ERROR(ME_UNDERFLOW, "beta");
 	    /* return ML_UNDERFLOW; pointless giving incorrect value */
 	}
+#endif
 	return exp(val);
     }
 }
