@@ -1,7 +1,7 @@
 /*
  *  Mathlib : A C Library of Special Functions
  *  Copyright (C) 1998 Ross Ihaka
- *  Copyright (C) 2000 The R Development Core Team
+ *  Copyright (C) 2000-2014 The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,8 +14,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
+ *  along with this program; if not, a copy is available at
+ *  https://www.R-project.org/Licenses/
  *
  *  DESCRIPTION
  *
@@ -25,22 +25,25 @@
 #include "nmath.h"
 #include "dpq.h"
 
-double dlnorm(double x, double logmean, double logsd, int give_log)
+double dlnorm(double x, double meanlog, double sdlog, int give_log)
 {
     double y;
 
 #ifdef IEEE_754
-    if (ISNAN(x) || ISNAN(logmean) || ISNAN(logsd))
-	return x + logmean + logsd;
+    if (ISNAN(x) || ISNAN(meanlog) || ISNAN(sdlog))
+	return x + meanlog + sdlog;
 #endif
-    if(logsd <= 0) ML_ERR_return_NAN;
-
+    if(sdlog <= 0) {
+	if(sdlog < 0) ML_ERR_return_NAN;
+	// sdlog == 0 :
+	return (log(x) == meanlog) ? ML_POSINF : R_D__0;
+    }
     if(x <= 0) return R_D__0;
 
-    y = (log(x) - logmean) / logsd;
+    y = (log(x) - meanlog) / sdlog;
     return (give_log ?
-	    -(M_LN_SQRT_2PI   + 0.5 * y * y + log(x * logsd)) :
-	    M_1_SQRT_2PI * exp(-0.5 * y * y)  /	 (x * logsd));
+	    -(M_LN_SQRT_2PI   + 0.5 * y * y + log(x * sdlog)) :
+	    M_1_SQRT_2PI * exp(-0.5 * y * y)  /	 (x * sdlog));
     /* M_1_SQRT_2PI = 1 / sqrt(2 * pi) */
 
 }
