@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2012 The R Core Team
+ *  Copyright (C) 2000-2015 The R Core Team
  *
  *  Algorithm AS 226 Appl. Statist. (1987) Vol. 36, No. 2
  *  by Russell V. Lenth
@@ -19,7 +19,7 @@
 #include "nmath.h"
 #include "dpq.h"
 
-long double attribute_hidden
+LDOUBLE attribute_hidden
 pnbeta_raw(double x, double o_x, double a, double b, double ncp)
 {
     /* o_x  == 1 - x  but maybe more accurate */
@@ -31,9 +31,9 @@ pnbeta_raw(double x, double o_x, double a, double b, double ncp)
 				     see PR#11277 */
 
     double a0, lbeta, c, errbd, x0, temp, tmp_c;
-    int j, ierr;
+    int ierr;
 
-    long double ans, ax, gx, q, sumq;
+    LDOUBLE ans, ax, gx, q, sumq;
 
     if (ncp < 0. || a <= 0. || b <= 0.) ML_ERR_return_NAN;
 
@@ -61,7 +61,7 @@ pnbeta_raw(double x, double o_x, double a, double b, double ncp)
     ans = ax = q * temp;
 
 	/* recurse over subsequent terms until convergence is achieved */
-    j = (int) x0;
+    double j = floor(x0); // x0 could be billions, and is in package EnvStats
     do {
 	j++;
 	temp -= (double) gx;
@@ -87,14 +87,24 @@ pnbeta2(double x, double o_x, double a, double b, double ncp,
 	/* o_x  == 1 - x  but maybe more accurate */
 	int lower_tail, int log_p)
 {
-    long double ans = pnbeta_raw(x, o_x, a,b, ncp);
+    LDOUBLE ans = pnbeta_raw(x, o_x, a,b, ncp);
 
     /* return R_DT_val(ans), but we want to warn about cancellation here */
-    if (lower_tail) return (double) (log_p ? logl(ans) : ans);
+    if (lower_tail)
+#ifdef HAVE_LONG_DOUBLE
+	return (double) (log_p ? logl(ans) : ans);
+#else
+	return log_p ? log(ans) : ans;
+#endif
     else {
-	if(ans > 1 - 1e-10) ML_ERROR(ME_PRECISION, "pnbeta");
-	ans = fmin2(ans, 1.0);  /* Precaution */
+	if (ans > 1. - 1e-10) ML_ERROR(ME_PRECISION, "pnbeta");
+	if (ans > 1.0) ans = 1.0;  /* Precaution */
+#if defined(HAVE_LONG_DOUBLE) && defined(HAVE_LOG1PL)
+	return (double) (log_p ? log1pl(-ans) : (1. - ans));
+#else
+	/* include standalone case */
 	return (double) (log_p ? log1p((double)-ans) : (1. - ans));
+#endif
     }
 }
 
