@@ -1,7 +1,7 @@
 /*
  *  Mathlib : A C Library of Special Functions
  *  Copyright (C) 1998 Ross Ihaka
- *  Copyright (C) 2000 The R Development Core Team
+ *  Copyright (C) 2000 The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, a copy is available at
- *  http://www.r-project.org/Licenses/
+ *  https://www.R-project.org/Licenses/
  *
  *  SYNOPSIS
  *
@@ -35,7 +35,7 @@
 
 double rchisq(double df)
 {
-    if (!R_FINITE(df) || df <= 0.0) ML_ERR_return_NAN;
+    if (!R_FINITE(df) || df < 0.0) ML_ERR_return_NAN;
 
     return rgamma(df / 2.0, 2.0);
 }
