@@ -14,7 +14,7 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, a copy is available at
-  http://www.r-project.org/Licenses/
+  https://www.R-project.org/Licenses/
 
   SYNOPSIS
 
@@ -315,7 +315,7 @@ double rwilcox(double m, double n)
     for (i = 0; i < k; i++)
 	x[i] = i;
     for (i = 0; i < n; i++) {
-	j = (int) floor(k * unif_rand());
+	j = (int) R_unif_index(k);
 	r += x[j];
 	x[j] = x[--k];
     }
