@@ -31,18 +31,25 @@ if(interactive()) {
                                         #   ^^^^^^ to show Matrix.msg()s
 
 ### Matrix() ''smartness''
-(d4 <- d40 <- Matrix(diag(4)))
+(d40 <- Matrix( diag(4)))
 (z4 <- Matrix(0*diag(4)))
 (o4 <- Matrix(1+diag(4)))
 (tr <- Matrix(cbind(1,0:1)))
 (M4 <- Matrix(m4 <- cbind(0,rbind(6*diag(3),0))))
 dM4 <- Matrix(M4, sparse = FALSE)
 d4. <- diag(4); dimnames(d4.) <- dns <- rep(list(LETTERS[1:4]), 2)
+d4a <- diag(4); dimnames(d4a) <- dna <- list(LETTERS[1:4], letters[1:4])# "a"symmetric
+m1a <- matrix(0, dimnames=list("A","b"))# "a"symmetric
 d4di<- as(d4., "diagonalMatrix")
+d4da<- as(d4a, "diagonalMatrix")
 d4d <- as(d4., "denseMatrix")
+d4aS <- Matrix(d4a, sparse=TRUE, doDiag=FALSE)
+d1aS <- Matrix(m1a, sparse=TRUE, doDiag=FALSE)
 stopifnot(identical(d4di@x, numeric()), # was "named" unnecessarily
-          identical(dimnames(d4 <- Matrix(d4.)), dns), identical(unname(d4), d40),
+          identical(dimnames(d4 <- Matrix(d4.)), dns),
+          identical4(d40, Matrix(diag(4)), unname(d4), unname(d4da)),
           identical3(d4, as(d4., "Matrix"), as(d4., "diagonalMatrix")),
+          is(d4aS, "dtCMatrix"), # not "dsC*", as asymmetric dimnames
           is(d4d, "denseMatrix"))
 
 class(mN <-  Matrix(NA, 3,4)) # NA *is* logical
@@ -69,6 +76,7 @@ stopifnot(all(is.na(sL@x)), ## not yet:  all(is.na(sL)),
           validObject(Matrix(c(NA,0), 4, 4)))
 stopifnotValid(Matrix(c(NA,0,0,0), 4, 4), "sparseMatrix")
 I <- i1 <- I1 <- Diagonal(1)
+## TODO? stopifnot(identical(I, Matrix(1, sparse=TRUE))) # doDiag=TRUE default
 I1[1,1] <- i1[1, ] <- I [ ,1] <- NA
 stopifnot(identical3(I,i1,I1))
 image(d4) # gave infinite recursion
@@ -100,6 +108,43 @@ stopifnot(E | as.vector(F), identical(E | F, F | E),
 	  all(e | f), all(E | F), # <- failed  Ops.spv.spv
 	  identical(Fv, base::as.vector(F)),
 	  is.logical(Fv), which(Fv) == c(2,5,8))
+F[-8:-1] # was partly "illegal" (length = 0 is ok; nnz '3' is not)
+F.ineg <- lapply(1:8, function(k) F[-8:-k])
+F.pos <-  lapply(1:8, function(k) F[seq_len(k-1)])
+F.vec <-  lapply(1:8, function(k) Fv[seq_len(k-1)])
+str(F.vec, vec=8)
+(nT <- vapply(F.vec, sum, 1L)) # == 0 0  1 1 1  2 2 2
+str(whichT <- lapply(F.vec, which))
+i.lengths <- function(L) vapply(L, function(.) length(.@i), 1L)
+stopifnot(identical(lengths(F.vec), 0:7)
+         ,
+          identical(lengths(F.ineg), 0:7)
+         ,
+          identical(lengths(F.pos),  0:7)
+         ,
+          identical(i.lengths(F.pos),  nT)
+         ,
+          identical(i.lengths(F.ineg), nT) # failed before 2018-03-19
+         ,
+          identical(lapply(F.pos, slot, "i"), whichT)
+         ,
+          identical(lapply(F.ineg, slot, "i"), whichT) # failed before 2018-03-19
+)
+## Here, sparseVector '[' is really wrong:
+SV <- new("nsparseVector", length = 30L,
+          i = c(1L, 8L, 9L, 12L, 13L, 18L, 21L, 22L))
+NI <- -c(1:5, 7:10, 12:16, 18:27, 29,30)
+sv <- SV[1:14]; ni <- -(1:14)[-c(6,11)] # smaller example
+selectMethod("[", c("nsparseVector","index","missing","missing"))# the culprit
+if(FALSE)
+  trace("[", browser, signature=c("nsparseVector","index","missing","missing"))
+stopifnot(
+    SV[NI] == as.vector(SV)[NI] ## badly failed before 2018-03-19
+    ,
+    sv[ni] == as.vector(sv)[ni] ## ditto
+)
+if(FALSE)
+    untrace("[", signature=c("nsparseVector","index","missing","missing"))
 
 dT <- new("dgTMatrix",
 	  i = c(1:2,1:2), j=rep(1:2, each=2), Dim = c(4L, 4L), x = c(1, 1, NA, 2))
@@ -111,6 +156,16 @@ isValid(lc <- c1 > c2,"lgCMatrix")
 isValid(lt <- dT > dt,"lgCMatrix")
 stopifnot(identical(lc,lt))
 
+## Versions of Diagonal()
+dD <- Diagonal(x = 5:1)
+sD <- .symDiagonal(5, 5:1)
+tD <- .trDiagonal (5, 5:1) # x=<integer> had failed for both
+stopifnot(all(sD == dD)
+        , all(tD == dD)
+        , identical(sD, .symDiagonal(x = 5:1))
+        , identical(tD, .sparseDiagonal(x=5:1))
+        , identical(tD, .trDiagonal (x = 5:1)))# 'n' now has default
+
 M <- Diagonal(4); M[1,2] <- 2 ; M
 cM <- crossprod(M) # >> as_cholmod_l_triplet(): could not reallocate for internal diagU2N()
 stopifnot(identical(cM, tcrossprod(t(M))))
@@ -269,11 +324,10 @@ e1 <- try(Lrg == Lrg) # ==> Cholmod error 'problem too large' at file ../Core/ch
 ## (error message almost ok)
 
 (memGB <- Sys.memGB("MemFree")) # from test-tools-1.R
-##                                             __vv__
-system.time( # ~10 sec.
+system.time( # ~10 sec.                            __vv__
     e2 <- if(doExtras && is.finite(memGB) && memGB > 30) { # need around 18 GB
               try(!Lrg) # now *works* on 64-bit machines with enough RAM
-              ## and immediately errors if LONG_VECTORs are not annvailable
+              ## and immediately errors if LONG_VECTORs are not available
     })
 str(e2) # error, NULL or "worked" (=> 50000 x 50000 lgeMatrix)
 ina <- is.na(Lrg)# "all FALSE"
@@ -284,6 +338,59 @@ stopifnot(grep("too large", e1) == 1,
 stopifnot(suppressWarnings(any(Lrg)))# (double -> logical  warning)
 rm(e2)# too large...
 
+RNGversion("3.6.0")# future proof
+if(doExtras && is.finite(memGB) && memGB > 24) # need around .. GB
+{
+    cat("computing SM .. \n")
+    showSys.time(m <- matrix(0, 3e6, 1024))
+    ##  user  system elapsed
+    ## 2.475  10.688  13.196  (faster in past ??)
+    set.seed(1); inot0 <- unique(sort(c(1, length(m), sample(length(m), 20))))
+    ai0 <- arrayInd(inot0, .dim=dim(m), useNames=FALSE)
+    showSys.time(m[inot0] <- 1:22)
+    ##  user  system elapsed
+    ## 5.931  11.184  17.162
+    showSys.time(SM  <- as(m, "sparseMatrix")) # ~ 8 sec
+    ## gave 'Error in asMethod(object) : negative length vectors are not allowed'
+    ## now works - via new C  matrix_to_Csparse()
+    showSys.time(n0.m <- c(m) != 0) # logical (full, base R) matrix, 12 GB
+    ##   user  system elapsed
+    ## 14.901  10.789  25.776
+    try( ## _FIXME_ in R: Error ... long vectors not supported yet
+        in0.m <- which(n0.m)
+    )
+    ## DONE: now very fast! [previously did coerce the whole matrix to dense first !]
+    subS <- SM[inot0]
+    selectMethod("[", c("dgCMatrix","numeric","missing","missing"))# -> .M.vectorSub(x,i)
+    ## Directly via arrayInd(), is *FAST*:
+    subSij <- SM[ai0]
+    stopifnot(subS == 1:22, identical(subS, subSij))
+    cat(" [Ok]\n")
+    rm(m)
+    str(SM)
+    ## checking SM:
+    TM <- as(SM, "TsparseMatrix")
+    stopifnot(as.matrix(summary(TM)) == cbind(ai0, 1:22))
+    ## cleanup:
+    rm(SM, TM)
+}
+
+## Constructing *packed* dense symmetric (dsp*) | triangular (dtp*) Matrices:
+if(doExtras && is.finite(memGB) && memGB > 35) { # need around 17.2 GB
+    m <- as.integer(2^16) ## = 65536
+    showSys.time(x <- rep(as.numeric(1:100), length.out=m*(m+1)/2))
+    ##  user  system elapsed
+    ## 6.028   8.964  15.074
+    print(object.size(x)) # 17'180'131'368 bytes: ~ 17 GB
+    mat <- new("dspMatrix", x = x, Dim = c(m, m)) # failed with
+    ## long vectors not supported yet: ../../src/include/Rinlinedfuns.h:...
+    validObject(mat)
+    mat <- new("dtpMatrix", x = x, Dim = c(m, m)) # failed .......
+    validObject(mat)
+    ## cleanup
+    rm(mat)
+}
+
 ## with dimnames:
 v <- c(a=1, b=2:3)
 m <- as.matrix(v)
@@ -635,9 +742,18 @@ stopifnot(## something like the equivalent of  all(I. == Diagonal(3125)) :
 
 ## printSpMatrix() ;  "suppressing (columns | rows) .." {and do it correctly!}
 IT3
-op <- options(width = 70, max.print = 1000)
+op0 <- options(width = 70, max.print = 1000)
 T125[-(1:50),] ## suppression ... is it correctly done?
 
+## Still buggy -- FIXME: see ../TODO --- even if we'd require max.print >= 5 or so
+for(mm in 1:21) {
+    options(max.print=mm)
+    cat("----------\n\nmax.print=",mm,":\n", sep="")
+    cat("\n>> U:   ") ; show(U)
+    cat("\n>> slp: ") ; show(slp)
+}
+
+options(op0)# revert to max.print = 1000
 
 ###-- row- and column operations  {was ./rowcolOps.R }
 
@@ -1028,7 +1144,10 @@ if(doExtras) {
     Sys.setlocale("LC_COLLATE", "C")    # to keep ls() reproducible
     for(nm in ls()) if(is(.m <- get(nm), "Matrix")) {
 	cat("\n", rep("-",nchar(nm)),"\n",nm, ":\n", sep='')
-	checkMatrix(.m)
+        if(nm %in% c("d4da")) # asymmetric dimnames in symmetricMatrix ..
+            cat(" __skipping__ check for ",nm," for now !! \n")
+        else
+            checkMatrix(.m)
     }
     cat('Time elapsed: ', proc.time() - .pt,'\n') # "stats"
 }
@@ -1059,6 +1178,6 @@ sessionInfo()
 c(Matrix = packageDescription("Matrix")$Built)
 if(SysI[["sysname"]] == "Linux" && require("sfsmisc")) local({
     nn <- names(.Sc <- sfsmisc::Sys.cpuinfo())
-    nn <- names(.Sc <- .Sc[nn != "flags"])
-    print(.Sc[grep("\\.[0-9]$", nn, invert=TRUE)])
+    nn <- names(.Sc <- .Sc[!grepl("^flags", nn)])
+    print(.Sc[ grep("\\.[0-9]+$", nn, invert=TRUE) ])
 })
