@@ -74,7 +74,8 @@ aeq(sfit$surv, truth0$surv)
 
 fit <- coxph(Surv(time, status) ~x, test1, eps=1e-8)
 aeq(round(fit$coef,6), 1.676857)
-truth <- byhand(fit$coef, 0)
+truebeta <- log(cos(acos((45/23)*sqrt(3/23))/3) * 2* sqrt(23/3))
+truth <- byhand(truebeta, 0)
 aeq(truth$loglik, fit$loglik[2])
 aeq(1/truth$imat, fit$var)
 aeq(truth$mart, fit$resid[c(2:6,1)])
