@@ -4,29 +4,30 @@
 ## so there may be differences from the reference ouptut,
 ## especially when alternative BLAS are used.
 
-options(digits=4)
+options(digits = 4L)
 
 ##    -------  examples from ?svd ---------
 
 hilbert <- function(n) { i <- 1:n; 1 / outer(i - 1, i, "+") }
 Eps <- 100 * .Machine$double.eps
 
-X <- hilbert(9)[,1:6]
-(s <- svd(X)); D <- diag(s$d)
+## The signs of the vectors are not determined here, so don't print
+X <- hilbert(9L)[, 1:6]
+s <- svd(X); D <- diag(s$d)
 stopifnot(abs(X - s$u %*% D %*% t(s$v)) < Eps)#  X = U D V'
 stopifnot(abs(D - t(s$u) %*% X %*% s$v) < Eps)#  D = U' X V
 
-# The signs of the vectors are not determined here.
+## ditto
 X <- cbind(1, 1:7)
 s <- svd(X); D <- diag(s$d)
 stopifnot(abs(X - s$u %*% D %*% t(s$v)) < Eps)#  X = U D V'
 stopifnot(abs(D - t(s$u) %*% X %*% s$v) < Eps)#  D = U' X V
 
 # test nu and nv
-s <- svd(X, nu = 0)
-s <- svd(X, nu = 7) # the last 5 columns are not determined here
-stopifnot(dim(s$u) == c(7,7))
-s <- svd(X, nv = 0)
+s <- svd(X, nu = 0L)
+s <- svd(X, nu = 7L) # the last 5 columns are not determined here
+stopifnot(dim(s$u) == c(7L,7L))
+s <- svd(X, nv = 0L)
 
 # test of complex case
 
@@ -39,7 +40,7 @@ stopifnot(abs(D - Conj(t(s$u)) %*% X %*% s$v) < Eps)
 
 ##  -------  tests of random real and complex matrices ------
 fixsign <- function(A) {
-    A[] <- apply(A, 2, function(x) x*sign(Re(x[1])))
+    A[] <- apply(A, 2L, function(x) x*sign(Re(x[1L])))
     A
 }
 ##			       100  may cause failures here.
@@ -49,7 +50,7 @@ eigenok <- function(A, E, Eps=1000*.Machine$double.eps)
     print(zapsmall(E$values))
     V <- E$vectors; lam <- E$values
     stopifnot(abs(A %*% V - V %*% diag(lam)) < Eps,
-              abs(lam[length(lam)]/lam[1]) < Eps || # this one not for singular A :
+              abs(lam[length(lam)]/lam[1]) < Eps | # this one not for singular A :
               abs(A - V %*% diag(lam) %*% t(V)) < Eps)
 }
 
