@@ -241,7 +241,7 @@ assert.EQ.mat(pd., as(pd1,"matrix"))
 assert.EQ.mat(pd..,as(pd2,"matrix"))
 
 ## check that 'solve' and '%*%' are inverses
-set.seed(1)
+suppressWarnings(RNGversion("3.5.0")); set.seed(1)
 A <- Matrix(rnorm(25), nc = 5)
 y <- rnorm(5)
 all.equal((A %*% solve(A, y))@x, y)
@@ -254,9 +254,11 @@ mLeft  <- matrix(data = double(0), nrow = 3, ncol = 0)
 mRight <- matrix(data = double(0), nrow = 0, ncol = 4)
 MLeft   <- Matrix(data = double(0), nrow = 3, ncol = 0)
 MRight  <- Matrix(data = double(0), nrow = 0, ncol = 4)
-stopifnot(identical3(class(mLeft), class(mRight), "matrix"))
-stopifnot(class(MLeft) ==  class(MRight),
-          class(MLeft) ==  "dgeMatrix")
+stopifnot(exprs = {
+    class(mLeft) ==  class(mRight)
+    class(MLeft) ==  class(MRight)
+    class(MLeft) == "dgeMatrix"
+})
 
 Qidentical3 <- function(a,b,c) Q.eq(a,b) && Q.eq(b,c)
 Qidentical4 <- function(a,b,c,d) Q.eq(a,b) && Q.eq(b,c) && Q.eq(c,d)
